/*************************************************************************
 *                          ACTIVE SILICON LIMITED                      
 *                                                                        
 *  File name   : xpl_sem.c
 *  Project     : ASLEXAMP maintained Snapper example code
 *  Author      : Mike Harrison, Martin Bone
 *  Systems     : ANSI C, POSIX & WIN32 threading.
 *  Written     : 4-MAR-1998
 *  Updated     : 
 *  Version     : 1:00:00                                                     
 *                                                                        
 *  Copyright 
 **************************************************************************
 * Comments: 
 * Assumptions:
 * Revisions:
 */

/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>


/* Semaphores depend on OS */
#if defined _LYNXOS || defined _SOLARIS2
/***************************************************************************/
ui32 ASL_Sem_Create( struct SemInfo *psSemInfo, ui32 dwInitialCount, ui32 dwMaximumCount )
{
   static char *szFnName = "ASL_Sem_Create";
#if  defined _LYNXOS 
   pthread_condattr_create( &psSemInfo->pcaCondAttr );   
   if( pthread_cond_init(   &psSemInfo->pcCondVar, &psSemInfo->pcaCondAttr ) ==  -1 )
   {
      DBG_WriteString( 8, "Failed to initialise condition variable attributes\n");
      return ASL_ERROR;
   }
   pthread_mutexattr_create( &psSemInfo->ptmaSemAttr );
   pthread_mutex_init( &psSemInfo->ptmSem, psSemInfo->ptmaSemAttr );   
#elif defined _SOLARIS2
   pthread_condattr_init( &psSemInfo->pcaCondAttr );
   if( pthread_cond_init( &psSemInfo->pcCondVar, &psSemInfo->pcaCondAttr ) ==  -1 )
   {
      DBG_WriteString( 8, "Failed to initialise condition variable attributes\n");
      return ASL_ERROR;
   }
   pthread_mutexattr_init( &psSemInfo->ptmaSemAttr );
   pthread_mutex_init( &psSemInfo->ptmSem, &psSemInfo->ptmaSemAttr );
#endif

   return ASL_OK;
}

ui32 ASL_Sem_Destroy( struct SemInfo *psSemInfo )
{
   static char *szFnName = "ASL_Sem_Destroy";
#if defined _LYNXOS   
   /* Closedown all thread mutex and signalling variables */
   pthread_cond_destroy( &psSemInfo->pcCondVar );
   pthread_condattr_delete( &psSemInfo->pcaCondAttr );
   /* Destroy thread mutex and attributes - always succeeds */
   pthread_mutex_destroy( &psSemInfo->ptmSem );
   pthread_mutexattr_delete( &psSemInfo->ptmaSemAttr );
#elif defined _SOLARIS2
   pthread_cond_destroy( &psSemInfo->pcCondVar );
   pthread_condattr_destroy( &psSemInfo->pcaCondAttr );
   /* Destroy thread mutex and attributes - always succeeds */
   pthread_mutex_destroy( &psSemInfo->ptmSem );
   pthread_mutexattr_destroy( &psSemInfo->ptmaSemAttr );
#endif
   return ASL_OK;
}

ui32 ASL_Sem_Signal( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "ASL_Sem_Signal";
   pthread_cond_signal( &psSemInfo->pcCondVar );
   return ASL_OK;
}

ui32 ASL_Sem_SaveParent( struct SemInfo *psParentSemInfo, struct SemInfo *psChildSemInfo  )
{
   static char *szFnName = "ASL_Sem_SaveParent";
   psChildSemInfo->ppcParentCondVar = &psParentSemInfo->pcCondVar;
   psChildSemInfo->pptmParentSem =    &psParentSemInfo->ptmSem;
   return ASL_OK;
}

ui32 ASL_Sem_SignalParent( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "ASL_Sem_SignalParent";
   pthread_cond_signal( psSemInfo->ppcParentCondVar );
   return ASL_OK;
}

/* Function to wait on mutex with timeout */
ui32 ASL_Sem_WaitWithTimeout( struct SemInfo *psSemInfo, ui32 dwWait )
{
   static char *szFnName = "ASL_Sem_WaitWithTimeout";
   struct timespec stTime;
   ui32 dwStatus = ASL_OK;
   
   /* Grab the lock associated with the condition variable */
   pthread_mutex_lock( &psSemInfo->ptmSem );

#if defined _LYNXOS
   /* Set wait timeout interval */
   stTime.tv_sec = dwWait; 
   stTime.tv_nsec = 0;
   if ( pthread_cond_timedwait( &psSemInfo->pcCondVar, &psSemInfo->ptmSem, &stTime ) != ASL_OK )
   {
      DBG_WriteString( 8, "%s: Failed to set suspend on timed wait\n", szFnName );
      if ( errno == EINTR )
      {
         DBG_WriteString( 8,"%s: signaled or timed out \n", szFnName );
         dwStatus = ASL_OK;
      }
      else if ( errno == EINVAL ) 
      {
         DBG_WriteString( 8, "%s:Condition arguments incorrect \n", szFnName );
         dwStatus = ASLERR_BAD_PARAM;
      }
      else
      {
         DBG_WriteString( 8, "%s: Unknown errno 0x%4x during timed wait\n", szFnName, errno );
         dwStatus = ASL_ERROR;
      }
    }

#elif defined _SOLARIS2
   /* Set wait timeout interval */
   stTime.tv_sec = time(NULL) + dwWait;
   stTime.tv_nsec = 0;
   if ( pthread_cond_timedwait( &psSemInfo->pcCondVar, &psSemInfo->ptmSem, &stTime ) != ASL_OK )
   {
        if ( errno == EINTR)
        {
           DBG_WriteString( 8, "%s: Wait on callback received system signal (^C ? ) n", szFnName );
           dwStatus = ASLERR_THREAD_ERROR;
        }
        else if ( errno == ETIMEDOUT )
        {
           DBG_WriteString( 8, "%s: Wait on condition variable timed out\n", szFnName );
           dwStatus = ASLERR_TIMEOUT;
        }
        else if ( errno == EINVAL ) 
        {
           DBG_WriteString( 8, "%s: Condition arguments incorrect\n", szFnName );
           dwStatus = ASLERR_BAD_PARAM;
        }
        else if ( errno == EFAULT )
        {
           DBG_WriteString( 8, "%s: Illegal address in argument passed to pthread_cond_timedwait\n", szFnName );
           dwStatus = ASLERR_BAD_PARAM;
        }
        else
        {
           DBG_WriteString( 8, "%s: Unknown errno 0x%04x in timed wait \n", szFnName, errno );
           dwStatus = ASLERR_SYSTEM_CALL_FAILED;
        }
   }
#endif 
   pthread_mutex_unlock( &psSemInfo->ptmSem );
   return dwStatus;
}
/* end of _LYNXOS & _SOLARIS2 */


#elif defined _WINNT || defined _WIN95
/***************************************************************************/
ui32 ASL_Sem_Create( struct SemInfo *psSemInfo, ui32 dwInitialCount, ui32 dwMaximumCount )
{
   static char *szFnName = "ASL_Sem_Create";
   if ( ( psSemInfo->hSem = CreateSemaphore( NULL, dwInitialCount, dwMaximumCount, NULL ) ) != NULL )
      return ASL_OK;
   else 
      return ASL_ERROR;
}

ui32 ASL_Sem_Destroy( struct SemInfo *psSemInfo )
{
   static char *szFnName = "ASL_Sem_Destroy";
   if( CloseHandle( (HANDLE)psSemInfo->hSem ) == TRUE )
      return ASL_OK;
   else
      return ASL_ERROR;
}

ui32 ASL_Sem_Signal( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "ASL_Sem_Signal";
   
   if ( ReleaseSemaphore( psSemInfo->hSem, (LONG) dwCount, NULL) == TRUE )
   {
      DBG_WriteString( 4, "%s: Semaphore signalled OK %d\n", szFnName, dwCount );
      return ASL_OK;
   }
   else
   {
      DBG_WriteString( 4, "%s: Semaphore signal failed %d\n", szFnName, dwCount );
      return ASL_ERROR;
   }
}

ui32 ASL_Sem_SaveParent( struct SemInfo *psParentSemInfo, struct SemInfo *psChildSemInfo  )
{
   static char *szFnName = "ASL_Sem_SaveParent";
   psChildSemInfo->hParentSem = psParentSemInfo->hSem;
   return ASL_OK;
}

ui32 ASL_Sem_SignalParent( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "ASL_Sem_SignalParent";
   if ( ReleaseSemaphore( psSemInfo->hParentSem, (LONG) dwCount, NULL ) == TRUE )
      return ASL_OK;
   else
      return ASL_ERROR;
}

/* Function to wait on mutex with timeout */
ui32 ASL_Sem_WaitWithTimeout( struct SemInfo *psSemInfo, ui32 dwWait )
{
   static char *szFnName = "ASL_Sem_WaitWithTimeout";
   ui32 dwStatus = ASL_OK;
   
   dwStatus = WaitForSingleObject( psSemInfo->hSem, dwWait * 1000 /* ms */ );
   if( dwStatus == WAIT_OBJECT_0 )
   {
      DBG_WriteString( 6, "%s: signalled successfully\n", szFnName);
      dwStatus = ASL_OK;
   }
   else if( dwStatus == WAIT_TIMEOUT )
   {
      DBG_WriteString( 8, "%s: wait timed out\n", szFnName);
      dwStatus = ASLERR_TIMEOUT;
   }
   else if( dwStatus == WAIT_ABANDONED ) 
   {
      DBG_WriteString( 8, "%s: waiting thread terminated\n", szFnName);
      dwStatus = ASLERR_THREAD_ERROR;
   }
   else if( dwStatus == WAIT_FAILED )
   {
       char *szBuffer;
       FormatMessage( 
       FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
       NULL,
       GetLastError(),
       MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
       (LPTSTR) &szBuffer,
       0,
       NULL 
       );
       dwStatus = ASLERR_SYSTEM_CALL_FAILED;
   }
   else
      dwStatus = ASL_ERROR; /* Unknown error */
   
   return dwStatus;
}
/* End of _WINNT and _WIN95 */


#elif defined_MACINTOSH
/***************************************************************************/
ui32 ASL_Sem_Create( struct SemInfo *psSemInfo, ui32 dwInitialCount, ui32 dwMaximumCount )
{
   static char *szFnName = "(MAC)ASL_Sem_Create";
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );
}

ui32 ASL_Sem_Destroy( struct SemInfo *psSemInfo )
{
   static char *szFnName = "(MAC)ASL_Sem_Destroy";
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );
}

ui32 ASL_Sem_Signal( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "(MAC)ASL_Sem_Signal";
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );
}

ui32 ASL_Sem_SaveParent( struct SemInfo *psParentSemInfo, struct SemInfo *psChildSemInfo  )
{
   static char *szFnName = "(MAC)ASL_Sem_SaveParent";
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );
}

ui32 ASL_Sem_SignalParent( struct SemInfo *psSemInfo, ui32 dwCount )
{
   static char *szFnName = "(MAC)ASL_Sem_SignalParent";
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );
}

/* Function to wait on mutex with timeout */
ui32 ASL_Sem_WaitWithTimeout( struct SemInfo *psSemInfo, ui32 dwWait )
{
   static char *szFnName = "(MAC)ASL_Sem_WaitWithTimeout";
   ui32 dwStatus = ASL_OK;

   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS doesn't support semaphors, mutexes or threads", 0, szFnName );   
}
/* end of _MACINTOSH */

#else
#if ! defined __MWERKS__
#error Undefined OS in thread.c
#endif
#endif
