/**************************************************************************
 *             ACTIVE SILICON LIMITED / ACTIVE_IMAGING LIMITED            *
 *                                                                        *
 *  Function    : Very Simple Snapper-8/24 Demo                           *
 *  Project     : Snappers                                                *
 *  Author      : Chris Beynon                                            *
 *  Systems     : Sun C v3                                                *
 *  Written     : Aug 1994                                                *
 *  Updated     : 01-Apr-96                                               *
 *  Version     : see below                                               *
 *  Comments    :                                                         *
 *                                                                        *
 **************************************************************************
 */

#include <asl_inc.h>


int main(ui16 argc, char** argv)
{
   char *fn_name = "main";
   Thandle Hsnp24;       /* Handle to 8/24 bit video capture board */
   Thandle Hbase;        /* Handle to SBus board */
   Thandle Hvid_image, Hrgb_image;
   Terr status;
   ui16 strip, total_strips;
   Terr module_ID;
   ui16 param;           /* base create parameter for board number */
   char *title = "s24sol";
   ui32 snapper_width;



  /*
   * Initialize baseboard and snapper module
   * ---------------------------------------
   */
   if (argc == 1)
      param = BASE_AUTO;
   else
      param = (ui16) (BASE_DEVICE | (ui16) atoi(argv[1]));

   if (ASL_is_err(status = BASE_create(param) ) )
   {
      fprintf(stderr, "\n%s: Failed to initialize Snapper hardware\n", title);
      exit(0);
   }
   else
   {
      Hbase = ASL_get_ret(status);
   }

   Hsnp24 = BASE_get_parameter(Hbase, BASE_MODULE_HANDLE);

   if (Hsnp24 == MODULE_NULL_HANDLE)
   {
      BASE_destroy(Hbase);
      Hbase = BASE_NULL_HANDLE;
      fprintf(stderr, "\n%s: No Snapper module fitted to baseboard (should be SNP-8 or SNP-24)\n", title);
      exit(0);
   }

   module_ID = ASL_get_ID(BASE_get_parameter(Hbase, BASE_MODULE_IDREV_VALUE));
   switch(module_ID & 0xF8)
   {
   case SNP24_ID:
   case SNP_PCI24_ID:
   case PMC_SNP24_ID:
      snapper_width = 24;
      break;
   case SNP8_ID:
   case SNP_PCI8_ID:
   case PMC_SNP8_ID:
      snapper_width = 8;
      break;
   default:
      BASE_destroy(Hbase);
      Hbase = BASE_NULL_HANDLE;
      fprintf(stderr, "\n%s: Incorrect Snapper module fitted (should be SNP-8 or SNP-24)\n", title);
      exit(0);
   }

   if (snapper_width == 8)
      fprintf(stderr, "\n%s: SBus Baseboard and Snapper-8 initialized OK\n", title);
   else
      fprintf(stderr, "\n%s: SBus Baseboard and Snapper-24 initialized OK\n", title);


   Hvid_image = TMG_image_create();
   Hrgb_image = TMG_image_create();


   /* Set required Snapper mode */
   SNP24_initialize(Hsnp24, SNP24_CCIR_DEFAULT);
   /* SNP24_initialize(Hsnp24, SNP24_EIA_DEFAULT); */

   SNP24_set_capture(Hsnp24, SNP24_SUB_X1 );

   if (snapper_width == 24)
      SNP24_set_format(Hsnp24, SNP24_FORMAT_RGB, TMG_XBGR32);
   else
      SNP24_set_format(Hsnp24, SNP24_FORMAT_Y8_ON_RED, TMG_Y8);

#if 0
   SNP24_set_sync(Hsnp24, SNP24_SYNC_OFF_CSYNC_NEG);
#endif


   /* Set up image parameters */
   SNP24_set_image(Hsnp24, Hvid_image);

   /* Delay to ensure Snapper locked before real image captured */
   SNP24_capture(Hsnp24, SNP24_START_AND_WAIT);
   SNP24_capture(Hsnp24, SNP24_START_AND_WAIT);
   SNP24_capture(Hsnp24, SNP24_START_AND_WAIT);
   SNP24_capture(Hsnp24, SNP24_START_AND_WAIT);

   if (ASL_is_ok(SNP24_capture(Hsnp24, SNP24_START_AND_WAIT)))
   {
      if (ASL_is_ok(SNP24_read_video_data(Hsnp24, Hvid_image, TMG_RUN)))
      {
         if (snapper_width == 24)
         {
            TMG_image_convert(Hvid_image, Hrgb_image, TMG_RGB24, 0, TMG_RUN);
            TMG_image_set_outfilename(Hrgb_image, "rgb24.tif");
            TMG_image_write(Hrgb_image, TMG_NULL, TMG_TIFF, TMG_RUN);
            printf("\n24 bit RGB TIFF file 'rgb24.tif' written OK\n");
         }
         else
         {
            TMG_image_set_outfilename(Hvid_image, "y8.tif");
            TMG_image_write(Hvid_image, TMG_NULL, TMG_TIFF, TMG_RUN);
            printf("\n8 bit mono TIFF file 'y8.tif' written OK\n");
         }
      }
      else
         printf("\nFailed to read video data");

   }
   else
      printf("\nFailed to capture video data");

   BASE_destroy(BASE_ALL_HANDLES);
   TMG_image_destroy(TMG_ALL_HANDLES);
}
