/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : asl_gen.h                                               *
 *  Function    : General purpose type definitions                        *
 *  Project     : General purpose                                         *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI                                                    *
 *  Written     : October 1993                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _ASL_GEN_H_
#define _ASL_GEN_H_

/*
 * Debug levels:
 *  1 = low level detail ... 5 = higher level algorithms
 *  > 9 = release level - i.e. all errors are "DPRINTF(10)".
 *  0 = All sorts of debug!
 */

#if defined _VXWORKS || defined _SOLARIS || defined _MACOSX || defined _LINUX
/* Use a global variable to control debugging
 * Who cares about the inefficiency of a conditional branch !?
 */
extern int _SnapperDebugLevel;
#define DEBUG_PRINTF(L) if (L >= _SnapperDebugLevel) printf
#else
/* Efficient as compiled out where not used */
#ifndef _DEBUG_LVL   /* If compiled without debug - LEAVE AT 10 for RELEASE SOFTWARE */
#define DEBUG_PRINTF(L) if (L >= 10) printf   /* LEAVE AT 10 for RELEASE SOFTWARE */
#else
#define DEBUG_PRINTF(L) if (L >= _DEBUG_LVL) printf
#endif
#endif /* _VXWORKS / Solaris */

/*
 * General Type Definitions
 * ------------------------
 */
typedef ui32  Tboolean;
typedef i16   Tcoord;   /* NOTE - delete in future release */
typedef ui32  Terr;
typedef ui32  Thandle;
typedef ui32  Tparam;

#define Tparam_BITS              32

/*
 * Error Returns - see ASL_err_display() for comments on the meaning
 * -------------   of each of these error returns
 *
ErrorReturnCodes(ForBrief)
 */

#define MAX_ERROR_STRING_LENGTH  512   /* used in ASL_display_err */

#define ASL_OK                        ((Terr) 0)
#define ASL_ERROR                     ((Terr) 0xffffffff)

/* BAD_xxx errors */
#define ASLERR_BAD_HANDLE             ((Terr) ((Terr)   1 << 16))
#define ASLERR_BAD_REG                ((Terr) ((Terr)   2 << 16))
#define ASLERR_BAD_IMAGE              ((Terr) ((Terr)   3 << 16))

/* 'Not possible' errors */
#define ASLERR_NOT_SUPPORTED          ((Terr) ((Terr)  20 << 16))
#define ASLERR_INCOMPATIBLE           ((Terr) ((Terr)  21 << 16))
#define ASLERR_NOT_IMPLEMENTED        ((Terr) ((Terr)  22 << 16))
#define ASLERR_NOT_RECOGNIZED         ((Terr) ((Terr)  23 << 16))

/* Function parameter errors */
#define ASLERR_BAD_PARAM              ((Terr) ((Terr)  40 << 16))
#define ASLERR_OUT_OF_RANGE           ((Terr) ((Terr)  41 << 16))
#define ASLERR_PARAM_CONFLICT         ((Terr) ((Terr)  42 << 16))

/* Hardware related errors */
#define ASLERR_HW_FAILURE             ((Terr) ((Terr)  60 << 16))
#define ASLERR_HW_NOT_FOUND           ((Terr) ((Terr)  61 << 16))
#define ASLERR_UNKNOWN_MODULE         ((Terr) ((Terr)  62 << 16))
#define ASLERR_UNKNOWN_BASEBOARD      ((Terr) ((Terr)  63 << 16))
#define ASLERR_NO_VIDEO               ((Terr) ((Terr)  64 << 16))
#define ASLERR_TIMEOUT                ((Terr) ((Terr)  65 << 16))
#define ASLERR_OUT_OF_LOCK            ((Terr) ((Terr)  66 << 16))
#define ASLERR_OVERFLOW               ((Terr) ((Terr)  67 << 16))
#define ASLERR_PARITY                 ((Terr) ((Terr)  68 << 16))
#define ASLERR_UNDERFLOW              ((Terr) ((Terr)  69 << 16))


/* Software / OS errors */
#define ASLERR_OUT_OF_MEMORY          ((Terr) ((Terr)  80 << 16))
#define ASLERR_THREAD_ERROR           ((Terr) ((Terr)  81 << 16))
#define ASLERR_DRIVER_CALL_FAILED     ((Terr) ((Terr)  82 << 16))
#define ASLERR_SYSTEM_CALL_FAILED     ((Terr) ((Terr)  83 << 16))

/* File etc errors */
#define ASLERR_OPEN_FAILED            ((Terr) ((Terr) 100 << 16))
#define ASLERR_CLOSE_FAILED           ((Terr) ((Terr) 101 << 16))
#define ASLERR_READ_FAILED            ((Terr) ((Terr) 102 << 16))
#define ASLERR_WRITE_FAILED           ((Terr) ((Terr) 103 << 16))
#define ASLERR_SEEK_FAILED            ((Terr) ((Terr) 104 << 16))
#define ASLERR_CORRUPT                ((Terr) ((Terr) 105 << 16))

/* Miscellaneous */
#define ASLERR_OUT_OF_HANDLES         ((Terr) ((Terr) 120 << 16))
#define ASLERR_INTERNAL_ERR           ((Terr) ((Terr) 121 << 16))
#define ASLERR_IN_PROGRESS            ((Terr) ((Terr) 122 << 16))
#define ASLERR_INCOMPATIBLE_LIBRARIES ((Terr) ((Terr) 123 << 16))
#define ASLERR_INVALID_STATE          ((Terr) ((Terr) 124 << 16))

/* **** Note: 127 and above are warning codes **** */


/*
 * Warning "Error" Returns
 * -----------------------
 * To fit in with the Snapper error handler legacy and drivers already 
 * written, a warning code - is still an error code EXCEPT it will not
 * cause an/the error handler to be invoked.  So really it's not a warning
 * but an error code with the error handler always switched off.
 *
WarningReturnCodes(ForBrief)
 */
#define ASLWRN_CODE     0x00800000L
#define ASLWRN_TIMEOUT                ((Terr) ASLWRN_CODE | ASLERR_TIMEOUT)



/* General Definitions
 * -------------------
 */
#define MILLISECONDS * (ui32)    1
#define SECONDS      * (ui32) 1000 MILLISECONDS
#define SECOND       SECONDS
#define MINUTES      * (ui32)   60 SECONDS
#define MINUTE       MINUTES       /* so that "1 MINUTE" is valid too */

#define ASL_MAX_UI32 4294967295UL


/*
 * Boolean Definitions
 * -------------------
 */
#ifndef TRUE
#define TRUE     (1==1)
#define FALSE    (!TRUE)
#endif
#ifndef ON
#define ON       (TRUE)
#define OFF      (FALSE)
#endif


/* Command Line Parsing Defines
 * ----------------------------
 */
#ifdef _DOS16    /* DOS16 has limited memory */
#define  MAX_FIELD_LENGTH 64
#else
#define  MAX_FIELD_LENGTH 128
#endif
#define  MAX_OPTIONS 3
#define  MAX_PARAMS 3

typedef struct Tcmd
{
   ui16  no_of_fields;
   char  cmd_name[MAX_FIELD_LENGTH];
   ui16  no_of_options;
   char  options[MAX_OPTIONS][MAX_FIELD_LENGTH];
   ui16  no_of_params;
   char  params[MAX_PARAMS][MAX_FIELD_LENGTH];
} Tcmd;

/*
 * RBT file related defines
 * ------------------------
 * Note some of these definitions are duplicated in driver header files.
 */
#define RBT_NUM_OF_EOL_CHARS       1       /* i.e. \n                     */
#define RBT_NAME_LENGTH            100     /* Max length of filename      */
#define RBT_LINE_LENGTH            121     /* Max length of line in rbt file */
#define RBT_PREAMBLE_LENGTH        4       /* 4 bits preamble code        */
#define RBT_COMMENT_LINES          7       /* 7 lines of comments         */
#define RBT_LENGTH_COUNT           24      /* 24 bit field in RBT header  */
#define RBT_OUTPUT_HEADER_LENGTH   (RBT_LENGTH_COUNT + RBT_PREAMBLE_LENGTH + 12) /* NOTE: used ? */
                                          /* 12 = preamble (8) + dummy (4) */
#define RBT_BITS_BEFORE_DOT        1000   /* How many programming bits between '.'s  */
#define RBT_LINES_BEFORE_DOT       20     /* How many programming lines between '.'s */
#define RBT_ASL_DATE_LENGTH        18     /* #characters in ASL rbt date string */
#define RBT_ASL_NAME_LENGTH        16     /* #characters in ASL rbt name string */


/*
 * Function parameter defines
 * --------------------------
 */

/* ASL_err_set_reporting()
 * -----------------------
 */
#define ASL_ERR_SET_HANDLER         ((ui32) 0x0001)



/*
 * ROI defines
 * -----------
 */
#define ASL_ROI_X_START         0
#define ASL_ROI_Y_START         1
#define ASL_ROI_X_LENGTH        2
#define ASL_ROI_Y_LENGTH        3
#define ASL_SIZE_2D_ROI         4


/*
 * Malloc Macros
 * -------------
 */
/* for ASL_get_malloc_info function: */
#define ASL_NUM_FREES      0
#define ASL_NUM_MALLOCS    1
#define ASL_TOTAL_MALLOCS  2

#ifdef _MALLOC_DEBUG
#define _ASL_Malloc(x)   ASL_malloc(x)
#define _ASL_Free(x)     {ASL_free(x); x=NULL;}
#else

/* Note: these next definitions also appear in asl_gen.c inside ASL_free
 * and ASL_malloc.  Thus any changes need to be reflected in both.
 */
#ifdef _CPU16
#ifdef __BORLANDC__
#define _ASL_Malloc(x)   farmalloc(x)
#define _ASL_Free(x)     {farfree(x); x=NULL;}
#elif defined __SC__
#define _ASL_Malloc(x)   farmalloc(x)
#define _ASL_Free(x)     {farfree(x); x=NULL;}
#else   /* MS Visual C++ */
#define _ASL_Malloc(x)   _halloc(x,1)
#define _ASL_Free(x)     {_hfree(x); x=NULL;}    
#endif  /* _#ifdef __BORLAND__ */

#else   /* for 32 bit compilers: */
#ifdef _SOLARIS
#define _ASL_Malloc(x)   memalign(64, x)
#define _ASL_Free(x)     {free(x); x=NULL;}
#elif defined _VXWORKS
/* NOTE:CLR
 * It is vital that the libraries be compiled with the correct CPU= directive,
 * as the _CACHE_ALIGN_SIZE is a constant defined in the header files on a 
 * per CPU basis. Different CPU's in the same family have different values 
 * for this constant.
 */
#define _ASL_Malloc(x)   memalign(_CACHE_ALIGN_SIZE,(x))
#define _ASL_Free(x)     { free((x)) ; (x) = NULL; }
#else
#define _ASL_Malloc(x)   malloc(x)
#define _ASL_Free(x)     {free(x); x=NULL;}
#endif  /* _SOLARIS */
#endif  /* else 32 bit compilers */
#endif  /* if not _MALLOC_DEBUG  */


/* General Macros
 * --------------
GeneralMacros(ForBrief)
 *
 * _ASL_RoundUp   - Rounds up x to y alignment.
 *                  (Note does not cast - careful on 16 bit compilers)
 * _ASL_RoundDown - Rounds down x to y alignment.
 *                  (Note does not cast - careful on 16 bit compilers)
 */
#define _ASL_RoundUp(x,y)     (((x) + ((y)-1)) & ~((y)-1))
#define _ASL_RoundDown(x,y)   ((x) & ~((y)-1))

#define limit_to_0(x)     { if ((x) < 0) { (x) = 0; } }
#define limit_to_255(x)   { if ((x) > 255) { (x) = 255; } }
#define limit_0_to_255(x) { if ((x) < 0) { (x) = 0; } else if ((x) > 255) { (x) = 255; } }
#define limit_0_to_359(x) { if ((x) < 0) { (x) = 0; } else if ((x) > 359) { (x) = 359; } }
#define ASL_get_ID(x)     ((x) & 0xf8)
#define ASL_get_rev(x)    ((x) & 0x07)
#define ASL_IS_ODD(x)     ((x) & 0x01)  /* Assumes FALSE == 0 */
#define ASL_IS_EVEN(x)    (~((x) & 0x01))
#define ASL_MAX(a,b)      (((a) > (b)) ? (a) : (b))
#define ASL_MIN(a,b)      (((a) < (b)) ? (a) : (b))
#define _ASL_Min3(a,b,c)  ASL_MIN(a,ASL_MIN(b,c))

#define ASL_check_mode(requested,check)             ( ((requested) & (check)) == (check) )

#define ASL_check_mode_conflict(req,chk1,chk2)      ( ( ((req) & (chk1)) == (chk1) ) && ( ((req) & (chk2)) == (chk2) ) )


#ifndef INTEGER_FILE_IO
#define INTEGER_FILE_IO
/*
 * The VxWorks PPC compiler (and others!) cannot cope with TMG_tiff.c using 
 * these macros as the function becomes too long for the optimiser, 
 * exhausting virtual memory.
 */
#if defined _MACOSX
void EXPORT_FN put_ui32m(FILE*, ui32);
void EXPORT_FN put_ui16m(FILE*, ui16);
void EXPORT_FN put_ui8(FILE*, ui8);
void EXPORT_FN put_ui16i(FILE*, ui16);
void EXPORT_FN put_ui32i(FILE*, ui32);
#elif 1
extern void EXPORT_FN put_ui32m(FILE*, ui32);
extern void EXPORT_FN put_ui16m(FILE*, ui16);
extern void EXPORT_FN put_ui8(FILE*, ui8);
extern void EXPORT_FN put_ui16i(FILE*, ui16);
extern void EXPORT_FN put_ui32i(FILE*, ui32);
#else  /* Note: Keep the macros just in case */
/* ms byte first - big endian (Motorola type) */
#define put_ui32m(Pfile, b) putc( (char)(((ui32)b>>24) & 0x0ff), Pfile ); putc( (char)(((ui32)b>>16) & 0x0ff), Pfile ); putc( (char)(((ui32)b>>8)  & 0x0ff), Pfile ); putc( (char)((ui32)b & 0x0ff), Pfile )
#define put_ui16m(Pfile, b) putc( (((ui16)b>>8) & 0x0ff), Pfile ); putc( ((ui16)b & 0x0ff), Pfile )
#define put_ui8(Pfile, b)  putc( ((ui8)b & 0x0ff), Pfile )
/* ls byte first - Intel style */
#define put_ui16i(Pfile, b)  putc( ((ui16)b & 0x0ff), Pfile ); putc( (((ui16)b>>8)  & 0x0ff), Pfile )
#define put_ui32i(Pfile, b)  putc( (char)((ui32)b & 0x0ff), Pfile ); putc( (char)(((ui32)b>>8)  & 0x0ff), Pfile ); putc( (char)(((ui32)b>>16) & 0x0ff), Pfile ); putc( (char)(((ui32)b>>24) & 0x0ff), Pfile )
#endif
#endif


/* Global Terr Processing Macros
 * -----------------------------
 *
ErrorMacros(ForBrief)
 */
/* NOTE - need to check appropriate error returns for given OS, CJB */
#define ASL_exit_on_err(f)       { Terr r;\
                                   r = f;\
                                   if ASL_is_err(r) exit(0);}
#define ASL_get_ret(x)           ((ui16) (x))
#define ASL_get_err(x)           ((x) & (ui32) 0x00ff0000L)
#define ASL_get_err_ID(x)        ((x) & (ui32) 0xf8000000L)
#define ASL_is_ok(x)             (ASL_get_err(x) == 0)
#define ASL_is_err(x)            (ASL_get_err(x) != 0)

#define ASL_ok_ret(x)            return(ASL_OK | ((Terr) x & (Terr) 0xffff))

#define ASL_is_warning(x)        ((ui32) x & (ui32) ASLWRN_CODE)

/* roe = Return On Error */
#define ASL_roe(f)               { Terr r;\
                                   r = f;\
                                   if ASL_is_err(r) return(r);}

/* proe = Print and Return On Error (print dependent on error reporting state)
 * Note: ASL_proe cannot be used on the line after an `else' - {} are needed
 * even on single line if-else statements.
 */
#define ASL_proe(f,d,s)          { Terr e;\
                                   e = f;\
                                   if ASL_is_err(e)\
                                   { ASL_err_process(s,e,d);\
                                     return(e);\
                                 } }



/* Local definitions for error handling (in alphabetical order)
 * ------------------------------------------------------------
 * Note: ASLGEN_err_ret cannot be used on the line after an `else' - {} are
 * needed even on single line if-else statements.
 */
/* A reminder of the args format used for ...
   void  EXPORT_FN     ASL_err_process(char*, Terr, char*);
*/

#define ASLGEN_IDREV  0x18

#if defined _lint
extern Terr EXPORT_FN ASLGEN_Dummy_err_ret(Terr, char *, Terr, char *);
#define ASLGEN_err_ret(e,d,v,s)  return(ASLGEN_Dummy_err_ret(e,d,v,s))
#elif defined _DOS16  /* real mode DOS - we lose some the comments to save space */
#define ASLGEN_err_ret(e,d,v,s)  {ASL_err_process(s,ASLGEN_make_ret(e,v),"");return(ASLGEN_make_ret(e,v));}
#else
#define ASLGEN_err_ret(e,d,v,s)  {ASL_err_process(s,ASLGEN_make_ret(e,v),d);return(ASLGEN_make_ret(e,v));}
#endif

#define ASLGEN_make_ret(e,v)     ((ASLGEN_IDREV << 24) | (e) | ((Terr) v & (Terr) 0xffff)) 


/* Thread Structure
 * ----------------
 */
struct Tthread_info
{
   Thandle Hsnapper;
   Thandle Himage;
   volatile Terr  err;
   volatile ui32  status;
#define ASL_THREAD_RUNNING    ((ui32) 1)  /* currently active          */
#define ASL_THREAD_FINISHED   ((ui32) 2)  /* active, but finished task */
#define ASL_THREAD_IDLE       ((ui32) 3)  /* not running & ready start */

   ui32    dwPriority;  /* not used in Solaris or MacOS */
#define ASL_THREAD_PRIORITY_HIGH   ((ui32) 1)
#define ASL_THREAD_PRIORITY_NORMAL ((ui32) 2)
#define ASL_THREAD_PRIORITY_LOW    ((ui32) 3)

#if defined _SOLARIS2
   thread_t  thread_ID;
   void *    (*Pfunc)(struct Tthread_info *);
   int       nConcurrency;
#elif defined _WIN95 || defined _WINNT
   /*HANDLE hThread;*//* for CreateThread */
   ui32 hThread;      /* for _beginthread */
   ui32 dwThreadId;   /* note this is not the same as hThread */
   void (*Pfunc)(struct Tthread_info*);
/*
#elif defined _LYNXOS 
   pthread_t ptThread; 
   void (*Pfunc)(struct Tthread_info*);
*/
#elif defined _OS_THREADS && defined _WIN31 /* 07-Jul-96 */
   HINSTANCE hThread;
   void (CALLBACK *Pfunc)(struct Tthread_info FAR*);
#elif defined _OS_THREADS && defined _MAC_LVR /* LVR 04-Sep-96 */
   ThreadID  thread_ID;
   ThreadEntryProcPtr Pfunc;
#elif defined _OS_THREADS && defined _MACINTOSH
   ThreadID   thread_ID;
   ThreadEntryProcPtr Pfunc;
#endif
};


#endif /* _ASL_GEN_H_ */
