/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : asl_l24.h                                               *
 *  Function    : Type definitions for Snapper Linus 2.4.X driver         *
 *  Project     : Snapper                                                 *
 *  Author      : Chris Read, Active Silicon Ltd                          *
 *  Systems     : ANSI C                                                  *
 *  Version     : 4.10.18                                                 *
 *  Release Date: 02-Oct-2002                                             *
 *                                                                        *
 *  Copyright (c) 1993-2002 Active Silicon Ltd, Active Silicon Ltd.       *
 *************************************************************************
 * Comments:
 * --------
 * This is the Snapper Linux 2.4.X driver include file
 *
 * File History
 * ------------
 * 02-Oct-2002, CLR, 4.10 Initial version
 *
 */

/*
 * Driver Revision Level
 * ---------------------
 * This revision level is sent back to the Snapper library as a check that 
 * compatible libraries and driver are being used.
 */
#define SNAPPER_DRIVER_REV_LEVEL  (SNP_REV_LEVEL)

/*
 * Driver Build Options
 * --------------------
 * _DEBUG_PARALLEL_PORT - When defined, allows the macro 
 *                        _DRV_DEBUG_ParallelPortOut to be used to drive
 *                        pins in the parallel port high/low etc for 
 *                        timing/speed measurements.
 *                        This is only supportable on certain hardware platforms
 *                        and is enabled by the user filling in the appropriate
 *                        stub functions
 *                        
 *
 *
 */

#if !defined (MAXLONG)
#define MAXLONG ( (ui32)(~0) )
#endif

#define FAR

typedef enum {
    STATUS_SUCCESS = 0,
    STATUS_UNSUCCESSFUL,
    STATUS_NOT_IMPLEMENTED,
    STATUS_UNSUPPORTED,
    STATUS_INTERNAL_ERROR,
    STATUS_INVALID_DEVICE_REQUEST,
    STATUS_INVALID_BUFFER_SIZE,
    STATUS_ADAPTER_HARDWARE_ERROR,
    STATUS_INVALID_PARAMETER
} DRV_STATUS;

typedef enum {
    DRV_FALSE = 0,
    DRV_TRUE
} DRV_BOOLEAN;

/*
 * System Include Files
 * --------------------
 */
/*
 * General #defines
 * ----------------
 */
#define DRV_MAX_NUM_PCI_BUSES 256   /* Maximum number of PCI buses          */

/*
 * Malloc macros
 * -------------
MALLOC_PHYS_MEMORY(ForBrief)
FREE_PHYS_MEMORY(ForBrief)
 */
/* VxWorks does not perform paging, therefore the standard memory
 * allocator is OK here
 */
#define  MALLOC_PHYS_MEMORY(n)  _ASL_Malloc((n))
#define  FREEE_PHYS_MEMORY(n)   _ASL_Free((n))


/* Snapper I/O control codes
 * These are used to give code compatibility between the standard
 * driver environment (e.g. WindowsNT) and VxWorks. However, in
 * VxWorks, there is not an separate driver.
 * An enum is used in preference to #defines, as these are symbolically 
 * visible at run time in the debugger
 */
enum SnapperIoControlCode {
   IOCTL_SNAPPER_OPEN_DEVICE = 0,
   IOCTL_SNAPPER_WRITE_UI8,
   IOCTL_SNAPPER_READ_UI8,
   IOCTL_SNAPPER_WRITE_UI16,
   IOCTL_SNAPPER_READ_UI16,
   IOCTL_SNAPPER_WRITE_UI32,
   IOCTL_SNAPPER_READ_UI32,
   IOCTL_SNAPPER_DYLAN_WRITE_UI32,
   IOCTL_SNAPPER_DYLAN_READ_UI32,
   IOCTL_SNAPPER_FPGA_SEND_INFO,
   IOCTL_SNAPPER_FPGA_SEND_DATA,
   IOCTL_SNAPPER_FPGA_RESET,
   IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS,
   IOCTL_SNAPPER_BURST_IN_UI32,
   IOCTL_SNAPPER_BURST_IN_UI32_ASYNC,
   IOCTL_SNAPPER_READ_COMPLETE,
   IOCTL_SNAPPER_TABLE_LOAD,
   IOCTL_SNAPPER_GET_ERROR_STRING,
   IOCTL_SNAPPER_ABORT_OPERATION,
   IOCTL_SNAPPER_TIMEOUT_SET,
   IOCTL_SNAPPER_SERIAL_TX,
   IOCTL_SNAPPER_SERIAL_RX,
   IOCTL_SNAPPER_SERIAL_SEND_INFO,
   IOCTL_SNAPPER_SERIAL_GET_INFO,
   IOCTL_SNAPPER_INTERRUPT_SET,
   IOCTL_SNAPPER_INTERRUPT_STATUS,
   IOCTL_SNAPPER_SGTABLE_SEND_INFO,
   IOCTL_SNAPPER_PHYSMEM_LOCK,
   IOCTL_SNAPPER_PHYSMEM_UNLOCK,
   IOCTL_SNAPPER_GET_DEVICE_INFO,
   IOCTL_SNAPPER_MARK_DEVICE_IN_USE,
   IOCTL_SNAPPER_NIOCTLS
};

struct tSnapper_PCIDeviceInfo{
   ui32 dwDriverRevLevel;
   ui32 dwDeviceInUse;
   ui32 dwBasePhysAddr;
   ui32 dwInterruptLine;
   ui16 wPciVendorID;
   ui16 wPciDeviceID;
   ui16 wPciSubVendorID;
   ui16 wPciSubSystemID;
};

struct tSnapper_Event{
   ui32 dwEvent;
   ui32 dwData;
};

struct tSnapper_Ioctl{
   void* pBufferIn;
   void *pBufferOut;
   size_t sizeBufferIn;
   size_t sizeBufferOut;
   size_t dwBytesOut;
};

