/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : asl_sol.h                                                *
 *  Function    : Type definitions for Snapper Solaris driver             *
 *  Project     : Snapper                                                 *
 *  Author      : Chris Read, Active Silicon Ltd                          *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10.38                                                 *
 *  Release Date: 17-Feb-98                                               *
 *                                                                        *
 *  Copyright (c) 1993-98 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * This is the Snapper VxWorks driver include file
 * It is included vy both the libraries and the drivers
 * Note there is a separate include file for each of the actual
 * device drivers (SBus and PCI at present) This include file is located
 * in the appropriate driver source directory
 *
 * File History
 * ------------
 * 17-Feb-98, CLR, 3.10r38 Initial version
 *
 */

#include <snp_rev.h>

#if defined _KERNEL
/* This file is being included as part of the driver build.
 * Define snapper types from the kernel model independent types.
 * This in readiness for Solaris 64 bit kernel implementations
 */
typedef uint64_t ui64;
typedef uint32_t ui32;
typedef uint16_t ui16;
typedef uint8_t  ui8;
typedef int64_t  i64;
typedef int32_t  i32;
typedef int16_t  i16;
typedef int8_t   i8;
#if defined FAR
#undef FAR
#endif

#if defined EXPORT
#undef EXPORT
#endif

#define EXPORT
#define FAR

#endif

#if !defined DIG16_BIT_INT_MAIN_TRANSFER_EN
#define DIG16_BIT_INT_MAIN_TRANSFER_EN   ((ui32) 0x00010000)
#endif

#if !defined MODULE_READ_VPORT_UI32
#define MODULE_READ_VPORT_UI32          ((ui8) 5)
#endif

#if !defined BASE_PCIMAX_EEPROM_LENGTH
#define BASE_PCIMAX_EEPROM_LENGTH  256
#endif

/* Snapper I/O control codes
 * An enum is used in preference to #defines, as these are symbolically 
 * visible at run time in the debugger
 */
enum SnapperIoControlCode {
   IOCTL_SNAPPER_OPEN_DEVICE,
   IOCTL_SNAPPER_WRITE_UI8,
   IOCTL_SNAPPER_READ_UI8,
   IOCTL_SNAPPER_WRITE_UI16,
   IOCTL_SNAPPER_READ_UI16,
   IOCTL_SNAPPER_WRITE_UI32,
   IOCTL_SNAPPER_READ_UI32,
   IOCTL_SNAPPER_DYLAN_WRITE_UI32,
   IOCTL_SNAPPER_DYLAN_READ_UI32,
   IOCTL_SNAPPER_FPGA_SEND_INFO,
   IOCTL_SNAPPER_FPGA_SEND_DATA,
   IOCTL_SNAPPER_FPGA_RESET,
   IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS,
   IOCTL_SNAPPER_BURST_IN_UI16,
   IOCTL_SNAPPER_BURST_IN_UI32,
   IOCTL_SNAPPER_BURST_IN_UI32_ASYNC,
   IOCTL_SNAPPER_READ_COMPLETE,
   IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL,
   IOCTL_SNAPPER_BURST_OUT_UI16,
   IOCTL_SNAPPER_TABLE_LOAD,
   IOCTL_SNAPPER_PAGE_WRITE_UI16,
   IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL,
   IOCTL_SNAPPER_GET_ERROR_STRING,
   IOCTL_SNAPPER_ABORT_OPERATION,
   IOCTL_SNAPPER_TIMEOUT_SET,
   IOCTL_SNAPPER_SERIAL_TX,
   IOCTL_SNAPPER_SERIAL_RX,
   IOCTL_SNAPPER_SERIAL_SEND_INFO,
   IOCTL_SNAPPER_SERIAL_GET_INFO,
   IOCTL_SNAPPER_INTERRUPT_SET,
   IOCTL_SNAPPER_INTERRUPT_STATUS,
   IOCTL_SNAPPER_GET_BOARD_TYPE,
   IOCTL_SNAPPER_GET_PROPERTY,
   IOCTL_SNAPPER_PREFILL,
   IOCTL_SNAPPER_WAIT_DMA_ARMED
};

int  SnapperIoControl(int,    /* File descriptor */
                      enum SnapperIoControlCode, /* Code for requested operation */
                      void*,  /* Pointer to input parameters */
                      size_t, /* Size of input parameters structure */
                      void*,  /* Pointer rto output parameters */
                      size_t, /* Size of output parameters structure */
                      ui32*,  /* Number of bytes written to output structure */
                      void*); /* Not used */

struct SnapperIoInfo {
   void   *pBufferDirect;
   void   *pBufferBufferred;
   size_t sizeBufferDirect;
   size_t sizeBufferBufferred;
   ui32   nInformation;
};

enum SnapperBoardType{
   Unknown,
   PCI_BIB,
   PCI_JPG,
   SBUS_BIB,
   ISA_BIB,
   ISA_JPG
};

#if defined (_KERNEL) && defined (_MULTI_DATAMODEL)
/* All the driver/library interface structures need defining explicitly
 * for 32 bit program/libraries in a 64 bit kernel.
 */
struct SnapperIoInfo32 {
   caddr32_t pBufferDirect;
   caddr32_t pBufferBufferred;
   size32_t  sizeBufferDirect;
   size32_t  sizeBufferBufferred;
   ui32      nInformation;
};

/*
struct_Ttable_in(ForBrief)
*/
struct Ttable_info32
{
   ui32 hw_addr_offset;  /* address of table relative to base address */
   ui32 count;           /* number of elements */
   caddr32_t Ptable_ui32;    /* actual table data - for 32 bit table */
   caddr32_t Ptable_ui16;    /* actual table data - for 16 bit table */
   caddr32_t Ptable_ui8;     /* actual table data - for  8 bit table */
   ui32 mode;
   ui32 ConfigAlgorithm;    /* pass information about how to configure */
};

/*
 * A structure to pass port address offset and byte count of amount of 
 * data to transfer etc.
 *
struct_TBurstTransfer(ForBrief)
 */
struct TBurstTransfer32 {
   ui32 dwAddrOffset;  /* port address offset         */
   ui32 dwByteCount;   /* number of bytes to transfer */
   ui32 dwMapping;     /* mapping required            */
   caddr32_t pData;        /* For outgoing data           */
};

/*
 * A structure to pass the information about board type/address to and 
 * from the driver.
 * Note we make all structure elements 32 bit to eliminate potential 
 * problems with structure data alignment between library and drivers.
 *
TBoardInfo(ForBrief)
 */
struct TBoardInfo32 {

   /* Passed into driver: */

   ui32 dwBaseAddress;     /* 0, 300 or 8001 etc */
   ui32 dwBoardType;       /* ISA or PCI         */
   ui32 dwLibDeviceNum;    /* if base address = 8001, then this = 1 */

   /* Returned from driver: */

   /* Note: dwBaseAddress is returned from NT driver */
   ui32 dwInterruptLine;   /* Interrupt line number (can be interesting) */
   ui32 dwIDrev;           /* Baseboard IDrev      */
   caddr32_t pdwEeprom;        /* To shadow PCI EEPROM - mallocs     */
                           /* BASE_PCIMAX_EEPROM_LENGTH*4 bytes  */
                           /* Used by Windows 95 */
   ui32 dwDriverRevLevel;
   ui32 dwEeprom[BASE_PCIMAX_EEPROM_LENGTH]; /* NT driver uses this - the "proper" way */
};

/*
 * A structure for sending/receiving a buffer of characters to/from
 * the driver.
 *
struct_TSerialBuffer(ForBrief)
 */
struct TSerialBuffer32 {
   caddr32_t pBuffer;
   caddr32_t pdwCount;
   ui32 dwTimeoutTx;
   ui32 dwTimeoutRx;
};
/*
 * FPGA Info Structure
 * -------------------
 * The members are all ui32s to reduce structure alignment hassles when
 * interfacing to a driver.
 *
struct_Tfpga_info(ForBrief)
 */
struct Tfpga_info32 {
   ui32  fpga_type;         /* type ui32 for VxD alignment issues */
   caddr32_t fpga_data;
   ui32  fpga_bit_length;

   ui32  data_reg;
   ui32  data0_clock0;
   ui32  data0_clock1;
   ui32  data1_clock0;
   ui32  data1_clock1;

   ui32  reset_reg;
   ui32  reset_assert;
   ui32  reset_deassert;

   ui32  status_reg;
   ui32  dp_mask;

   ui32  ConfigAlgorithm;    /* pass information about how to configure */
};

#endif

