/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : base_pro.h                                              *
 *  Function    : Prototype definitions for base functions                *
 *  Project     : Snappers                                                *
 *  Author      : Warren Frost, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : December 1993                                           *
 *  Updated     : 17-Jun-96                                               *
 *  Version     : 3.11                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _BASE_PRO_H_
#define _BASE_PRO_H_

/*
 * Baseboard Prototype Definitions - for base.c
 * --------------------------------------------
 */
#ifdef __cplusplus             
extern "C" {      
#endif
Terr    EXPORT_FN BASE_configure_fpga(Thandle, char*);
Terr    EXPORT_FN BASE_create(ui16);
Terr    EXPORT_FN BASE_create_base_registers(Thandle);
Thandle EXPORT_FN BASE_create_sw(void);
Terr    EXPORT_FN BASE_destroy(Thandle);
Terr    EXPORT_FN BASE_destroy_baseboard(Thandle);
Terr    EXPORT_FN BASE_get_parameter(Thandle, ui16);
Terr    EXPORT_FN BASE_initialize(Thandle);
Terr    EXPORT_FN BASE_PhysMem_Lock(Thandle hBase, void *pData, ui32 dwByteCount, ui32 **ppSGTable);
Terr    EXPORT_FN BASE_PhysMem_Unlock(Thandle hBase, void *pData, ui32 *pSGTable);
Terr    EXPORT_FN BASE_set_ptr(Thandle hBase, ui32 dwType, void *pData);
Terr    EXPORT_FN BASE_set_default_options(Thandle);
Terr    EXPORT_FN BASE_toggle_VBANK(Thandle);
#ifdef __cplusplus
};                
#endif


/*
 * Low Level Baseboard Prototype Definitions - for base_low.c
 * ----------------------------------------------------------
 */
#ifdef __cplusplus             
extern "C" {      
#endif
Terr EXPORT_FN BASE_configure_fpga_rbt(Thandle, ui16, char*, char*);
Terr EXPORT_FN BASE_delay(Thandle, ui32);
Terr EXPORT_FN BASE_determine_baseboard_type(Thandle);
Terr EXPORT_FN BASE_get_property(Thandle, char*, char*);
Terr EXPORT_FN BASE_set_property(Thandle, char *, char *);
Terr EXPORT_FN BASE_construct_property_table(Thandle);
void EXPORT_FN_PTR DIG16_get_ptr(Thandle, ui32);
Terr EXPORT_FN BASE_get_register_value(Thandle, ui16);
Terr EXPORT_FN BASE_hard_reset(Thandle);
Tboolean EXPORT_FN BASE_is_module_fitted(Thandle);
Terr EXPORT_FN BASE_mapper_get(Thandle, ui16);
Terr EXPORT_FN BASE_mapper_set(Thandle, ui16, ui16, ui16);
Terr EXPORT_FN BASE_mapper_write_control(Thandle, ui32, ui8);

Terr BASE_module(Thandle, ui8, ...);

Terr EXPORT_FN BASE_reset_fpga(Thandle, ui16);

Terr EXPORT_FN BASE_serial_set_dylan(Thandle Hbase, ui32 parameter, ui32 value);
Terr EXPORT_FN BASE_serial_set_parameter(Thandle, ui32, ui32);
Terr EXPORT_FN BASE_serial_get_parameter(Thandle, ui32);
Terr EXPORT_FN BASE_serial_transmit_buffer(Thandle, ui8*, ui32*, ui32);
Terr EXPORT_FN BASE_serial_receive_buffer(Thandle, ui8*, ui32*, ui32);

Terr EXPORT_FN BASE_set_registers(Thandle);
Terr EXPORT_FN BASE_set_register_value(Thandle, ui16, ui32);
Terr EXPORT_FN BASE_set_timer(Thandle, Tparam, ui32);
Terr EXPORT_FN BASE_table_load(Thandle, struct Ttable_info *);
#ifdef __cplusplus
};
#endif


/*
 * Baseboard I/O Prototype Definitions - for base_<os>.c
 * -----------------------------------------------------
 */
#ifdef __cplusplus             
extern "C" {      
#endif
#ifdef _WIN95  /* Make available for debugging */
void EXPORT_FN LCL_Output_GlobalMemoryStatus(char*);
#endif

#if defined _SOLARIS
Terr EXPORT_FN OS_sbus_dma_prefill(Thandle hBase);
Terr EXPORT_FN OS_get_board_type(Thandle hBase);
#endif

Terr EXPORT_FN OS_abort_operation(Thandle, ui32);
Terr EXPORT_FN OS_burst_in_ui16(Thandle, ui32, IM_UI16*, ui32);
Terr EXPORT_FN OS_burst_in_ui16_special(Thandle, ui32, IM_UI16*, ui32, ui16);
Terr EXPORT_FN OS_burst_in_ui32(Thandle, ui32, IM_UI32*, ui32);
Terr EXPORT_FN OS_burst_in_ui32_async(Thandle, ui32, IM_UI32*, ui32); /* NT/16 bit under Win95 */
Terr EXPORT_FN OS_ReadComplete(Thandle hBase, ui32 dwMode, ui32 *pdwStatus);
Terr EXPORT_FN OS_IsAsyncDMAComplete(Thandle);                        /* 16 bit under Win95 */
Terr EXPORT_FN OS_burst_out_ui16(Thandle, ui32, IM_UI16*, ui32);
Terr EXPORT_FN OS_burst_out_ui16_special(Thandle, ui32, IM_UI16*, ui32, ui16);
Terr EXPORT_FN OS_close(Thandle);
Terr EXPORT_FN OS_delay( Thandle, ui32 );
Terr EXPORT_FN OS_fpga_configure(Thandle, struct Tfpga_info*);
Terr EXPORT_FN OS_fpga_get_config_status(Thandle, struct Tfpga_info*);
Terr EXPORT_FN OS_fpga_reset(Thandle, struct Tfpga_info*);
Terr EXPORT_FN OS_get_base_property(Thandle, char*, char*);
Terr EXPORT_FN OS_GetTimeStamp_ms(ui32*);
Terr EXPORT_FN OS_interrupt_set(Thandle hBase, ui32 dwIntControlAddr, ui32 dwIntStatusAddr, ui32 dwData);
#if defined _VXWORKS
void OS_InterruptThread(Thandle);
#elif defined _WINNT || defined _WIN95
void OS_InterruptThread(void *pdwHandleNum);

#elif defined _LYNXOS /* MH 8.12.97 For thread creation */
void OS_InterruptThread( void* );   

#elif defined _SOLARIS2 || defined _MACOSX || defined _LINUX
void *OS_InterruptThread( void* );

#elif defined _DOS32 || defined _WIN31
/* DOS does not use a separate interrupt thread */
#elif defined _MAC_LVR || defined _MACINTOSH
void OS_InterruptThread(void *pdwHandleNum);
#else
#error Undefined OS_InterruptThread function (in base_pro.h)
#endif

void EXPORT_FN OS_page_write_ui16(Thandle, ui16, ui16, ui16, ui16);
Terr EXPORT_FN OS_open(Thandle);
ui32 EXPORT_FN OS_processor_read_ui32(Thandle, ui32);
Terr EXPORT_FN OS_processor_write_ui32(Thandle, ui32, ui32);
ui8  EXPORT_FN OS_read_ui8(Thandle, ui32);
ui16 EXPORT_FN OS_read_ui16(Thandle, ui32);
ui32 EXPORT_FN OS_read_ui32(Thandle, ui32);
Terr EXPORT_FN OS_serial_send_info(Thandle);
Terr EXPORT_FN OS_serial_get_info(Thandle);
Terr EXPORT_FN OS_serial_transmit_buffer(Thandle, ui8*, ui32*, ui32);
Terr EXPORT_FN OS_serial_receive_buffer(Thandle, ui8*, ui32*, ui32);
Terr EXPORT_FN OS_set_timeout(Thandle, ui32, ui32);
Terr EXPORT_FN OS_table_load(Thandle, struct Ttable_info*);
#ifdef _OS_THREADS
Terr EXPORT_FN OS_thread_create(Thandle, struct Tthread_info*);
Terr EXPORT_FN OS_thread_destroy(Thandle, struct Tthread_info*);
void EXPORT_FN OS_thread_exit(Thandle, struct Tthread_info*);
Terr EXPORT_FN OS_thread_kill(Thandle, struct Tthread_info*);
Terr EXPORT_FN OS_thread_start(Thandle, struct Tthread_info*);
Terr EXPORT_FN OS_thread_sync(Thandle, struct Tthread_info*);
void EXPORT_FN OS_thread_yield(void);
void EXPORT_FN OS_process_yield(void);
#endif
Tboolean EXPORT_FN OS_Timeout(ui32);
void EXPORT_FN OS_write_ui8(Thandle, ui32, ui8);
void EXPORT_FN OS_write_ui8_abs(Thandle, ui32, ui8);
void EXPORT_FN OS_write_ui16(Thandle, ui32, ui16);
void EXPORT_FN OS_write_ui32(Thandle, ui32, ui32);

Terr EXPORT_FN OS_PhysMem_Lock(Thandle hBase, void *pData, ui32 dwByteCount, ui32 **ppSGTable);
Terr EXPORT_FN OS_PhysMem_Unlock(Thandle hBase, void *pData, ui32 *pSGTable);

Terr EXPORT_FN OS_set_debug(Thandle, Tboolean);

#if defined _MACINTOSH
/* Undocumented debuggin function */
ui32 EXPORT_FN OS_get_max_dma_length(Thandle);
#endif
/* Some debug functions */
#ifdef _LOGFILE
void EXPORT_FN OS_write_ui16_logfile(Thandle, ui32, ui16);
void EXPORT_FN OS_burst_out_ui16_logfile(Thandle, ui32, IM_UI16*, ui32);
#endif

#ifdef __cplusplus
};                
#endif

/* functions for initialisation via SnapperTool config file  */
Terr  EXPORT_FN SNPINI_settings_load( Thandle hSnapper, ui32 dwFormat, const void* pvData );
Terr  EXPORT_FN SNPINI_settings_load_from_file( Thandle hSnapper, char* szSnapTypeIDString, char* szINIFileName, struct InitTag** pstRootPtr );
void  EXPORT_FN_PTR SNPINI_find_param( struct InitTag* pstRootPtr, const char* szParamName );


/*
 * Base Library Globals
 * --------------------
 */

/* array of baseboard handles */
#if defined _MACINTOSH
extern EXPORT_DATA struct Tbase * Pbase_array[BASE_NUM_HANDLES+1];
#else
extern struct Tbase * EXPORT_DATA Pbase_array[BASE_NUM_HANDLES+1];
#endif

/* array of pixels read per Snapper access */
extern const ui8 FAR pix_per_read_array[MODULE_MAX_VALUE+1];

/* array of Datamapper structures */
extern const struct Tdm_fpga FAR dm_fpga_array[MODULE_MAX_VALUE+1][TMG_MAX_VALUE+1];

#ifdef _LOGFILE
extern FILE *Plogfile_ui16;  /* for logging reads/writes to logfile */
#endif

#endif /* _base_pro_h */
