/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : cl550.h                                                 *
 *  Function    : C-Cube CL550 Register Definitions                       *
 *  Project     : PC JPEG Card                                            *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : Zortech C, v3.0 (ANSI)                                  *
 *  Written     : July 1993                                               *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _cl550_h
#define _cl550_h


/* CL550 Register Index Assignments */
/* -------------------------------- */
#define _Codec              (ui16) 0x0000
#define _HPeriod            (ui16) 0x8000
#define _HSync              (ui16) 0x8004
#define _HDelay             (ui16) 0x8008
#define _HActive            (ui16) 0x800c
#define _VPeriod            (ui16) 0x8010
#define _VSync              (ui16) 0x8014
#define _VDelay             (ui16) 0x8018
#define _VActive            (ui16) 0x801c
#define _DCT0               (ui16) 0x8800
#define _DCT1               (ui16) 0x8804
#define _DCT2               (ui16) 0x8808
#define _DCT3               (ui16) 0x880c
#define _DCT4               (ui16) 0x8810
#define _DCT5               (ui16) 0x8814
#define _DCT6               (ui16) 0x8818
#define _DCT7               (ui16) 0x881c
#define _Init_5             (ui16) 0x8820
#define _Init_6             (ui16) 0x8824
#define _Config             (ui16) 0x9000
#define _Huff_Enable        (ui16) 0x9004
#define _S_Reset            (ui16) 0x9008
#define _Start              (ui16) 0x900c
#define _HV_Enable          (ui16) 0x9010
#define _Flags              (ui16) 0x9014
#define _NMRQ_Mask          (ui16) 0x9018
#define _DRQ_Mask           (ui16) 0x901c
#define _StartOfFrame       (ui16) 0x9020
#define _Version            (ui16) 0x9024
#define _Init_1             (ui16) 0x9800
#define _Init_2             (ui16) 0x9804
#define _HuffTableSequence  (ui16) 0xa000
#define _DPCM_SeqHigh       (ui16) 0xa004
#define _DPCM_SeqLow        (ui16) 0xa008
#define _CoderAttr          (ui16) 0xa00c
#define _CodingIntH         (ui16) 0xa010
#define _CodingIntL         (ui16) 0xa014
#define _DecLength          (ui16) 0xa80c
#define _DecMarker          (ui16) 0xa810
#define _DecResume          (ui16) 0xa814
#define _DecDPCM_Reset      (ui16) 0xa818
#define _DecCodeOrder       (ui16) 0xa81c
#define _Init_3             (ui16) 0xb600
#define _QTableBase         (ui16) 0xb800
#define _QTable1            (ui16) 0xb800
#define _QTable2            (ui16) 0xb900
#define _QTable3            (ui16) 0xba00
#define _QTable4            (ui16) 0xbb00
#define _QuantABSelect      (ui16) 0xbc00
#define _QuantSync          (ui16) 0xbe00
#define _QuantYCSequence    (ui16) 0xbe08
#define _QuantABSequence    (ui16) 0xbe0c
#define _Matrix00           (ui16) 0xc000
#define _Matrix01           (ui16) 0xc004
#define _Matrix02           (ui16) 0xc008
#define _Matrix10           (ui16) 0xc00c
#define _Matrix11           (ui16) 0xc010
#define _Matrix12           (ui16) 0xc014
#define _Matrix20           (ui16) 0xc018
#define _Matrix21           (ui16) 0xc01c
#define _Matrix22           (ui16) 0xc020
#define _VideoLatency       (ui16) 0xc030
#define _HControl           (ui16) 0xc034
#define _VControl           (ui16) 0xc038
#define _VertLineCount      (ui16) 0xc03c
#define _Init_4             (ui16) 0xcf00
#define _Init_7             (ui16) 0xd400
#define _FIFO_Base          (ui16) 0xd800
#define _HuffTables         (ui16) 0xe000
#define _HuffYAC            (ui16) 0xe000
#define _HuffYDC            (ui16) 0xec00
#define _HuffCAC            (ui16) 0xf000
#define _HuffCDC            (ui16) 0xfc00



/*  CL550 Status Flags -
   These are masks that correspond to the CL550 status indicators.
   They are used when reading the Flags register or when writing to
   the NMRQ and DRQ mask registers.
*/

#define _Flags_FIFO_NotFull       (ui16) 0x8000L
#define _Flags_FIFO_NotEmpty      (ui16) 0x4000L
#define _Flags_Codec_NotBusy      (ui16) 0x2000L
#define _Flags_BusError           (ui16) 0x1000L
#define _Flags_MarkerCode         (ui16) 0x0800L
#define _Flags_VSync              (ui16) 0x0400L
#define _Flags_VideoNotActive     (ui16) 0x0200L
#define _Flags_FIFO_Empty         (ui16) 0x0080L
#define _Flags_FIFO_QFull         (ui16) 0x0040L
#define _Flags_FIFO_HalfFull      (ui16) 0x0020L
#define _Flags_FIFO_3QFull        (ui16) 0x0010L
#define _Flags_FIFO_NotQFull      (ui16) 0x0008L
#define _Flags_FIFO_NotHalfFull   (ui16) 0x0004L
#define _Flags_FIFO_Not3QFull     (ui16) 0x0002L
#define _Flags_FIFO_Late          (ui16) 0x0001L

/* these are the CL550 init register values for GREYSCALE DECOMPRESSION */

#define GRAYSCALE_DECOMP_INIT1              (ui16) 0x003E
#define GRAYSCALE_DECOMP_INIT2              (ui16) 0x0037
#define GRAYSCALE_DECOMP_INIT3              (ui16) 0x01FF
#define GRAYSCALE_DECOMP_INIT4              (ui16) 0x0049
#define GRAYSCALE_DECOMP_INIT5              (ui16) 0x0005
#define GRAYSCALE_DECOMP_INIT6              (ui16) 0x0000
#define GRAYSCALE_DECOMP_INIT7              (ui16) 0x0022
#define GRAYSCALE_DECOMP_QUANTSYNC          (ui16) 0x043E
#define GRAYSCALE_DECOMP_QUANTYCSEQUENCE    (ui16) 0x2000
#define GRAYSCALE_DECOMP_QUANTABSEQUENCE    (ui16) 0x0000
#define GRAYSCALE_DECOMP_VIDEOLATENCY       (ui16) 0x00BF
#define GRAYSCALE_DECOMP_CONFIG             (ui16) 0x0188


/* these are the CL550 init register values for GREYSCALE COMPRESSION */

#define GRAYSCALE_COMP_INIT1              (ui16) 0x0002
#define GRAYSCALE_COMP_INIT2              (ui16) 0x0001
#define GRAYSCALE_COMP_INIT3              (ui16) 0x0141
#define GRAYSCALE_COMP_INIT4              (ui16) 0x00F7
#define GRAYSCALE_COMP_INIT5              (ui16) 0x0000
#define GRAYSCALE_COMP_INIT6              (ui16) 0x0000
#define GRAYSCALE_COMP_INIT7              (ui16) 0x005D
#define GRAYSCALE_COMP_QUANTSYNC          (ui16) 0x0406
#define GRAYSCALE_COMP_QUANTYCSEQUENCE    (ui16) 0x2000
#define GRAYSCALE_COMP_QUANTABSEQUENCE    (ui16) 0x0000
#define GRAYSCALE_COMP_VIDEOLATENCY       (ui16) 0x00BF
#define GRAYSCALE_COMP_CONFIG             (ui16) 0x0088

/* these are the CL550 config register values for GRAYSCALE */

#define GRAYSCALE_HUFFTABLESEQUENCE  (ui16) 0x0000
#define GRAYSCALE_DPCMSEQHIGH        (ui16) 0x0000
#define GRAYSCALE_DPCMSEQLOW         (ui16) 0x0000
#define GRAYSCALE_CODERATTR          (ui16) 0x0011
#define GRAYSCALE_CODINGINTH         (ui16) 0x0000
#define GRAYSCALE_CODINGINTL         (ui16) 0x0100    /* 0x0001 in Appendix A, 0x000f in manual, 0x0100 in demo code */
#define GRAYSCALE_DECLENGTH          (ui16) 0x0008
#define GRAYSCALE_DECCODEORDER       (ui16) 0x0001
#define GRAYSCALE_QUANTABSELECT      (ui16) 0x0000

/* these are the CL550 init register values for RGB->YUV_422 COMPRESSION */

#define RGB_YUV_COMP_INIT1              (ui16) 0x003C
#define RGB_YUV_COMP_INIT2              (ui16) 0x003D
#define RGB_YUV_COMP_INIT3              (ui16) 0x007B
#define RGB_YUV_COMP_INIT4              (ui16) 0x00F1
#define RGB_YUV_COMP_INIT5              (ui16) 0x000A
#define RGB_YUV_COMP_INIT6              (ui16) 0x0000
#define RGB_YUV_COMP_INIT7              (ui16) 0x0017
#define RGB_YUV_COMP_QUANTSYNC          (ui16) 0x0400
#define RGB_YUV_COMP_QUANTYCSEQUENCE    (ui16) 0x2099
#define RGB_YUV_COMP_QUANTABSEQUENCE    (ui16) 0x0000
#define RGB_YUV_COMP_VIDEOLATENCY       (ui16) 0x0185
#define RGB_YUV_COMP_CONFIG             (ui16) 0x0038

/* these are the CL550 init register values for RGB->YUV_422 DECOMPRESSION */

#define RGB_YUV_DECOMP_INIT1              (ui16) 0x003E
#define RGB_YUV_DECOMP_INIT2              (ui16) 0x0037
#define RGB_YUV_DECOMP_INIT3              (ui16) 0x01FF
#define RGB_YUV_DECOMP_INIT4              (ui16) 0x0049
#define RGB_YUV_DECOMP_INIT5              (ui16) 0x0005
#define RGB_YUV_DECOMP_INIT6              (ui16) 0x0000
#define RGB_YUV_DECOMP_INIT7              (ui16) 0x0022
#define RGB_YUV_DECOMP_QUANTSYNC          (ui16) 0x043E
#define RGB_YUV_DECOMP_QUANTYCSEQUENCE    (ui16) 0x2033
#define RGB_YUV_DECOMP_QUANTABSEQUENCE    (ui16) 0x0000
#define RGB_YUV_DECOMP_VIDEOLATENCY       (ui16) 0x017F
#define RGB_YUV_DECOMP_CONFIG             (ui16) 0x0138


/* these are the CL550 config register values for RGB_YUV */

#define RGB_YUV_HUFFTABLESEQUENCE  (ui16) 0x00CC
#define RGB_YUV_DPCMSEQHIGH        (ui16) 0x0088
#define RGB_YUV_DPCMSEQLOW         (ui16) 0x0044
#define RGB_YUV_CODERATTR          (ui16) 0x0004
#define RGB_YUV_CODINGINTH         (ui16) 0x0000
#define RGB_YUV_CODINGINTL         (ui16) 0x0008    /* 0x0001 in Appendix A, 0x000f in manual, 0x0100 in demo code */
#define RGB_YUV_DECLENGTH          (ui16) 0x0008
#define RGB_YUV_DECCODEORDER       (ui16) 0x0001
#define RGB_YUV_QUANTABSELECT      (ui16) 0x0000

/* these are the CL550 init register values for YUV_422 COMPRESSION */

#define YUV_422_COMP_INIT1              (ui16) 0x0042
#define YUV_422_COMP_INIT2              (ui16) 0x0001
#define YUV_422_COMP_INIT3              (ui16) 0x0081
#define YUV_422_COMP_INIT4              (ui16) 0x00F7
#define YUV_422_COMP_INIT5              (ui16) 0x0000
#define YUV_422_COMP_INIT6              (ui16) 0x0000
#define YUV_422_COMP_INIT7              (ui16) 0x001D
#define YUV_422_COMP_QUANTSYNC          (ui16) 0x0406
#define YUV_422_COMP_QUANTYCSEQUENCE    (ui16) 0x2099
#define YUV_422_COMP_QUANTABSEQUENCE    (ui16) 0x0000
#define YUV_422_COMP_VIDEOLATENCY       (ui16) 0x017F
#define YUV_422_COMP_CONFIG             (ui16) 0x0018

/* these are the CL550 init register values for YUV_422 DECOMPRESSION */

#define YUV_422_DECOMP_INIT1              (ui16) 0x003E
#define YUV_422_DECOMP_INIT2              (ui16) 0x0037
#define YUV_422_DECOMP_INIT3              (ui16) 0x01FF
#define YUV_422_DECOMP_INIT4              (ui16) 0x0049
#define YUV_422_DECOMP_INIT5              (ui16) 0x0005
#define YUV_422_DECOMP_INIT6              (ui16) 0x0000
#define YUV_422_DECOMP_INIT7              (ui16) 0x0022
#define YUV_422_DECOMP_QUANTSYNC          (ui16) 0x043E
#define YUV_422_DECOMP_QUANTYCSEQUENCE    (ui16) 0x2033
#define YUV_422_DECOMP_QUANTABSEQUENCE    (ui16) 0x0000
#define YUV_422_DECOMP_VIDEOLATENCY       (ui16) 0x017F
#define YUV_422_DECOMP_CONFIG             (ui16) 0x0118


/* these are the CL550 config register values for YUV */

#define YUV_422_HUFFTABLESEQUENCE  (ui16) 0x00CC
#define YUV_422_DPCMSEQHIGH        (ui16) 0x0088
#define YUV_422_DPCMSEQLOW         (ui16) 0x0044
#define YUV_422_CODERATTR          (ui16) 0x0004
#define YUV_422_CODINGINTH         (ui16) 0x0000
#define YUV_422_CODINGINTL         (ui16) 0x0001    /* 0x0001 in Appendix A, 0x000f in manual, 0x0100 in demo code */
#define YUV_422_DECLENGTH          (ui16) 0x0008
#define YUV_422_DECCODEORDER       (ui16) 0x0001
#define YUV_422_QUANTABSELECT      (ui16) 0x0000


/* these are the CL550 init register values for RGB_444 COMPRESSION */

#define RGB_444_COMP_INIT1              (ui16) 0x0040
#define RGB_444_COMP_INIT2              (ui16) 0x0000
#define RGB_444_COMP_INIT3              (ui16) 0x007F
#define RGB_444_COMP_INIT4              (ui16) 0x00F5
#define RGB_444_COMP_INIT5              (ui16) 0x000E
#define RGB_444_COMP_INIT6              (ui16) 0x0000
#define RGB_444_COMP_INIT7              (ui16) 0x001B
#define RGB_444_COMP_QUANTSYNC          (ui16) 0x0404
#define RGB_444_COMP_QUANTYCSEQUENCE    (ui16) 0x2044
#define RGB_444_COMP_QUANTABSEQUENCE    (ui16) 0x0088
#define RGB_444_COMP_VIDEOLATENCY       (ui16) 0x0181
#define RGB_444_COMP_CONFIG             (ui16) 0x0048

/* these are the CL550 init register values for RGB_444 DECOMPRESSION */

#define RGB_444_DECOMP_INIT1              (ui16) 0x003E
#define RGB_444_DECOMP_INIT2              (ui16) 0x0037
#define RGB_444_DECOMP_INIT3              (ui16) 0x01FF
#define RGB_444_DECOMP_INIT4              (ui16) 0x0049
#define RGB_444_DECOMP_INIT5              (ui16) 0x0005
#define RGB_444_DECOMP_INIT6              (ui16) 0x0000
#define RGB_444_DECOMP_INIT7              (ui16) 0x0022
#define RGB_444_DECOMP_QUANTSYNC          (ui16) 0x043E
#define RGB_444_DECOMP_QUANTYCSEQUENCE    (ui16) 0x2088
#define RGB_444_DECOMP_QUANTABSEQUENCE    (ui16) 0x0011
#define RGB_444_DECOMP_VIDEOLATENCY       (ui16) 0x017F
#define RGB_444_DECOMP_CONFIG             (ui16) 0x0148

/* these are the CL550 config register values for RGB 444 */

#define RGB_444_HUFFTABLESEQUENCE  (ui16) 0x0000
#define RGB_444_DPCMSEQHIGH        (ui16) 0x0024
#define RGB_444_DPCMSEQLOW         (ui16) 0x0012
#define RGB_444_CODERATTR          (ui16) 0x0003
#define RGB_444_CODINGINTH         (ui16) 0x0000
#define RGB_444_CODINGINTL         (ui16) 0x0001    /* 0x0001 in Appendix A, 0x000f in manual, 0x0100 in demo code */
#define RGB_444_DECLENGTH          (ui16) 0x0006
#define RGB_444_DECCODEORDER       (ui16) 0x0001
#define RGB_444_QUANTABSELECT      (ui16) 0x0000


#define HUFF_LOAD_ENABLE   (ui16) 0x00ff
/* #define SRESET          (ui16) 0x0000   */
#define START              (ui16) 0x0000
#define HVENABLE           (ui16) 0x0000
#define DRQMASK            (ui16) 0x0000 /* NOTE may not need this */

/* RGB-YUV colour transformation matrix - decompression */

#define Y2R                (ui16) 0x0400
#define U2R                (ui16) 0x0FFE
#define V2R                (ui16) 0x059C
#define Y2G                (ui16) 0x0400
#define U2G                (ui16) 0x0EA2
#define V2G                (ui16) 0x0D23
#define Y2B                (ui16) 0x0400
#define U2B                (ui16) 0x071B
#define V2B                (ui16) 0x0FFD

/* RGB-YUV colour transformation matrix - compression */

#define R2Y                (ui16) 0x0133
#define G2Y                (ui16) 0x0259
#define B2Y                (ui16) 0x0074
#define R2U                (ui16) 0x0F54
#define G2U                (ui16) 0x0EAD
#define B2U                (ui16) 0x01FF
#define R2V                (ui16) 0x01FF
#define G2V                (ui16) 0x0E53
#define B2V                (ui16) 0x0FAE

typedef struct TCL550_registers {

ui16  Codec;              
ui16  HPeriod;            
ui16  HSync;              
ui16  HDelay;             
ui16  HActive;            
ui16  VPeriod;            
ui16  VSync;              
ui16  VDelay;             
ui16  VActive;            
ui16  DCT0;                
ui16  DCT1;                
ui16  DCT2;                
ui16  DCT3;                
ui16  DCT4;                
ui16  DCT5;                
ui16  DCT6;                
ui16  DCT7;                
ui16  Init_5;             
ui16  Init_6;             
ui16  Config;             
ui16  Huff_Enable;        
ui16  S_Reset;            
ui16  Start;              
ui16  HV_Enable;          
ui16  Flags;              
ui16  NMRQ_Mask;          
ui16  DRQ_Mask;           
ui16  StartOfFrame;       
ui16  Version;            
ui16  Init_1;             
ui16  Init_2;             
ui16  HuffTableSequence;  
ui16  DPCM_SeqHigh;       
ui16  DPCM_SeqLow;        
ui16  CoderAttr;          
ui16  CodingIntH;         
ui16  CodingIntL;         
ui16  DecLength;          
ui16  DecMarker;          
ui16  DecResume;          
ui16  DecDPCM_Reset;      
ui16  DecCodeOrder;       
ui16  Init_3;             
ui16  QTableBase;         
ui16  QTable1;            
ui16  QTable2;            
ui16  QTable3;            
ui16  QTable4;            
ui16  QuantABSelect;      
ui16  QuantSync;          
ui16  QuantYCSequence;    
ui16  QuantABSequence;    
ui16  Matrix00;           
ui16  Matrix01;           
ui16  Matrix02;           
ui16  Matrix10;           
ui16  Matrix11;           
ui16  Matrix12;           
ui16  Matrix20;           
ui16  Matrix21;           
ui16  Matrix22;           
ui16  VideoLatency;       
ui16  HControl;           
ui16  VControl;           
ui16  VertLineCount;      
ui16  Init_4;             
ui16  Init_7;             
ui16  FIFO_Base;          
ui16  HuffTables;         
ui16  HuffYAC;            
ui16  HuffYDC;            
ui16  HuffCAC;            
ui16  HuffCDC;            

} TCL550_registers;

#endif /*!_cl550_h*/
