/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : crunch.h                                                *
 *  Function    : Definitions for Crunch DOS Library (CRUNCH) functions   *
 *  Project     : Snapper                                                 *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : October 1993                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _crunch_h
#define _crunch_h


/* Crunch Structure - */
/* ------------------ */
typedef struct Tcrunch {
   ui32 strip_buf_size;        /* 1024 pixels wide or 4096  (24K or 96K) */
   Tboolean strip_buf_enabled; /* set to TRUE if the strip buffer is enabled */
   ui16 clock_speed;           /* 30 or 10 (MHz)    */
   ui16 hbus_fifo_size;        /* 512 or 1024, 0 = no external FIFO  */
   ui16 compress;              /* set to TRUE for use in compression */
   ui16 Q_factor;              /* used in compression only */
   ui16 pixel_mode;            /* pixel bus mode */
#define     CRUNCH_GRAYSCALE       0x0000
#define     CRUNCH_YUV_422         0x0001
#define     CRUNCH_RGB_YUV         0x0002
#define     CRUNCH_RGB_444         0x0003
   ui16 mode;                  /* operational mode */
#define     CRUNCH_JPEG_ENGINE     0x0001
#define     CRUNCH_RAW_VIDEO       0x0002
#define     CRUNCH_JPEG_VIDEO      0x0003
#define     CRUNCH_MODULE_ENGINE   0x0004
   struct TCL550_registers *PCL550_reg;    /* pointer to Crunch register structure */
} Tcrunch;


/* XFIFO defines  - "X" for external FIFO */
/* -------------------------------------- */
#define CRUNCH_XFIFO_SELCONF  0x0100
#define CRUNCH_XCONF_REG_0    0x0
#define CRUNCH_XCONF_REG_1    0x1
#define CRUNCH_XCONF_REG_2    0x2
#define CRUNCH_XCONF_REG_3    0x3
#define CRUNCH_XCONF_REG_4    0x4
#define CRUNCH_DECOMP_XFIFO_FLAGS 0x8BA9
#define CRUNCH_COMP_XFIFO_FLAGS   0xCDCF
#define CRUNCH_XNEARLY_EMPTY_AB   0x001
#define CRUNCH_XNEARLY_FULL_AB    0x200
#define CRUNCH_XNEARLY_EMPTY_BA   0x200
#define CRUNCH_XNEARLY_FULL_BA    0x001


/*
 * FIFO_FRACTION - This defines the ratio of the largest block
 * that is read out of the pixel side of the card without
 * checking to see if the external FIFO has drained to below
 * half way.  Generally this should be set to 4.  Greater than
 * 4 is less efficient, but perhaps safer.  Less than 4 is 
 * risky (and not much more efficient).
 * It should also always be an even number.
 */
#define CRUNCH_FIFO_FRACTION 4


/* Huffman #Defines */
/* ---------------- */
#define DefaultLengthOfValues_YDC   12;
#define DefaultLengthOfValues_CDC   12;
#define DefaultLengthOfValues_YAC   162;
#define DefaultLengthOfValues_CAC   162;


/* CRUNCH Prototype Definitions */
/* ---------------------------- */
#ifdef __cplusplus             
extern "C" {      
#endif
Terr EXPORT_FN CRUNCH_calculateHControl(Thandle, ui16);
Terr EXPORT_FN CRUNCH_calculateVControl(Thandle, ui16, ui16, ui16);
Terr EXPORT_FN CRUNCH_calculateVDelay(Thandle, ui16, ui16, ui16);
Terr EXPORT_FN CRUNCH_check_underflow(Thandle, char*);
Terr EXPORT_FN CRUNCH_clear_Huffman_tables(void);
Terr EXPORT_FN CRUNCH_compress(Thandle, Thandle, Thandle, ui16);      /* user function */
Terr EXPORT_FN CRUNCH_compress_image_to_image(Thandle, Thandle, Thandle, ui16, ui16); /* user function */
Terr EXPORT_FN CRUNCH_compress_1_strip(Thandle Hcrunch, Thandle, Thandle, ui16);
Terr EXPORT_FN CRUNCH_configure_crunch(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_configure_crunch_XFIFO(Thandle);
Terr EXPORT_FN CRUNCH_copy_H_tables_to_structure(Thandle);
Terr EXPORT_FN CRUNCH_copy_Q_table_to_structure(Thandle, ui8*, ui16);
Terr EXPORT_FN CRUNCH_copy_ui16_table(ui16*, ui16*);
i16  EXPORT_FN CRUNCH_count_bits(ui16);
Terr EXPORT_FN CRUNCH_decompress(Thandle, Thandle, Thandle, ui16);  /* user function */
Terr EXPORT_FN CRUNCH_decompress_image_to_image(Thandle, Thandle, Thandle, ui16, ui16); /* user function */
Terr EXPORT_FN CRUNCH_decompress_1_strip(Thandle, Thandle, Thandle);
Terr EXPORT_FN CRUNCH_empty_fifos(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_fill_external_FIFO(Thandle, Thandle, ui16);
Terr EXPORT_FN CRUNCH_fill_xFIFO_from_file(Thandle, Thandle, ui16*, ui16);
Terr EXPORT_FN CRUNCH_fill_xFIFO_from_memory(Thandle, Thandle, ui16*, ui16);
Terr EXPORT_FN CRUNCH_get_Pcrunch(Thandle, struct Tcrunch*);
Terr EXPORT_FN CRUNCH_load_table(Thandle, ui16, ui16, ui16*, ui16);
Terr EXPORT_FN CRUNCH_make_CL550_Q_table(Thandle, ui8*, ui16*);
Terr EXPORT_FN CRUNCH_make_JPEG_Q_table(ui16, ui16*, ui8*);
Terr EXPORT_FN CRUNCH_module_compress_1_strip(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_read_CL550_reg(Thandle, ui16, ui16);  
Terr EXPORT_FN CRUNCH_read_XFIFO_config(Thandle, ui16);
Terr EXPORT_FN CRUNCH_sequence_replay(Thandle, Thandle, Thandle, ui16, ui16); /* user function */
Terr EXPORT_FN CRUNCH_set_CL550_registers(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_set_clock_speed(Thandle, ui16);
Terr EXPORT_FN CRUNCH_set_default_Huffman_tables(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_set_mode_crunch(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_compress(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_colour_RGB_to_YUV(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_decompress(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_grayscale(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_module_engine(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_RGB_444(Thandle);
Terr EXPORT_FN CRUNCH_set_mode_YUV_422(Thandle);
Terr EXPORT_FN CRUNCH_set_Quality_factor(Thandle, ui16);       /* user function */
Terr EXPORT_FN CRUNCH_set_Quantization_factor(Thandle, ui16);  /* user function */
Terr EXPORT_FN CRUNCH_set_quantizer_tables(Thandle, Thandle);
Terr EXPORT_FN CRUNCH_soft_reset_crunch(Thandle);
Terr EXPORT_FN CRUNCH_start_CL550(Thandle);
Terr EXPORT_FN CRUNCH_write_XFIFO_config(Thandle, ui16, ui16);
/* mod_high.c functions */
Terr EXPORT_FN CRUNCH_compress_from_snapper(Thandle, Thandle, ui16);       /* user function */
Terr EXPORT_FN CRUNCH_sequence_record(Thandle, Thandle, ui32, ui16, ui16); /* user function */

Terr EXPORT_FN CRUNCH_compress_from_s16(Thandle, Thandle, ui16);   /* non-doc/user */
Terr EXPORT_FN CRUNCH_compress_from_s24(Thandle, Thandle, ui16);   /* non-doc/user */
Terr EXPORT_FN CRUNCH_sequence_record_s16(Thandle, Thandle, ui32, ui16, ui16); /* non-doc/user */
Terr EXPORT_FN CRUNCH_s16_compress(Thandle, Thandle, ui16);        /* non-doc/user */
Terr EXPORT_FN CRUNCH_s24_compress(Thandle, Thandle, ui16);        /* non-doc/user */
#ifdef __cplusplus
};                
#endif


/*
 * Macro Definitions
 * -----------------
 */
/* None */


/*
 * Externals
 * ---------
 */
extern struct Tcrunch gTcrunch;  /* global - used locally for crunch DLL */
extern struct Tcrunch *gPcrunch;

/*
 * Huffman table arrays
 * Windows needs numbers in for large model DLL
 */
extern ui8 DefaultBitsNVals_YDC[28];
extern ui8 DefaultBitsNVals_CDC[28];
extern ui8 DefaultBitsNVals_YAC[178];
extern ui8 DefaultBitsNVals_CAC[178];

extern ui16 CompDefaultHuffmanYDC[25];
extern ui16 CompDefaultHuffmanCDC[25];
extern ui16 CompDefaultHuffmanYAC[513];
extern ui16 CompDefaultHuffmanCAC[513];

extern ui16 DecompDefaultHuffmanYDC[29];
extern ui16 DecompDefaultHuffmanCDC[25];
extern ui16 DecompDefaultHuffmanYAC[241];
extern ui16 DecompDefaultHuffmanCAC[245];

extern ui16 CL550HuffmanYDC[32]; 
extern ui16 CL550HuffmanCDC[32];
extern ui16 CL550HuffmanYAC[704];
extern ui16 CL550HuffmanCAC[704];

/*
 * Q table arrays
 */
extern ui16 Default_Y_Visi_Table[64];
extern ui16 Default_C_Visi_Table[64];
extern ui16 Zero_Visi_Table[64];
extern double CompKFactor[64];
extern double DecompKFactor[64];

extern ui8 JPEG_Q_Table1[64];
extern ui8 JPEG_Q_Table2[64];
extern ui8 JPEG_Q_Table3[64];
extern ui8 JPEG_Q_Table4[64];

extern ui16 CL550_Q_Table1[64];
extern ui16 CL550_Q_Table2[64];
extern ui16 CL550_Q_Table3[64];
extern ui16 CL550_Q_Table4[64];

extern ui16 Zig_Zag[64];


#endif /* _crunch_h */

