/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : dig16.h                                                 *
 *  Function    : Header for DIG-16 - #defines and types                  *
 *  Project     : Snappers - DC10                                         *
 *  Authors     : Chris Beynon, Martin Bone, Active Silicon Ltd           *
 *  Systems     : ANSI C                                                  *
 *  Written     : February 1995                                           *
 *  Updated     : 04-Jul-97                                               *
 *  Version     : 3.13                                                    *
 *                                                                        *
 *  Copyright (c) 1995-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _DIG16_H_
#define _DIG16_H_

#ifndef _MALLOC_DEBUG
#define DIG16_COMPATIBILITY   /* defined for release */
#endif

/* TYPE DEFINITIONS
 * ----------------
 */


/* ARRAY SIZES
 * -----------
 */
#define DIG16_NUM_HANDLES          16
#define DIG16_NUM_PAGES            16
#define DIG16_NUM_LCA_REGS         (8*DIG16_NUM_PAGES)

#define DIG16_SIZ_LUT8             256
#define DIG16_SIZ_LUT16            ((ui32) 65536)

#define DIG16_MIN_ROI_LENGTH        8
#define DIG16_MIN_LINESCAN_Y_LENGTH 1
#define DIG16_MAX_X_LENGTH          8192
#define DIG16_MAX_Y_LENGTH          8192
#define DIG16_MAX_LINESCAN_LENGTH   ((ui32) 65536)


#define DIG16_FIFO16_BYTES         ((ui32) 32768)  /* FIFO size */
#define DIG16_FIFO32_BYTES         ((ui32) 65536)  /* FIFO size */


#if (defined _WINNT || defined _WIN95 || defined _VXWORKS || defined _LYNXOS || defined _SOLARIS || defined _MACOSX || defined _LINUX) && !(defined _FOR_WIN95)

#define DIG16_MIN_TRANSFER_BYTES   ((ui32) 8)
/* NOTE: Minimum in "normal" readout is:
 * 17 (DIG16_PAE_OFFSET+1) x 4 bytes (in DIG16 FIFO) +
 * 2 -> 4 x 4 in Data Mapper +
 * 4 x 4 in AMCC = 112 bytes!
 */
#define DIG16_FLUSH_BYTES          ((ui32) 384 )   /* Was 512 (Must be less than FIFO size) */
#else                                              /* *** DO NOT CHANGE ***                 */
#define DIG16_FLUSH_BYTES          ((ui32) 16384)  /* Must be less than FIFO size */
#endif

/* FIFO Flag Offsets - note that these are in 16/32 bit words, not bytes */
#define DIG16_PAE_OFFSET        16     /* Must be at least 6 for Aug-96 Data Mappers */
#define DIG16_PAF_OFFSET        1024   /* Arbitary */

/* NOTE In threads mode PAF_OFFSET was set to the following, but this has not
 * been used since before DMA supported under DOS/WIN31
 * #define DIG16_PAF_OFFSET        ((DIG16_FIFO_BYTES / 2) - 2048)
 */


/* ERROR CODES
 * -----------
 * The error return code consists of 4 bytes.  The lower 2 contain any
 * parameter information from the function.  The next byte contains an
 * error code, and the highest byte contains the ID of the module
 * signalling the error.  This technique allows the software to
 * return an error although the original call may have been made
 * indirectly through another library call.
 */
/* ANY ADDITIONAL IDS MUST BE INCLUDED IN BASE_XXX.C AS WELL AS THE DIG16 LIBRARY */
#define DIG16_ID              ((ui8) 0xC0)
#define DIG16_PMC_ID          ((ui8) 0xC8)
#define DIG16_50M_ID          ((ui8) 0x80)
#define DIG16_50M_PMC_ID      ((ui8) 0x88)
#define DIG_PMC16_ID          DIG16_PMC_ID /* For Compatibility */
#define PMC_DIG16_ID          DIG16_PMC_ID /* For Compatibility */
#define DIG16_FAMILY_ID       DIG16_ID


/* REGISTER ADDRESSES
 * ------------------
 */
#define DIG16_PAGE_NONE       0x00
#define DIG16_PAGE_GENERAL    0x00
#define DIG16_PAGE_ROI_X      0x01
#define DIG16_PAGE_ROI_Y      0x02
#define DIG16_PAGE_GENERAL2   0x03
#define DIG16_PAGE_LUT        0x04

#define DIG16_REG_GEN_CTRL    ((DIG16_PAGE_NONE     << 3) | 0x00)
#define DIG16_REG_ID          ((DIG16_PAGE_NONE     << 3) | 0x00)
#define DIG16_REG_CAPTURE     ((DIG16_PAGE_NONE     << 3) | 0x01)
#define DIG16_REG_CAP_STAT    ((DIG16_PAGE_NONE     << 3) | 0x01)
#define DIG16_REG_INT_CTRL    ((DIG16_PAGE_NONE     << 3) | 0x02)
#define DIG16_REG_INT_STAT    ((DIG16_PAGE_NONE     << 3) | 0x02)
#define DIG16_REG_PAGE        ((DIG16_PAGE_NONE     << 3) | 0x03)
#define DIG16_REG_GEN_STAT    ((DIG16_PAGE_NONE     << 3) | 0x03)

#define DIG16_REG_CAMERA_OP   ((DIG16_PAGE_GENERAL  << 3) | 0x04)
#define DIG16_REG_CAMERA_IP   ((DIG16_PAGE_GENERAL  << 3) | 0x04)
#define DIG16_REG_CAMERA_FMT  ((DIG16_PAGE_GENERAL  << 3) | 0x05)
#define DIG16_REG_TRIG_CTRL   ((DIG16_PAGE_GENERAL  << 3) | 0x06)
#define DIG16_REG_DATA_MASK   ((DIG16_PAGE_GENERAL  << 3) | 0x07)

#define DIG16_REG_X_START_LO  ((DIG16_PAGE_ROI_X    << 3) | 0x04)
#define DIG16_REG_X_START_HI  ((DIG16_PAGE_ROI_X    << 3) | 0x05)
#define DIG16_REG_X_LENGTH_LO ((DIG16_PAGE_ROI_X    << 3) | 0x06)
#define DIG16_REG_X_LENGTH_HI ((DIG16_PAGE_ROI_X    << 3) | 0x07)

#define DIG16_REG_Y_START_LO  ((DIG16_PAGE_ROI_Y    << 3) | 0x04)
#define DIG16_REG_Y_START_HI  ((DIG16_PAGE_ROI_Y    << 3) | 0x05)
#define DIG16_REG_Y_LENGTH_LO ((DIG16_PAGE_ROI_Y    << 3) | 0x06)
#define DIG16_REG_Y_LENGTH_HI ((DIG16_PAGE_ROI_Y    << 3) | 0x07)

#define DIG16_REG_LUT_CTRL    ((DIG16_PAGE_GENERAL2 << 3) | 0x04)
#define DIG16_REG_TEST_LUT    ((DIG16_PAGE_GENERAL2 << 3) | 0x05)
#define DIG16_REG_FIFO_SERIAL ((DIG16_PAGE_GENERAL2 << 3) | 0x06)
#define DIG16_REG_CLK_FREQ    ((DIG16_PAGE_GENERAL2 << 3) | 0x07)

#define DIG16_REG_LUT         ((DIG16_PAGE_LUT      << 3) | 0x04)


/* Line scan registers
 * -------------------
 */
#define DIG16_REG_LINE_START_OUT ((DIG16_PAGE_ROI_Y   << 3) | 0x04)
#define DIG16_REG_LINESCAN_CTRL1 ((DIG16_PAGE_ROI_Y   << 3) | 0x05)
#define DIG16_REG_LINESCAN_CTRL2 ((DIG16_PAGE_ROI_Y   << 3) | 0x06)
#define DIG16_REG_LINE_EXP_TIMER ((DIG16_PAGE_ROI_Y   << 3) | 0x07)


/* Data stream registers
 * ---------------------
 */
#define DIG16_REG_DSTRM_COUNT_0  ((DIG16_PAGE_ROI_X   << 3) | 0x04)
#define DIG16_REG_DSTRM_COUNT_1  ((DIG16_PAGE_ROI_X   << 3) | 0x05)
#define DIG16_REG_DSTRM_COUNT_2  ((DIG16_PAGE_ROI_X   << 3) | 0x06)
#define DIG16_REG_DSTRM_COUNT_3  ((DIG16_PAGE_ROI_X   << 3) | 0x07)

#define DIG16_REG_DSTRM_CTRL     ((DIG16_PAGE_ROI_Y   << 3) | 0x05)



/* DIG16_REGISTER_BITS
 * ===================
 */

/* General Control Register Register
 * ---------------------------------
 */
#define DIG16_HMASK_EXP_ON_OUT_A    DIG16_BIT_EXP_ON_OUT_A
#define DIG16_BIT_OUT_A_ON_OUT_A    ((ui8) 0x00)
#define DIG16_BIT_EXP_ON_OUT_A      ((ui8) 0x01)

#define DIG16_HMASK_PCLK_POS_NEG    DIG16_BIT_PCLK_POS
#define DIG16_BIT_PCLK_NEG          ((ui8) 0x00)
#define DIG16_BIT_PCLK_POS          ((ui8) 0x02)

#define DIG16_HMASK_EXP_POS_NEG     DIG16_BIT_EXP_POS
#define DIG16_BIT_EXP_NEG           ((ui8) 0x00)
#define DIG16_BIT_EXP_POS           ((ui8) 0x04)

#define DIG16_HMASK_FIFO_SER_LD     DIG16_BIT_FIFO_SER_LD
#define DIG16_BIT_FIFO_SER_RUN      ((ui8) 0x00)
#define DIG16_BIT_FIFO_SER_LD       ((ui8) 0x08)

#define DIG16_HMASK_FIFO_MRST       DIG16_BIT_FIFO_MRST
#define DIG16_BIT_FIFO_MRUN         ((ui8) 0x00)
#define DIG16_BIT_FIFO_MRST         ((ui8) 0x10)

#define DIG16_HMASK_FIFO_PRST       DIG16_BIT_FIFO_PRST
#define DIG16_BIT_FIFO_PRUN         ((ui8) 0x00)
#define DIG16_BIT_FIFO_PRST         ((ui8) 0x20)

#define DIG16_HMASK_FIFO_16_32      DIG16_BIT_FIFO_16
#define DIG16_BIT_FIFO_32           ((ui8) 0x00)
#define DIG16_BIT_FIFO_16           ((ui8) 0x40)

#define DIG16_HMASK_FIELD_INVERT    DIG16_BIT_FIELD_INVERT
#define DIG16_BIT_FIELD_NORMAL      ((ui8) 0x00)
#define DIG16_BIT_FIELD_INVERT      ((ui8) 0x80)


/* Capture Register
 * ----------------
 */
#define DIG16_BIT_CAPTURE           ((ui8) 0x01)

#define DIG16_HMASK_INT_NON_INT     DIG16_BIT_INTERLACED
#define DIG16_BIT_NON_INTERLACED    ((ui8) 0x00)
#define DIG16_BIT_INTERLACED        ((ui8) 0x02)

#define DIG16_HMASK_WHICH_FIELD     DIG16_BIT_1ST_FIELD
#define DIG16_BIT_NEXT_FIELD        ((ui8) 0x00)
#define DIG16_BIT_1ST_FIELD         ((ui8) 0x04)

#define DIG16_HMASK_FIELD_FRAME     DIG16_BIT_FIELD_MODE
#define DIG16_BIT_FRAME_MODE        ((ui8) 0x00)
#define DIG16_BIT_FIELD_MODE        ((ui8) 0x08)

#define DIG16_HMASK_SEQUENCE_SINGLE DIG16_BIT_SEQUENCE_CAPTURE
#define DIG16_BIT_SINGLE_CAPTURE    ((ui8) 0x00)
#define DIG16_BIT_SEQUENCE_CAPTURE  ((ui8) 0x10)

#define DIG16_HMASK_SUBSAMPLE_MODE  ((ui8) 0x60)
#define DIG16_BITS_SUB_X1           ((ui8) 0x00)
#define DIG16_BITS_SUB_X2           ((ui8) 0x20)
#define DIG16_BITS_SUB_X4           ((ui8) 0x40)
#define DIG16_BITS_SUB_X8           ((ui8) 0x60)

#define DIG16_HMASK_EARLY_TC        DIG16_BIT_EARLY_TC_EN
#define DIG16_BIT_EARLY_TC_DIS      ((ui8) 0x00)
#define DIG16_BIT_EARLY_TC_EN       ((ui8) 0x80)


/* Capture Status Register
 * -----------------------
 * Note - capture armed bit doubles as capture triggered bit
 */
#define DIG16_STAT_CAPTURE_ENABLED_SW    ((ui8) 0x01)
#define DIG16_STAT_CAPTURE_ARMED         ((ui8) 0x02)
#define DIG16_STAT_CAPTURE_SYNCED        ((ui8) 0x04)
#define DIG16_STAT_CAPTURE_IN_PROGRESS   ((ui8) 0x08)
#define DIG16_STAT_CAPTURE_ENABLED_HW    ((ui8) 0x10)
#define DIG16_STAT_FIELD1_CAPTURED       ((ui8) 0x80)


/* Interrupt Control & Status Register
 * -----------------------------------
 * Note: Any bits in the lower 16 bits represent hardware generated 
 * interrupts. Any bits in the upper 16 bits represent virtual interrupts
 * generated by the driver which appear like hardware interrupts to the user
 * but in fact are software generated interrupts.
 * Note: 0x8000000 as a software interrupt is reserved for abort/closing of 
 * interrupt thread (it is used in base_<os>.c and the driver).
 */
#define DIG16_HMASK_INT_FRAME       DIG16_BIT_INT_FRAME_EN
#define DIG16_BIT_INT_FRAME_DIS     ((ui32) 0x0000)
#define DIG16_BIT_INT_FRAME_EN      ((ui32) 0x0001)

#define DIG16_HMASK_INT_TRIGGER     DIG16_BIT_INT_TRIGGER_EN
#define DIG16_BIT_INT_TRIGGER_DIS   ((ui32) 0x0000)
#define DIG16_BIT_INT_TRIGGER_EN    ((ui32) 0x0002)

#define DIG16_HMASK_INT_FIFO        DIG16_BIT_INT_FIFO_EN
#define DIG16_BIT_INT_FIFO_DIS      ((ui32) 0x0000)
#define DIG16_BIT_INT_FIFO_EN       ((ui32) 0x0004)

#define DIG16_HMASK_INT_CAPTURE     DIG16_BIT_INT_CAPTURE_EN
#define DIG16_BIT_INT_CAPTURE_DIS   ((ui32) 0x0000)
#define DIG16_BIT_INT_CAPTURE_EN    ((ui32) 0x0008)

#define DIG16_HMASK_INT_LINE        DIG16_BIT_INT_LINE_EN
#define DIG16_BIT_INT_LINE_DIS      ((ui32) 0x0000)
#define DIG16_BIT_INT_LINE_EN       ((ui32) 0x0010)

#define DIG16_HMASK_INT_1ST_DATA    DIG16_BIT_INT_1ST_DATA_EN
#define DIG16_BIT_INT_1ST_DATA_DIS  ((ui32) 0x0000)
#define DIG16_BIT_INT_1ST_DATA_EN   ((ui32) 0x0020)

/* "Software" Interrupts: */

#define DIG16_HMASK_INT_MAIN_TRANSFER  DIG16_BIT_INT_MAIN_TRANSFER_EN
#define DIG16_BIT_INT_MAIN_TRANSFER_DIS  ((ui32) 0x00000000)
#define DIG16_BIT_INT_MAIN_TRANSFER_EN   ((ui32) 0x00010000)

#define DIG16_HMASK_INT_RESERVED  DIG16_BIT_INT_RESERVED_EN
#define DIG16_BIT_INT_RESERVED_DIS  ((ui32) 0x00000000)
#define DIG16_BIT_INT_RESERVED_EN   ((ui32) 0x80000000)

#define DIG16_HMASK_HW_INTERRUPTS  ((ui32) 0x0000ffff)
#define DIG16_HMASK_SW_INTERRUPTS  ((ui32) 0xffff0000)

#define DIG16_HMASK_INTERRUPTS_IN_USE \
                               (DIG16_HMASK_INT_FRAME    | \
                                DIG16_HMASK_INT_TRIGGER  | \
                                DIG16_HMASK_INT_FIFO     | \
                                DIG16_HMASK_INT_CAPTURE  | \
                                DIG16_HMASK_INT_LINE     | \
                                DIG16_HMASK_INT_1ST_DATA)


/* General Status Register
 * -----------------------
 * Note - empty and almost empty are both active low
 * Only bits 0x01 to 0x08 are returned by the function DIG16_get_FIFO_status.
 * This function also ORs the FIFO reset status into the higher bits.
 */
#define DIG16_BITS_FIFO_STATUS       ((ui8) 0x0F)
#define DIG16_STAT_FIFO_EMPTY        ((ui8) 0x01)
#define DIG16_STAT_FIFO_ALMOST_EMPTY ((ui8) 0x02)
#define DIG16_STAT_FIFO_ALMOST_FULL  ((ui8) 0x04)
#define DIG16_STAT_FIFO_FULL         ((ui8) 0x08)

#define DIG16_STAT_SER422_232        DIG16_STAT_SER422
#define DIG16_STAT_SER232            ((ui8) 0x00)
#define DIG16_STAT_SER422            ((ui8) 0x40)

#define DIG16_STAT_FRAME             ((ui8) 0x80)


/* Camera Outputs Register
 * -----------------------
 */
#define DIG16_BIT_IO_A              ((ui8) 0x01)
#define DIG16_BIT_IO_B              ((ui8) 0x02)
#define DIG16_BIT_IO_C              ((ui8) 0x04)
#define DIG16_BIT_IO_D              ((ui8) 0x08)
#define DIG16_BIT_OUT_A             ((ui8) 0x10)
#define DIG16_BIT_OUT_B             ((ui8) 0x20)
#define DIG16_BIT_OUT_C             ((ui8) 0x40)
#define DIG16_BIT_OUT_D             ((ui8) 0x80)


/* Camera Inputs Register
 * ----------------------
 */
#define DIG16_STAT_IO_A_IN          ((ui8) 0x01)
#define DIG16_STAT_IO_B_IN          ((ui8) 0x02)
#define DIG16_STAT_IO_C_IN          ((ui8) 0x04)
#define DIG16_STAT_IO_D_IN          ((ui8) 0x08)
#define DIG16_BITS_IO_STATUS        ((ui8) 0x0F)


/* Camera I/O Format Register
 * --------------------------
 */
#define DIG16_HMASK_IO_A_OUT_IN     DIG16_BIT_IO_A_OUT
#define DIG16_BIT_IO_A_IN           ((ui8) 0x00)
#define DIG16_BIT_IO_A_OUT          ((ui8) 0x01)

#define DIG16_HMASK_IO_B_OUT_IN     DIG16_BIT_IO_B_OUT
#define DIG16_BIT_IO_B_IN           ((ui8) 0x00)
#define DIG16_BIT_IO_B_OUT          ((ui8) 0x02)

#define DIG16_HMASK_IO_C_OUT_IN     DIG16_BIT_IO_C_OUT
#define DIG16_BIT_IO_C_IN           ((ui8) 0x00)
#define DIG16_BIT_IO_C_OUT          ((ui8) 0x04)

#define DIG16_HMASK_IO_D_OUT_IN     DIG16_BIT_IO_D_OUT
#define DIG16_BIT_IO_D_IN           ((ui8) 0x00)
#define DIG16_BIT_IO_D_OUT          ((ui8) 0x08)

#define DIG16_HMASK_PCLK_OUT_IN     DIG16_BIT_PCLK_OUT
#define DIG16_BIT_PCLK_IN           ((ui8) 0x00)
#define DIG16_BIT_PCLK_OUT          ((ui8) 0x10)

#define DIG16_HMASK_OUT_X_EN        DIG16_BIT_OUT_X_EN
#define DIG16_BIT_OUT_X_DIS         ((ui8) 0x00)
#define DIG16_BIT_OUT_X_EN          ((ui8) 0x20)

#define DIG16_HMASK_SER422_232      DIG16_BIT_SER422
#define DIG16_BIT_SER232            ((ui8) 0x00)
#define DIG16_BIT_SER422            ((ui8) 0x40)

#define DIG16_HMASK_SER232_POL      DIG16_BIT_SER232_ON_NEG
#define DIG16_BIT_SER232_ON_POS     ((ui8) 0x00)
#define DIG16_BIT_SER232_ON_NEG     ((ui8) 0x80)


/* Acquistion Trigger Control Register
 * -----------------------------------
 */
#define DIG16_HMASK_TRIG_IN_SELECT  DIG16_BIT_TRIG_IN_ENABLE
#define DIG16_BIT_TRIG_IN_DISABLE   ((ui8) 0x00)
#define DIG16_BIT_TRIG_IN_ENABLE    ((ui8) 0x01)

#define DIG16_HMASK_TRIG_IN_POS_NEG DIG16_BIT_TRIG_IN_POS
#define DIG16_BIT_TRIG_IN_NEG       ((ui8) 0x00)
#define DIG16_BIT_TRIG_IN_POS       ((ui8) 0x02)

#define DIG16_HMASK_TRIG_SOURCE     ((ui8) 0x1C)
#define DIG16_BITS_TRIG_IN_IO_A     ((ui8) 0x00)
#define DIG16_BITS_TRIG_IN_IO_B     ((ui8) 0x04)
#define DIG16_BITS_TRIG_IN_IO_C     ((ui8) 0x08)
#define DIG16_BITS_TRIG_IN_IO_D     ((ui8) 0x0C)
#define DIG16_BITS_TRIG_IN_TTL1     ((ui8) 0x10)
#define DIG16_BITS_TRIG_IN_TTL2     ((ui8) 0x14)
#define DIG16_BITS_TRIG_IN_FRAME    ((ui8) 0x18)
#define DIG16_BITS_TRIG_IN_LINE     ((ui8) 0x1C)

#define DIG16_HMASK_TRIG_LVL_EDGE   DIG16_BIT_TRIG_LEVEL
#define DIG16_BIT_TRIG_EDGE         ((ui8) 0x00)
#define DIG16_BIT_TRIG_LEVEL        ((ui8) 0x80)

#define DIG16_HMASK_TRIG_GATED      DIG16_BIT_TRIG_GATED
#define DIG16_BIT_TRIG_NON_GATED    ((ui8) 0x00)
#define DIG16_BIT_TRIG_GATED        ((ui8) 0x40)


/* Data Mask Register
 * ------------------
 */
#define DIG16_BITS_8BIT             ((ui8) 0x0F)
#define DIG16_BITS_10BIT            ((ui8) 0x0E)
#define DIG16_BITS_12BIT            ((ui8) 0x0C)
#define DIG16_BITS_14BIT            ((ui8) 0x08)
#define DIG16_BITS_16BIT            ((ui8) 0x00)



/* LUT Control Register
 * --------------------
 */

#define DIG16_HMASK_LUT_ADDR_RST    DIG16_BIT_LUT_ADDR_RST
#define DIG16_BIT_LUT_ADDR_COUNT    ((ui8) 0x00)
#define DIG16_BIT_LUT_ADDR_RST      ((ui8) 0x01)

#define DIG16_HMASK_LUT_PROG_USE    DIG16_BIT_LUT_PROG
#define DIG16_BIT_LUT_USE           ((ui8) 0x00)
#define DIG16_BIT_LUT_PROG          ((ui8) 0x02)

#define DIG16_HMASK_LUT_16_8        DIG16_BIT_LUT_16
#define DIG16_BIT_LUT_8             ((ui8) 0x00)
#define DIG16_BIT_LUT_16            ((ui8) 0x04)

#define DIG16_HMASK_LUT_PROG_WR     DIG16_BIT_LUT_PROG_WRITE
#define DIG16_BIT_LUT_PROG_READ     ((ui8) 0x00)
#define DIG16_BIT_LUT_PROG_WRITE    ((ui8) 0x08)

#define DIG16_HMASK_LUT_HI_LO_PROG  DIG16_BIT_LUT_HI_PROG
#define DIG16_BIT_LUT_LO_PROG       ((ui8) 0x00)
#define DIG16_BIT_LUT_HI_PROG       ((ui8) 0x10)

#define DIG16_HMASK_CLK_INT_EXT     DIG16_BIT_CLK_INT
#define DIG16_BIT_CLK_EXT           ((ui8) 0x00)
#define DIG16_BIT_CLK_INT           ((ui8) 0x20)



/* FIFO Serial Register
 * --------------------
 */
#define DIG16_FIFO_MODE_IDT_STD     ((ui8) 0x00)
#define DIG16_FIFO_MODE_FWFT        ((ui8) 0x01)



/* Clock Frequency Register
 * ------------------------
 */
#define DIG16_BITS_CLK_0M           ((ui8) 0x3F)   /* The best we can do */
#define DIG16_BITS_CLK_0M25         ((ui8) 0x3F)
#define DIG16_BITS_CLK_0M5          ((ui8) 0x2F)
#define DIG16_BITS_CLK_1M           ((ui8) 0x1F)
#define DIG16_BITS_CLK_2M           ((ui8) 0x30)
#define DIG16_BITS_CLK_2M5          ((ui8) 0x39)
#define DIG16_BITS_CLK_3M13         ((ui8) 0x3A)
#define DIG16_BITS_CLK_4M           ((ui8) 0x20)
#define DIG16_BITS_CLK_5M           ((ui8) 0x29)
#define DIG16_BITS_CLK_6M27         ((ui8) 0x2A)
#define DIG16_BITS_CLK_8M           ((ui8) 0x10)
#define DIG16_BITS_CLK_10M          ((ui8) 0x19)
#define DIG16_BITS_CLK_12M53        ((ui8) 0x1A)
#define DIG16_BITS_CLK_16M          ((ui8) 0x00)
#define DIG16_BITS_CLK_20M          ((ui8) 0x09)
#define DIG16_BITS_CLK_25M06        ((ui8) 0x0A)
#define DIG16_BITS_CLK_33M29        ((ui8) 0x0C)
#define DIG16_BITS_CLK_40M          ((ui8) 0x0B)
#define DIG16_BITS_CLK_50M11        ((ui8) 0x02)


/* Line Scan Control Register 1
 * ----------------------------
 */
#define DIG16_HMASK_LSTART_IN_POL   DIG16_BIT_LSTART_IN_POS
#define DIG16_BIT_LSTART_IN_NEG     ((ui8) 0x00)
#define DIG16_BIT_LSTART_IN_POS     ((ui8) 0x02)

#define DIG16_HMASK_LSTART_OUT_POL  DIG16_BIT_LSTART_OUT_POS
#define DIG16_BIT_LSTART_OUT_NEG    ((ui8) 0x00)
#define DIG16_BIT_LSTART_OUT_POS    ((ui8) 0x04)

#define DIG16_HMASK_PCLK_OUT_EN     DIG16_BIT_PCLK_OUT_EN
#define DIG16_BIT_PCLK_OUT_DIS      ((ui8) 0x00)
#define DIG16_BIT_PCLK_OUT_EN       ((ui8) 0x08)

#define DIG16_HMASK_LSTART_OUT_EN   DIG16_BIT_LSTART_OUT_EN
#define DIG16_BIT_LSTART_OUT_DIS    ((ui8) 0x00)
#define DIG16_BIT_LSTART_OUT_EN     ((ui8) 0x10)

#define DIG16_HMASK_LSTART_IN_SEL   DIG16_BIT_LSTART_IN_INT
#define DIG16_BIT_LSTART_IN_EXT     ((ui8) 0x00)
#define DIG16_BIT_LSTART_IN_INT     ((ui8) 0x20)

#define DIG16_HMASK_LINEACC_MODE    ((ui8) 0xC0)
#define DIG16_BITS_LINE_X1          ((ui8) 0x00)
#define DIG16_BITS_LINE_X2          ((ui8) 0x40)
#define DIG16_BITS_LINE_X4          ((ui8) 0x80)
#define DIG16_BITS_LINE_X8          ((ui8) 0xC0)


/* Line Scan Control Register 2
 * ----------------------------
 */
#define DIG16_HMASK_LTRIG_SELECT    DIG16_BIT_LTRIG_IN_EXT
#define DIG16_BIT_LTRIG_IN_TIMER    ((ui8) 0x00)
#define DIG16_BIT_LTRIG_IN_EXT      ((ui8) 0x01)

#define DIG16_HMASK_LTRIG_IN_POL    DIG16_BIT_LTRIG_IN_POS
#define DIG16_BIT_LTRIG_IN_NEG      ((ui8) 0x00)
#define DIG16_BIT_LTRIG_IN_POS      ((ui8) 0x02)


/* Datastream Control Register
 * ---------------------------
 */
#define DIG16_HMASK_DSTRM_LINE_ACQ_SEL      DIG16_BIT_DSTRM_LINE_ACQ_ENABLE
#define DIG16_BIT_DSTRM_LINE_ACQ_DISABLE    ((ui8) 0x00)
#define DIG16_BIT_DSTRM_LINE_ACQ_ENABLE     ((ui8) 0x01)

#define DIG16_HMASK_DSTRM_FRAME_ACQ_SEL     DIG16_BIT_DSTRM_FRAME_ACQ_ENABLE
#define DIG16_BIT_DSTRM_FRAME_ACQ_DISABLE   ((ui8) 0x00)
#define DIG16_BIT_DSTRM_FRAME_ACQ_ENABLE    ((ui8) 0x02)

#define DIG16_HMASK_DSTRM_LINE_START_SEL    DIG16_BIT_DSTRM_LINE_START_ENABLE
#define DIG16_BIT_DSTRM_LINE_START_DISABLE  ((ui8) 0x00)
#define DIG16_BIT_DSTRM_LINE_START_ENABLE   ((ui8) 0x04)

#define DIG16_HMASK_DSTRM_FRAME_START_SEL   DIG16_BIT_DSTRM_FRAME_START_ENABLE
#define DIG16_BIT_DSTRM_FRAME_START_DISABLE ((ui8) 0x00)
#define DIG16_BIT_DSTRM_FRAME_START_ENABLE  ((ui8) 0x08)


/* SOFTWARE PARAMETER FLAGS
 * ========================
 */

/*
DIG16_capture_to_image()
------------------------
*/
#define DIG16_START_AND_WAIT        ((Tparam) 0x00000001L)
#define DIG16_START_AND_RETURN      ((Tparam) 0x00000002L)
#define DIG16_CAPTURE_ABORT         ((Tparam) 0x00000004L)
#define DIG16_CAPTURE_END           ((Tparam) 0x00000008L)
#define DIG16_START_AND_RETURN_IMMEDIATE \
                                    ((Tparam) 0x00000010L)


/*
DIG16_configure_fpga()
----------------------
*/
#define DIG16_MODE_AREA_SCAN        ((ui16) 0x0001)
#define DIG16_MODE_LINE_SCAN        ((ui16) 0x0002)
#define DIG16_MODE_DATA_STREAM      ((ui16) 0x0003)


/* 
DIG16_get_capture_status()
--------------------------
*/
#define DIG16_CAPTURE_COMPLETE      ((ui16) 0x0001L)
#define DIG16_CAPTURE_IN_PROGRESS   ((ui16) 0x0002L)


/*
DIG16_get_data_width()
----------------------
*/
/* #define DIG16_CAMERA_DATA_WIDTH  defined under DIG16_set_camera_info() */
#define DIG16_IMAGE_DATA_WIDTH      ((Tparam) 0x00000002L)


/* 
DIG16_get_parameter()
---------------------
* See list under DIG16_set_parameter()
*/



/*
DIG16_get_ptr()
---------------
*/
#define DIG16_SNAPPER_STRUCT        ((Tparam) 0x00000001L)
#define DIG16_SNAPPER_FUNCTION      ((Tparam) 0x00000002L)



/*
DIG16_initialize()
------------------
*/
#define DIG16_AIA_MONO8             ((ui16) 0x0001L)  /* No longer used ? */
#define DIG16_AIA_MONO16            ((ui16) 0x0002L)  /* No Longer Used ? */
#define DIG16_KODAK_MPLUS14         ((ui16) 0x0003L)
#define DIG16_KODAK_MPLUS16         ((ui16) 0x0004L)
#define DIG16_KODAK_MPLUS42         ((ui16) 0x0005L)
#define DIG16_HAMAMATSU_C4742       ((ui16) 0x0006L)
#define DIG16_PULNIX_TM1000         ((ui16) 0x0007L)
#define DIG16_PULNIX_TM1001         ((ui16) 0x0008L)
#define DIG16_PULNIX_TM9700         ((ui16) 0x0009L)
#define DIG16_PULNIX_TM9701         ((ui16) 0x000AL)
/* Note that XILLIX_1400 used to use 0x0B */
#define DIG16_CUSTOM_CAMERA         ((ui16) 0x000CL)
#define DIG16_HWTEST_16             ((ui16) 0x000DL)
#define DIG16_KODAK_MPLUS14I        ((ui16) 0x000EL)
#define DIG16_CAPELLA1400           ((ui16) 0x000FL)  /* must not re-order numbers */
#define DIG16_XILLIX1400_PMI        ((ui16) 0x0010L)
#define DIG16_DVC_08                ((ui16) 0x0011L)
#define DIG16_DALSA_CAD1            ((ui16) 0x0012L)
#define DIG16_DVC_10                ((ui16) 0x0013L)
#define DIG16_PULNIX_TM1040         ((ui16) 0x0014L)


/* This section below requires custom FPGAs for AreaScan */
#define DIG16_INIT_SPECIAL_FPGA     ((ui16) 0x1000L)
#define DIG16_XILLIX1400_12BIT      ((ui16) 0x1001L)
#define DIG16_XILLIX1400_12BIT_BIN2 ((ui16) 0x1002L)
#define DIG16_XILLIX1400_10BIT      ((ui16) 0x1003L)
#define DIG16_XILLIX1400_10BIT_BIN2 ((ui16) 0x1004L)
/* Retained for compatability with the interim release */
#define DIG16_XILLIX_1400           DIG16_XILLIX1400_12BIT
#define DIG16_XILLIX_1400_BIN2      DIG16_XILLIX1400_12BIT_BIN2


#define DIG16_INIT_DATA_STREAM      ((ui16) 0x4000L)
#define DIG16_CUSTOM_DATA_STREAM    ((ui16) 0x4001L)
#define DIG16_AIA_DATA_STREAM       ((ui16) 0x4002L)

#define DIG16_INIT_LINE_SCAN        ((ui16) 0x8000L)
#define DIG16_CUSTOM_LINESCAN       ((ui16) 0x8001L)
#define DIG16_DALSA_CLCX_1CH        ((ui16) 0x8002L)
#define DIG16_DALSA_CLCX_2CH        ((ui16) 0x8004L)
#define DIG16_BASLER_L120_1CH       ((ui16) 0x8005L)
#define DIG16_BASLER_L120_2CH       ((ui16) 0x8006L)

#define DIG16_INIT_FULL_DATA_WIDTH  ((ui16) 0)


/*
DIG16_set_alignment()
---------------------
*/
#define DIG16_ALIGN_MSB             ((Tparam) 0x00000001L)
#define DIG16_ALIGN_LSB             ((Tparam) 0x00000002L)
#define DIG16_ALIGN_NONE            ((Tparam) 0x00000004L)

#define DIG16_ALIGN_INVALID_PARAM   ((Tparam) 0xFFFFFFF8L)



/*
DIG16_set_C4742_ctrl()
----------------------
*/
#define DIG16_C4742_MODE            ((Tparam) 0x00000100L)
#define DIG16_C4742_PULSE_EXT_IN    ((Tparam) 0x00000200L)

#define DIG16_C4742_SMASK_NUMERIC   ((Tparam) 0x000000FFL)
#define DIG16_C4742_INVALID_PARAM   ((Tparam) 0xFFFFFC00L)


/*
DIG16_set_callback()
--------------------
*/
/* Modes: */
#define DIG16_CALLBACK_INIT          ((Tparam) 0x00000001L)
#define DIG16_CALLBACK_SET           ((Tparam) 0x00000002L)
#define DIG16_CALLBACK_INVALID_PARAM ((Tparam) 0xFFFFFFFCL)


/*
DIG16_set_camera_info()
-----------------------
*/
#define DIG16_CAMERA_MSB            ((Tparam) 0x00000100L)
#define DIG16_CAMERA_NONINTERLACED  ((Tparam) 0x00000200L)
#define DIG16_CAMERA_INTERLACED     ((Tparam) 0x00000400L)
#define DIG16_CAMERA_DATA_WIDTH     ((Tparam) 0x00000800L)
#define DIG16_CAMERA_TWO_CHANNEL    ((Tparam) 0x00001000L)
#define DIG16_CAMERA_DATA_CLK       ((Tparam) 0x00002000L) /* alternative to continous clock */
#define DIG16_CAMERA_CONTINUOUS_CLK ((Tparam) 0x00004000L) /* alternative to data clock */

#define DIG16_CAMERA_SMASK_NUMERIC  ((Tparam) 0x000000FFL)

#define DIG16_CAMERA_INVALID_PARAM  ((Tparam) 0xFFFF8000L)


/*
DIG16_set_capture()
-------------------
*/
#define DIG16_CAPTURE_INIT          ((Tparam) 0x00000001L)

#define DIG16_SUB_MODE_MASK         ((Tparam) 0x0000003EL)
#define DIG16_SUB_X1                ((Tparam) 0x00000002L)
#define DIG16_SUB_X1_SINGLE_FIELD   ((Tparam) 0x00000004L)
#define DIG16_SUB_X2                ((Tparam) 0x00000008L)
#define DIG16_SUB_X4                ((Tparam) 0x00000010L)
#define DIG16_SUB_X8                ((Tparam) 0x00000020L)

#define DIG16_START_1ST_FIELD       ((Tparam) 0x00000040L)
#define DIG16_START_NEXT_FIELD      ((Tparam) 0x00000080L)
#define DIG16_START_2ND_FIELD       ((Tparam) 0x00000400L)

#define DIG16_TRIG_IN_DISABLE       ((Tparam) 0x00000100L)
#define DIG16_TRIG_IN_ENABLE        ((Tparam) 0x00000200L)

#define DIG16_SINGLE_CAPTURE_MODE   ((Tparam) 0x00000800L)
#define DIG16_SEQUENCE_MODE         ((Tparam) 0x00001000L)

#define DIG16_SET_CAP_INVALID_PARAM ((Tparam) 0xFFFFE000L)

#ifdef DIG16_COMPATIBILITY
/* Note: These #defines must match ones above */
#define DIG16_NEXT_FRAME            ((Tparam) 0x00000040L)  /* = START_1ST_FIELD  */
#define DIG16_NEXT_2_FIELDS         ((Tparam) 0x00000040L)  /* = START_1ST_FIELD  */
#define DIG16_NEXT_1ST_FIELD        ((Tparam) 0x00000080L)  /* = START_NEXT_FIELD */
#define DIG16_NEXT_FIELD            ((Tparam) 0x00000080L)  /* = START_NEXT_FIELD */
#endif


/*
DIG16_set_clk()
---------------
*/
#define DIG16_CLK_IN_POS            ((Tparam) 0x10000002L)
#define DIG16_CLK_IN_NEG            ((Tparam) 0x10000004L)
#define DIG16_CLK_OUT_POS           ((Tparam) 0x20000008L)
#define DIG16_CLK_OUT_NEG           ((Tparam) 0x20000010L)
#define DIG16_CLK_OUT_FREQUENCY     ((Tparam) 0x00000020L)
#define DIG16_CLK_IO_POS            ((Tparam) 0x40000040L)
#define DIG16_CLK_IO_NEG            ((Tparam) 0x40000080L)

#define DIG16_CLK_IN_MASK           ((Tparam) 0x10000000L)
#define DIG16_CLK_OUT_MASK          ((Tparam) 0x20000000L)
#define DIG16_CLK_IO_MASK           ((Tparam) 0x40000000L)

#define DIG16_CLK_INVALID_PARAM     ((Tparam) 0x8FFFFF01L)


/*
DIG16_set_comms()
-----------------
*/
#define DIG16_COMMS_INIT            ((Tparam) 0x00001000L)
#define DIG16_COMMS_SMASK_NUMERIC   ((Tparam) 0x00000FFFL)

#define DIG16_COMMS_RS232           ((Tparam) 0x00002000L)
#define DIG16_COMMS_RS422           ((Tparam) 0x00004000L)
#define DIG16_COMMS_DEFAULT         ((Tparam) 0x00010000L)
#define DIG16_COMMS_RS232_ON_NEG    ((Tparam) 0x00020000L)
/* Note: Also need baud, bits, stop bits, intercharacter spacing, parity ... */

#define DIG16_COMMS_INVALID_PARAM   ((Tparam) 0xFFFC8FFFL)
#define DIG16_COMMS_UNKNOWN         ((Tparam) 0x00008000L)  /* Used by DIG16_get_parameter */


/*
DIG16_set_ctrl_io()
-------------------
*/
#define DIG16_CTRL_INIT             ((Tparam) 0x00000001L)
#define DIG16_CTRL_IO_A_IN          ((Tparam) 0x00000002L)
#define DIG16_CTRL_IO_A_OUT_0       ((Tparam) 0x00000004L)
#define DIG16_CTRL_IO_A_OUT_1       ((Tparam) 0x00000008L)
#define DIG16_CTRL_IO_B_IN          ((Tparam) 0x00000010L)
#define DIG16_CTRL_IO_B_OUT_0       ((Tparam) 0x00000020L)
#define DIG16_CTRL_IO_B_OUT_1       ((Tparam) 0x00000040L)
#define DIG16_CTRL_IO_C_IN          ((Tparam) 0x00000080L)
#define DIG16_CTRL_IO_C_OUT_0       ((Tparam) 0x00000100L)
#define DIG16_CTRL_IO_C_OUT_1       ((Tparam) 0x00000200L)
#define DIG16_CTRL_IO_D_IN          ((Tparam) 0x00000400L)
#define DIG16_CTRL_IO_D_OUT_0       ((Tparam) 0x00000800L)
#define DIG16_CTRL_IO_D_OUT_1       ((Tparam) 0x00001000L)
#define DIG16_CTRL_OUT_A_0          ((Tparam) 0x00002000L)
#define DIG16_CTRL_OUT_A_1          ((Tparam) 0x00004000L)
#define DIG16_CTRL_OUT_A_EXP_POS    ((Tparam) 0x00008000L)
#define DIG16_CTRL_OUT_B_0          ((Tparam) 0x00010000L)
#define DIG16_CTRL_OUT_B_1          ((Tparam) 0x00020000L)
#define DIG16_CTRL_OUT_C_0          ((Tparam) 0x00040000L)
#define DIG16_CTRL_OUT_C_1          ((Tparam) 0x00080000L)
#define DIG16_CTRL_OUT_D_0          ((Tparam) 0x00100000L)
#define DIG16_CTRL_OUT_D_1          ((Tparam) 0x00200000L)
#define DIG16_CTRL_OUT_EN           ((Tparam) 0x00400000L)
#define DIG16_CTRL_OUT_DIS          ((Tparam) 0x00800000L)
#define DIG16_CTRL_OUT_A_EXP_NEG    ((Tparam) 0x01000000L)
#define DIG16_CTRL_OUT_A_EXP_MASK   ((Tparam) 0x02000000L)
#define DIG16_CTRL_OUT_A_EXP_UNMASK ((Tparam) 0x04000000L)

#define DIG16_CTRL_IO_A_MASK        ((Tparam) 0x0000000EL)
#define DIG16_CTRL_IO_B_MASK        ((Tparam) 0x00000070L)
#define DIG16_CTRL_IO_C_MASK        ((Tparam) 0x00000380L)
#define DIG16_CTRL_IO_D_MASK        ((Tparam) 0x00001C00L)
#define DIG16_CTRL_OUT_A_MASK       ((Tparam) 0x0700E000L)
#define DIG16_CTRL_INVALID_PARAM    ((Tparam) 0xF8000000L)


/*
DIG16_set_data_stream_ctrl()
---------------------------
*/
#define DIG16_DSTRM_FRAME_ACQ_ENABLE    ((Tparam) 0x00000002L)
#define DIG16_DSTRM_FRAME_ACQ_DISABLE   ((Tparam) 0x00000004L)
#define DIG16_DSTRM_LINE_ACQ_ENABLE     ((Tparam) 0x00000008L)
#define DIG16_DSTRM_LINE_ACQ_DISABLE    ((Tparam) 0x00000010L)
#define DIG16_DSTRM_FRAME_START_ENABLE  ((Tparam) 0x00000020L)
#define DIG16_DSTRM_FRAME_START_DISABLE ((Tparam) 0x00000040L)
#define DIG16_DSTRM_LINE_START_ENABLE   ((Tparam) 0x00000080L)
#define DIG16_DSTRM_LINE_START_DISABLE  ((Tparam) 0x00000100L)

#define DIG16_DSTRM_INVALID_PARAM       ((Tparam) 0xFFFFFE01L)


/*
DIG16_set_fifo()
----------------
*/
#define DIG16_FIFO_INIT             ((Tparam) 0x00000001L)
#define DIG16_FIFO_RESET            ((Tparam) 0x00000002L)
#define DIG16_FIFO_RUN              ((Tparam) 0x00000003L)


/*
DIG16_set_format()
------------------
*/
#define DIG16_FORMAT_Y8             ((Tparam) 0x00000001L)
#define DIG16_FORMAT_Y16            ((Tparam) 0x00000002L)


/*
DIG16_set_interrupts()
----------------------
*/
/* Types of interrupt that can get installed */
#define DIG16_INT_INIT               ((Tparam) 0x00000001L)
#define DIG16_INT_MAIN_TRANSFER_COMPLETE ((Tparam) 0x00000002L)
#define DIG16_INT_CAPTURE_COMPLETE   ((Tparam) 0x00000004L)
#define DIG16_INT_START_OF_FRAME     ((Tparam) 0x00000008L)
#define DIG16_INT_END_OF_LINE        ((Tparam) 0x00000010L)
#define DIG16_INT_ACQ_TRIGGER        ((Tparam) 0x00000020L)
#define DIG16_INT_FIFO_OVERFLOW      ((Tparam) 0x00000040L)
#define DIG16_INT_INITIAL_DATA       ((Tparam) 0x00000080L)
#define DIG16_INT_INVALID_PARAM      ((Tparam) 0xFFFFFF00L)


/*
DIG16_set_linescan_ctrl()
-------------------------
*/
#define DIG16_LSCAN_INIT             ((Tparam) 0x00000001L)
#define DIG16_LSCAN_LINES_X1         ((Tparam) 0x00000002L)
#define DIG16_LSCAN_LINES_X2         ((Tparam) 0x00000004L)
#define DIG16_LSCAN_LINES_X4         ((Tparam) 0x00000008L)
#define DIG16_LSCAN_LINES_X8         ((Tparam) 0x00000010L)
#define DIG16_LSCAN_LSTART_IN_POS    ((Tparam) 0x00000100L)
#define DIG16_LSCAN_LSTART_IN_NEG    ((Tparam) 0x00000200L)
#define DIG16_LSCAN_LSTART_IN_NONE   ((Tparam) 0x00000400L)
#define DIG16_LSCAN_LSTART_OUT_POS   ((Tparam) 0x00000800L)
#define DIG16_LSCAN_LSTART_OUT_NEG   ((Tparam) 0x00001000L)
#define DIG16_LSCAN_LSTART_OUT_NONE  ((Tparam) 0x00002000L)
#define DIG16_LSCAN_LSTART_OUT_WIDTH ((Tparam) 0x00004000L)
#define DIG16_LSCAN_LTRIG_IN_POS     ((Tparam) 0x00008000L)
#define DIG16_LSCAN_LTRIG_IN_NEG     ((Tparam) 0x00010000L)
#define DIG16_LSCAN_LTRIG_IN_TIMER   ((Tparam) 0x00020000L)

#define DIG16_LSCAN_LINES_MASK       ((Tparam) 0x0000001EL)
#define DIG16_LSCAN_LSTART_IN_MASK   ((Tparam) 0x00000700L)
#define DIG16_LSCAN_LSTART_OUT_MASK  ((Tparam) 0x00003800L)

#define DIG16_LSCAN_INVALID_PARAM    ((Tparam) 0xFFFC00E0L)


/*
DIG16_set_LUTs()
----------------
*/
#define DIG16_LUT_INIT              ((Tparam) 0x00000001L)
#define DIG16_LUT_SET               ((Tparam) 0x00000002L)


/*
DIG16_set_mplus_ctrl()
----------------------
*/
#define DIG16_MPLUS_CONTINUOUS      ((Tparam) 0x00000001L)
#define DIG16_MPLUS_TRIGGERED       ((Tparam) 0x00000002L)
#define DIG16_MPLUS_CONTROLLED      ((Tparam) 0x00000004L)
#define DIG16_MPLUS_SHUTTER_RUN     ((Tparam) 0x00000008L)
#define DIG16_MPLUS_SHUTTER_OPEN    ((Tparam) 0x00000010L)
#define DIG16_MPLUS_FRAME_RESET     ((Tparam) 0x00000020L)
#define DIG16_MPLUS_FRAME_EXPOSE    ((Tparam) 0x00000040L)

#define DIG16_MPLUS_MODE_MASK       ((Tparam) 0x00000007L)
#define DIG16_MPLUS_INVALID_PARAM   ((Tparam) 0xFFFFFF80L)


/*
DIG16_get_parameter()
DIG16_set_parameter()
---------------------
*/
#define DIG16_BASEBOARD_HANDLE       ((ui16) 0x0001)
#define DIG16_PIXEL_COUNT            ((ui16) 0x0002)
#define DIG16_COMMS_JUMPER           ((ui16) 0x0004)
#define DIG16_FIFO_WIDTH             ((ui16) 0x0008)
#define DIG16_FIFO_BYTES             ((ui16) 0x0010)
/* Timeout types: */
#define DIG16_TIMEOUT_BEFORE_CAPTURE ((ui16) 0x0020)
#define DIG16_TIMEOUT_DATA_TRANSFER  ((ui16) 0x0040)
#define DIG16_TWO_CHANNEL            ((ui16) 0x0080)
#define DIG16_TIMEOUT_TRIGGER        ((ui16) 0x0100)
#define DIG16_IS_CAMERA_INTERLACED   ((ui16) 0x0200)
#define DIG16_ID_VALUE               ((ui16) 0x0400)
#define DIG16_IDREV_VALUE            ((ui16) 0x0800)
#define DIG16_FAMILY_VALUE           ((ui16) 0x1000)
#define DIG16_FAMILY                 DIG16_FAMILY_VALUE /* For Compatibility */
#define DIG16_REV_VALUE              ((ui16) 0x2000)
#define DIG16_FPGA_STUB_NAME         ((ui16) 0x4000)


/*
DIG16_set_ROI()
---------------
*/
#define DIG16_ROI_SET               ((Tparam) 0x00000001L)
#define DIG16_ROI_CHECK             ((Tparam) 0x00000002L)
#define DIG16_ROI_IGNORE_MINIMUM    ((Tparam) 0x80000000L)



/*
DIG16_set_timer()
-----------------
*/
#define DIG16_TIMER_INIT            ((Tparam) 0x00000001L)

#define DIG16_TIMER_SET_EXPOSURE    ((Tparam) 0x00000002L)
#define DIG16_TIMER_SET_TIMER_FN    ((Tparam) 0x00000004L)

#define DIG16_TIMER_INVALID_PARAM   ((Tparam) 0xFFFFFFF8L)



/*
DIG16_set_trigger()
-------------------
*/
#define DIG16_TRIG_INIT             ((Tparam) 0x00000001L)
#define DIG16_TRIG_IN_POS           ((Tparam) 0x00000002L)
#define DIG16_TRIG_IN_NEG           ((Tparam) 0x00000004L)
#define DIG16_TRIG_IN_IO_A          ((Tparam) 0x00000008L)
#define DIG16_TRIG_IN_IO_B          ((Tparam) 0x00000010L)
#define DIG16_TRIG_IN_IO_C          ((Tparam) 0x00000020L)
#define DIG16_TRIG_IN_IO_D          ((Tparam) 0x00000040L)
#define DIG16_TRIG_IN_TTL1          ((Tparam) 0x00000080L)
#define DIG16_TRIG_IN_TTL2          ((Tparam) 0x00000100L)
#define DIG16_TRIG_IN_EDGE          ((Tparam) 0x00000200L)
#define DIG16_TRIG_IN_LEVEL         ((Tparam) 0x00000400L)
#define DIG16_TRIG_GATED            ((Tparam) 0x00000800L)
#define DIG16_TRIG_NON_GATED        ((Tparam) 0x00001000L)
#define DIG16_TRIG_IN_FRAME         ((Tparam) 0x00002000L)
#define DIG16_TRIG_IN_LINE          ((Tparam) 0x00004000L)

#define DIG16_TRIG_IN_MASK          ((Tparam) 0x000061F8L)
#define DIG16_TRIG_INVALID_PARAM    ((Tparam) 0xFFFF8000L)

/*
DIG16_set_X1400PMI_send_serial_config()
----------------------
*/
/* Used to define serial buffer size used to configure camera */
#define DIG16_X1400PMI_CAM_DATA_SIZE  14

/*
DIG16_set_X1400_ctrl()
----------------------
*/
#define DIG16_X1400_FREQ_500KHZ     ((Tparam) 0x00000001L)
#define DIG16_X1400_FREQ_1MHZ       ((Tparam) 0x00000002L)
#define DIG16_X1400_FREQ_4MHZ       ((Tparam) 0x00000004L)
#define DIG16_X1400_FREQ_8MHZ       ((Tparam) 0x00000008L)

#define DIG16_X1400_FREQ_MASK       ((Tparam) 0x0000000FL)
#define DIG16_X1400_INVALID_PARAM   ((Tparam) 0xFFFFFFF0L)


/*
DIG16_settings_load
----------------------
*/
#define DIG16_SETTINGS_FORMAT_FILE  ((Tparam) 0x00000001L)

#ifdef DIG16_COMPATIBILITY
/* Compatibility modes - must be same as TRIG_IN_POS & TRIG_IN_NEG */
#define DIG16_TRIG_IN_RISING        ((Tparam) 0x00000002L)
#define DIG16_TRIG_IN_FALLING       ((Tparam) 0x00000004L)
#endif


/* STRUCTURE DEFINITIONS
 * ---------------------
 *
struct_Tdig16(ForBrief)
 */
struct Tdig16
{
   Terr     (*Pfunc)(Thandle, ui8, ...);
   Thandle  Hbaseboard;
   ui8      ID;
   ui8      rev;
   ui16     mode;                           /* Area scan, line scan */
   ui16     highest_bit;                    /* Bit position of greater of msb & lsb */
   ui16     msb;                            /* Bit position of MSB from camera */
   ui16     lsb;                            /* Bit position of LSB from camera */
   ui16     camera_data_width;              /* Number of data bits from camera */
   ui16     image_data_width;               /* Number of valid data bits in Timage */
   ui16     camera_type;                    /* Model of camera connected */
   Tparam   camera_info;                    /* interlaced / data clock */
   Tparam   alignment;                      /* LSB / MSB / none */
   Tparam   subsample_mode;                 /* x1 / x2 / etc */
   ui32     pixel_count;                    /* Pixel count in datastream mode */
   ui32     fSequenceMode;                  /* = continuous - no flushing */
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _SOLARIS || defined _MACOSX || defined _LINUX
   Terr     (*timer_fn)(Thandle);           /* Function to generate exposure pulse(s) */
#else
   Terr     (EXPORT_FN *timer_fn)(Thandle); /* Function to generate exposure pulse(s) */
#endif
   Tboolean flush_read;                     /* TRUE if image so small that it is entirely read in flush mode */
   Tboolean A_EXP;                          /* TRUE if OUT_A being used as exposure pulse */
   Tboolean two_channel;                    /* TRUE if twin tap camera in use */
   i16      roi[ASL_SIZE_2D_ROI];           /* ROI setting */
   i16      active_area[ASL_SIZE_2D_ROI];   /* Max ROI setting allowed */
   ui16     x_round;                        /* Required 2^n rounding for x_length */
   ui16     y_round;                        /* Required 2^n rounding for y_length */
   ui16     module_format;                  /* e.g. xxYY, xxY1Y0 */
   ui16     TMG_format;                     /* e.g. TMG_RGB16 */
   ui8      LCA_reg[DIG16_NUM_LCA_REGS];
   char     snp_fpga_date[RBT_ASL_DATE_LENGTH];   /* Date read from Snapper FPGA */ 

   ui32     dwExposureTime;                 /* Microseconds */

   /* timeout in milliseconds - see dig16ini.c for default values */
   ui32     dwTimeoutBefore;                /* Before acquisition starts */
   ui32     dwTimeoutDuring;                /* For acquisition period    */
   ui32     dwTimeoutTrigger;               /* Before trigger pulse received */

#ifdef _OS_THREADS
   struct   Tthread_info thread_info;       /* for separate read thread */
#endif

   ui32 dwInterruptControl;                 /* ORable. Lower 16 bits = HW, upper 16 = SW */

   ui16 wLinesSoFar;                        /* for read_video_data */
   ui16 wLocalAction;                       /* for read_video_data */

   volatile ui32 fCaptureInProgress;        /* Boolean */
   volatile ui32 fMainReadInProgress;       /* Used as part of interrupt callback */
   ui32 dwTotalTransferBytes;               /* Total amout of data to read */
   ui32 dwMainReadBytes;                    /* Main DMA amount */
   ui32 dwFlushBytes;                       /* Flush amount    */
   ui32 *pdwData;                           /* Pointer to where acquired data will go */

   volatile Tboolean fReadAborted;          /* for signaling to polling loop in read_strip */
   char Prbt_file[MAX_FIELD_LENGTH];        /* FPGA file most recently downloaded */
};


/* MACRO DEFINITIONS not for error handling (in alphabetical order)
 * ----------------------------------------------------------------
 */
#define DIG16_get_offset(r)         (r & 0x07)
#define DIG16_get_page(r)           (r >> 3)

#if defined _lint
struct Tdig16 * EXPORT_FN DUMMY_get_Pdig16(char *, Thandle);
#define DIG16_get_Pdig16(s,Ps,Hs)   Ps = DUMMY_get_Pdig16(s,Hs);

#elif defined _MALLOC_DEBUG
#define DIG16_get_Pdig16(s,Ps,Hs) {if (((Hs) < 1) || ((Hs) >= DIG16_NUM_HANDLES))\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Handle out of range",s)\
                                   else if ((Ps = Pdig16_array[Hs]) == NULL)\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Null pointer",s)\
                                  }
#else
#define DIG16_get_Pdig16(s,Ps,Hs)   {if ((Ps = Pdig16_array[Hs]) == NULL) ASL_proe(ASLERR_BAD_HANDLE,"",s)}
#endif



/* MACRO DEFINITIONS for error handling (in alphabetical order)
 * ------------------------------------------------------------
 * Note: DIG16_err_ret cannot be used on the line after an `else' - {} are
 * needed even on single line if-else statements.
 */
#if defined _lint
extern Terr EXPORT_FN DIG16_Dummy_err_ret(Thandle, Terr, char *, Terr, char *);
#define DIG16_err_ret(Hs,e,d,v,s)   return(DIG16_Dummy_err_ret(Hs,e,d,v,s))
#define DIG16_make_ret(Hs,e,v)      (( (Pdig16_array[Hs]->ID | Pdig16_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))
#else
#define DIG16_err_ret(Hs,e,d,v,s)   {ASL_err_process(s,DIG16_make_ret(Hs,e,v),d);return(DIG16_make_ret(Hs,e,v));}
#define DIG16_make_ret(Hs,e,v)      (( (Pdig16_array[Hs]->ID | Pdig16_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))
#endif


#endif /* _DIG16_H_ */
