/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : dig16pro.h                                              *
 *  Function    : Header for DIG-16 - prototypes and externs              *
 *  Project     : Snappers - DC10                                         *
 *  Author      : Chris Beynon, Martin Bone, Active Silicon Ltd           *
 *  Systems     : ANSI C                                                  *
 *  Written     : February 1995                                           *
 *  Updated     : 26-Feb-96                                               *
 *  Version     : 3.11                                                    *
 *                                                                        *
 *  Copyright (c) 1995-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _DIG16PRO_H_
#define _DIG16PRO_H_


/* FUNCTION PROTOTYPES (in alphabetical order)
 * -------------------------------------------
 */
#ifdef __cplusplus
extern "C" {
#endif

Terr EXPORT_FN DIG16_capture_to_image(Thandle, Thandle, Tparam);
Terr EXPORT_FN DIG16_configure_fpga(Thandle, char*, ui16);
Terr EXPORT_FN DIG16_create_dig16(void);
Terr EXPORT_FN DIG16_destroy_dig16(Thandle);
Terr EXPORT_FN DIG16_generate_timer_pulse(Thandle);

Terr EXPORT_FN DIG16_get_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_get_camera_LSB(Thandle);
Terr EXPORT_FN DIG16_get_camera_MSB(Thandle);
Terr EXPORT_FN DIG16_get_camera_type(Thandle);
Terr EXPORT_FN DIG16_get_capture_status(Thandle, Tparam);
Terr EXPORT_FN DIG16_get_ctrl_io_status(Thandle);
Terr EXPORT_FN DIG16_get_data_width(Thandle, Tparam);
Terr EXPORT_FN DIG16_get_FIFO_status(Thandle);
Terr EXPORT_FN DIG16_get_ID(Thandle);
Terr EXPORT_FN DIG16_get_LUTs_HW(Thandle, IM_UI16 *, ui32);
ui32 EXPORT_FN DIG16_get_LUT_max_addr(Thandle Hdig16);
Terr EXPORT_FN DIG16_get_module_format(Thandle);
Terr EXPORT_FN DIG16_get_parameter(Thandle, ui16);
Terr EXPORT_FN DIG16_get_property(Thandle, char *, char *);
Terr EXPORT_FN DIG16_get_rev(Thandle);
Terr EXPORT_FN DIG16_get_ROI(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_get_ROI_max(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_get_snapper_mode(Thandle);
Terr EXPORT_FN DIG16_get_subsample(Thandle);
Terr EXPORT_FN DIG16_get_TMG_format(Thandle);

Terr EXPORT_FN DIG16_interrupt_process(Thandle hDig16, struct TCallbackEventInfo *psCallbackEventInfo);

Terr EXPORT_FN DIG16_initialize(Thandle, ui16, ui16);
Terr EXPORT_FN DIG16_initialize_LUTs(Thandle, ui16, ui16);

#ifdef _MAC_LVR  /* NOTE: Temporary fix for MacOS/Photometrics */
Tboolean EXPORT_FN DIG16_is_data_ready(Thandle);
#endif
Tboolean EXPORT_FN DIG16_is_field1_captured(Thandle);
Tboolean EXPORT_FN DIG16_is_in_trigger_sel(Thandle);
Tboolean EXPORT_FN DIG16_is_trigger_started(Thandle);

Terr EXPORT_FN DIG16_read_dig16_reg(Thandle, Tparam);

Terr EXPORT_FN DIG16_read_data_start(Thandle hDig16, Thandle hImage);
Terr EXPORT_FN DIG16_read_strip_ui32(Thandle, Thandle, Tboolean);
Terr EXPORT_FN DIG16_read_video_data(Thandle, Thandle, ui16);

Terr EXPORT_FN DIG16_restore_dig16(Thandle);

Terr EXPORT_FN DIG16_set_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_set_alignment(Thandle, Tparam);
#ifdef _QUICKC_
Terr DIG16_set_baseboard(Thandle, Thandle, Terr (*) (Thandle, ui8, ...));
#else
Terr DIG16_set_baseboard(Thandle, Thandle, Terr (*Pfunc) (Thandle, ui8, ...));
#endif
Terr EXPORT_FN DIG16_set_C4742_ctrl(Thandle, Tparam);
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _MACOSX
Terr EXPORT_FN DIG16_set_callback(Thandle, Tparam, void (*)(Thandle, ui32, void*), void* );
#else
Terr EXPORT_FN DIG16_set_callback(Thandle, Tparam, void (EXPORT_FN*)(Thandle, ui32, void*), void* );
#endif
Terr EXPORT_FN DIG16_set_capture(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_clk(Thandle, Tparam, ui16 *);
Terr EXPORT_FN DIG16_set_comms(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_ctrl_io(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_camera_info(Thandle, Tparam, ui8, ui8);
Terr EXPORT_FN DIG16_set_data_stream_ctrl(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_fifo(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_format(Thandle, Tparam, ui16);
Terr EXPORT_FN DIG16_set_ID(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_image(Thandle, Thandle);
Terr EXPORT_FN DIG16_set_image_data_width(Thandle, ui16);
Terr EXPORT_FN DIG16_set_interrupts(Thandle, Tparam, Tboolean);
Terr EXPORT_FN DIG16_set_linescan_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_set_linescan_ctrl(Thandle, Tparam, ui16);
Terr EXPORT_FN DIG16_set_linescan_ROI(Thandle, Tparam, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_set_LUTs(Thandle, Tparam, i16, IM_UI16 *);
Terr EXPORT_FN DIG16_set_LUTs_HW(Thandle, IM_UI16 *, ui32);
Terr EXPORT_FN DIG16_set_module_format(Thandle, ui16);
Terr EXPORT_FN DIG16_set_mplus_ctrl(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_parameter(Thandle, ui16, ui32);
Terr EXPORT_FN DIG16_set_rev(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_ROI(Thandle, Tparam, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN DIG16_set_ROI_rounding(Thandle, ui16, ui16);
Terr EXPORT_FN DIG16_set_TMG_format(Thandle, ui16);
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _MACOSX
Terr EXPORT_FN DIG16_set_timer(Thandle, Tparam, ui32, Terr (*)(Thandle) );
#else
Terr EXPORT_FN DIG16_set_timer(Thandle, Tparam, ui32, Terr (EXPORT_FN *)(Thandle) );
#endif
Terr EXPORT_FN DIG16_set_trigger(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_X1400_ctrl(Thandle, Tparam);
Terr EXPORT_FN DIG16_set_X1400PMI_send_serial_config(Thandle, ui8* Serial_config_string, ui8* line_status );

#ifdef _OS_THREADS /* LVR 04-Sep-96 */

#if defined _WIN31 && defined _OS_THREADS /* 7-Jul-96 */
void CALLBACK DIG16_thread_read_video_data(struct Tthread_info FAR *thread_info);
#elif defined _MAC_LVR || defined _MACINTOSH
pascal void* DIG16_thread_read_video_data( void* thread_info );
#else
#error DIG16_thread_read_video_data not defined
#endif

#endif /* def _OS_THREADS */

Terr EXPORT_FN DIG16_settings_load(Thandle, ui32, const void *);
Terr EXPORT_FN DIG16_settings_implement( Thandle, struct InitTag* );
Terr EXPORT_FN DIG16_write_dig16_page(Thandle, Tparam);
Terr EXPORT_FN DIG16_write_dig16_reg(Thandle, Tparam);

#ifdef __cplusplus
};
#endif



/* EXTERNS
 * -------
 */

/* Global Array of DIG16 Handles
 * -----------------------------
 */
extern struct Tdig16 *Pdig16_array[DIG16_NUM_HANDLES+1];

#endif /* _DIG16PRO_H_ */
