/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : dwin_drv.h                                              *
 *  Function    : Definitions for DOS and Windows 3.1 "driver"            *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : October 1993                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _dwin_drv_h
#define _dwin_drv_h

/*
 * Size of bursts for DOS/Windows driver
 * Note they are in long words (ui32).
 */
#if defined _DOS16
#define BURST_SIZE_PIO_DOS16 ((ui32) 4096)
#endif
#ifdef _DOS32
#define BURST_SIZE_PIO_DOS32  ((ui32) 32768)
#define BURST_SIZE_DMA_DOS32  ((ui32) 1024)  /* 4096 bytes = 1 page */
#endif
#ifdef _WIN31
#define BURST_SIZE_PIO_WIN31  ((ui32) 4096)
#define BURST_SIZE_DMA_WIN31  ((ui32) 1024)  /* 4096 bytes = 1 page */
#endif

/*
 * DDS structure - DMA Descriptor Structure - for PCI Bus DMA under Win 3.1
 */
struct TDDS {
   ui32 dwSize;          /* (00h) size of DMA region (bytes) */
   ui32 dwOffset;        /* (04h) 32 bit offset              */
   ui32 dwSegSel;        /* (08h) segment/selector           */
   ui32 dwPhysicalAddr;  /* (0Ch) number available           */ 
};

#ifdef _WIN31
/*
 * DMA Region structure - for PCI DMA under Windows 3.1
 */
struct TRegionListElement {
   IM_UI8 *pData;  /* pointer to DMA region - if 0, no region */
   ui32 PhysAddr;  /* actual physical address - if 0 unlocked/non-contiguous  */
                   /* Note also if Pbase->pData == NULL, region is not locked */
   ui32 NumBytes;  /* size of region in bytes */
};
#endif  /* _WIN31 */

/*
 * Serial Comms
 * ------------
 */
/* In a "real" operating system, the structures "TSerialInfo_Driver" and
 * "TSerialInfo_Common" would be defined here.  For DOS and Windows
 * they are conveniently put in the base structure.
 */

#endif /* _dwin_drv_h */



