/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : isa_base.h                                              *
 *  Function    : Definitions for Snapper ISA baseboard hardware          *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : October 1993                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _isa_base_h
#define _isa_base_h


#ifdef _NEVER_USE_TRACE  /* write_CL550_reg with trace output */
#define write_CL550_reg(Hbase, addr, data, offset) { \
   OS_page_write_ui16(Hbase, ISA_HOST_INDEX_REG, (ui16) (addr+(offset<<2)), ISA_HOST_DATA_PORT, data); \
   printf("Write  Reg: %04x  Data: %x (%d)\n",addr+(offset<<2),data,data); }
#else
#define write_CL550_reg(Hbase, addr, data, offset) { \
   OS_page_write_ui16(Hbase, ISA_HOST_INDEX_REG, (ui16) (addr+(offset<<2)), ISA_HOST_DATA_PORT, data); }
/*                           addr of page reg,   data for page,             reg addr in page mem, data */
#endif


/* ISA baseboard default base address */
/* ---------------------------------- */
#define ISA_DEFAULT_BASE_ADDRESS 0x300

/* ISA baseboard FPGA D/P pin data mask */
#define  ISA_DP_MASK 0x0001

/* module FPGA configuration data values */
#define  ISA_MODULE_DATA0_CL0  0x0002
#define  ISA_MODULE_DATA1_CL0  0x0003
#define  ISA_MODULE_DATA0_CL1  0x0000
#define  ISA_MODULE_DATA1_CL1  0x0001
#define  ISA_MODULE_RESET_0    0x0004
#define  ISA_MODULE_RESET_1    0x0000


/* module FPGA D/P pin data mask */
#define  ISA_MODULE_DP_MASK    0x0100


/* ISA Baseboard Register Addresses - Offsets only */
/* ----------------------------------------------- */
#define ISA_FPGA_RESET_REG       0x0
#define ISA_FPGA_DATA_REG        0x2
#define ISA_FPGA_STATUS_REG      0x4

#define ISA_HOST_DATA_PORT       0x6
#define ISA_HOST_INDEX_REG       0x8
#define ISA_PIXEL_PORT           0xA
#define ISA_CONTROL_REG1         0xC
#define ISA_CONTROL_REG2         0xE
#define ISA_STATUS_REG           0x10
#define ISA_IRQ_MASK_REG         0x12
#define ISA_MODULE_FPGA_CONFIG   0x14
#define ISA_VID_MODULE_PORT      0x16


/* ISA Status Register - for Compression Operations */
/* ------------------------------------------------ */
#define ISA_PIXEL_RDY           0x01
#define ISA_XFIFO_ALMOST_EMPTY  0x02
#define ISA_XFIFO_FULL          0x04
#define ISA_XFIFO_HALF_FULL     0x08
#define ISA_XFIFO_EMPTY         0x10


/* ISA Baseboard FPGA configuration data values */
/* -------------------------------------------- */
#define ISA_DATA0_CL0  0x0001
#define ISA_DATA1_CL0  0x0003
#define ISA_DATA0_CL1  0x0000
#define ISA_DATA1_CL1  0x0002
#define ISA_RESET_0    0x0001
#define ISA_RESET_1    0x0000

#endif /* _isa_base_h */
