/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : Lnxioctl.h                                             *
 *  Function    : Snapper Lynx I/O control codes.                         *
 *  Project     : Snapper                                                 *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C under LynxOS
 *  Version     :                                                         *
 *  Release Date:                                                         *
 *                                                                        *
 *  Copyright (c) 1993-97 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * I/O control codes for communication between the Snapper library and 
 * driver.
 *
 * File History
 * ------------
 */

#ifndef _Lynx_ioctl_h
#define _Lynx_ioctl_h

/*
 * IOCTL Codes
 * -----------
 * Note: They are in the order in which there were written and tested.
 * This ordering may be useful for programmers doing operating system
 * ports etc.
 */

#define CTL_CODE(a,b) ( ((a)<<16) + (b) )
#define FILE_DEVICE_SNAPPER  0x40
#define IOCTL_SNAPPER_TEST_DEVICE            CTL_CODE( FILE_DEVICE_SNAPPER, 0x900)
#define IOCTL_SNAPPER_OPEN_DEVICE            CTL_CODE( FILE_DEVICE_SNAPPER, 0x901)
#define IOCTL_SNAPPER_WRITE_UI8              CTL_CODE( FILE_DEVICE_SNAPPER, 0x902)
#define IOCTL_SNAPPER_READ_UI8               CTL_CODE( FILE_DEVICE_SNAPPER, 0x903)
#define IOCTL_SNAPPER_WRITE_UI16             CTL_CODE( FILE_DEVICE_SNAPPER, 0x904)
#define IOCTL_SNAPPER_READ_UI16              CTL_CODE( FILE_DEVICE_SNAPPER, 0x905)
#define IOCTL_SNAPPER_WRITE_UI32             CTL_CODE( FILE_DEVICE_SNAPPER, 0x906)
#define IOCTL_SNAPPER_READ_UI32              CTL_CODE( FILE_DEVICE_SNAPPER, 0x907)
#define IOCTL_SNAPPER_DYLAN_WRITE_UI32       CTL_CODE( FILE_DEVICE_SNAPPER, 0x908)
#define IOCTL_SNAPPER_DYLAN_READ_UI32        CTL_CODE( FILE_DEVICE_SNAPPER, 0x909)
#define IOCTL_SNAPPER_FPGA_SEND_INFO         CTL_CODE( FILE_DEVICE_SNAPPER, 0x90A)
#define IOCTL_SNAPPER_FPGA_SEND_DATA         CTL_CODE( FILE_DEVICE_SNAPPER, 0x90B)
#define IOCTL_SNAPPER_FPGA_RESET             CTL_CODE( FILE_DEVICE_SNAPPER, 0x90C)
#define IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS CTL_CODE( FILE_DEVICE_SNAPPER, 0x90D)
#define IOCTL_SNAPPER_BURST_IN_UI16          CTL_CODE( FILE_DEVICE_SNAPPER, 0x90E)
#define IOCTL_SNAPPER_BURST_IN_UI32          CTL_CODE( FILE_DEVICE_SNAPPER, 0x90F)
#define IOCTL_SNAPPER_BURST_IN_UI32_ASYNC    CTL_CODE( FILE_DEVICE_SNAPPER, 0x910)
#define IOCTL_SNAPPER_READ_COMPLETE          CTL_CODE( FILE_DEVICE_SNAPPER, 0x911)
#define IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL  CTL_CODE( FILE_DEVICE_SNAPPER, 0x912)
#define IOCTL_SNAPPER_BURST_OUT_UI16         CTL_CODE( FILE_DEVICE_SNAPPER, 0x913)
#define IOCTL_SNAPPER_TABLE_LOAD             CTL_CODE( FILE_DEVICE_SNAPPER, 0x914)
#define IOCTL_SNAPPER_PAGE_WRITE_UI16        CTL_CODE( FILE_DEVICE_SNAPPER, 0x915)
#define IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL CTL_CODE( FILE_DEVICE_SNAPPER, 0x916)
#define IOCTL_SNAPPER_GET_ERROR_STRING       CTL_CODE( FILE_DEVICE_SNAPPER, 0x917)
#define IOCTL_SNAPPER_ABORT_OPERATION        CTL_CODE( FILE_DEVICE_SNAPPER, 0x918)
#define IOCTL_SNAPPER_TIMEOUT_SET            CTL_CODE( FILE_DEVICE_SNAPPER, 0x919)
#define IOCTL_SNAPPER_SERIAL_TX              CTL_CODE( FILE_DEVICE_SNAPPER, 0x91A)
#define IOCTL_SNAPPER_SERIAL_RX              CTL_CODE( FILE_DEVICE_SNAPPER, 0x91B)
#define IOCTL_SNAPPER_SERIAL_SEND_INFO       CTL_CODE( FILE_DEVICE_SNAPPER, 0x91C)
#define IOCTL_SNAPPER_SERIAL_GET_INFO        CTL_CODE( FILE_DEVICE_SNAPPER, 0x91D)
#define IOCTL_SNAPPER_INTERRUPT_SET          CTL_CODE( FILE_DEVICE_SNAPPER, 0x91E)
#ifdef NEVERNEVER   /* This ioctl is not used */
#define IOCTL_SNAPPER_GETTIMESTAMP           CTL_CODE( FILE_DEVICE_SNAPPER, 0x91F)
#endif
#define IOCTL_SNAPPER_BURST_IN_UI32_PRE      CTL_CODE( FILE_DEVICE_SNAPPER, 0x920)
#define IOCTL_SNAPPER_INTERRUPT_STATUS       CTL_CODE( FILE_DEVICE_SNAPPER, 0x921)
#define IOCTL_SNAPPER_TABLE_LOAD_PRE         CTL_CODE( FILE_DEVICE_SNAPPER, 0x922)
#endif  /* _Lynx_ioctl_h */

