/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : module.h                                                *
 *  Function    : Generic Module Interface Header                         *
 *  Project     : Snapper                                                 *
 *  Author      : Martin Bone, Active Silicon Ltd                         *
 *  Systems     : ANSI                                                    *
 *  Written     : October 1993                                            *
 *  Updated     : 19-Jun-96                                               *
 *  Version     : 3.11                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _MODULE_H_
#define _MODULE_H_


/* Error Returns
 * -------------
 */
/* NOTE: Delete these two error defines before next release if no use found 
 * for them. June 1996.
 * #define MODULE_BAD_HANDLE    ((Terr) 0x30010000)
 * #define MODULE_UNDEFINED     ((Terr) 0x30030000)
 */

/* Module Control Flags
 * --------------------
 */
#define DEFAULT_FPGA                    (char*) ""  /* empty string */
#define MODULE_READ_UI8                 ((ui8) 0)
#define MODULE_WRITE_UI8                ((ui8) 1)
#define MODULE_CONFIG_FPGA              ((ui8) 2)
#define MODULE_CONFIG_FPGA_RBT          ((ui8) 3)
#define MODULE_READ_VPORT_UI16          ((ui8) 4)
#define MODULE_READ_VPORT_UI32          ((ui8) 5)  /* NOTE: If changed, see snapper.h */
#define MODULE_INIT_VPORT               ((ui8) 6)
#define MODULE_RESET_VPORT              ((ui8) 7)
#define MODULE_SET_DATAMAPPER           ((ui8) 8)
#define MODULE_GET_DATAMAPPER           ((ui8) 9)
#define MODULE_GET_BASEBOARD_DATAWIDTH  ((ui8) 10)
#define MODULE_IS_BASE_MAPPER_FITTED    ((ui8) 11)
#define MODULE_THREAD_START             ((ui8) 12)
#define MODULE_THREAD_EXIT              ((ui8) 13)
#define MODULE_THREAD_SYNC              ((ui8) 14)
#define MODULE_THREAD_CREATE            ((ui8) 15)
#define MODULE_THREAD_DESTROY           ((ui8) 16)
#define MODULE_THREAD_KILL              ((ui8) 17)
#define MODULE_IS_DMA_ENABLED           ((ui8) 18)
#define MODULE_DO_LARGE_DMA             ((ui8) 19)
#define MODULE_SET_TIMER                ((ui8) 20)
#define MODULE_TABLE_LOAD               ((ui8) 21)
#define MODULE_ABORT_OPERATION          ((ui8) 22)  /* Selected driver operation */
#define MODULE_TIMEOUT_SET              ((ui8) 23)
#define MODULE_INTERRUPT_SET            ((ui8) 24)  /* Also used as interrupt thread abort flag for consistency */


/* Module Format Flags
 * --------------------
 */
#define MODULE_UNDEFINED_FORMAT  ((ui16) 0)

/* First used by SNP-16 */
#define MODULE_FORMAT_XXCY       ((ui16) 1)
#define MODULE_FORMAT_XXY1Y0     ((ui16) 2)

/* First used by SNP-8 / SNP-24 */
#define MODULE_FORMAT_XXXY       ((ui16) 3)
#define MODULE_FORMAT_XXYX       ((ui16) 4)
#define MODULE_FORMAT_XYXX       ((ui16) 5)
#define MODULE_FORMAT_XBGR       ((ui16) 6)

/* First used by SNP-DIG16 */
#define MODULE_FORMAT_XXYY       ((ui16) 7)
#define MODULE_FORMAT_XXY0Y1     ((ui16) 8)  /* NOT TO BE USED ON NEW MODULES */

/* First used by PMC-DIG16 */
#define MODULE_FORMAT_YY1YY0     ((ui16) 9)
#define MODULE_FORMAT_Y3Y2Y1Y0   ((ui16) 10)

/* MAX_VALUE needed by data mapper functions. This is one greater than
 * the highest number used above because the last entry is for a test
 * mapper.
 * Note: see g_base.c - to update dm_fpga_array if this number changes
 */
#define MODULE_MAX_VALUE         ((ui16) 11)

/* Datamapper #defines: */

#define MODULE_TEST_MAPPER       MODULE_MAX_VALUE


/* Prototypes
 * ----------
 */
#ifdef __cplusplus
extern "C" {
#endif

#ifdef _QUICKC_
Terr MODULE_create_module(Thandle, ui8, Terr (*) (Thandle, ui8, ...));
#else
/* for Pascal calling convention - we can't have EXPORT_FN */
Terr MODULE_create_module(Thandle, ui8, Terr (*Pfunc) (Thandle, ui8, ...));
#endif

Terr MODULE_destroy_module(Thandle, ui8);

#ifdef __cplusplus
};
#endif

#endif /* _MODULE_H_ */
