/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : nt_ioctl.h                                              *
 *  Function    : Snapper NT I/O control codes.                           *
 *  Project     : Snapper                                                 *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10                                                    *
 *  Release Date: ??-Apr-97                                               *
 *                                                                        *
 *  Copyright (c) 1993-97 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * I/O control codes for communication between the Snapper library and 
 * driver.
 *
 * File History
 * ------------
 * 02-Apr-97, HCP, First release.
 *
 */

#ifndef _nt_ioctl_h
#define _nt_ioctl_h


/*
 * IOCTL Codes
 * -----------
 * Function codes above 32768 (0x8000) are not affected by winioctl.h
 * conventions.
 * Note: They are in the order in which there were written and tested.
 * This ordering may be useful for programmers doing operating system
 * ports etc.
 */

/* Device type - in the "User Defined" range (0x8000 - 0xFFFF).
 * Standard definitions in NTDDK.H.
 */
#define FILE_DEVICE_SNAPPER  0xACE1

/* Note that METHOD_OUT_DIRECT means that the driver has direct access to
 * Win32's DeviceIoControl's OutBuffer. The input buffer will be copied.
 * (This is for sending data to the driver and onto the hardware ...)
 * And METHOD_IN_DIRECT means that the driver has direct access to
 * Win32's DeviceIoControl's InBuffer. The output buffer will be copied.
 * NOTE: In practice its confusing and appears not to behave quite like this.
 * The direct buffer always seems to be the 5th parameter to DeviceIoControl 
 * irrespective of METHOD_IN/OUT. Anyway all the ioctls here work fine with 
 * no wasted time in extra buffering. A useful example to refer to is 
 * 16 bit burst read and write functions.
 * Microsoft PSS ID Q115758 states (as I have discovered) that the output 
 * buffer is always the one that is locked down and an MDL built etc!
 */

#define IOCTL_SNAPPER_OPEN_DEVICE \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x901, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI8 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x902, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_READ_UI8 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x903, METHOD_IN_DIRECT, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x904, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_READ_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x905, METHOD_IN_DIRECT, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x906, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_READ_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x907, METHOD_IN_DIRECT, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_DYLAN_WRITE_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x908, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_DYLAN_READ_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x909, METHOD_IN_DIRECT, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_FPGA_SEND_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90A, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_FPGA_SEND_DATA \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90B, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_FPGA_RESET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90C, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90D, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90E, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90F, METHOD_NEITHER, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI32_ASYNC \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x910, METHOD_NEITHER, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_READ_COMPLETE \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x911, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x912, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_OUT_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x913, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_TABLE_LOAD \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x914, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_PAGE_WRITE_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x915, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x916, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_GET_ERROR_STRING \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x917, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_ABORT_OPERATION \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x918, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_TIMEOUT_SET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x919, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_SERIAL_TX \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91A, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_SERIAL_RX \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91B, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_SERIAL_SEND_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91C, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_SERIAL_GET_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91D, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_INTERRUPT_SET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91E, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_INTERRUPT_STATUS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91F, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

/*
 * Note: These next three ioctls are (currently) NT specific and used for
 * DMAing direct to a physical device, Colin, 24-Dec-97.
 */
#define IOCTL_SNAPPER_PHYSMEM_LOCK \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x920, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_PHYSMEM_UNLOCK \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x921, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_SGTABLE_SEND_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x922, METHOD_OUT_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI8_ABS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x923, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#ifdef NEVERNEVER   /* This ioctl is not used */
#define IOCTL_SNAPPER_GETTIMESTAMP \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x920, METHOD_BUFFERED, FILE_ANY_ACCESS )
#endif

#endif  /* _nt_ioctl_h */

