/*  Snapper OS/2 Device Driver for Watcom C/C++ V10.0
 *
 * device.h     Header file for device.c
 *
 * :set ts=4
 *
 * This file contains all the device driver general constants as well 
 * as the exported data and function declarations for Device.C
 *
 * Copyright (C) MPC Data Limited.  All rights reserved.
 *
 * Modification History:
 *
 *      Dec 1994 Graham Anstey MPC Data Limited. Original implementation.
 */

#if !defined(_INCL_DEVICE)
#define _INCL_DEVICE

/* Ensure device-driver types and structures available
 */
#ifdef DRIVER
#if !defined(_INCL_DRVLIB)
#  include "drvlib.h"
#endif
#endif

void BREAKPOINT(void);
#pragma aux BREAKPOINT =   \
	"Int 3"


/* The driver 'category'. This allows us to ignore IOCTLs which are not
 * relevant to this driver.
 */
#define SNAPPER_CATEGORY 128

/* The set of IOCTL functions supported.
 */
typedef enum {
	IOCTL_SNAPPER_READ_PORT_UCHAR,
	IOCTL_SNAPPER_WRITE_PORT_UCHAR,
	IOCTL_SNAPPER_READ_PORT_USHORT,
	IOCTL_SNAPPER_WRITE_PORT_USHORT,
	IOCTL_SNAPPER_READ_PORT_ULONG,
	IOCTL_SNAPPER_WRITE_PORT_ULONG,
	IOCTL_SNAPPER_READ_PROCESSOR_ULONG,
	IOCTL_SNAPPER_WRITE_PROCESSOR_ULONG,
	IOCTL_SNAPPER_WRITE_RGBX32,
	IOCTL_SNAPPER_READ_RGBX32,
	IOCTL_SNAPPER_PAGE_WRITE,
	IOCTL_SNAPPER_SEND_FPGA_INFO,
	IOCTL_SNAPPER_SET_PORT_ADDRESS,
	IOCTL_SNAPPER_LOAD_TABLE,
	IOCTL_SNAPPER_GET_BASE_ADDRESS
};

/* The set of IOCTL return codes.
 */
typedef enum {
	SNAPPER_ERR_NONE=1,
	SNAPPER_ERR_NOT_SUPPORTED
};

#if defined DRIVER

int display(USHORT x, USHORT y, USHORT len, FPUCHAR s);

/* Device.C Exported Functions
 */
USHORT	DevOpen(void);
USHORT	DevClose(void);
USHORT  DevRead(FPRP_READWRITE rp);
USHORT  DevWrite(FPRP_READWRITE rp);
USHORT	DevIOCtl(FPRP_IOCTL2 IOCtl2);

void MoveBytes(char _far *Dest, const char _far *Src, USHORT nBytes);

void DevStrcpy(char _far *Dest, const char _far *Src);
USHORT DevStrlen(const char _far *Src);
FPUCHAR DevNumToAscii(USHORT Len, FARPOINTER Var);

#define MAX_CMD_LINE_LEN 128
extern UCHAR CommandLine[MAX_CMD_LINE_LEN];
extern FPUCHAR CmdLinePtr;

/* Default base port, and # of ports */
#define DEFAULT_BASE_ADDR	0x300
#define NUMBER_PORTS		0x20

#define RBT_PREAMBLE_LENGTH   	4   /* 4 bits preamble code        */
#define PCI_DYLAN_TIMEOUT	10000
#pragma pack (1)

typedef struct {
   USHORT fpga_type;
#			define BASE_FPGA      0
#			define MODULE_FPGA    1
#			define MAPPER_FPGA    2
	ULONG  fpga_data;		/* Not used */
	ULONG  fpga_bit_length;

   ULONG data_reg;
   ULONG data0_clock0;
   ULONG data0_clock1;
   ULONG data1_clock0;
   ULONG data1_clock1;

   ULONG reset_reg;
   ULONG reset_assert;
   ULONG reset_deassert;

   ULONG status_reg;
   ULONG dp_mask;
} OS2FPGA, _far *FPOS2FPGA;

enum eBoardType { IsaBib, IsaCrunch, PciBib };

// driver local data structure specific to each device object
typedef struct {
	UCHAR		Debug[5];
	USHORT		PortBase;		// base port address
	USHORT		PortCount;		// Number of IO ports following PortBase
	OS2FPGA		FpgaInfo;		// information required to program fpga
	ULONG		BitCount;		// Current bit index in fpga configuration
	BOOLEAN		Programming;	// Set to indicate that write operation in FPGA data
	USHORT		PixelPort;		// Port to use for R/W operations
	enum eBoardType	BoardType;
	ULONG		BoardRevLevel;
	USHORT		ReadRGBX32;
	USHORT		WriteRGBX32;
} LOCAL_DEVICE_INFO, *PLOCAL_DEVICE_INFO, _far *FPLOCAL_DEVICE_INFO;

/* Local Device Information.
 */
extern LOCAL_DEVICE_INFO LDI;

/* Macro to test string equality
 */
#define  STREQU(s1, s2)  (strcmp(s1, s2) == 0)

#endif

#endif
