/* os2_drv.h
 *
 * This header file contains definitions intended to go along with
 * DRVLIB.LIB, a C-callable subroutine library.
 */

#if !defined(_INCL_DRVLIB)
#define _INCL_DRVLIB


#if !defined(NULL)
#  define NULL ((void *)0)
#endif

#ifndef __OS2_H__

typedef unsigned char      UCHAR;
typedef unsigned short     USHORT;
typedef unsigned long      ULONG;
typedef UCHAR _near       *PUCHAR;
typedef USHORT _near      *PUSHORT;
typedef char _near        *PCHAR;
typedef short _near       *PSHORT;
typedef long _near        *PLONG; 
typedef ULONG _near       *PULONG;

#endif

typedef unsigned short     BOOLEAN;
typedef UCHAR _far        *FPUCHAR;
typedef USHORT _far       *FPUSHORT;
typedef ULONG _far        *FPULONG;
typedef void _near        *POINTER;
typedef POINTER _near     *PPOINTER;
typedef void _far         *FARPOINTER;
typedef FARPOINTER _near  *PFARPOINTER;
typedef FARPOINTER _far   *FPFARPOINTER;

#ifndef __OS2_H__

typedef USHORT             SEL;       /* 16-bit selector                      */
typedef SEL _near         *PSEL;      /* Pointer to a selector                */
typedef USHORT             PID;       /* Process ID                           */
typedef USHORT             TID;       /* Thread ID                            */
typedef USHORT             SHANDLE;   /* Short (16-bit) handle                */
typedef ULONG              LHANDLE;   /* Long  (32-bit) handle                */

#endif

typedef USHORT             ERRCODE;   /* Error code returned                  */
typedef ERRCODE _far      *FPERRCODE; /* Pointer to an error code             */
typedef UCHAR              FLAG;      /* 8-bit flag                           */
typedef FLAG _near        *PFLAG;     /* Pointer to 8-bit flag                */
typedef FLAG _far         *FPFLAG;    /* Pointer to 8-bit flag                */
typedef SEL _far          *FPSEL;     /* _far pointer to selector             */
typedef USHORT             SEG;       /* 16-bit segment                       */
typedef USHORT             OFF;       /* 16-bit offset                        */
typedef ULONG              LOFF;      /* 32-bit offset                        */
typedef ULONG              PHYSADDR;  /* 32-bit physical address              */
typedef ULONG              LINADDR;   /* 32-bit linear address                */
typedef LINADDR  _far     *FPLINADDR; /* Pointer to 32 bit linear address     */
typedef FPLINADDR _far    *FPFPLINADDR;/* Pointer to linear address pointer   */
typedef PHYSADDR _far     *FPPHYSADDR;/* Pointer to 32-bit physical address   */
typedef char _near        *PSTRING;   /* Pointer to character string          */
typedef char _far         *FPSTRING;  /* _far pointer to string               */
typedef SHANDLE _far      *PSHANDLE;  /* Pointer to a short handle            */
typedef LHANDLE _far      *FPLHANDLE; /* Pointer to a long handle             */

/* Pointers to functions
 */
typedef int (pascal _near         *PFUNCTION)();
typedef int (pascal _near * _near *PPFUNCTION)();
typedef int (pascal _far          *FPFUNCTION)();
typedef int (pascal _far  * _near *PFPFUNCTION)();

/* Macros
 */
#define FALSE   0
#define TRUE    1

/* Access modes for DevVerifyAccess().
 */
#define VERIFY_READONLY 0
#define VERIFY_READWRITE 1

#define NP _near pascal


/* Far pointer from selector-offset
 */
#define MAKEFP(sel, off)     ( (void _far *) MAKEULONG(off, sel) )

#ifndef __OS2_H__

/* Get selector or offset from _far pointer
 */
#define SELECTOROF(p)       ( ((USHORT _far *) &(p)) [1])
#define OFFSETOF(p)         ( ((USHORT _far *) &(p)) [0])

#endif

/* Combine l(ow) & h(igh) to form a 32 bit quantity.
 */
#define MAKEULONG(l, h)  ((ULONG)(((USHORT)(l)) | ((ULONG)((USHORT)(h))) << 16))
#define MAKELONG(l, h)   ((LONG)MAKEULONG(l, h))
#define MAKEBIGOFFSETOF(p) ((ULONG) (OFFSETOF (p)))

/* Combine l(ow) & h(igh) to form a 16 bit quantity.
 */
#define MAKEUSHORT(l, h) (((USHORT)(l)) | ((USHORT)(h)) << 8)
#define MAKESHORT(l, h)  ((SHORT)MAKEUSHORT(l, h))

#ifndef __OS2_H__

/* Get high and low order parts of a 16 and 32 bit quantity
 */
#define LOBYTE(w)       LOUCHAR(w)
#define HIBYTE(w)       HIUCHAR(w)
#define LOUCHAR(w)      ((UCHAR)(w))
#define HIUCHAR(w)      (((USHORT)(w) >> 8) & 0xff)
#define LOUSHORT(l)     ((USHORT)(l))
#define HIUSHORT(l)     ((USHORT)(((ULONG)(l) >> 16) & 0xffff))

#endif

/* Device Driver Attributes Word
 */
#define DAW_CHR    0x8000           /* 1=Char, 0=block                      */
#define DAW_IDC    0x4000           /* 1=IDC available in this DD           */
#define DAW_IBM    0x2000           /* 1=Non-IBM block format               */
#define DAW_SHR    0x1000           /* 1=Supports shared device access      */
#define DAW_OPN    0x0800           /* 1=Open/close, or removable media     */
#define DAW_LEVEL1 0x0080           /* Level 1                              */
#define DAW_LEVEL2 0x0100           /* Level 2 DosDevIOCtl2                 */
#define DAW_LEVEL3 0x0180           /* Level 3 bit strip                    */
#define DAW_GIO    0x0040           /* 1=Generic IOCtl supported            */
#define DAW_CLK    0x0008           /* 1=CLOCK device                       */
#define DAW_NUL    0x0004           /* 1=NUL device                         */
#define DAW_SCR    0x0002           /* 1=STDOUT (screen)                    */
#define DAW_KBD    0x0001           /* 1=STDIN  (keyboard)                  */

/* Capabilities bit strip
 */
#define CBS_SHD    0x0001           /* 1=Shutdown/DevIOCtl2                 */
#define CBS_HMEM   0x0002           /* Hign memory map for adapters         */
#define CBS_PP     0x0004           /* Supports parallel ports              */

/* SaveMessage structure
 */
typedef struct MessageTable {
   USHORT      Id;
   USHORT      FillInItem;
   FARPOINTER  Item1;
   FARPOINTER  Item2;
   FARPOINTER  ItemLast;
} MESSAGETABLE;


/* OS/2 circular character queues
 */
#define QUEUE_SIZE  512             /* Size of queues                       */
typedef struct CharQueue {
   USHORT   Qsize;                  /* Number of bytes in queue             */
   USHORT   Qchrout;                /* Index of next char to put out        */
   USHORT   Qcount;                 /* Number of charactes in queue         */
   UCHAR    Qbuf[QUEUE_SIZE]; 
} CHARQUEUE;

typedef CHARQUEUE _near *PCHARQUEUE;


/* AttachDD inter device driver communication data area
 */
typedef struct AttachArea {
   OFF RealOFF;                     /* Real-mode offset of idc entry point  */
   SEG RealCS;                      /* Real-mode CS of IDC entry point      */
   SEG RealDS;                      /* Real-mode DS of IDC DD               */
   OFF ProtOFF;                     /* Protect-mode offset of entry point   */
   SEL ProtCS;                      /* Protect-mode CS of entry point       */
   SEL ProtDS;                      /* Protect-mode DS of other DD          */
} ATTACHAREA;

typedef ATTACHAREA _near *PATTACHAREA;


/* DRIVER REQUEST PACKET AND IT'S SUB-STRUCTURES...
 *
 * Sub-Structures first...
 */

    typedef struct {                /* Init                                 */
      UCHAR      Units;             /* Number of units                      */
      FPFUNCTION DevHlp;            /* &DevHlp                              */
      char _far *Args;              /* &Args                                */
      UCHAR      Drive;             /* Drive #                              */
    } RP_INIT_IN;                        

    typedef RP_INIT_IN  _far *FPRP_INIT_IN;      


    typedef struct { 
      UCHAR      Units;             /* Same as input                        */
      OFF        FinalCS;           /* Final offset, 1st code segment       */
      OFF        FinalDS;           /* Final offset, 1st data segment       */
      FARPOINTER BPBarray;          /* &BPB                                 */
    } RP_INIT_OUT;

    typedef RP_INIT_OUT  _far *FPRP_INIT_OUT;      

    typedef struct {                /* Read, Write, Write W/Verify          */
      UCHAR      Media;             /* Media descriptor                     */
      PHYSADDR   Buffer;            /* Transfer address                     */
      USHORT     Count;             /* Bytes/sectors                        */
      ULONG      StartSector;       /* Starting sector number               */
      USHORT     reserved; 
    } RP_READWRITE;                  

    typedef RP_READWRITE _far *FPRP_READWRITE;


    typedef struct {                /* Cached Read, Write, Write W/Verify   */
      UCHAR      Media;             /* Media descriptor                     */
      PHYSADDR   Buffer;            /* Transfer address                     */
      USHORT     Count;             /* Bytes/sectors                        */
      ULONG      StartSector;       /* Starting sector#                     */
      USHORT     reserved; 
    } RP_CREADWRITE;

    typedef RP_CREADWRITE _far *FPRP_CREADWRITE;

      
    typedef struct {                /* System Shutdown                      */
      UCHAR      SubCode;           /* Sub request code                     */
      ULONG      reserved;
    } RP_SHUTDOWN;

    typedef RP_SHUTDOWN _far *FPRP_SHUTDOWN;


    typedef struct {                /* Open/Close                           */
      USHORT     SysFileNum;        /* System file number                   */
    } RP_OPENCLOSE;

    typedef RP_OPENCLOSE _far *FPRP_OPENCLOSE;


    typedef struct {                /* IOCtl2                               */
      UCHAR      Category;          /* Category code                        */
      UCHAR      Function;          /* Function code                        */
      FARPOINTER Parameters;        /* &Parameters                          */
      FARPOINTER Buffer;            /* &Buffer                              */
		USHORT     SysFileNum;			/* System file number                   */
		USHORT     ParmBufLen;			/* Length of parameter buffer           */
		USHORT     DataBufLen;			/* Length of data buffer                */
    } RP_IOCTL2; 

    typedef RP_IOCTL2 _far *FPRP_IOCTL2;


    typedef struct {                /* IOCtl                                */
      UCHAR      Category;          /* Category code                        */
      UCHAR      Function;          /* Function code                        */
      FARPOINTER Parameters;        /* &Parameters                          */
      FARPOINTER Buffer;            /* &Buffer                              */
    } RP_IOCTL;                      

    typedef RP_IOCTL _far *FPRP_IOCTL;


    typedef struct {                /* Read, No Wait                        */
      UCHAR      CharReturned;      /* Char to return                       */
    } RP_READNOWAIT;                 

    typedef RP_READNOWAIT _far *FPRP_READNOWAIT;


    typedef struct {                /* Media Check                          */
      UCHAR      Media;             /* Media descriptor                     */
      UCHAR      ReturnSode;        /* See #defines                         */
      FARPOINTER PrevVolume;        /* &Previous volume ID                  */
    } RP_MEDIACHECK;                 

    typedef RP_MEDIACHECK _far *FPRP_MEDIACHECK;


    typedef struct {                /* Build BPB                            */
      UCHAR      Media;             /* Media descriptor                     */
      FARPOINTER Buffer;            /* 1-Sector buffer FAT                  */
      FARPOINTER BPBarray;          /* &BPB array                           */
      UCHAR      Drive;             /* Drive #                              */
    } RP_BUILDBPB;                   

    typedef RP_BUILDBPB _far *FPRP_BUILDBPB;

                                                                             
    typedef struct {                /* Query Partitionalble Fixed Disks     */
      UCHAR      Count;             /* # disks                              */
      ULONG      reserved;
    } RP_QPARTITIONABLE;              

    typedef RP_QPARTITIONABLE _far *FPRP_QPARTITIONABLE;

     
    typedef struct {                /* Fixed Disk LU Map                    */
      ULONG      Units;             /* Units supported                      */
      ULONG      reserved;
    } RP_FIXEDMAP;  

    typedef RP_FIXEDMAP _far *FPRP_FIXEDMAP;


    typedef struct {                /* Get Driver Capabilities              */
      UCHAR      reserved[3];
      FARPOINTER CapStruct;         /* 16:16 pointer to DCS                 */
      FARPOINTER VolCharStruct;     /* 16:16 pointer to VCS                 */
    } RP_GETDRIVERCAPS;

    typedef RP_GETDRIVERCAPS _far *FPRP_GETDRIVERCAPS;


typedef struct ReqPacket {
    UCHAR   RPlength;                       /* Request packet length         */
    UCHAR   RPunit;                         /* Unit code for block DD only   */
    UCHAR   RPcommand;                      /* Command code                  */
    USHORT  RPstatus;                       /* Status word                   */
    UCHAR   RPreserved[4];                  /* Reserved bytes                */
    ULONG   RPqlink;                        /* Queue linkage                 */
    union {                                 /* Command-specific data         */
        UCHAR               avail[19];      /* Rest of packet                */         
        RP_INIT_IN          Init;           /* Init input params             */
        RP_INIT_OUT         InitExit;       /* Init exit params              */
        RP_READWRITE        ReadWrite;      /* Read & Write params           */
        RP_CREADWRITE       CReadWrite;     /* Cached Read & Write params    */
        RP_SHUTDOWN         Shutdown;       /* Shutdown params               */
        RP_OPENCLOSE        OpenClose;      /* Open & Close params           */
        RP_IOCTL            IOCtl;          /* Device I/O Control params     */
        RP_IOCTL2           IOCtl2;         /* Device I/O Control params     */
        RP_READNOWAIT       ReadNoWait;     /* ReadNoWait params             */
        RP_MEDIACHECK       MediaCheck;     /* Media Check Params            */
        RP_BUILDBPB         BuildBPB;       /* Build BPB params              */
        RP_QPARTITIONABLE   Partitionable;  /* Query Partitionable Fixed Disks*/
        RP_FIXEDMAP         GetFixedMap;    /* Get Fixed Disk Map            */
        RP_GETDRIVERCAPS    DriverCaps;     /* Get Driver Capabilities       */
    } u;                                    /* Command parameters union      */
} REQPACKET;

typedef REQPACKET  _far *PREQPACKET;
typedef PREQPACKET _far *PPREQPACKET;
typedef PREQPACKET       QHEAD;             /* Queue Head is &ReqPacket      */
typedef QHEAD _near     *PQHEAD;


/* Global Info Seg
 */
typedef struct {      
    ULONG   Time;
    ULONG   Msecs;
    UCHAR   Hour;
    UCHAR   Minutes;
    UCHAR   Seconds;
    UCHAR   Hundredths;
    USHORT  Timezone;
    USHORT  CusecTimerInterval;
    UCHAR   Day;
    UCHAR   Month;
    USHORT  Year;
    UCHAR   Weekday;
    UCHAR   uchMajorVersion;
    UCHAR   uchMinorVersion;
    UCHAR   chRevisionLetter;
    UCHAR   sgCurrent;
    UCHAR   sgMax;
    UCHAR   cHugeShift;
    UCHAR   fProtectModeOnly;
    USHORT  pidForeground;
    UCHAR   fDynamicSched;
    UCHAR   csecMaxWait;
    USHORT  cmsecMinSlice;
    USHORT  cmsecMaxSlice;
    USHORT  BootDrive;
    UCHAR   amecRAS[32];
    UCHAR   csgWindowableVioMax;
    UCHAR   csgPMMax;
} GINFOSEG;

typedef GINFOSEG _far *PGINFOSEG;

/* local info seg  */

typedef struct {      
    PID     pidCurrent;
    PID     pidParent;
    USHORT  prtyCurrent;
    TID     tidCurrent;
    USHORT  sgCurrent;
    UCHAR   rfProcStatus;
    UCHAR   dummy1;
    USHORT  fForeground;
    UCHAR   typeProcess;
    UCHAR   dummy2;
    SEL     selEnvironment;
    USHORT  offCmdLine;
    USHORT  cbDataSegment;
    USHORT  cbStack;
    USHORT  cbHeap;
    USHORT  hmod;
    SEL     selDS;
} LINFOSEG;

typedef LINFOSEG _far *PLINFOSEG;


/* Stack Usage Structure
 */
typedef struct _REGSTACK {          
    USHORT  usStruct;               /* Set to 14 before using               */
    USHORT  usFlags;                /* 0x01 means that the interrupt proc   */
                                    /* Enables interrupts. All others resvd */
    USHORT  usIRQ;                  /* IRQ of interrupt handler             */
    USHORT  usStackCLI;             /* # of stack bytes with interrupts off */
    USHORT  usStackSTI;             /* # of stack bytes with interrupts on  */
    USHORT  usStackEOI;             /* Number of bytes needed after EOI     */
    USHORT  usNest;                 /* Max number of nested levels          */
} REGSTACK;

typedef REGSTACK _near *PREGSTACK;


/* Page List Structure
 */
typedef struct _PAGELIST {
    ULONG pl_Physaddr;
    ULONG pl_cb;
} PAGELIST;

typedef PAGELIST _far *PPAGELIST;
/* Device Monitor flag values for Open/Close request packets
 */
#define MON_OPEN_STATUS   0x08      /* Open from DosMonOpen                 */
#define MON_CLOSE_STATUS  0x08      /* Close from DosMonClose               */


/* Media Descriptor Byte
 */
#define MDB_REMOVABLE     0x04      /*  1=Removable                         */
#define MDB_EIGHT_SECTORS 0x02      /*  1=8 Sectors per track               */
#define MDB_DOUBLE_SIDED  0x01      /*  1=Double-sided media                */


/* Return codes from MediaCheck
 */
#define MC_MEDIA_UNCHANGED 0x01
#define MC_MEDIA_CHANGED   0xFF
#define MC_MEDIA_UNSURE    0x00


/* Event numbers for SendEvent
 */
#define EVENT_SM_MOUSE   0x00       /* Session switch via mouse             */
#define EVENT_CTRLBRK    0x01       /* Control break                        */
#define EVENT_CTRLC      0x02       /* Control C                            */
#define EVENT_CTRLNUMLK  0x03       /* Control num lock                     */
#define EVENT_CTRLPRTSC  0x04       /* Control printscreen                  */
#define EVENT_SHFTPRTSC  0x05       /* Shift printscreen                    */
#define EVENT_SM_KBD     0x06       /* Session switch hot key               */


/* Defines for 1.x movedata function
 */
#define   MOVE_PHYSTOPHYS   0       /* Move bytes from phys to phys memory  */
#define   MOVE_PHYSTOVIRT   1       /* Move bytes from phys to virt memory  */
#define   MOVE_VIRTTOPHYS   2       /* Move bytes from virt to phys memory  */
#define   MOVE_VIRTTOVIRT   3       /* Move bytes from virt to virt memory  */

/* Defines for Lock function, LockType and WaitType lock type
 */
#define   LWT_BLOCK         0       /* Block until available                */
#define   LWT_NOWAIT        1       /* Return if not immediately available  */
#define   LT_SHORTTERM      0       /* Lock (for up to 2s)                  */ 
#define   LT_LONGTERM       1       /* Lock can last > 2s                   */
#define   LT_PERMHIMEM      3       /* Init time permament Himem lock       */
#define   LT_VERIFY         4       /* Verification lock                    */
/* DEVICE DRIVER HELPER FUNCTIONS
 */

/* Micro Channel specific
 */
BOOLEAN NP      ABIOSCall (USHORT, USHORT, FARPOINTER);
BOOLEAN NP      ABIOSComm (USHORT, FARPOINTER);
BOOLEAN NP      FreeLIDEntry (USHORT);
BOOLEAN NP      GetDeviceBlock(USHORT, FARPOINTER);
BOOLEAN NP      GetLIDEntry (USHORT, USHORT, USHORT, FPUSHORT);

BOOLEAN NP      MoveData (FARPOINTER, FARPOINTER, USHORT, USHORT);

/* System Services and Misc.
 */
BOOLEAN  NP     AttachDD(PSTRING, PATTACHAREA);
BOOLEAN  NP     GetDOSVar(UCHAR Id, FPFARPOINTER DosVar);
BOOLEAN  NP     InternalError(PSTRING,USHORT);
BOOLEAN  NP     ProtToReal(void);
BOOLEAN  NP     RealToProt(void);
BOOLEAN  NP     SaveMessage(FPSTRING);
BOOLEAN  NP     SchedClockAddr(PFARPOINTER);
BOOLEAN  NP     SendEvent(USHORT, USHORT);
BOOLEAN  NP     SetROMVector(USHORT,PFUNCTION,PFUNCTION,FARPOINTER);

/* Process Management
 */
BOOLEAN  NP     Block(ULONG, ULONG, USHORT, FARPOINTER);
void     NP     DevDone(PREQPACKET);
void     NP     Run(ULONG);
void     NP     TCYield(void);
BOOLEAN  NP     VideoPause(USHORT);
void     NP     Yield(void);

/* Memory Management
 */
typedef enum {ACCESS_RO, ACCESS_RW} ACCESS;

BOOLEAN  NP    AllocPhys(ULONG, USHORT, FPPHYSADDR);
BOOLEAN  NP    FreePhys(PHYSADDR);
BOOLEAN  NP    Lock(SEL Sel, USHORT LockType,USHORT WaitType, FPLHANDLE LHandle);
BOOLEAN  NP    UnLock(LHANDLE);
BOOLEAN  NP    VerifyAccess(FARPOINTER Ptr, USHORT Length, ACCESS Access);

/* Address Conversion
 */
BOOLEAN  NP     AllocGDTSelector(USHORT NumSelectors, FPUSHORT SelectorArray);
BOOLEAN  NP     PhysToGDTSelector(ULONG Physaddr,USHORT Len, USHORT Sel);
BOOLEAN  NP     PhysToUVirt(ULONG Physaddr, USHORT Len, UCHAR Type,
                            FPFARPOINTER Virt);
BOOLEAN  NP     PhysToVirt(PHYSADDR Addr, USHORT Len, USHORT Typ,
                           FPFARPOINTER Virt);
BOOLEAN  NP     UnPhysToVirt(void);
BOOLEAN  NP     VirtToPhys(FARPOINTER, FPPHYSADDR);

/* Request Packet Queues
 */
BOOLEAN  NP     AllocReqPacket(USHORT, PPREQPACKET);
void     NP     FreeReqPacket(PREQPACKET);
BOOLEAN  NP     PullParticular(PQHEAD, PREQPACKET);
BOOLEAN  NP     PullReqPacket(PQHEAD, PPREQPACKET);
void     NP     PushReqPacket(PQHEAD, PREQPACKET);
void     NP     SortReqPacket(PQHEAD, PREQPACKET);

/* Driver Semaphores
 */
void     NP     SemClear(LHANDLE);
BOOLEAN  NP     SemHandle(LHANDLE, FLAG, FPLHANDLE);
BOOLEAN  NP     SemRequest(FPLHANDLE LHandle, ULONG msWait, FPERRCODE Errcode);

/* Circular Character Queues
 * 
 * (Please note that there these routines are not very efficient, since the
 * fairly simple queueing operations are buried two or three function levels
 * deep.   If desired, use these routines early in development, then code
 * your own replacements).
 */
void     NP     QueueFlush(PCHARQUEUE);
void     NP     QueueInit(PCHARQUEUE);
BOOLEAN  NP     QueueRead(PCHARQUEUE, FPUCHAR);
BOOLEAN  NP     QueueWrite(PCHARQUEUE, UCHAR);

/* Interrupt handling
 */
void     NP     ClaimInterrupt(void);
BOOLEAN  NP     EOI(USHORT);
void     NP     RefuseInterrupt(void);
BOOLEAN  NP     RegisterStackUsage(PREGSTACK);
BOOLEAN  NP     SetIRQ(USHORT, PFUNCTION, USHORT);
BOOLEAN  NP     UnSetIRQ(USHORT);

/* System Timers
 */
BOOLEAN  NP     ResetTimer(PFUNCTION);
BOOLEAN  NP     SetTimer(PFUNCTION);
BOOLEAN  NP     TickCount(PFUNCTION, USHORT);


/* Device Monitor support
 */
BOOLEAN  NP     DeRegister(SHANDLE, PID, FPERRCODE);
BOOLEAN  NP     MonCreate(PSHANDLE, FARPOINTER, FARPOINTER, FPERRCODE);
BOOLEAN  NP     MonFlush(SHANDLE, FPERRCODE);
BOOLEAN  NP     MonWrite(SHANDLE, POINTER, USHORT, USHORT, ULONG, FPERRCODE);
BOOLEAN  NP     Register(SHANDLE, USHORT, PID, FARPOINTER, OFF, FPERRCODE);

/* 2.0  Specfic
 */
BOOLEAN  NP     AllocateCtxHook(OFF, ULONG, FPLHANDLE);   
BOOLEAN  NP     ArmCtxHook(ULONG, LHANDLE, ULONG);        
BOOLEAN  NP     Beep(USHORT, USHORT);              
BOOLEAN  NP     CloseEventSem(LHANDLE);     
BOOLEAN  NP     DynamicAPI(FARPOINTER, USHORT, USHORT, FPUSHORT);
BOOLEAN  NP     FreeCtxHook(LHANDLE);       
BOOLEAN  NP     FreeGDTSelector(USHORT Selector);   
BOOLEAN  NP     GetDescInfo(SEL, FPUSHORT, FPULONG, FPULONG);       
BOOLEAN  NP     LinToGDTSelector(SEL, LINADDR, ULONG);  
BOOLEAN  NP     LinToPageList(LINADDR, ULONG, LINADDR, FPULONG);     
BOOLEAN  NP     OpenEventSem(LHANDLE);      
BOOLEAN  NP     PageListToGDTSelector(SEL, ULONG, LINADDR, USHORT, FPUSHORT);
BOOLEAN  NP     PageListToLin(ULONG, LINADDR, FPLINADDR);
BOOLEAN  NP     PhysToGDTSel(PHYSADDR, ULONG, SEL, USHORT, FPUSHORT);      
BOOLEAN  NP     PostEventSem(LHANDLE);      
BOOLEAN  NP     RegisterBeep(FPFUNCTION);      
BOOLEAN  NP     RegisterPDD(FPUCHAR, FPFUNCTION);       
BOOLEAN  NP     RegisterTmrDD(FPFUNCTION, FPFARPOINTER, FPFARPOINTER);     
BOOLEAN  NP     ResetEventSem(LHANDLE, FPULONG);     
BOOLEAN  NP     VMAlloc(FPLINADDR, ULONG, ULONG, FPLINADDR);           
BOOLEAN  NP     VMFree(PHYSADDR);            
BOOLEAN  NP     VMGlobalToProcess(LINADDR, ULONG, ULONG, FPLINADDR); 
BOOLEAN  NP     VMLock(LINADDR, ULONG, LINADDR, LINADDR, ULONG, FPULONG);            
BOOLEAN  NP     VMProcessToGlobal(LINADDR, ULONG, ULONG, FPLINADDR); 
BOOLEAN  NP     VMSetMem(LINADDR, ULONG, ULONG);          
BOOLEAN  NP     VMUnlock(LHANDLE);          
BOOLEAN  NP     VirtToLin(FARPOINTER, FPLINADDR);         


/* These are the only DynLink API calls available to the driver at Init time
 */
#ifndef __OS2_H__
#define APIENTRY _far pascal

USHORT APIENTRY DosBeep(USHORT, USHORT);
USHORT APIENTRY DosCaseMap(USHORT, FARPOINTER, FARPOINTER);
USHORT APIENTRY DosChgFilePtr(SHANDLE, long, USHORT, FARPOINTER);
USHORT APIENTRY DosClose(SHANDLE);
USHORT APIENTRY DosDelete(FARPOINTER, ULONG);
USHORT APIENTRY DosDevConfig(FARPOINTER, USHORT, USHORT);
USHORT APIENTRY DosDevIOCtl(FARPOINTER, FARPOINTER, USHORT, USHORT, USHORT);
USHORT APIENTRY DosFindClose(SHANDLE);
USHORT APIENTRY DosFindFirst(FARPOINTER, FARPOINTER, USHORT, FARPOINTER, USHORT, FARPOINTER, ULONG);
USHORT APIENTRY DosFindNext(SHANDLE, FARPOINTER, USHORT, FARPOINTER);
USHORT APIENTRY DosGetEnv(FARPOINTER, FARPOINTER);
USHORT APIENTRY DosGetMessage(FARPOINTER, USHORT, FARPOINTER, USHORT, USHORT, FARPOINTER, FARPOINTER);
USHORT APIENTRY DosOpen(FARPOINTER, FARPOINTER, FARPOINTER, ULONG, USHORT, USHORT, USHORT, ULONG);
USHORT APIENTRY DosPutMessage(SHANDLE, USHORT, FARPOINTER);
USHORT APIENTRY DosQCurDir(USHORT, FARPOINTER, FARPOINTER);
USHORT APIENTRY DosQCurDisk(FARPOINTER, FARPOINTER);
USHORT APIENTRY DosQFileInfo(SHANDLE, USHORT, FARPOINTER, USHORT);
USHORT APIENTRY DosQFileMode(FARPOINTER, FARPOINTER, ULONG);
USHORT APIENTRY DosRead(SHANDLE, FARPOINTER, USHORT, FARPOINTER);
USHORT APIENTRY DosWrite(SHANDLE, FARPOINTER, USHORT, FARPOINTER);

#endif

/* DevHlp entry point.
 */
extern FPFUNCTION DevHlp;

extern USHORT selector;
int display(USHORT x, USHORT y, USHORT len, FPUCHAR s);

/* Device.C Exported Functions
 */
USHORT	DevOpen(void);
USHORT	DevClose(void);
USHORT  DevRead(FPRP_READWRITE rp);
USHORT  DevWrite(FPRP_READWRITE rp);
USHORT	DevIOCtl(FPRP_IOCTL2 IOCtl2);

void MoveBytes(char _far *Dest, const char _far *Src, USHORT nBytes);

void DevStrcpy(char _far *Dest, const char _far *Src);
USHORT DevStrlen(const char _far *Src);
FPUCHAR DevNumToAscii(USHORT Len, FARPOINTER Var);

#define MAX_CMD_LINE_LEN 128
extern UCHAR CommandLine[MAX_CMD_LINE_LEN];
extern FPUCHAR CmdLinePtr;

/* Default base port, and # of ports */
#define DEFAULT_BASE_ADDR	0x300
#define NUMBER_PORTS		0x1c

#define RBT_PREAMBLE_LENGTH   	4   /* 4 bits preamble code        */

#pragma pack (1)

typedef struct {
   USHORT fpga_type;
#			define BASE_FPGA      0
#			define MODULE_FPGA    1
#			define MAPPER_FPGA    2
	ULONG  fpga_data;		/* Not used */
	ULONG  fpga_bit_length;

   ULONG data_reg;
   ULONG data0_clock0;
   ULONG data0_clock1;
   ULONG data1_clock0;
   ULONG data1_clock1;

   ULONG reset_reg;
   ULONG reset_assert;
   ULONG reset_deassert;

   ULONG status_reg;
   ULONG dp_mask;
} OS2FPGA, _far *FPOS2FPGA;

// driver local data structure specific to each device object
typedef struct {
	UCHAR		Debug[5];
	USHORT		PortBase;		// base port address
	USHORT		PortCount;		// Number of IO ports following PortBase
	OS2FPGA		FpgaInfo;		// information required to program fpga
	ULONG		BitCount;		// Current bit index in fpga configuration
	BOOLEAN		Programming;	// Set to indicate that write operation in FPGA data
	USHORT		PixelPort;		// Port to use for R/W operations
} LOCAL_DEVICE_INFO, *PLOCAL_DEVICE_INFO, _far *FPLOCAL_DEVICE_INFO;

/* Local Device Information.
 */
extern LOCAL_DEVICE_INFO LDI;

/* Macro to test string equality
 */
#define  STREQU(s1, s2)  (strcmp(s1, s2) == 0)
#endif
