/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : os_sys.h                                                *
 *  Function    : Operating System dependent definitions                  *
 *  Project     : Snapper                                                 *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI                                                    *
 *  Written     : September 1994                                          *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _OS_SYS_H_
#define _OS_SYS_H_


/*
 * General Type Definitions
 * ------------------------
 */
#if defined _VXWORKS || defined _MACOSX || defined _LINUX
typedef signed char      i8;
typedef unsigned char   ui8;
typedef signed short    i16;
typedef unsigned short ui16;
typedef signed int      i32;
typedef unsigned int   ui32;
#elif (defined (_ILP32) || defined (_LP64)) && defined _SOLARIS2
/* Running under the Solaris 32/64 bit environment */
typedef int8_t          i8;
typedef uint8_t         ui8;
typedef int16_t         i16;
typedef uint16_t        ui16;
typedef int32_t         i32;
typedef uint32_t        ui32;
typedef int64_t         i64;
typedef uint64_t        ui64;
#elif defined _SOLARIS2
/* Solaris 32/64 user environment */
typedef signed char     i8;
typedef unsigned char   ui8;
typedef signed short    i16;
typedef unsigned short  ui16;
typedef signed int      i32;
typedef unsigned int    ui32;
#else
typedef char             i8;
typedef unsigned char   ui8;
typedef short           i16;
typedef unsigned short ui16;
typedef long            i32;
typedef unsigned long  ui32;
#endif


/*
 * Operating System Compiler Switch
 * options are : _WIN95 _WINNT, _WIN31, _DOS16, _DOS32, _SOLARIS2, _OS2, _MAC_LVR
 *		           _LYNXOS, _VXWORKS, CLR_MAC.
 *
 * For 16 bit DOS programs use _DOS16.
 * For 32 bit DOS programs use _DOS32 (Watcom and Symantec)
 * For Windows 3.1 applications use _WIN31 (and _WINDOWS - which is automatic).
 * For Windows 95 applications use _WIN95.
 * For Windows NT applications use _WINNT.
 * For OS2 applications compiled with the Watcom compiler, use _OS2.
 * For Macintosh system 7.5 and later on PowerPC's using Metrowerks CW9 and later, use _MACINTOSH.
 * For LynxOS 2.5 applications use _LYNXOS along with _X86 or _POWERPC or _CETIA_LYNXOS
 * For VxWorks use _VXWORKS - NOTE this is for PowerPC604 (64 bit long etc).
 *
 * Related Switches:     _ASL_ASM - use assembler for burst I/O on ISA & PCI.
 * (for DOS/Windows 3.1) _ASL_DMA - use DMA on the PCI baseboard.
 */

/* Microsoft vs Symantec inline assembler directive */
/* MS C expects _asm and Symantec expects asm.  We always use _asm */
#ifdef __SC__   /* for 32 bit DOSX and Symantec 16 bit */
#define _asm asm
#endif

/*
 * Windows / Non-Windows dependent stuff
 */
#ifdef _WINNT    /* Win32 (?) and Windows NT */
#ifdef _MALLOC_DEBUG
#define _WIN_DEBUG
#endif
#ifndef EXPORT_FN /* Allow user to redefine to prevent export from user DLL */
 #define EXPORT_FN     __declspec(dllexport)
#endif
#ifndef EXPORT_FN_PTR
 #define EXPORT_FN_PTR __declspec(dllexport)*
#endif
#define EXPORT_DATA
#define SPRINTF sprintf
#ifdef __cplusplus
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  ::MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  ::MessageBox(NULL,string,NULL,MB_OK); return(err); }
#else
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  MessageBox(NULL,string,NULL,MB_OK); return(err); }
#endif
#ifdef __cplusplus
#define WDPRINTF(fn_name,string,parameter) { \
  char message[128];char param_string[32]; \
  strcpy(message,fn_name);strcat(message," - ");\
  strcat(message,string);sprintf(param_string,"%08lX",(ui32)parameter);\
  strcat(message,param_string);::MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#define OS_ERR_PRINT(string) {::MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#else
#define WDPRINTF(fn_name,string,parameter) { \
  char message[128];char param_string[32]; \
  strcpy(message,fn_name);strcat(message," - "); \
  strcat(message,string);sprintf(param_string,"%08lX",(ui32)parameter); \
  strcat(message,param_string); \
  MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#define OS_ERR_PRINT(string) {MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#endif
#define DPRINTF(L)   if (FALSE)   /* always compiled out - replace with pop-up */
#define IM_UI8       ui8
#define IM_UI16      ui16
#define IM_UI32      ui32
#define IM_I32       i32
#define IM_I16       i16
#define CMAP_PTR     struct Tcmap
#define _CPU32       TRUE
//#define _OS_THREADS  TRUE   /* No longer defined */
#define _OS_INTERRUPTS  TRUE  /* Win NT */

#ifdef _MALLOC_DEBUG
/*
 * MACRO: Outputs a long word as part of a debug string.
 */
#define WIN32_OutputDebug(pszFnName, pszString, dwParameter) { \
  char _szWork[128]; char _szParamString[32]; strcpy(_szWork, pszFnName); \
  strcat(_szWork, " - "); strcat(_szWork, pszString); \
  wsprintf(_szParamString, " 0x%lx (%ld)\n", (ui32) dwParameter, (ui32) dwParameter); \
  strcat(_szWork, _szParamString); \
  OutputDebugString( _szWork ); }

/*
 * MACRO: Outputs a string as part of a debug string.
 */
#define WIN32_OutputDebugString(fn_name, string, parameter) { \
  char szWork[128]; char szParamString[32]; strcpy(szWork, fn_name); \
  strcat(szWork, " - "); strcat(szWork, string); \
  wsprintf(szParamString, " %s\r", (ui32) parameter); \
  strcat(szWork, szParamString); \
  OutputDebugString( szWork ); }

#else

#define WIN32_OutputDebug(fn_name, string, parameter)
#define WIN32_OutputDebugString(fn_name, string, parameter)

#endif  /* #else _MALLOC_DEBUG */

#elif defined _WIN95   /* NOTE: use _WIN32 for Windows 95 (and Windows NT) in future? */
#ifdef _MALLOC_DEBUG
#define _WIN_DEBUG
#endif
#ifndef EXPORT_FN /* Allow user to redefine to prevent export from user DLL */
 #define EXPORT_FN     __declspec(dllexport)
#endif
#ifndef EXPORT_FN_PTR
 #define EXPORT_FN_PTR __declspec(dllexport)*
#endif
#define EXPORT_DATA
#define SPRINTF sprintf
#ifdef __cplusplus
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  ::MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  ::MessageBox(NULL,string,NULL,MB_OK); return(err); }
#else
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  MessageBox(NULL,string,NULL,MB_OK); return(err); }
#endif
#ifdef __cplusplus
#define WDPRINTF(fn_name,string,parameter) { \
  char message[128];char param_string[32]; \
  strcpy(message,fn_name);strcat(message," - ");\
  strcat(message,string);sprintf(param_string,"%08lX",(ui32)parameter);\
  strcat(message,param_string);::MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#define OS_ERR_PRINT(string) {::MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#else
#define WDPRINTF(fn_name,string,parameter) { \
  char message[128];char param_string[32]; \
  strcpy(message,fn_name);strcat(message," - "); \
  strcat(message,string);sprintf(param_string,"%08lX",(ui32)parameter); \
  strcat(message,param_string); \
  MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#define OS_ERR_PRINT(string) {MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#endif
/* always compiled out - replace with pop-up */
#define DPRINTF(L)   /*lint -save -e505 -e774 */ if (FALSE) (void) /*lint -restore */
#define IM_UI8       ui8
#define IM_UI16      ui16
#define IM_UI32      ui32
#define IM_I32       i32
#define IM_I16       i16
#define CMAP_PTR     struct Tcmap
#define _CPU32       TRUE
//#define _OS_THREADS  TRUE
#define _OS_INTERRUPTS  TRUE   /* Win 95 */

#ifdef _MALLOC_DEBUG
/*
 * MACRO: Outputs a long word as part of a debug string.
 */
/*
 * MACRO: Outputs a long word as part of a debug string.
 */
#define WIN32_OutputDebug(pszFnName, pszString, dwParameter) { \
  char _szWork[128]; char _szParamString[32]; strcpy(_szWork, pszFnName); \
  strcat(_szWork, " - "); strcat(_szWork, pszString); \
  wsprintf(_szParamString, " 0x%lx (%ld)\n", (ui32) dwParameter, (ui32) dwParameter); \
  strcat(_szWork, _szParamString); \
  OutputDebugString( _szWork ); }

/*
 * MACRO: Outputs a string as part of a debug string.
 */
#define WIN32_OutputDebugString(fn_name, string, parameter) { \
  char szWork[128]; char szParamString[32]; strcpy(szWork, fn_name); \
  strcat(szWork, " - "); strcat(szWork, string); \
  wsprintf(szParamString, " %s\r", (ui32) parameter); \
  strcat(szWork, szParamString); \
  OutputDebugString( szWork ); }

#else

#define WIN32_OutputDebug(fn_name, string, parameter)
#define WIN32_OutputDebugString(fn_name, string, parameter)

#endif  /* #else _MALLOC_DEBUG */


#elif defined _WIN31      /* MS Windows 3.1 */
#ifdef _MALLOC_DEBUG
#define _WIN_DEBUG
#endif
/* Note: #define the Snappers and DMA/ASM options because the Win 3.1 compiler
 * has restrictions on the number of #defines that can be made via the project
 * options.
 */
#ifndef _FOR_WIN95    /* 16 bit under Windows 95 is for DIG16 only */
#ifndef _FOR_PM_W31   /* For Photometrics/Windows 3.1 */
#define _SNP16
#define _SNP24
#endif
#endif
#define _DIG16
#define _ASL_DMA
#define _ASL_ASM
/* __far is used so that users can write in the small memory model if
 * they wish too (no longer supported). __pascal is used so that 
 * Visual Basic can be used.
 */
#define EXPORT_FN      __export __far __pascal
#define EXPORT_FN_PTR *__export __far __pascal
#define EXPORT_DATA    __export
#define SPRINTF wsprintf
#ifdef __cplusplus
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  ::MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  ::MessageBox(NULL,string,NULL,MB_OK); return(err); }
#define OS_ERR_PRINT(string) {::MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#else
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  MessageBox(NULL,message,NULL,MB_OK); return(err); }
#define return_and_print(err, string) { \
  MessageBox(NULL,string,NULL,MB_OK); return(err); }
#define OS_ERR_PRINT(string) {MessageBox(NULL,string,"ERROR MESSAGE",MB_OK);}
#endif
#ifdef __cplusplus
#define WDPRINTF(fn_name,string,parameter) {\
  char message[128];char param_string[32]; \
  strcpy(message,fn_name);strcat(message," - "); \
  strcat(message,string);sprintf(param_string,"%08lX",(ui32)parameter); \
  strcat(message,param_string); \
  ::MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#else
#define WDPRINTF(fn_name,string,parameter) { \
  char message[128];char param_string[32];strcpy(message,fn_name); \
  strcat(message," - ");strcat(message,string); \
  sprintf(param_string,"%08lX",(ui32)parameter); \
  strcat(message,param_string);MessageBox(NULL,message,"DEBUG PRINT",MB_OK);}
#endif
#define DPRINTF(L)  if (FALSE)   /* always compiled out - replace with pop-up */
/* CJB allow level 4 compiler warnings  #define DPRINTF(L)  dummy12234 */
#define IM_UI8         ui8  __huge
#define IM_UI16        ui16 __huge
#define IM_UI32        ui32 __huge
#define IM_I32         i32 __huge
#define IM_I16         i16 __huge
#define CMAP_PTR       struct Tcmap __far
#define _CPU16         TRUE

#ifdef _FOR_PM_W31
#define _OS_THREADS    TRUE  /* define for Win3.1 / Photometrics release / Yield() */
#endif
#ifdef _FOR_WIN95            /* This is defined for Photometrics release/Win95 */
#define _OS_THREADS    TRUE  /* (the 16 bit library uses the VxD  */
#define _OS_INTERRUPTS TRUE  /* "_FOR_WIN95" - interrupts are supported */
#endif

#ifdef _MALLOC_DEBUG
/*
 * MACRO: Outputs a long word as part of a debug string.
 */
#define WIN31_OutputDebug(fn_name, string, parameter) { \
  char szWork[128]; char szParamString[32]; szWork[0] = '\n'; \
  szWork[1] = '\r';  strcpy( &(szWork[2]), fn_name); \
  strcat(szWork, " - "); strcat(szWork, string); \
  wsprintf(szParamString, " 0x%lx\n", (ui32) parameter); \
  strcat(szWork, szParamString); \
  OutputDebugString( szWork ); }
/*
 * MACRO: Outputs a string as part of a debug string.
 */
#define WIN31_OutputDebugString(fn_name, string, parameter) { \
  char szWork[128]; char szParamString[32]; szWork[0] = '\n'; \
  szWork[1] = '\r';  strcpy( &(szWork[2]), fn_name); \
  strcat(szWork, " - "); strcat(szWork, string); \
  wsprintf(szParamString, " '%s'\n", parameter); \
  strcat(szWork, szParamString); \
  OutputDebugString( szWork ); }
#else
#define WIN31_OutputDebug(fn_name, string, parameter)
#define WIN31_OutputDebugString(fn_name, string, parameter)
#endif  /* #else _MALLOC_DEBUG */

/* end _WIN31 */

#elif defined _DOS32  /* 32 bit protected mode DOS */

#ifndef _DOS
#define _DOS TRUE
#endif

#ifdef _FOR_WATCOM  /* so that pc_drv/pci_asm call be compiled using Symantec */
#define FP_SEG ASL_FP_SEG
#define strncmp fstrncmp  /* the actual Watcom function is _fstrncmp - Symantec */
                          /* will put an "_" on the front for us. */
int bprintf(char*, unsigned int, const char*, ...);  /* prototype ready for Watcom linking */
#endif

#ifdef __WATCOMC__  /* stack based calling convention only supported */
#define EXPORT_FN __cdecl
#define EXPORT_FN_PTR __cdecl *
#define EXPORT_DATA
#define Pbase_array _Pbase_array    /* needed to link with pc_drv.obj */
#else
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#endif  /* not Watcom */

#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { printf("%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE
/* no _OS_THREADS */
/* no _OS_INTERRUPTS */   /* DOS32 */


#elif defined _DOS16     /* 16 bit real mode DOS */
#ifndef _DOS
#define _DOS TRUE
#endif
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { printf("%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU16         TRUE
/* no _OS_THREADS */
/* no _OS_INTERRUPTS */  /* DOS16 */

#elif defined _SOLARIS2     /* Solaris 2.3 and above */
#ifndef _SOLARIS
#define _SOLARIS TRUE
#endif
#define _ASL_UNIX
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { fprintf(stderr, "%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE  /* Solaris 2 */

/* no _OS_THREADS */  /* obsolete anyway Solaris 2 */
#define _OS_INTERRUPTS TRUE
#ifndef INVALID_THREAD_ID
#define INVALID_THREAD_ID (-1) 
#endif


#elif defined _LYNXOS     /* LYNXOS 2.3.0 and above */
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { fprintf(stderr, "%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE
/*#define _OS_THREADS  TRUE    Not used anymore */
#define _OS_INTERRUPTS TRUE
#ifndef INVALID_HANDLE_VALUE
#define INVALID_HANDLE_VALUE (-1)
#endif
#ifndef INVALID_THREAD_ID
#define INVALID_THREAD_ID (-1) 
#endif

#elif defined _OS2      /* OS/2 */

#ifdef __IBMC__
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#else   /* Watcom 10.0 - for DLL build */
#define EXPORT_FN      __export
#define EXPORT_FN_PTR *__export
#define EXPORT_DATA
#endif
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { DPRINTF(10)("%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE   /* OS/2 */
/* #define _OS_THREADS    TRUE   NOTE: Not supported */
/* no _OS_INTERRUPTS */  /* OS/2 */


#elif defined _MACINTOSH  /* CLR MacOS8 port */
#define EXPORT_FN     __declspec(dllexport)
/* CLR - 20-Oct-98 removed export of Mac data */
#define EXPORT_DATA __declspec(dllexport)
/* The followind definition appears to cause problems when used - 
 * Fixed in code using extra conditional compilation.
 */
#define EXPORT_FN_PTR __declspec(dllexport) *

#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) do { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); } while (0)
#define return_and_print(err, string) do { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); } while(0)
#if 1
#define OS_ERR_PRINT(string) { DPRINTF(10)("%s", string); }
#else
#define OS_ERR_PRINT(string) { printf("%s", string); }
#endif
/* ASL_err_display output routine */
#define WDPRINTF(fn_name,string,parameter) {}
#if defined _DEBUG_LVL
#define DPRINTF(L)     DEBUG_PRINTF(L)
#else
#define DPRINTF(L)
#endif
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE
#define _OS_INTERRUPTS
#define _OS_THREADS
/* end _MACINTOSH */

#elif defined _MAC_LVR  /* MacOS 7.5 & Metrowerks CW9 */

#ifndef EXPORT_FN /* Allow user to redefine to prevent export from user DLL */
 #ifndef _MAC  /* _MAC signifies MS Mac cross-compiler - compiles under Wintel */
  #define EXPORT_FN     __declspec(export)
  #define EXPORT_FN_PTR __declspec(export) *
  #define EXPORT_DATA   __declspec(export)
 #else
  #define EXPORT_FN     __declspec(dllexport)
  #define EXPORT_FN_PTR __declspec(dllexport) *
  #define EXPORT_DATA   __declspec(dllexport)
 #endif
#endif

#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) do { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); } while (0)
#define return_and_print(err, string) do { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); } while(0)
/* ASL_err_display output routine */
static inline int _OS_DebuggerPresent( void)
  { return 0 != (0x20 & (*(unsigned char*)0xbff)); }
#define OS_ERR_PRINT(string) do \
  { if ( _OS_DebuggerPresent()) debugstr( string); else fprintf( stderr, string); } while(0)
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE
#define _OS_THREADS    TRUE
/* no _OS_INTERRUPTS in V2.03 driver*/  /* MacOS */

#elif defined _VXWORKS       /* VXWORKS 1.0.1 and above */
#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { fprintf(stderr, "%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE  /* VXWORKS */
#define _OS_INTERRUPTS       /* VXWORKS */
/* #define _OS_THREADS  No longer relevant */

#elif defined _MACOSX       /* _MACOSX 10.0.3 and above */
#define EXPORT_FN 
#define EXPORT_FN_PTR *
#define EXPORT_DATA 
#define SPRINTF sprintf
#define ASL_print_and_ret(err, string, fn_name) { \
  char message[128]; strcpy(message, string); \
  strcat(message, " in - "); strcat(message, fn_name); \
  DPRINTF(10)("\nReturning - '%s'", message); return(err); }
#define return_and_print(err, string) { \
  DPRINTF(10)("\nReturning - '%s'", string); return(err); }
#define OS_ERR_PRINT(string) { fprintf(stderr, "%s", string); }
#define WDPRINTF(fn_name,string,parameter) {}
#define DPRINTF(L)     DEBUG_PRINTF(L)
#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap
#define _CPU32         TRUE  /* _MACOSX */
#define _OS_INTERRUPTS       /* _MACOSX */
/* #define _OS_THREADS  No longer relevant */

#elif defined _LINUX /*=================================================*/

/*
_LINUX(ForBrief)
--------------------
*/
#define _CPU32            /* _TMG_CPU16 may be useful, so we'll keep this */


#define _TMG_Malloc(x) malloc(x)
#define _TMG_Free(x)   free(x)


#define IM_UI8         ui8
#define IM_UI16        ui16
#define IM_UI32        ui32
#define IM_I32         i32
#define IM_I16         i16
#define CMAP_PTR       struct Tcmap

#ifndef TRUE
#define TRUE     (1==1)
#define FALSE    (!TRUE)
#endif


#define EXPORT_FN
#define EXPORT_FN_PTR *
#define EXPORT_DATA

#define _TMG_ErrPrint(szMessage, szTitle) {printf("\nERROR - %s: %s\n", szTitle, szMessage);}


#define _TMG_Assert assert

#define DPRINTF(L)     DEBUG_PRINTF(L)
#define OS_ERR_PRINT(string) { printf("%s", string); }

#ifdef _TMG_DEBUG

/* _TMG_Debug: Outputs a long word as part of a debug string.
   ---------- */
#define _TMG_Debug(pszMessage, dwParameter1, dwParameter2) \
  { printf("TMG_Debug (%s) - %s: 0x%02lx (%02ld) : 0x%02lx (%02ld)\n", szFnName, pszMessage, (ui32) dwParameter1, (ui32) dwParameter1, (ui32) dwParameter2, (ui32) dwParameter2); }

/* _TMG_DebugString: Outputs a string as part of a debug string.
   --------------- */
#define _TMG_DebugString(pszMessage, pszParameter) { \
  char _szWork[TMG_DEBUG_STRING_LEN]; char _szParamString[TMG_DEBUG_STRING_LEN]; strcpy(_szWork, szFnName); \
  strcat(_szWork, " - "); strcat(_szWork, pszMessage); \
  sprintf(_szParamString, " %s\r", (ui32) pszParameter); \
  strcat(_szWork, _szParamString); \
  printf( _szWork ); }

/* _TMG_DebugPopup: Outputs a debug string in a popup.
   -------------- */
#ifdef __cplusplus
#define _TMG_DebugPopup(pszMessage, pszTitle)
#else
#define _TMG_DebugPopup(pszMessage, pszTitle)
#endif

#define _TMG_DPRINTF(L)  if (L>10) printf

#else   /* no debugging - _TMG_DEBUG not defined */

#define _TMG_Debug(pszMessage, dwParameter1, dwParameter2)
#define _TMG_DebugString(pszMessage, pszParameter)
#define _TMG_DebugPopup(pszMessage, pszTitle)

#define _TMG_DPRINTF(L)   if (FALSE) printf

#endif  /* #else part of "#ifdef _TMG_DEBUG" */

/* end of _LINUX */
#else
#error You have a missing compiler directive (e.g. _WINNT) !!!!!
#endif

/*
 * Processor word/integer size dependent stuff
 */
#ifdef  _CPU16         /* 16 bit x86 processor */
#ifndef FAR 
#define FAR __far
#define _near __near
#define far __far

#ifndef __BORLANDC__
#define FP_SEG _FP_SEG
#define FP_OFF _FP_OFF
#endif

#endif  /* _CPU16 */

#elif defined _CPU32   /* 32 bit x86/SPARC/Pentium processor etc */
#ifdef  FAR
#undef  FAR
#endif
#define FAR

#else
#error  Neither _CPU16 or _CPU32 are defined.
#endif

#endif /* _OS_SYS_H_ */
