/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        * 
 *  File name   : pci_base.h                                              * 
 *  Function    : Definitions for PCI Bus Interface Board hardware        * 
 *  Project     : Snapper                                                 * 
 *  Author      : Warren Frost, Active Silicon Ltd                        * 
 *  Systems     : ANSI C                                                  * 
 *  Written     : August 1994                                             * 
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        * 
 *  Copyright (c) 1994-96 Active Silicon, Active Imaging Ltd.              *
 **************************************************************************
 *  Comments:
 */

#ifndef _pci_base_h
#define _pci_base_h

/*
 * Maximum number of PCI boards.
 */
#define MAX_NUM_PCI_BOARDS 3

/*
 * PCI Vendor IDs
 */
#define PCI_AMCC_VENDOR_ID  ((ui16) 0x10E8)  /* AMCCs PCI vendor ID  */
#define PCI_ASL_VENDOR_ID   ((ui16) 0x5341)  /* 'A''S' PCI vendor ID */

/*
 * PCI device IDs
 */
#define PCI_GP_DEVICE  ((ui16) 0x4750)  /* AMCC S5933 default device code */
#define PCI_BIB_ID40   ((ui16) 0x0040)  /* PCI BIB IDrev 40 - device code */

/* 
 * S5933 Operation Register definitions (offsets)
 * ==============================================
 */
#define PCI_REG_OMB1   ((ui32) 0x00)
#define PCI_REG_OMB2   ((ui32) 0x04)
#define PCI_REG_OMB3   ((ui32) 0x08)
#define PCI_REG_OMB4   ((ui32) 0x0C)
#define PCI_REG_IMB1   ((ui32) 0x10)
#define PCI_REG_IMB2   ((ui32) 0x14)
#define PCI_REG_IMB3   ((ui32) 0x18)
#define PCI_REG_IMB4   ((ui32) 0x1C)
#define PCI_REG_FIFO   ((ui32) 0x20)
#define PCI_REG_MWAR   ((ui32) 0x24)
#define PCI_REG_MWTC   ((ui32) 0x28)
#define PCI_REG_MRAR   ((ui32) 0x2C)
#define PCI_REG_MRTC   ((ui32) 0x30)
#define PCI_REG_MBEF   ((ui32) 0x34)
#define PCI_REG_INTCSR ((ui32) 0x38)
#define PCI_REG_MCSR   ((ui32) 0x3C)

/* 
 * Dylan registers
 * ===============
 */

#define DYLAN_REG_CONTROL        ((ui32) 0x00)
#define DYLAN_REG_STATUS         ((ui32) 0x01)
#define DYLAN_REG_SNAP_READ      ((ui32) 0x02)
#define DYLAN_REG_SNAP_WRITE     ((ui32) 0x03)
#define DYLAN_REG_SERW           ((ui32) 0x04)
#define DYLAN_REG_SW_TIMER       ((ui32) 0x05)
#define DYLAN_REG_HW_TIMER       ((ui32) 0x06)
#define DYLAN_REG_SERFMT         ((ui32) 0x07)
#define DYLAN_REG_FPGA_PROG      ((ui32) 0x08)
#define DYLAN_REG_TOGGLE_VBANK   ((ui32) 0x09)
#define DYLAN_REG_MAPPER_CTRL_WR ((ui32) 0x0A)
#define DYLAN_REG_SNAP_READ32    ((ui32) 0x0B)
#define DYLAN_REG_SNAP_WRITE32   ((ui32) 0x0C)
#define DYLAN_REG_INT_CONTROL    ((ui32) 0x0D)

/* 
 * Dylan magic number
 * ==================
 */
#define DYLAN_MAGIC_NUMBER 0x69

/* 
 * Bit position definitions
 * ========================
 */

/* S5933 Mailbox empty/full flags */
#define PCI_BIT_IMBEF4_3 ((ui32) 0x80000000)
#define PCI_BIT_IMBEF4_2 ((ui32) 0x40000000)
#define PCI_BIT_IMBEF4_1 ((ui32) 0x20000000)
#define PCI_BIT_IMBEF4_0 ((ui32) 0x10000000)
#define PCI_BIT_IMBEF3_3 ((ui32) 0x08000000)
#define PCI_BIT_IMBEF3_2 ((ui32) 0x04000000)
#define PCI_BIT_IMBEF3_1 ((ui32) 0x02000000)
#define PCI_BIT_IMBEF3_0 ((ui32) 0x01000000)
#define PCI_BIT_IMBEF2_3 ((ui32) 0x00800000)
#define PCI_BIT_IMBEF2_2 ((ui32) 0x00400000)
#define PCI_BIT_IMBEF2_1 ((ui32) 0x00200000)
#define PCI_BIT_IMBEF2_0 ((ui32) 0x00100000)
#define PCI_BIT_IMBEF1_3 ((ui32) 0x00080000)
#define PCI_BIT_IMBEF1_2 ((ui32) 0x00040000)
#define PCI_BIT_IMBEF1_1 ((ui32) 0x00020000)
#define PCI_BIT_IMBEF1_0 ((ui32) 0x00010000)

#define PCI_BIT_OMBEF4_3 ((ui32) 0x00008000)
#define PCI_BIT_OMBEF4_2 ((ui32) 0x00004000)
#define PCI_BIT_OMBEF4_1 ((ui32) 0x00002000)
#define PCI_BIT_OMBEF4_0 ((ui32) 0x00001000)
#define PCI_BIT_OMBEF3_3 ((ui32) 0x00000800)
#define PCI_BIT_OMBEF3_2 ((ui32) 0x00000400)
#define PCI_BIT_OMBEF3_1 ((ui32) 0x00000200)
#define PCI_BIT_OMBEF3_0 ((ui32) 0x00000100)
#define PCI_BIT_OMBEF2_3 ((ui32) 0x00000080)
#define PCI_BIT_OMBEF2_2 ((ui32) 0x00000040)
#define PCI_BIT_OMBEF2_1 ((ui32) 0x00000020)
#define PCI_BIT_OMBEF2_0 ((ui32) 0x00000010)
#define PCI_BIT_OMBEF1_3 ((ui32) 0x00000008)
#define PCI_BIT_OMBEF1_2 ((ui32) 0x00000004)
#define PCI_BIT_OMBEF1_1 ((ui32) 0x00000002)
#define PCI_BIT_OMBEF1_0 ((ui32) 0x00000001)

/* S5933 Bus Master Control-Status Register */
#define PCI_BIT_MBEF_RESET   ((ui32) 0x08000000)
#define PCI_BIT_IFSTAT_RESET ((ui32) 0x04000000)
#define PCI_BIT_OFSTAT_RESET ((ui32) 0x02000000)
#define PCI_BIT_RESET        ((ui32) 0x01000000)
#define PCI_BIT_RTE          ((ui32) 0x00004000)
#define PCI_BIT_RDFMGMT      ((ui32) 0x00002000)
#define PCI_BIT_RDWT_PRIO    ((ui32) 0x00001000)
#define PCI_BIT_WTE          ((ui32) 0x00000400)  /* Bit 10 */
#define PCI_BIT_WTFMGMT      ((ui32) 0x00000200)  /* Bit  9 */
#define PCI_BIT_WTRD_PRIO    ((ui32) 0x00000100)
#define PCI_BIT_IBTC0        ((ui32) 0x00000080)
#define PCI_BIT_OBTC0        ((ui32) 0x00000040)
#define PCI_BIT_IFE          ((ui32) 0x00000020)
#define PCI_BIT_IF4PLUS      ((ui32) 0x00000010)
#define PCI_BIT_IFF          ((ui32) 0x00000008)
#define PCI_BIT_OFE          ((ui32) 0x00000004)
#define PCI_BIT_OF4PLUS      ((ui32) 0x00000002)
#define PCI_BIT_OFF          ((ui32) 0x00000001)

/* S5933 Interrupt Control-Status Register */
#define PCI_BIT_INT_ASS      ((ui32) 0x00800000)
#define PCI_BIT_TGT_ABRT     ((ui32) 0x00200000)
#define PCI_BIT_MST_ABRT     ((ui32) 0x00100000)
#define PCI_BIT_RTC          ((ui32) 0x00080000)
#define PCI_BIT_WTC          ((ui32) 0x00040000)
#define PCI_BIT_IMB          ((ui32) 0x00020000)
#define PCI_BIT_OMB          ((ui32) 0x00010000)
#define PCI_BIT_RTC_EN       ((ui32) 0x00008000)
#define PCI_BIT_WTC_EN       ((ui32) 0x00004000)
#define PCI_BIT_IMB_EN       ((ui32) 0x00001000) /* interrupt enable for IMB */
#define PCI_BIT_OMB_EN       ((ui32) 0x00000010)

#define PCI_BIT_IMB4B3_EN    ((ui32) 0x00000F00) /* IMB4 Byte 3 */


#define PCI_BIT_8051_MRS     ((ui32) 0x00000001)
#define PCI_BIT_8051_MCK     ((ui32) 0x00000001)

#define PCI_SW_RESET         0x01

#define PCI_DATA0_CL0        ((ui32) 0x0001)
#define PCI_DATA1_CL0        ((ui32) 0x0003)
#define PCI_DATA0_CL1        ((ui32) 0x0000)
#define PCI_DATA1_CL1        ((ui32) 0x0002)
#define PCI_RESET_0          ((ui32) 0x0001)
#define PCI_RESET_1          ((ui32) 0x0000)

#define PCI_MODULE_DATA0_CL0 ((ui32) 0x0001)
#define PCI_MODULE_DATA1_CL0 ((ui32) 0x0003)
#define PCI_MODULE_DATA0_CL1 ((ui32) 0x0000)
#define PCI_MODULE_DATA1_CL1 ((ui32) 0x0002)
#define PCI_MODULE_RESET_0   ((ui32) 0x0001)
#define PCI_MODULE_RESET_1   ((ui32) 0x0000)

#define PCI_BIT_DYLAN_SERIAL_TX      ((ui32) 0x00000001)
#define PCI_BIT_DYLAN_SERIAL_RX      ((ui32) 0x00000002)
#define PCI_BIT_DYLAN_SERIAL_OVER    ((ui32) 0x00000004)
#define PCI_BIT_DYLAN_SERIAL_PARITY  ((ui32) 0x00000008)
#define PCI_BIT_DYLAN_TIMER_RUNNING  ((ui32) 0x00000010)
#define PCI_BIT_DYLAN_INT_TX         ((ui32) 0x00010000)
#define PCI_BIT_DYLAN_INT_RX         ((ui32) 0x00020000)
#define PCI_BIT_DYLAN_INT_TIMER      ((ui32) 0x00100000)

#define PCI_BIT_DYLAN_INTEN_TX       ((ui32) 0x00000001)
#define PCI_BIT_DYLAN_INTEN_RX       ((ui32) 0x00000002)
#define PCI_BIT_DYLAN_INTEN_TIMER    ((ui32) 0x00000004)
#define PCI_BITS_DYLAN_INTEN_ALL     (PCI_BIT_DYLAN_INTEN_TX | PCI_BIT_DYLAN_INTEN_RX | PCI_BIT_DYLAN_INTEN_TIMER)

/* 
 * Mask definitions
 * ================
 */
/* S5933 All interrupt source */
#define PCI_MASK_ALL_INT_SOURCES ( PCI_BIT_TGT_ABRT | \
                                   PCI_BIT_MST_ABRT | \
                                   PCI_BIT_RTC |      \
                                   PCI_BIT_WTC |      \
                                   PCI_BIT_IMB |      \
                                   PCI_BIT_OMB  )

/* S5933 Bus master Control-Status Register */
#define PCI_MASK_NVRAM_CTRL   ((ui32) 0xE0000000)
#define PCI_MASK_NVRAM_DATA   ((ui32) 0x00FF0000)

/* S5933 Interrupt Control-Status Register */
#define PCI_MASK_FIFOEND_CTRL ((ui32) 0xFF000000)
#define PCI_MASK_IMB_INTSEL   ((ui32) 0x00000C00)
#define PCI_MASK_IMB_BINTSEL  ((ui32) 0x00000300)
#define PCI_MASK_OMB_INTSEL   ((ui32) 0x0000000C)
#define PCI_MASK_OMB_BINTSEL  ((ui32) 0x00000003)

#define PCI_MASK_INTCSR ( PCI_MASK_FIFOEND_CTRL | \
                          PCI_MASK_IMB_INTSEL   | \
                          PCI_MASK_IMB_BINTSEL  | \
                          PCI_MASK_OMB_INTSEL   | \
                          PCI_MASK_OMB_BINTSEL  | \
                          PCI_BIT_RTC_EN |        \
                          PCI_BIT_WTC_EN |        \
                          PCI_BIT_IMB_EN |        \
                          PCI_BIT_OMB_EN  )

/* Dylan Status Register */
#define PCI_BIT_DM_DP      ((ui32) 0x40)   /* Datamapper FPGA DONE/PROGRAM */
#define PCI_BIT_SN_DP      ((ui32) 0x80)   /* Snapper FPGA DONE/PROGRAM */



/* Serial Format Register
 * ----------------------
 */
#define PCI_HMASK_PARITY        ((ui32) 0x00000007)
#define PCI_BITS_PARITY_EVEN    ((ui32) 0x00000000)
#define PCI_BITS_PARITY_ODD     ((ui32) 0x00000001)
#define PCI_BITS_PARITY_MARK    ((ui32) 0x00000002)
#define PCI_BITS_PARITY_SPACE   ((ui32) 0x00000003)
#define PCI_BITS_PARITY_NONE    ((ui32) 0x00000004)
#define PCI_HMASK_DATABITS      PCI_BIT_8DATABITS
#define PCI_BIT_7DATABITS       ((ui32) 0x00000000)
#define PCI_BIT_8DATABITS       ((ui32) 0x00000010)
#define PCI_HMASK_STOPBITS      PCI_BIT_2STOPBITS
#define PCI_BIT_1STOPBIT        ((ui32) 0x00000000)
#define PCI_BIT_2STOPBITS       ((ui32) 0x00000020)
#define PCI_HMASK_BAUD_N        PCI_BIT_BAUD_N
#define PCI_BIT_BAUD_N          ((ui32) 0x00000040)
#define PCI_HMASK_BAUD_X        PCI_BIT_BAUD_X
#define PCI_BIT_BAUD_X          ((ui32) 0x00000080)
 
#define PCI_BIT_TX_RDY          ((ui32) 0x00000001)
#define PCI_BIT_RX_RDY          ((ui32) 0x00000002)
#define PCI_BIT_RX_OVR          ((ui32) 0x00000004)
#define PCI_BIT_PARITY          ((ui32) 0x00000008)
#define PCI_BIT_CTS             ((ui32) 0x00000010)

/* Datamapper Snapper clock control bits
 * -------------------------------------
 */
#define PCI_BITS_SNAPCLK_2       ((ui32) 0xF)
#define PCI_BITS_SNAPCLK_4       ((ui32) 0xE)
#define PCI_BITS_SNAPCLK_8       ((ui32) 0x8)
#define PCI_BITS_SNAPCLK_16      ((ui32) 0x0)
#define PCI_BITS_SNAPCLK_20      ((ui32) 0x9)
#define PCI_BITS_SNAPCLK_25      ((ui32) 0xA)
#define PCI_BITS_SNAPCLK_33      ((ui32) 0xC)
#define PCI_BITS_SNAPCLK_40      ((ui32) 0xB)
#define PCI_BITS_SNAPCLK_50      ((ui32) 0xD)

#endif  /* _pci_base_h */
