/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        * 
 *  File name   : pci_bios.h                                              *
 *  Function    : PCI BIOS definitions                                    * 
 *  Project     : Snapper                                                 * 
 *  Author      : Warren Frost, Active Silicon Ltd                        * 
 *  Systems     : ANSI C                                                  * 
 *  Written     : August 1994                                             * 
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        * 
 *  Copyright (c) 1994-96 Active Silicon, Active Imaging Ltd.              *
 **************************************************************************
 *  Comments:
 */

#ifndef _pci_bios_h
#define _pci_bios_h


/*
 * PCI BIOS function codes
 */
#define PCI_FUNCTION_ID             0xB1
#define PCI_BIOS_PRESENT            0x01
#define PCI_FIND_PCI_DEVICE         0x02
#define PCI_FIND_PCI_CLASS_CODE     0x03
#define PCI_GENERATE_SPECIAL_CYCLE  0x06
#define PCI_READ_CS_BYTE            0x08
#define PCI_READ_CS_WORD            0x09
#define PCI_READ_CS_DWORD           0x0A
#define PCI_WRITE_CS_BYTE           0x0B
#define PCI_WRITE_CS_WORD           0x0C
#define PCI_WRITE_CS_DWORD          0x0D

/*
 * PCI BIOS return codes
 */
#define PCI_SUCCESSFUL           0x00
#define PCI_NOT_SUCCESSFUL       0x01
#define PCI_FUNC_NOT_SUPPORTED   0x81
#define PCI_BAD_VENDOR_ID        0x83
#define PCI_DEVICE_NOT_FOUND     0x86
#define PCI_BAD_REGISTER_NUMBER  0x87

/*
 * PCI configuration space registers
 */
#define PCI_CS_VENDOR_ID         0x00
#define PCI_CS_DEVICE_ID         0x02
#define PCI_CS_COMMAND           0x04
#define PCI_CS_STATUS            0x06
#define PCI_CS_REVISION_ID       0x08
#define PCI_CS_CLASS_CODE        0x09
#define PCI_CS_CACHE_LINE_SIZE   0x0C
#define PCI_CS_MASTER_LATENCY    0x0D
#define PCI_CS_HEADER_TYPE       0x0E
#define PCI_CS_BIST              0x0F
#define PCI_CS_BASE_ADDRESS_0    0x10
#define PCI_CS_BASE_ADDRESS_1    0x14
#define PCI_CS_BASE_ADDRESS_2    0x18
#define PCI_CS_BASE_ADDRESS_3    0x1C
#define PCI_CS_BASE_ADDRESS_4    0x20
#define PCI_CS_BASE_ADDRESS_5    0x24
#define PCI_CS_EXPANSION_ROM     0x30
#define PCI_CS_INTERRUPT_LINE    0x3C
#define PCI_CS_INTERRUPT_PIN     0x3D
#define PCI_CS_MIN_GNT           0x3E
#define PCI_CS_MIN_LAT           0x3F

#endif /* _pci_bios_h */

