/**************************************************************************
 *                ACTIVE SILICON LIMITED / DATA CELL LIMITED              * 
 *                                                                        *
 *  File name   : sbusbase.h                                              *
 *  Function    : Header file for SBus Bus Interface Board                *
 *  Project     : DC04 SBus Bus Interface Board                           *
 *  Author      : Chris Beynon                                            *
 *  Systems     : Solaris 2 (ANSI C)                                      *
 *  Written     : 30-Aug-94                                               *
 *  Updated     : 28-Jun-95                                               *
 *  Version     : 2.6                                                     *
 *                                                                        *
 *  Copyright (c) 1994, 1995 Active Silicon Ltd, Data Cell Ltd.           *
 **************************************************************************
 *  Comments:
 *  SOL1 & SOL2 in comments flag differences between Solaris 1 & 2
 */

#ifndef _sbus_base_h
#define _sbus_base_h


/* GENERAL DEFINITIONS
 * -------------------
 */
#define MAX_NUM_SBUS_BOARDS  8


/* FPGAS
 * -----
 */
#define SBUS_BUSIF_FPGA_BYTES    18444
#define SBUS_MAPPER_FPGA_BYTES   5764
#define SBUS_SNAPPER_FPGA_BYTES  18444


/* REGISTER ADDRESSES
 * ------------------
 */
#define SBUS_REG_SBUS_CONTROL   0x0800
#define SBUS_REG_SBUS_STATUS    0x0804
#define SBUS_REG_DMA_ADDR       0x0808
#define SBUS_REG_DMA_COUNT      0x080C
#define SBUS_REG_FPGA_ID        0x0810
#define SBUS_REG_I2C_ACCESS     0x0814
#define SBUS_REG_FIFO_FLAGS     0x0818
#define SBUS_REG_8051_STATUS    0x081C
#define SBUS_REG_8051_FPGA      0x0A00
#define SBUS_REG_8051_CODE      0x0A04
#define SBUS_REG_8051_SERW      0x0A08
#define SBUS_REG_8051_SERR      0x0A0C
#define SBUS_REG_8051_BAUD      0x0A10
#define SBUS_REG_8051_SERFMT    0x0A14
#define SBUS_REG_8051_CTRL1     0x0A18
#define SBUS_REG_8051_CTRL2     0x0A1C
#define SBUS_REG_8051_CCLK8     0x0A20
#define SBUS_REG_8051_TIMER_LO  0x0A24
#define SBUS_REG_8051_TIMER_HI  0x0A28
#define SBUS_REG_8051_TIMER_SW  0x0A2C
#define SBUS_REG_8051_COMMAND1  0x0A38
#define SBUS_REG_8051_COMMAND2  0x0A3C
#define SBUS_REG_FIFO_RW        0x0C00
#define SBUS_REG_FIFO_BYPASS    0x0C04
#define SBUS_REG_FIFO_A2B_AE    0x0C10
#define SBUS_REG_FIFO_A2B_AF    0x0C14
#define SBUS_REG_FIFO_B2A_AE    0x0C18
#define SBUS_REG_FIFO_B2A_AF    0x0C1C
#define SBUS_REG_MAPPER_DATA    0x0E00
#define SBUS_REG_MAPPER_LOAD    0x0E04
#define SBUS_REG_SNAPPER_BASE   0x0F00


/* REGISTER BITS
 * =============
 *
 * SBus Control Register
 */
#define SBUS_BIT_PREFILL         ((ui32) 0x20000000)
#define SBUS_BIT_IEN_TX_RDY      ((ui32) 0x00000002)
#define SBUS_BIT_IEN_RX_RDY      ((ui32) 0x00000001)
#define SBUS_BIT_DMA_EN          ((ui32) 0x01000000)
#define SBUS_BIT_IGNORE_FIFO_RDY ((ui32) 0x00400000)
#define SBUS_BIT_IEN_END         ((ui32) 0x00800000)
#define SBUS_BIT_IEN_MAP_SNP     ((ui32) 0x00000100)
#define SBUS_BITS_SIZ_SHIFT      ((ui32) 26)


/* SBus Status Register
 */
#define SBUS_BIT_INT_PEND0       ((ui32) 0x40000000)
#define SBUS_BIT_INT_PEND1       ((ui32) 0x80000000)
#define SBUS_BIT_DMA_DONE        ((ui32) 0x20000000)
#define SBUS_BIT_I_TX_RDY        ((ui32) 0x10000000)
#define SBUS_BIT_I_RX_RDY        ((ui32) 0x08000000)
#define SBUS_BIT_I_END           ((ui32) 0x04000000)
#define SBUS_BIT_DMA_ERR         ((ui32) 0x02000000)
#define SBUS_BIT_DMA_TC          ((ui32) 0x01000000)
#define SBUS_BIT_DMA_RUNNING     ((ui32) 0x00800000)
#define SBUS_BIT_DMA_LERR        ((ui32) 0x00400000)
#define SBUS_BIT_STAT_DMA_EN     ((ui32) 0x00200000)
#define SBUS_BIT_I_MAP_SNP       ((ui32) 0x00000100)


/* DMA Address Register
 *
 * DMA Count Register
 *
 * FPGA ID Register
 * ----------------
 */
#define SBUS_FPGAID_POWUP        ((ui8) 0x01)
#define SBUS_FPGAID_TEST         ((ui8) 0x02)
#define SBUS_FPGAID_WRONLY       ((ui8) 0x11)
#define SBUS_FPGAID_RDONLY       ((ui8) 0x12)
#define SBUS_FPGAID_WR_RD        ((ui8) 0x13)

/* I2C_ACCESS Register
 * 
 * FIFO Flags Register
 * -------------------
 */
#define SBUS_BIT_A2B_AF          ((ui32) 0x00000001)
#define SBUS_BIT_A2B_F           ((ui32) 0x00000002)
#define SBUS_BIT_B2A_AE          ((ui32) 0x00000004)
#define SBUS_BIT_B2A_E           ((ui32) 0x00000008)

/*                         
 * Mapper Data/Load registers
 * --------------------------
 */
#define SBUS_BIT_MAPPER_DATA     ((ui32) 0x00000001)

/* 8051 Status Register
 * --------------------
 */
#define SBUS_BIT_CTS             ((ui32) 0x00000001)
#define SBUS_BIT_COMMAND_BUSY    ((ui32) 0x00400000)
#define SBUS_BIT_BUSY            ((ui32) 0x00800000)
#define SBUS_BIT_TX_RDY          ((ui32) 0x01000000)
#define SBUS_BIT_RX_RDY          ((ui32) 0x02000000)
#define SBUS_BIT_RX_OVR          ((ui32) 0x04000000)
#define SBUS_BIT_PARITY          ((ui32) 0x08000000)
#define SBUS_BIT_DP_M            ((ui32) 0x10000000)
#define SBUS_BIT_DP_S            ((ui32) 0x20000000)
#define SBUS_BIT_TIMER_RUNNING   ((ui32) 0x20000000)
#define SBUS_BIT_CODE            ((ui32) 0x40000000)
#define SBUS_BIT_ERROR           ((ui32) 0x80000000)

/* 8051 Control Registers
 * ----------------------
 */
#define SBUS_BIT_DIN             ((ui32) 0x01000000)
#define SBUS_BITS_SNP_PROG       ((ui32) 0x0C000000)
#define SBUS_BITS_MAP_PROG       ((ui32) 0x30000000)
/* #define SBUS_BIT_MAP_RST         ((ui32) 0x20000000) */

#define SBUS_HMASK_SNAPCLK       ((ui32) 0x0F000000)
#define SBUS_BITS_CLK2           ((ui32) 0x0F000000)
#define SBUS_BITS_CLK4           ((ui32) 0x07000000)
#define SBUS_BITS_CLK8           ((ui32) 0x06000000)
#define SBUS_BITS_CLK16          ((ui32) 0x00000000)
#define SBUS_BITS_CLK20          ((ui32) 0x09000000)
#define SBUS_BITS_CLK25          ((ui32) 0x0A000000)
#define SBUS_BITS_CLK33          ((ui32) 0x0C000000)
#define SBUS_BITS_CLK40          ((ui32) 0x01000000)
#define SBUS_BITS_CLK50          ((ui32) 0x02000000)

#define SBUS_BIT_FIF_RST         ((ui32) 0x10000000)
#define SBUS_BIT_MAP_PRST        ((ui32) 0x20000000)
#define SBUS_BIT_LED             ((ui32) 0x40000000)

/* Serial Format Register
 * ----------------------
 */
#define SBUS_HMASK_PARITY        ((ui32) 0x07000000)
#define SBUS_BITS_PARITY_EVEN    ((ui32) 0x00000000)
#define SBUS_BITS_PARITY_ODD     ((ui32) 0x01000000)
#define SBUS_BITS_PARITY_MARK    ((ui32) 0x02000000)
#define SBUS_BITS_PARITY_SPACE   ((ui32) 0x03000000)
#define SBUS_BITS_PARITY_NONE    ((ui32) 0x04000000)
#define SBUS_HMASK_DATABITS      SBUS_BIT_8DATABITS
#define SBUS_BIT_7DATABITS       ((ui32) 0x00000000)
#define SBUS_BIT_8DATABITS       ((ui32) 0x10000000)
#define SBUS_HMASK_STOPBITS      SBUS_BIT_2STOPBITS
#define SBUS_BIT_1STOPBIT        ((ui32) 0x00000000)
#define SBUS_BIT_2STOPBITS       ((ui32) 0x20000000)
#define SBUS_HMASK_BAUD_N        SBUS_BIT_BAUD_N
#define SBUS_BIT_BAUD_N          ((ui32) 0x40000000)
#define SBUS_HMASK_BAUD_X        SBUS_BIT_BAUD_X
#define SBUS_BIT_BAUD_X          ((ui32) 0x80000000)
 

/* 8051 Command Registers
 * ----------------------
 */
#define SBUS_CMD8051_INITIALIZE  0x0F
#define SBUS_CMD8051_NULL        0x00
#define SBUS_CMD8051_FPGA_DATA   0x01
#define SBUS_CMD8051_RECONFIGURE 0x02
#define SBUS_CMD8051_NEW_CODE    0x03
#define SBUS_CMD8051_JUMP        0x04
#define SBUS_CMD8051_STROBE_CCLK 0x05
#define SBUS_CMD8051_UPDATE_DP   0x06
#define SBUS_CMD8051_UPDATE_DP_M 0x06
#define SBUS_CMD8051_UPDATE_DP_S 0x86
#define SBUS_CMD8051_SECURITY    0x07


/* STRUCTURE DEFINITIONS
 * ---------------------
 */

#endif /* _sbus_base_h */

