/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : snp16.h                                                 *
 *  Function    : Header for SNP-16 - #defines and types                  *
 *  Project     : Snappers - DC03                                         *
 *  Author      : Martin Bone, Active Silicon Ltd                         *
 *  Systems     : ANSI C                                                  *
 *  Written     : September 1993                                          *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _SNP16_H_
#define _SNP16_H_


/* Type Definitions
 * ----------------
 */
/* none */


/* Interrupt Definitions
 * ---------------------
 */
#define SNP16_HW_INT     0    /* NOTE unused at present */


/* Array Sizes
 * -----------
 */
#define SNP16_NUM_I2C_REG   0x19
#define SNP16_NUM_LCA_REG   0x08
#define SNP16_NUM_HANDLES   16


/* Error Codes
 * -----------
 * The error return code consists of 4 bytes.  The lower 2 contain any
 * parameter information from the function.  The next byte contains an
 * error code, and the highest byte contains the ID of the module
 * signalling the error.  This technique allows the software to
 * return an error although the original call may have been made
 * indirectly through another library call.
 */

#define SNP16_ID               ((ui8) 0xA0) /* Snapper HW ID  */
#define SNP16_PCI_ID           ((ui8) 0xA8) /* Combined Snp16 & PCI-BIB ID  */
#define SNP_PCI16_ID           SNP16_PCI_ID /* For Compatibility */
#define SNP16_FAMILY_ID        SNP16_ID


/* Register Addresses
 * ------------------
 */
#define SNP16_LCA_REG00     0x00
#define SNP16_LCA_REG01     0x01
#define SNP16_LCA_REG02     0x02
#define SNP16_LCA_REG03     0x03
#define SNP16_LCA_REG04     0x04
#define SNP16_LCA_REG05     0x05
#define SNP16_LCA_REG06     0x06
#define SNP16_LCA_REG07     0x07

#define SNP16_I2C_REG00     0x00
#define SNP16_I2C_REG01     0x01
#define SNP16_I2C_REG02     0x02
#define SNP16_I2C_REG03     0x03
#define SNP16_I2C_REG04     0x04
#define SNP16_I2C_REG05     0x05
#define SNP16_I2C_REG06     0x06
#define SNP16_I2C_REG07     0x07
#define SNP16_I2C_REG08     0x08
#define SNP16_I2C_REG09     0x09
#define SNP16_I2C_REG0A     0x0A
#define SNP16_I2C_REG0B     0x0B
#define SNP16_I2C_REG0C     0x0C
#define SNP16_I2C_REG0D     0x0D
#define SNP16_I2C_REG0E     0x0E
#define SNP16_I2C_REG0F     0x0F
#define SNP16_I2C_REG10     0x10
#define SNP16_I2C_REG11     0x11
#define SNP16_I2C_REG12     0x12
#define SNP16_I2C_REG13     0x13
#define SNP16_I2C_REG14     0x14
#define SNP16_I2C_REG15     0x15
#define SNP16_I2C_REG16     0x16
#define SNP16_I2C_REG17     0x17
#define SNP16_I2C_REG18     0x18


/* Boolean Flags
 * -------------
 */
#define SNP16_FALLING_EDGE     TRUE
#define SNP16_RISING_EDGE      FALSE


/* Software Tokens
 * ---------------
 */
#define SNP16_INPUT_YC         0
#define SNP16_INPUT_COMPOSITE  1
#define SNP16_INPUT_MONO       2
#define SNP16_DECODE_PAL       0
#define SNP16_DECODE_NTSC      1
#define SNP16_DECODE_SECAM     2


/* SNP16_get_parameter
 */
#define SNP16_MAPPER_INTERFACE_TYPE ((ui16) 1)
#define SNP16_ID_VALUE              ((ui16) 2)
#define SNP16_IDREV_VALUE           ((ui16) 3)
#define SNP16_FAMILY_VALUE          ((ui16) 4)
#define SNP16_FAMILY                SNP16_FAMILY_VALUE /* For Compatibility */
#define SNP16_BASEBOARD_HANDLE      ((ui16) 5)
#define SNP16_REV_VALUE             ((ui16) 6)

/*
SNP16_settings_load
SNP16_settings_save
----------------------
*/
#define SNP16_SETTINGS_FORMAT_FILE  ((Tparam) 0x00000001L)



/* Hardware Specific Constants
 * ---------------------------
 */
#define SNP16_7191_WR_ADDR  0x8A
#define SNP16_7191_RD_ADDR  0x8B


/* Bit Positions
 * -------------
 */
/* LCA Read Register */
#define SNP16_BIT_TRIGGERED    ((ui8) 0x02)

/* LCA Register #0 */
#define SNP16_BIT_CAPTURE      ((ui8) 0x01)
#define SNP16_BIT_SW_BANK1     ((ui8) 0x02)
#define SNP16_BIT_HW_BANK_SEL  ((ui8) 0x04)
#define SNP16_BIT_HW_BANK_EN   ((ui8) 0x08)

/* LCA Register #1 */
#define SNP16_FORMAT_MONO8     ((Tparam) 0x00)
#define SNP16_FORMAT_MONO7     ((Tparam) 0x02)
#define SNP16_FORMAT_YUV422    ((Tparam) 0x04)
#define SNP16_FORMAT_TEST      ((Tparam) 0x01)
#define SNP16_BIT_MONO_OFFSET  ((ui8) 0x08)
#define SNP16_MASK_FORMAT      ((ui8) 0x07)
#define SNP16_BIT_IRQ_EN       ((ui8) 0x10)

/* LCA Register #2 */
#define SNP16_SUB_X1           ((ui8) 0x00)
#define SNP16_SUB_X2           ((ui8) 0x01)
#define SNP16_SUB_X4           ((ui8) 0x02)
#define SNP16_SUB_X1_FIELD_DUPLICATE   ((ui8) 0x03)
#define SNP16_SUB_FAST         SNP16_SUB_X1_FIELD_DUPLICATE   /* keep for compatibility */
#define SNP16_BIT_FILTER       ((ui8) 0x04)
#define SNP16_BIT_EXT_POL      ((ui8) 0x08)
#define SNP16_MASK_SUB         ((ui8) 0x03)

/* LCA Register #3 */
#define SNP16_BIT_REAL_TIME    ((ui8) 0x01)
#define SNP16_BIT_READ_RESET   ((ui8) 0x02)
#define SNP16_BIT_I2C_SCL      ((ui8) 0x04)
#define SNP16_BIT_I2C_SDA      ((ui8) 0x08)
#define SNP16_BIT_I2C_SCL_SDA  ((ui8) 0x0C)

/* LCA Register #5 */
#define SNP16_BIT_EXT_TRIGGER  ((ui8) 0x08)

/* I2C Registers */
#define SNP16_BIT_I2C_60Hz     ((ui8) 0x20)
#define SNP16_BIT_I2C_COLOUR   ((ui8) 0x01)
#define SNP16_BIT_I2C_LOCKED   ((ui8) 0x40)
#define SNP16_BIT_I2C_YC       ((ui8) 0x04)
#define SNP16_BIT_I2C_SECS     ((ui8) 0x01)
#define SNP16_BIT_I2C_FSEL     ((ui8) 0x40)
#define SNP16_BIT_I2C_VCR      ((ui8) 0x80)
#define SNP16_BIT_I2C_BYPS     ((ui8) 0x80)
#define SNP16_MASK_VIDEO_SRC   ((ui8) 0x03)
#define SNP16_VERT_NORMAL      ((ui8) 0x00)
#define SNP16_VERT_SEARCH      ((ui8) 0x01)
#define SNP16_VERT_AUTO        ((ui8) 0x02)
#define SNP16_VERT_BYPASS      ((ui8) 0x03)
#define SNP16_MASK_VERT_MODE   ((ui8) 0x03)


/* Default Active Area
 * -------------------
 */
#define SNP16_50HZ_X_OFFSET ((i16)   56)  /* NOTE: 58 shows right hand edge */
#define SNP16_50HZ_WIDTH    ((i16)  768)
#define SNP16_50HZ_X_BLANK  ((i16) -152)
#define SNP16_50HZ_Y_BLANK  ((i16)  -13)
#define SNP16_50HZ_HEIGHT   ((i16)  576)

#define SNP16_60HZ_X_OFFSET ((i16)   88)
#define SNP16_60HZ_WIDTH    ((i16)  640)
#define SNP16_60HZ_X_BLANK  ((i16) -144)
#define SNP16_60HZ_Y_BLANK  ((i16)   -8)
#define SNP16_60HZ_HEIGHT   ((i16)  480)

 /* Structure Definitions
 * ---------------------
 */
struct Tsnp16
{
   Terr     (*Pfunc)(Thandle, ui8, ...);
   Thandle  Hbaseboard;
   ui8      ID;                           /* B0 for SNP-24, B8 for SNP-8 */
   ui8      rev;

   ui32     trigger_timeout;  /* seconds */

   ui8      sync_start;
   ui8      sync_stop;
   ui8      clamp_start;
   ui8      clamp_stop;
   ui8      hue;
   ui8      chrominance;
   ui8      format;
   ui8      input_mode;
   ui8      decode_mode;
   ui8      video_source;
   ui8      vertical_mode;
   ui16     TMG_format;

   i16      x_start;
   i16      x_length;
   i16      y_start;
   i16      y_length;

   ui8      sub_sample;
   ui8      packing_byte;

   Tboolean mono7_offset;
   Tboolean filter;
   Tboolean irq_en;
   Tboolean real_time;
   Tboolean hw_bank_en;
   Tboolean hw_bank_sel;
   Tboolean sw_bank1;
   Tboolean deinterlace;

   Tboolean ext_trigger;
   Tboolean ext_polarity;

   Tboolean read_since_capture;

   volatile Tboolean capture_complete;

   ui8      LCA_reg[SNP16_NUM_LCA_REG];
   ui8      I2C_reg[SNP16_NUM_I2C_REG];

   char     snp_fpga_date[RBT_ASL_DATE_LENGTH];   /* Date read from Snapper FPGA */ 
   ui16 wLinesSoFar;                        /* for read_video_data */
   ui16 wLocalAction;                       /* for read_video_data */
};


/* MACRO DEFINITIONS not for error handling (in alphabetical order)
 * ----------------------------------------------------------------
 */

#if defined _lint
struct Tsnp16 * EXPORT_FN DUMMY_get_Psnp16(char *, Thandle);
#define SNP16_get_Psnp16(s,Ps,Hs)   Ps = DUMMY_get_Psnp16(s,Hs);

#elif defined _MALLOC_DEBUG
#define SNP16_get_Psnp16(s,Ps,Hs) {if (((Hs) < 1) || ((Hs) >= SNP16_NUM_HANDLES))\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Handle out of range",s)\
                                   else if ((Ps = Psnp16_array[Hs]) == NULL)\
                                      ASL_proe(ASLERR_BAD_HANDLE,"Null pointer",s)\
                                  }
#else
#define SNP16_get_Psnp16(s,Ps,Hs)   {if ((Ps = Psnp16_array[Hs]) == NULL) ASL_proe(ASLERR_BAD_HANDLE,"",s)}
#endif


/* MACRO DEFINITIONS for error handling (in alphabetical order)
 * ------------------------------------------------------------
 * Note: These macros cannot be used on the line  after an `else' -
 * {} are needed even on single line if-else statements.
 */
#if defined _lint
extern Terr EXPORT_FN SNP16_Dummy_err_ret(Thandle, Terr, char *, Terr, char *);
#define SNP16_err_ret(Hs,e,d,v,s)   return(SNP16_Dummy_err_ret(Hs,e,d,v,s))
#define SNP16_make_ret(Hs,e,v)      (( (Psnp16_array[Hs]->ID | Psnp16_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))

#elif defined _DOS16  /* real mode DOS - we lose some the comments to save space */
#define SNP16_err_ret(Hs,e,d,v,fn)  {ASL_err_process(fn,SNP16_make_ret(Hs,e,v),"");return(SNP16_make_ret(Hs,e,v));}
#define SNP16_make_ret(Hs,e,v)      (( (Psnp16_array[Hs]->ID | Psnp16_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))

#else
#define SNP16_err_ret(Hs,e,d,v,fn)  {ASL_err_process(fn,SNP16_make_ret(Hs,e,v),d);return(SNP16_make_ret(Hs,e,v));}
#define SNP16_make_ret(Hs,e,v)      (( (Psnp16_array[Hs]->ID | Psnp16_array[Hs]->rev) << 24) | (e) | ((Terr) v & (Terr) 0xffff))
#endif


#endif /* _SNP16_ */
