/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : snp16pro.h                                              *
 *  Function    : Header for SNP-16 - prototypes and externs              *
 *  Project     : Snappers - DC03                                         *
 *  Author      : Chris Beynon, Martin Bone, Active Silicon Ltd           *
 *  Systems     : ANSI C                                                  *
 *  Written     : October 1994                                            *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _SNP16PRO_H_
#define _SNP16PRO_H_


/* Function Prototypes
 * -------------------
 */
#ifdef __cplusplus
extern "C" {
#endif
Terr EXPORT_FN SNP16_capture(Thandle);
Terr EXPORT_FN SNP16_configure_fpga(Thandle, char*);
Terr EXPORT_FN SNP16_configure_gray_50Hz(Thandle);
Terr EXPORT_FN SNP16_configure_gray_60Hz(Thandle);
Terr EXPORT_FN SNP16_configure_NTSC(Thandle);
Terr EXPORT_FN SNP16_configure_PAL(Thandle);
Thandle EXPORT_FN SNP16_create_snapper(void);
Terr EXPORT_FN SNP16_destroy_snp16(Thandle);
Terr EXPORT_FN SNP16_get_chrominance(Thandle);
Terr EXPORT_FN SNP16_get_decode_mode(Thandle);
Terr EXPORT_FN SNP16_get_format(Thandle);
Terr EXPORT_FN SNP16_get_hue(Thandle);
Terr EXPORT_FN SNP16_get_ID(Thandle);
Terr EXPORT_FN SNP16_get_parameter(Thandle, ui16);
Terr EXPORT_FN SNP16_get_property(Thandle, char *, char *);
Terr EXPORT_FN SNP16_get_rev(Thandle);
Terr EXPORT_FN SNP16_get_sw_bank1(Thandle);
Terr EXPORT_FN SNP16_get_subsample(Thandle);
Terr EXPORT_FN SNP16_get_ROI(Thandle, i16*);
Terr EXPORT_FN SNP16_get_ROI_max(Thandle, i16*);
Terr EXPORT_FN SNP16_get_TMG_format(Thandle);
Terr EXPORT_FN SNP16_get_x_length(Thandle);
Terr EXPORT_FN SNP16_get_x_start(Thandle);
Terr EXPORT_FN SNP16_get_y_length(Thandle);
Terr EXPORT_FN SNP16_get_y_start(Thandle);
Terr EXPORT_FN SNP16_initialize(Thandle);
#ifdef NEVERNEVER  /* interrupts - not yet implemented */
Terr EXPORT_FN SNP16_int_intercept(Thandle);
int  EXPORT_FN SNP16_int_ISR(struct INT_DATA *);
Terr EXPORT_FN SNP16_int_restore(Thandle);
#endif
Tboolean EXPORT_FN SNP16_is_50Hz(Thandle);
Tboolean EXPORT_FN SNP16_is_60Hz(Thandle);
Tboolean EXPORT_FN SNP16_is_capture_complete(Thandle);
Tboolean EXPORT_FN SNP16_is_colour(Thandle);
Tboolean EXPORT_FN SNP16_is_locked(Thandle);
Tboolean EXPORT_FN SNP16_is_trigger_started(Thandle);
Terr EXPORT_FN SNP16_read_I2C_byte(Thandle);
Terr EXPORT_FN SNP16_read_I2C_reg(Thandle);
Terr EXPORT_FN SNP16_read_LCA_reg(Thandle, ui8);
Terr EXPORT_FN SNP16_read_strip_ui16(Thandle, Thandle);
Terr EXPORT_FN SNP16_read_strip_ui32(Thandle, Thandle);
Terr EXPORT_FN SNP16_read_strip_ui32_dos16(Thandle, Thandle);
Terr EXPORT_FN SNP16_read_video_data(Thandle, Thandle, ui16);
Terr EXPORT_FN SNP16_reset_read_pointer(Thandle);
Terr EXPORT_FN SNP16_restore_snapper(Thandle);
#ifdef _QUICKC_
Terr SNP16_set_baseboard(Thandle, Thandle, Terr (*) (Thandle, ui8, ...));
#else
Terr SNP16_set_baseboard(Thandle, Thandle, Terr (*Pfunc) (Thandle, ui8, ...));
#endif
Terr EXPORT_FN SNP16_set_chrominance(Thandle, ui8);
Terr EXPORT_FN SNP16_set_clamp_start(Thandle, ui8);
Terr EXPORT_FN SNP16_set_clamp_stop(Thandle, ui8);
Terr EXPORT_FN SNP16_set_decode_mode(Thandle, ui8);
Terr EXPORT_FN SNP16_set_deinterlace(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_ext_polarity(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_ext_trigger(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_filter(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_format(Thandle, Tparam, ui16);
Terr EXPORT_FN SNP16_set_hue(Thandle, ui8);
Terr EXPORT_FN SNP16_set_hw_bank_en(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_hw_bank_sel(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_ID(Thandle, ui8);
Terr EXPORT_FN SNP16_set_input_mode(Thandle, ui8);
Terr EXPORT_FN SNP16_set_image(Thandle, Thandle);
Terr EXPORT_FN SNP16_set_irq_en(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_mode_vcr(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_mono7_offset(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_real_time(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_rev(Thandle, ui8);
Terr EXPORT_FN SNP16_set_ROI(Thandle, Tcoord, Tcoord, Tcoord, Tcoord);
Terr EXPORT_FN SNP16_set_subsample(Thandle, ui16);
Terr EXPORT_FN SNP16_set_sw_bank1(Thandle, Tboolean);
Terr EXPORT_FN SNP16_set_sync_start(Thandle, ui8);
Terr EXPORT_FN SNP16_set_sync_stop(Thandle, ui8);
Terr EXPORT_FN SNP16_set_TMG_format(Thandle, ui16);
Terr EXPORT_FN SNP16_set_trigger_timeout(Thandle, ui32);
Terr EXPORT_FN SNP16_set_vertical_mode(Thandle, ui8);
Terr EXPORT_FN SNP16_set_video_source(Thandle, ui8);
Terr EXPORT_FN SNP16_set_x_length(Thandle, Tcoord);
Terr EXPORT_FN SNP16_set_x_start(Thandle, Tcoord);
Terr EXPORT_FN SNP16_set_y_length(Thandle, Tcoord);
Terr EXPORT_FN SNP16_set_y_start(Thandle, Tcoord);
Terr EXPORT_FN SNP16_settings_implement( Thandle, struct InitTag* );
Terr EXPORT_FN SNP16_settings_load(Thandle, ui32, const void *);
Terr EXPORT_FN SNP16_settings_save(Thandle, ui32, void *);
Terr EXPORT_FN SNP16_start_capture(Thandle);
Terr EXPORT_FN SNP16_write_I2C_byte(Thandle, ui8);
Terr EXPORT_FN SNP16_write_I2C_reg(Thandle, ui8);
Terr EXPORT_FN SNP16_write_LCA_reg(Thandle, ui8);
#ifdef __cplusplus
};
#endif


/*
 * External Definitions
 * --------------------
 */

/* Global Array of Snapper Handles
 * -------------------------------
 */
extern struct Tsnp16 *Psnp16_array[SNP16_NUM_HANDLES+1];


/*
 * FPGA configuration data
 */
extern ui8  FAR fpga_a1_data[];
extern ui8  FAR fpga_a2_data[];
extern ui32 FAR fpga_a1_bit_length;
extern ui32 FAR fpga_a2_bit_length;
extern char FAR fpga_a1_date[];
extern char FAR fpga_a2_date[];


#endif /* _SNP16PRO_ */
