/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : snp24pro.h                                              *
 *  Function    : Header for SNP-24 and SNP-8 - prototypes and externs    *
 *  Project     : Snappers - DC09                                         *
 *  Author      : Chris Beynon, Martin Bone, Active Silicon Ltd           *
 *  Systems     : ANSI C                                                  *
 *  Written     : June 1994                                               *
 *  Updated     : 15-Jan-96                                               *
 *  Version     : 3.10                                                    *
 *                                                                        *
 *  Copyright (c) 1994-96 Active Silicon, Active Imaging Ltd.             *
 **************************************************************************
 *  Comments:
 */

#ifndef _SNP24PRO_H_
#define _SNP24PRO_H_


/* FUNCTION PROTOTYPES (in alphabetical order)
 * -------------------------------------------
 */
#ifdef __cplusplus
extern "C" {
#endif

Terr EXPORT_FN SNP24_auto_gain(Thandle, ui8);
Terr EXPORT_FN SNP24_auto_offset(Thandle, ui8);
Terr EXPORT_FN SNP24_bt252_check_valid(Thandle, int);
Terr EXPORT_FN SNP24_bt252_clear_cmd(Thandle, int, Tparam);
Terr EXPORT_FN SNP24_bt252_set_cmd(Thandle, int, Tparam);
Terr EXPORT_FN SNP24_bt252_write_cmd(Thandle, int );
Terr EXPORT_FN SNP24_configure_fpga(Thandle, char*, ui16);
Terr EXPORT_FN SNP24_capture(Thandle, Tparam);
Thandle EXPORT_FN SNP24_create_snp24(void);
Terr EXPORT_FN SNP24_destroy_snp24(Thandle);
Terr EXPORT_FN SNP24_fram_dummy_read(Thandle, Tparam);
Terr EXPORT_FN SNP24_generate_timer_pulse(Thandle);

Terr EXPORT_FN SNP24_get_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_get_ctrl_io_status(Thandle);
Terr EXPORT_FN SNP24_get_ID(Thandle);
Terr EXPORT_FN SNP24_get_levels(Thandle, ui8 [SNP24_LVL_MAX_LEVELS]);
Terr EXPORT_FN SNP24_get_LUTs(Thandle, int, ui8 [SNP24_SIZE_LUT252]);
Terr EXPORT_FN SNP24_get_module_format(Thandle Hsnp24);
Terr EXPORT_FN SNP24_get_parameter(Thandle, ui16);
Terr EXPORT_FN_PTR SNP24_get_ptr(Thandle hSnp24, ui16 wPtrType);
Terr EXPORT_FN SNP24_get_property(Thandle, char *, char *);
Terr EXPORT_FN SNP24_get_rev(Thandle);
Terr EXPORT_FN SNP24_get_ROI(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_get_ROI_max(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_get_snapper_mode(Thandle);
Terr EXPORT_FN SNP24_get_subsample(Thandle);
Terr EXPORT_FN SNP24_get_TMG_format(Thandle);
Terr EXPORT_FN SNP24_get_video_standard(Thandle);

Terr EXPORT_FN SNP24_interrupt_process(Thandle hSnp24, struct TCallbackEventInfo *psCallbackEventInfo);

Terr EXPORT_FN SNP24_ics1522_init(Thandle);
Terr EXPORT_FN SNP24_ics1522_read_reg(Thandle, ui16);
Terr EXPORT_FN SNP24_ics1522_strobe_sclk(Thandle);
Terr EXPORT_FN SNP24_ics1522_write(Thandle);

Terr EXPORT_FN SNP24_initialize(Thandle, Tparam);

Tboolean EXPORT_FN SNP24_is_50Hz(Thandle);
Tboolean EXPORT_FN SNP24_is_capture_complete(Thandle);
Tboolean EXPORT_FN SNP24_is_data_ready(Thandle);
Tboolean EXPORT_FN SNP24_is_field1_captured(Thandle);
Tboolean EXPORT_FN SNP24_is_field1_incoming(Thandle);
Tboolean EXPORT_FN SNP24_is_in_trigger_sel(Thandle);
Tboolean EXPORT_FN SNP24_is_locked(Thandle);
Tboolean EXPORT_FN SNP24_is_read_bank_A(Thandle);
Tboolean EXPORT_FN SNP24_is_readout_deinterlace(Thandle);
Tboolean EXPORT_FN SNP24_is_sequence_dropped(Thandle);
Tboolean EXPORT_FN SNP24_is_sequence_mode(Thandle);
Tboolean EXPORT_FN SNP24_is_trigger_started(Thandle);
Tboolean EXPORT_FN SNP24_is_vsync_asserted(Thandle);

Terr EXPORT_FN SNP24_LM1882_settings_implement( Thandle hSnapper, struct InitTag* pstTag );
Terr EXPORT_FN SNP24_lm1882_prog(Thandle, Tparam, ui16 [SNP24_NUM_LM1882_REGS]);
Terr EXPORT_FN SNP24_LUT_settings_implement   ( Thandle hSnapper, struct InitTag* pstTag );

Terr EXPORT_FN SNP24_prepare_for_CRUNCH_read(Thandle);
Terr EXPORT_FN SNP24_read_snp24_reg(Thandle, Tparam);
Terr EXPORT_FN SNP24_read_strip_ui16(Thandle Hsnp24, Thandle Himage);
Terr EXPORT_FN SNP24_read_strip_ui32(Thandle Hsnp24, Thandle Himage);
Terr EXPORT_FN SNP24_read_video_data(Thandle, Thandle, ui16);
Terr EXPORT_FN SNP24_reset_read_pointer(Thandle, Tparam);
Terr EXPORT_FN SNP24_restore_snp24(Thandle);

#ifdef _QUICKC_
Terr SNP24_set_baseboard(Thandle, Thandle, Terr (*) (Thandle, ui8, ...));
#else
Terr SNP24_set_baseboard(Thandle, Thandle, Terr (*Pfunc) (Thandle, ui8, ...));
#endif
Terr EXPORT_FN SNP24_set_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _VXWORKS || defined _LYNXOS || defined _SOLARIS || defined _MACOSX || defined _LINUX
Terr EXPORT_FN SNP24_set_callback(Thandle Hdig16, Tparam dwMode, void (*pCallback)(Thandle, ui32, void*), void *pParameter );
#else
Terr EXPORT_FN SNP24_set_callback(Thandle Hdig16, Tparam dwMode, void (EXPORT_FN *pCallback)(Thandle, ui32, void*), void *pParameter );
#endif
Terr EXPORT_FN SNP24_set_capture(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_clamp(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_clk(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_ctrlout(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_format(Thandle, Tparam, ui16);
Terr EXPORT_FN SNP24_set_ID(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_image(Thandle, Thandle);
Terr EXPORT_FN SNP24_set_interrupts(Thandle hSnp24, Tparam dwType, Tboolean fSetOrNot);
Terr EXPORT_FN SNP24_set_levels(Thandle, Tparam, ui8 [SNP24_LVL_MAX_LEVELS]);
Terr EXPORT_FN SNP24_set_linescan_active_area(Thandle, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_set_linescan_ctrl(Thandle, Tparam, ui16);
Terr EXPORT_FN SNP24_set_linescan_freq(Thandle, ui16);
Terr EXPORT_FN SNP24_set_linescan_ROI(Thandle, Tparam, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_set_LUTs(Thandle, Tparam, int, ui8 [SNP24_SIZE_LUT252]);
Terr EXPORT_FN SNP24_set_module_format(Thandle, ui16);
Terr EXPORT_FN SNP24_set_parameter(Thandle, ui16, ui32);
Terr EXPORT_FN SNP24_set_pix_per_line(Thandle, ui16);
Terr EXPORT_FN SNP24_set_read_bank(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_rev(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_ROI(Thandle, Tparam, i16 [ASL_SIZE_2D_ROI]);
Terr EXPORT_FN SNP24_set_ROI_rounding(Thandle, ui16, ui16);
Terr EXPORT_FN SNP24_set_sync(Thandle, Tparam);
#if defined _WINNT || defined _WIN95 || defined _MAC_LVR || defined _MACINTOSH || defined _SOLARIS || defined _MACOSX || defined _LINUX /* LVR 04-Sep-96 */
Terr EXPORT_FN SNP24_set_timer(Thandle, Tparam, ui32, Terr (*)(Thandle) );
#else
Terr EXPORT_FN SNP24_set_timer(Thandle, Tparam, ui32, Terr (EXPORT_FN *)(Thandle) );
#endif

Terr EXPORT_FN SNP24_set_TMG_format(Thandle, ui16);
Terr EXPORT_FN SNP24_set_trigger(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_TTL422(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_video_standard(Thandle, Tparam);
Terr EXPORT_FN SNP24_set_video_src(Thandle, Tparam, int);

Terr EXPORT_FN SNP24_settings_implement( Thandle hSnapper, struct InitTag* pstTag );
Terr EXPORT_FN SNP24_settings_load(Thandle, ui32, const void *);

Terr EXPORT_FN SNP24_update_ctrlout(Thandle);
Terr EXPORT_FN SNP24_update_sync_source(Thandle);
Terr EXPORT_FN SNP24_update_trigger_source(Thandle);

Terr EXPORT_FN SNP24_write_snp24_reg(Thandle, Tparam);

#ifdef __cplusplus
};
#endif

/* EXTERNS
 * -------
 */

/* Global Array of SNP24 Handles
 * -----------------------------
 */
extern struct Tsnp24 *Psnp24_array[SNP24_NUM_HANDLES+1];

/* FPGA configuration data
 * -----------------------
 */
extern ui8  FAR fpga_b1_data[];
extern ui32 FAR fpga_b1_bit_length;
extern char FAR fpga_b1_date[];

#endif /* _SNP24PRO_H_ */
