/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : tmg_pro.h                                               *
 *  Function    : Prototype definitions for TMG library functions         *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Version     : 3.10                                                    *
 *  Release Date: 15-Jan-96                                               *
 *                                                                        *
 *  Copyright (c) 1993-96 Active Silicon Ltd, Active Imaging Ltd.         *
 **************************************************************************
 * Comments:
 * --------
 * "non-doc" means undocumented function that may be used occasionally in
 * applications.
 * "non-doc/user" means undocumented and not intended to be used outside the
 * libraries.
 * Note all the functions required by Snapper libraries are in tmg_snp.c, but
 * the functions are still grouped logically in this header file.
 *
 * File History
 * ------------
 * 11-Mar-97, HCP, "#ifndef Thandle" section added so that the file will
 *                 compile independently (of Snapper library).
 *
 */

#ifndef _TMG_PRO_H_
#define _TMG_PRO_H_

/* NOTE: Workaround for Snapper until "asl_types.h" */
#if !defined(_SOLARIS) && !defined(__WATCOMC__) && !defined (_LYNXOS) && !defined (_VXWORKS) && !defined _MAC_LVR && !defined (_MACINTOSH) && !defined (_MACOSX) && !defined(_LINUX)
typedef ui32 Thandle;
typedef ui32 Terr;
typedef ui32 Tboolean;
typedef ui32 Tdisplay_handle;
typedef ui32 Tparam;
#ifndef EXPORT_FN_PTR
#define EXPORT_FN_PTR  __declspec(dllexport)*
#endif
#endif



/*
 * Prototype Definitions
 * =====================
 */
#ifdef __cplusplus
extern "C" {
#endif

/*
 * TMG_cnv1.c - TMG image conversion routines - non-YUV422
 * ----------
 */
Terr EXPORT_FN TMG_image_convert(Thandle, Thandle, ui16, ui32, ui16);
Terr EXPORT_FN TMG_convert_to_paletted_LUT(Thandle, Thandle, ui16); /* non-doc */
Terr EXPORT_FN TMG_convert_to_RGB24(Thandle, Thandle, ui16);        /* non-doc */
Terr TMG_convert_to_RGB24_1_strip(Thandle, Thandle);                /* non-doc/user */
Terr EXPORT_FN TMG_convert_to_RGBX32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_convert_to_BGRX32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_convert_to_XBGR32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_convert_to_XRGB32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_convert_to_BGR24(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_convert_to_RGB16(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_convert_to_RGB15(Thandle, Thandle, ui16);        /* non-doc */
Terr TMG_convert_to_RGB15_1_strip(Thandle, Thandle);                /* non-doc/user */
Terr EXPORT_FN TMG_convert_to_RGB8(Thandle, Thandle, ui16);         /* non-doc */
Terr EXPORT_FN TMG_convert_to_RGB8_dither(Thandle, Thandle, ui16);  /* non-doc */
Terr EXPORT_FN TMG_paletted_to_RGB24_or_Y8(Thandle, Thandle, ui16); /* non-doc */
Terr EXPORT_FN TMG_convert_to_CMYK32(Thandle, Thandle, ui16);       /* non-doc */ 
Terr EXPORT_FN TMG_convert_to_Y8(Thandle, Thandle, ui16);           /* non-doc */
Terr EXPORT_FN TMG_convert_to_Y16(Thandle, Thandle, ui16);          /* non-doc */

/*
 * TMG_cnv2.c - TMG image conversion routines - YUV422
 * ----------
 */
Terr EXPORT_FN TMG_YUV422_to_HSI(Timage_handle Hin_image, Timage_handle Hout_image, ui16 TMG_action); /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGBX32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_BGRX32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_XBGR32(Thandle, Thandle, ui16);       /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_XRGB32(Timage_handle Hin_image, Timage_handle Hout_image, ui16 TMG_action);
Terr EXPORT_FN TMG_YUV422_to_RGB24(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_BGR24(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGB16(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGB15(Thandle, Thandle, ui16);        /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_paletted_LUT(Thandle, Thandle, ui16); /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGBX32_LUT(Thandle, Thandle, ui16);   /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_BGRX32_LUT(Thandle, Thandle, ui16);   /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_XBGR32_LUT(Thandle, Thandle, ui16);   /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_XRGB32_LUT(Timage_handle Hin_image, Timage_handle Hout_image, ui16 TMG_action);
Terr EXPORT_FN TMG_YUV422_to_RGB24_LUT(Thandle, Thandle, ui16);    /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_BGR24_LUT(Thandle, Thandle, ui16);    /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGB16_LUT(Thandle, Thandle, ui16);    /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_RGB15_LUT(Thandle, Thandle, ui16);    /* non-doc */
Terr EXPORT_FN TMG_YUV422_to_Y8(Thandle, Thandle, ui16);           /* non-doc */
Terr EXPORT_FN TMG_convert_to_YUV422(Thandle, Thandle, ui16);       /* non-doc */

/*
 * TMG_cnv3.c - TMG image conversion routines - LUT functions.
 * ----------
 */
Terr EXPORT_FN TMG_image_conv_LUT_generate(Thandle, ui16);
Terr EXPORT_FN TMG_image_conv_LUT_destroy(ui16);
Terr EXPORT_FN TMG_image_conv_LUT_load(Thandle, ui16, char*);
Terr EXPORT_FN TMG_image_conv_LUT_save(Thandle, ui16, char*);
Terr EXPORT_FN TMG_generate_RGB16_to_paletted_LUT(Thandle);  /* non-doc */
Terr EXPORT_FN TMG_generate_Y16_to_paletted_LUT(Thandle);    /* non-doc */
Terr EXPORT_FN TMG_generate_Y8_to_paletted_LUT(Thandle);     /* non-doc */
Terr EXPORT_FN TMG_destroy_RGB16_to_paletted_LUT(void);      /* non-doc */
Terr EXPORT_FN TMG_destroy_Y16_to_paletted_LUT(void);        /* non-doc */
Terr EXPORT_FN TMG_destroy_Y8_to_paletted_LUT(void);         /* non-doc */
Terr EXPORT_FN TMG_generate_YUV422_to_RGB16_LUT(void);       /* non-doc */
Terr EXPORT_FN TMG_generate_YUV422_to_RGB15_LUT(void);       /* non-doc */
Terr EXPORT_FN TMG_generate_YUV422_to_paletted_LUT(Thandle); /* non-doc */
Terr EXPORT_FN TMG_destroy_YUV422_to_RGB16_LUT(void);        /* non-doc */
Terr EXPORT_FN TMG_destroy_YUV422_to_RGB15_LUT(void);        /* non-doc */
Terr EXPORT_FN TMG_destroy_YUV422_to_paletted_LUT(void);     /* non-doc */

/*
 * tmg_crop.c - see also tmg_spl.c
 * ----------
 */
Terr EXPORT_FN TMG_IP_crop(Thandle, Thandle, i16[], ui16);

/*
 * TMG_dib.c - Windows 3.1 DIBs
 * ---------
 */
Terr EXPORT_FN TMG_convert_to_DIB_w31(Thandle, Thandle, ui32, ui16);   /* non-doc */
Terr EXPORT_FN TMG_read_BMP_file(Timage_handle, ui16);                 /* non-doc */
Terr EXPORT_FN TMG_write_BMP_file(Timage_handle, ui16);                /* non-doc */

/*
 * TMG_disp.c - TMG display routines
 * Including: tmg_w31.c, tmg_dos.c, tmg_sol.c, tmg_os2.c, tmg_nt.c
 * (and dci_asm.c)
 */
/* Generic to all operating systems */
/* -------------------------------- */
Terr     EXPORT_FN TMG_display_create(void);
Terr     EXPORT_FN TMG_display_destroy(Thandle);
Tboolean EXPORT_FN TMG_display_get_flags(Thandle, ui32);
Terr     EXPORT_FN TMG_display_get_parameter(Thandle, ui16);
Terr     EXPORT_FN TMG_display_get_ROI(Thandle, i16*);
Terr     EXPORT_FN TMG_display_image(Tdisplay_handle Hdisplay, Thandle Hin_image, ui16 TMG_action);
Terr     EXPORT_FN TMG_display_set_flags(Thandle, ui32, Tboolean);
Terr     EXPORT_FN TMG_display_set_parameter(Thandle, ui16, ui32);
Terr     EXPORT_FN TMG_display_set_ROI(Tdisplay_handle, Tparam, i16*);

#ifdef _TMG_FG_GRAPHICS   /* for DOS real mode and Symantec/Watcom protected mode */
Terr EXPORT_FN TMG_display_init(Tdisplay_handle, ui16);
Terr           TMG_DRV_display_image(Tdisplay_handle, Thandle, ui16);
Terr EXPORT_FN TMG_display_clear(Tdisplay_handle);
Terr EXPORT_FN TMG_display_box_fill(Tdisplay_handle, ui16, i16*);
Terr EXPORT_FN TMG_display_draw_text(Tdisplay_handle, char*, ui16, ui16);
Terr EXPORT_FN TMG_display_cmap_install(Tdisplay_handle, Thandle);
Terr EXPORT_FN TMG_display_cmap(Tdisplay_handle, Timage_handle, ui16);
Terr EXPORT_FN TMG_display_set_font(Tdisplay_handle, ui16);
#endif

#ifdef _TMG_X_GRAPHICS
Terr EXPORT_FN TMG_display_init(Thandle, ui16);
Terr EXPORT_FN TMG_display_cmap_install(Thandle, Thandle);
Terr EXPORT_FN TMG_display_clear(Thandle);
Terr EXPORT_FN TMG_DRV_display_image(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_display_set_Xid(Thandle, ui32, Window);
#endif

#ifdef _MACOSX
Terr EXPORT_FN TMG_display_init(Thandle, ui16);
Terr EXPORT_FN TMG_display_cmap_install(Thandle, Thandle);
Terr EXPORT_FN TMG_display_clear(Thandle);
Terr EXPORT_FN TMG_DRV_display_image(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_display_set_context(Thandle, CGLContextObj);
#endif

/*
 * Windows 3.1/95 display functions.  Note that _WINDOWS is also tested on -
 * this is so that pci_asm.c compiles under Symantec with the _WIN31 
 * directive without causing errors because HWND etc is not defined.
 */
#if defined _WINDOWS
#if defined _TMG_WIN31 || defined _TMG_WIN95 || defined _TMG_WINNT

/* User functions: */
Terr EXPORT_FN TMG_display_init(Tdisplay_handle, HWND);
Terr EXPORT_FN TMG_display_set_hWnd(Tdisplay_handle, HWND);
Terr EXPORT_FN TMG_display_set_paint_hDC(Tdisplay_handle, HDC);
HWND EXPORT_FN TMG_display_get_hWnd(Tdisplay_handle);
HDC  EXPORT_FN TMG_display_get_paint_hDC(Tdisplay_handle);
Terr EXPORT_FN TMG_DRV_display_image(Tdisplay_handle, Thandle, ui16);
Terr EXPORT_FN TMG_display_print_DIB(Tdisplay_handle, Thandle, i16, ui16);

/* Non-user functions: */
Terr EXPORT_FN TMG_display_DIB(Tdisplay_handle, Thandle, ui16);  /* non-doc */
Terr EXPORT_FN TMG_display_DDB(Tdisplay_handle, Thandle, ui16);  /* non-doc */

/* DCICopyNLines is in dci_asm.c for Windows 3.1 and tmg_dd.c for Windows 95 */
void far DCICopyNLines(short, long, long, long, long, short, short, long, short, short);

#if defined _WIN31
Terr EXPORT_FN TMG_display_direct_w31(Thandle, ui32, ui16, ui16, ui16, ui16, ui16); /* non-user */
HWINWATCH      TMG_display_get_DCIProvider(Thandle);             /* non-doc */
Terr EXPORT_FN TMG_display_DCI_w31(Thandle, Thandle, ui16);      /* non-doc */
#endif

#if defined _WIN95 || defined _WINNT
/* Display function that uses Direct Draw under Windows 95. */
Terr EXPORT_FN TMG_display_direct_draw_w95(Thandle, Thandle, ui16 ); /* non-doc */
#endif

#endif  /* Windows 3.1/95 specific display functions */
#endif  /* _WINDOWS */

#if defined _OS2  /* OS/2 display functions */
HWND EXPORT_FN TMG_display_get_hWnd(Tdisplay_handle);
Terr EXPORT_FN TMG_display_init(Tdisplay_handle, HWND);
Terr EXPORT_FN TMG_DRV_display_image(Tdisplay_handle, Thandle, ui16);
Terr EXPORT_FN TMG_display_set_paint(Tdisplay_handle);
#endif  /* OS/2 specific display functions */

#if defined _MAC_LVR || defined _MACINTOSH  /* MacOS display functions */
Terr EXPORT_FN TMG_display_init(Tdisplay_handle, PixMapHandle);
Terr EXPORT_FN TMG_DRV_display_image(Tdisplay_handle, Thandle, ui16);
Terr EXPORT_FN TMG_display_set_mask(Tdisplay_handle, RgnHandle);
#endif  /* MacOS specific display functions */

/*
 * TMG_eps.c - TMG encapsulated PostScript routines.
 * ---------
 */
Terr EXPORT_FN TMG_write_EPS_file(Thandle, ui16);
Terr EXPORT_FN TMG_read_EPS_file(Thandle, ui16);

/* 
 * TMG_gen.c - General Image Processing Routines
 * ---------
 */
#if defined _TMG_WIN32 || defined _MAC_LVR || defined _MACINTOSH || defined _MACOSX
ui32 EXPORT_FN TMG_ErrHandlerInstall(void (*pFnHandler)(ui32, char*, ui32, char*));
#else /* _TMG_DOS32 & others */
ui32 EXPORT_FN TMG_ErrHandlerInstall(void (EXPORT_FN *pFnHandler)(ui32, char*, ui32, char*));
#endif


Terr     EXPORT_FN     TMG_cmap_copy(Thandle, Thandle);
Terr     EXPORT_FN     TMG_image_calc_skip_bytes(ui32, ui32, ui32);
Terr     EXPORT_FN     TMG_image_calc_total_strips(Thandle);
Terr     EXPORT_FN     TMG_image_check(Thandle);
Terr     EXPORT_FN     TMG_image_copy(Thandle, Thandle);
Terr     EXPORT_FN     TMG_image_copy_parameters(Thandle, Thandle);
Thandle  EXPORT_FN     TMG_image_create(void);
Terr     EXPORT_FN     TMG_image_destroy(Thandle);
Terr     EXPORT_FN     TMG_image_free_data(Thandle);
ui32     EXPORT_FN     TMG_image_get_parameter(Thandle, ui16);
void     EXPORT_FN_PTR TMG_image_get_ptr(Thandle, ui16);
Terr     EXPORT_FN     TMG_image_get_flags(Thandle, ui32);
char FAR EXPORT_FN_PTR TMG_image_get_infilename(Thandle);
char FAR EXPORT_FN_PTR TMG_image_get_outfilename(Thandle);
Terr     EXPORT_FN     TMG_image_malloc_a_strip(Thandle);
Terr     EXPORT_FN     TMG_image_move(Thandle, Thandle);
Terr     EXPORT_FN     TMG_image_set_flags(Thandle, ui32, Tboolean);
Terr     EXPORT_FN     TMG_image_set_infilename(Thandle, char*);
Terr     EXPORT_FN     TMG_image_set_outfilename(Thandle, char*);
Terr     EXPORT_FN     TMG_image_set_parameter(Thandle, ui16, ui32);
Terr     EXPORT_FN     TMG_image_set_ptr(Thandle, ui16, void*);

/* one line functions - we'll keep for now */
Terr     EXPORT_FN     TMG_get_width(Thandle);            /* non-user/doc */
Terr     EXPORT_FN     TMG_get_height(Thandle);           /* non-user/doc */
Terr     EXPORT_FN     TMG_get_depth(Thandle);            /* non-user/doc */
Terr     EXPORT_FN     TMG_get_lines_this_strip(Thandle); /* non-user/doc */
Terr     EXPORT_FN     TMG_get_bytes_per_line(Thandle);   /* non-user/doc */

Terr     EXPORT_FN     TMG_write_to_memory(Thandle, Thandle, ui16);  /* non-doc */
Terr     EXPORT_FN     TMG_read_from_memory(Thandle, Thandle, ui16); /* non-doc */

/* internal non-user functions */
#if defined _MAC_LVR || defined _MACINTOSH
EXPORT_FN struct Tcmap* TMG_image_create_cmap(void);           /* non-user/doc */
#else
struct Tcmap EXPORT_FN_PTR TMG_image_create_cmap(void);        /* non-user/doc */
#endif
Terr     EXPORT_FN     TMG_copy_outfilename(Thandle, Thandle); /* non-user/doc */
IM_UI8   EXPORT_FN_PTR TMG_malloc_N_lines(Thandle, ui16);      /* non-user/doc */

/*
 * TMG_jpg.c
 * ---------
 */
Terr     EXPORT_FN     TMG_JPEG_build_image(Thandle, Thandle, ui16);
Terr     EXPORT_FN     TMG_JPEG_copy_parameters(Thandle, Thandle); /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_destroy_Tjpeg(Thandle);            /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_file_close(Thandle);
Terr     EXPORT_FN     TMG_JPEG_file_open(Thandle);
Terr     EXPORT_FN     TMG_JPEG_file_read(Thandle);
Terr     EXPORT_FN     TMG_JPEG_buffer_read(Thandle Himage, ui8 *pbData, ui32 dwBytesData);
Terr     EXPORT_FN     TMG_JPEG_file_write(Thandle, ui16);
Terr     EXPORT_FN     TMG_JPEG_buffer_write(Timage_handle Hjpeg_image, ui8 *pdData, ui32 *pdwCount, ui16 TMG_action);
Terr     EXPORT_FN     TMG_JPEG_free_data(Thandle);            /* non-doc/user */
FILE     EXPORT_FN_PTR TMG_JPEG_get_file_ptr(Thandle);         /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_get_parameter(Thandle, ui16);  /* non-doc/user */
Thandle  EXPORT_FN     TMG_JPEG_image_create(void);
Terr     EXPORT_FN     TMG_JPEG_malloc_data(Thandle);               /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_malloc_N_bytes_data(Thandle, ui32); /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_sequence_build(Thandle, Thandle);
Terr     EXPORT_FN     TMG_JPEG_sequence_calc_length(Thandle);
Terr     EXPORT_FN     TMG_JPEG_sequence_extract_frame(Thandle, Thandle, ui32);
Terr     EXPORT_FN     TMG_JPEG_sequence_set_start_frame(Thandle, ui32);
Terr     EXPORT_FN     TMG_JPEG_set_file_ptr(Thandle, FILE*);  /* non-doc/user */
Terr     EXPORT_FN     TMG_JPEG_set_image(Thandle, Thandle);

/*
 * TMG_jpeg.c
 * ----------
 */
Terr EXPORT_FN TMG_JPEG_compress_image_to_image(Thandle, Thandle, ui16, ui16);
Terr EXPORT_FN TMG_JPEG_compress(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_JPEG_decompress_image_to_image(Thandle, Thandle, ui16, ui16);
Terr EXPORT_FN TMG_JPEG_decompress(Timage_handle, Thandle, ui16);
Terr EXPORT_FN TMG_JPEG_set_Quality_factor(Thandle, ui16);
Terr EXPORT_FN TMG_JPEG_set_Quantization_factor(Thandle, ui16);
Terr EXPORT_FN TMG_JPEG_make_Q_tables(Thandle);                  /* non-doc/user */
Terr EXPORT_FN TMG_JPEG_set_default_H_tables(Thandle);           /* non-doc/user */
Terr EXPORT_FN TMG_JPEG_generate_H_codetable_LUTs(Thandle);      /* non-doc/user */
Terr EXPORT_FN TMG_JPEG_generate_Huffman_decode_tables(Thandle); /* non-doc/user */

/*
 * TMG_cmap.c - Optimum palette generation.
 * ----------
 */
#if defined _MAC_LVR || defined _MACINTOSH
EXPORT_FN struct Tcmap_info* TMG_cmap_create_info(void);             /* non-doc/user */
#else
struct Tcmap_info EXPORT_FN_PTR TMG_cmap_create_info(void);          /* non-doc/user */
#endif
Terr              EXPORT_FN     TMG_cmap_destroy_info(Thandle);      /* non-doc/user */
Terr     EXPORT_FN  TMG_convert_to_paletted(Thandle, Thandle, ui16); /* non-doc/user */
Terr     EXPORT_FN  TMG_cmap_generate(Thandle, ui16, ui16);
ui32     EXPORT_FN  TMG_cmap_get_occurrences(Thandle, ui16);
ui32     EXPORT_FN  TMG_cmap_get_RGB_colour(Thandle, ui16);
ui8      EXPORT_FN  TMG_cmap_find_closest_colour(Thandle, ui8, ui8, ui8);
/*Terr     EXPORT_FN  TMG_cmap_generate_cmap_info(Thandle);*/
Tboolean EXPORT_FN  TMG_cmap_is_grayscale(Thandle);
Terr     EXPORT_FN  TMG_cmap_set_colour(Thandle, ui16, ui16);
Terr     EXPORT_FN  TMG_cmap_set_RGB_colour(Thandle, ui16, ui8, ui8, ui8);
Terr     EXPORT_FN  TMG_cmap_set_type(Thandle, ui16);
Tboolean EXPORT_FN  TMG_image_is_colour(Thandle);

/*
 * TMG_scl.c - Image scaling and sub-sampling routines.
 * ----------
 */
Terr EXPORT_FN TMG_IP_subsample(Thandle, Thandle, ui16, ui16);
Terr EXPORT_FN TMG_IP_pixel_rep(Thandle, Thandle, ui16, ui16);

/*
 * TMG_raw.c - Raw data write & read.
 * ---------
 */
Terr EXPORT_FN TMG_write_raw_data_file(Thandle, ui16);

/*
 * TMG_rot.c - TMG image rotation.
 * ---------
 */
Terr EXPORT_FN TMG_image_rotate(Thandle, Thandle, ui16, ui16); /* Note: not in current libraries */

/*
 * TMG_rw.c - TMG image read/write wrapper.
 * --------
 */
Terr EXPORT_FN TMG_image_find_file_format(char*);
Terr EXPORT_FN TMG_image_read(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_image_write(Thandle, Thandle, ui16, ui16);

/*
 * TMG_spl.c - TMG special functions and image processing functions.
 * ---------
 */
Terr EXPORT_FN TMG_IP_extract_region(Thandle Hin_image, Thandle Hout_image, ui32 dwRegionType, ui32 dwX, ui32 dwY, ui32 dwRadius, ui16 TMG_action);
Terr EXPORT_FN TMG_IP_filter_3x3(Thandle, Thandle, i16*, ui16);
Terr EXPORT_FN TMG_IP_histogram_filter(struct tTMG_Histogram *psInHistogram, i32 nFilterOrder);
Terr EXPORT_FN TMG_IP_histogram_generate(Thandle Hin_image, struct tTMG_Histogram *psHistogram, ui16 TMG_action);
Terr EXPORT_FN TMG_IP_histogram_match(struct tTMG_Histogram *psRefHistogram, struct tTMG_Histogram *psInHistogram, i32 n32Plane, ui32 *pdwResult);
Terr EXPORT_FN TMG_IP_histogram_clear(struct tTMG_Histogram *psHistogram);
Terr EXPORT_FN TMG_IP_histogram_copy(struct tTMG_Histogram *psHistogramIn, struct tTMG_Histogram *psHistogramOut);
Terr EXPORT_FN TMG_IP_generate_averages(Thandle Hin_image, struct tTMG_Averages *psAverages, ui16 TMG_action);
Terr EXPORT_FN TMG_IP_mirror_image(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_IP_rotate_image(Thandle, Thandle, ui32);
Terr EXPORT_FN TMG_IP_scale(Thandle Hin_image, Thandle Hout_image, ui32 dwNewWidth, ui32 dwNewHeight, ui16 TMG_action);
Terr EXPORT_FN TMG_IP_threshold_grayscale(Thandle, Thandle, ui8, ui8, ui16);
Terr EXPORT_FN TMG_SPL_2fields_to_frame(Thandle, Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_SPL_Data32_to_Y8(Thandle Hin_image, Thandle Hout_image, ui16 wShiftRight, ui16 TMG_action);
Terr EXPORT_FN TMG_SPL_field_to_frame(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_SPL_interlace_frame(Thandle, Thandle, ui16);
Terr EXPORT_FN TMG_SPL_HSI_to_RGB_pseudo_colour(Thandle Hin_image, Thandle Hout_image, ui16 TMG_action);
Terr EXPORT_FN TMG_SPL_YUV422_to_RGB_pseudo_colour(Thandle Hin_image, Thandle Hout_image, ui16 TMG_action);
Terr EXPORT_FN TMG_SPL_XXXX32_to_Y8(Thandle, Thandle, ui16, ui16);
Terr EXPORT_FN TMG_SPL_Y8_to_pseudo_colour(Thandle, Thandle, ui16, ui16);

/*
 * TMG_test.c - Various development test programs.
 * ----------
 */
Terr  EXPORT_FN  TMG_print_Huffman_tables(Thandle);  /* non-doc/user */
Terr  EXPORT_FN  TMG_print_image(Thandle, char*);    /* non-doc/user */
Terr  EXPORT_FN  TMG_print_cmap(Thandle, char*);     /* non-doc/user */
Terr  EXPORT_FN  TMG_print_jpeg_image(Thandle);      /* non-doc/user */
Terr  EXPORT_FN  TMG_test_pattern(Thandle, ui16);    /* non-doc/user */

/*
 * TMG_tga.c - TGA reader and writer
 * ---------
 */
Terr EXPORT_FN TMG_write_TGA_file(Thandle, ui16);  /* non-doc */
Terr EXPORT_FN TMG_read_TGA_file(Thandle, ui16);   /* non-doc */

/*
 * TMG_tiff.c - TIFF reader and writer
 * ----------
 */
Terr EXPORT_FN TMG_write_TIFF_file(Thandle, ui16); /* non-doc */
Terr EXPORT_FN TMG_read_TIFF_file(Thandle, ui16);  /* non-doc */
ui16 EXPORT_FN TIFF_read_ui16(FILE*, ui16);        /* non-doc/user */
ui32 EXPORT_FN TIFF_read_ui32(FILE*, ui16);        /* non-doc/user */
ui32 EXPORT_FN TIFF_read_data(FILE*, ui16, ui16);  /* non-doc/user */

/*
 * TMG_chrm.c - Chroma keying and related functions.
 * ----------
 */
Thandle EXPORT_FN TMG_CK_create(void);
Terr    EXPORT_FN TMG_CK_destroy(Thandle);
Terr    EXPORT_FN TMG_CK_chroma_key(Thandle, Thandle, Thandle, ui16, Thandle, ui16, ui16);
Terr    EXPORT_FN TMG_CK_calibrate(Thandle, Thandle, ui16);
Terr    EXPORT_FN TMG_CK_set_parameter(Thandle, ui16, ui16);
Terr    EXPORT_FN TMG_CK_get_parameter(Thandle, ui16);
ui8     EXPORT_FN TMG_CK_get_component(ui16, ui16);
ui32    EXPORT_FN TMG_CK_get_YUV_values(ui16);
ui32    EXPORT_FN TMG_CK_get_YUV_values_RGB(ui16, ui16, ui16);
Terr    EXPORT_FN TMG_CK_generate_UV_to_hue_LUT(void);
Terr    EXPORT_FN TMG_CK_destroy_UV_to_hue_LUT(void);

/*
 * TMG_lut.c - LUT image editing functions - e.g. contrast, gamma etc.
 * ---------
 */
Thandle EXPORT_FN     TMG_LUT_create(ui32, ui16);
Terr    EXPORT_FN     TMG_LUT_destroy(Thandle);
Terr    EXPORT_FN     TMG_LUT_apply(Thandle, Thandle, Thandle, ui16);
Terr    EXPORT_FN     TMG_LUT_generate(Thandle, i16, i16, i16, i16, i16, i16);
void    EXPORT_FN_PTR TMG_LUT_get_ptr(Thandle, ui16);

#ifdef __cplusplus
};
#endif

#ifndef fileno
#define fileno(fp)   ((fp)->_file)
#endif


/*
 * Externals Definitions
 * ---------------------
 * Global Array of Snapper Handles
 */
#if defined _MAC_LVR
/* LVR 04-Sep-96: Made global data into EXPORT_DATA 'cos TMG shares 
 *them with SnapperLib.
 */
extern EXPORT_DATA struct Timage FAR *Pimage_array[];
extern EXPORT_DATA struct Ttmg_LUT FAR* pLUT_array[];
extern EXPORT_DATA struct Tdisplay FAR* Pdisplay_array[];
extern EXPORT_DATA struct Tchroma_key FAR* Pchroma_key_array[];

/*
 * YUV to 16 bit RGB LUT for fast colour display.
 * And RGB16/Y8 to paletted for fast paletted display.
 */
extern EXPORT_DATA ui8  FAR *RGB16_to_paletted_LUT;
extern EXPORT_DATA ui8  FAR *Y8_to_paletted_LUT;
extern EXPORT_DATA ui8  FAR *Y16_to_paletted_LUT;
extern EXPORT_DATA ui8  FAR *YUV_to_paletted_LUT;
extern EXPORT_DATA ui16 FAR *YUV_to_RGB16_LUT;
extern EXPORT_DATA ui16 FAR *YUV_to_RGB15_LUT;

#else

extern struct Timage FAR *Pimage_array[];
extern struct Ttmg_LUT FAR* pLUT_array[];
extern struct Tdisplay FAR* Pdisplay_array[];
extern struct Tchroma_key FAR* Pchroma_key_array[];

/*
 * YUV to 16 bit RGB LUT for fast colour display.
 * And RGB16/Y8 to paletted for fast paletted display.
 */
extern ui8  FAR *RGB16_to_paletted_LUT;
extern ui8  FAR *Y8_to_paletted_LUT;
extern ui8  FAR *Y16_to_paletted_LUT;
extern ui8  FAR *YUV_to_paletted_LUT;
extern ui16 FAR *YUV_to_RGB16_LUT;
extern ui16 FAR *YUV_to_RGB15_LUT;

#endif  /* else part of #ifdef _MAC_LVR */


/*
 * UV to hue (angle) LUT.
 */
#ifdef _CPU16
extern ui8 FAR *UV_to_hue_LUT;
#elif defined _MAC_LVR
extern EXPORT_DATA ui16 *UV_to_hue_LUT;
#else
extern ui16 *UV_to_hue_LUT;
#endif

/*
 * Software JPEG Huffman decode LUTs (used in tmg_jpeg.c).
 */
#ifdef _WIN31
extern struct Tdecodetable __huge *AC_Y_decode;
extern struct Tdecodetable __huge *AC_C_decode;
#elif defined _MAC_LVR
extern EXPORT_DATA struct Tdecodetable *AC_Y_decode;
extern EXPORT_DATA struct Tdecodetable *AC_C_decode;
#else
extern struct Tdecodetable *AC_Y_decode;
extern struct Tdecodetable *AC_C_decode;
#endif

/*
 * Malloc/free checking.
 */
#ifdef _MALLOC_DEBUG
extern ui16 ASL_mallocs;
extern ui16 ASL_frees;
extern float ASL_total_mallocs;
#endif

#endif  /* _TMG_PRO_H_ */

