/**************************************************************************
 *                ACTIVE SILICON LIMITED / ACTIVE IMAGING LIMITED         *
 *                                                                        *
 *  File name   : w95ioctl.h                                              *
 *  Function    : Definitions for Windows 95 Device I/O driver calls      *
 *  Project     : Snappers                                                *
 *  Author      : Colin Pearce, Active Silicon Ltd                        *
 *  Systems     : ANSI C                                                  *
 *  Written     : February 1996                                           *
 *  Updated     : 25-Feb-96                                               *
 *  Version     : 3.12                                                    *
 *                                                                        *
 *  Copyright (c) 1996 Active Silicon, Active Imaging Ltd.                *
 **************************************************************************
 *  Comments:
 */

#ifndef _w95ioctl_h
#define _w95ioctl_h

/*
 * These "Microsoft" definitions are duplicated so that the VxD doesn't 
 * need extra include files. If the system file, winioctl.h, is included,
 * it does mean that it should be included before this file.
 * NOTE:HCP: Probably don't need these if we include <devioctl> in driver.
 */
#ifndef CTL_CODE
#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)
#endif

#ifndef METHOD_BUFFERED
#define METHOD_BUFFERED       0
#define METHOD_IN_DIRECT      1
#define METHOD_OUT_DIRECT     2
#define METHOD_NEITHER        3
#endif

#ifndef FILE_ANY_ACCESS
#define FILE_ANY_ACCESS           0
#define FILE_READ_ACCESS   ( 0x0001 )    // file & pipe
#define FILE_WRITE_ACCESS  ( 0x0002 )    // file & pipe
#endif


/*
 * IOCTL Codes
 * -----------
 * Function codes above 32768 (0x8000) are not affected by winioctl.h
 * conventions.
 * Note: They are in the order in which there were written and tested.
 * This ordering may be useful for programmers doing operating system
 * ports etc.
 */

/* Device type - in the "User Defined" range." */
#define FILE_DEVICE_SNAPPER  0xACE1


#define IOCTL_SNAPPER_OPEN_DEVICE \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x901, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI8 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x902, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_READ_UI8 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x903, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x904, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_READ_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x905, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x906, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_READ_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x907, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_DYLAN_WRITE_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x908, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_DYLAN_READ_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x909, METHOD_BUFFERED, FILE_READ_ACCESS )

#define IOCTL_SNAPPER_FPGA_SEND_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90A, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_FPGA_SEND_DATA \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90B, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_FPGA_RESET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90C, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_FPGA_GET_CONFIG_STATUS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90D, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90E, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI32 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x90F, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI32_ASYNC \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x910, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_READ_COMPLETE \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x911, METHOD_IN_DIRECT, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_BURST_IN_UI16_SPECIAL \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x912, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_BURST_OUT_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x913, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_TABLE_LOAD \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x914, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_PAGE_WRITE_UI16 \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x915, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_BURST_OUT_UI16_SPECIAL \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x916, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_GET_ERROR_STRING \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x917, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_ABORT_OPERATION \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x918, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_TIMEOUT_SET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x919, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_SERIAL_TX \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91A, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_SERIAL_RX \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91B, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_SERIAL_SEND_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91C, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_SERIAL_GET_INFO \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91D, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_INTERRUPT_SET \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91E, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#define IOCTL_SNAPPER_INTERRUPT_STATUS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91F, METHOD_BUFFERED, FILE_ANY_ACCESS )

#define IOCTL_SNAPPER_WRITE_UI8_ABS \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x920, METHOD_BUFFERED, FILE_WRITE_ACCESS )

#ifdef NEVERNEVER   /* This ioctl is not used */
#define IOCTL_SNAPPER_GETTIMESTAMP \
   CTL_CODE( FILE_DEVICE_SNAPPER, 0x91F, METHOD_BUFFERED, FILE_WRITE_ACCESS )
#endif

#endif  /* _w95ioctl_h */
