/*************************************************************************
 *                          ACTIVE SILICON LIMITED                        
 *
 *  File name   : xpl_gui.h
 *  Project     : ASLexamp core example code GUI module
 *  Author      : Mike Harrison, Martin Bone
 *  Systems     : ANSI C, WIN32, X, DOS
 *  Written     : 24-FEB-1998
 *  Updated     :
 *  Version     : 1:00:00                                                     
 *                                                                        
 *  Copyright   : Active Silicon 1998
 **************************************************************************
 * Comments: Header file for OS-dependent GUI operations supporting the 
 *           Snapper example code modules.
 * Assumptions:
 * Revisions:
 */

#if ! defined _XPL_GUI_DEFINED
#define _XPL_GUI_DEFINED 

#if defined _WIN95 || defined _WINNT
/* Unmask one of these three to select a display method for Win32 display */
/*#define _WIN32_DISPLAY_DIB */
/*#define _WIN32_DISPLAY_DDB */
#define _WIN32_DISPLAY_DDI 

#if (! defined _WIN32_DISPLAY_DDB) && (! defined _WIN32_DISPLAY_DIB) && (! defined _WIN32_DISPLAY_DDI)
#error 'Windows image display method not specified' 
#endif
#endif


/* Default invalid settings */
#define GUI_INVALID_GC      ((ui32) 0)
#define GUI_INVALID_DISPLAY ((ui32) 0)
#define GUI_INVALID_WINDOW  ((ui32) 0)

#define GUI_INVALID_CMD     ((ui32) 0)
#define GUI_QUIT            ((ui32) 1)
#define GUI_INVALID_PARAM   ((ui32) 0)
#define EVENT_DEFINED

/* Window size depends on video mode and field sampling mode.
These constants are divided by 1, 2 and 4 as required. */
#define _EIA_WIN_MIN_WIDTH   640
#define _EIA_WIN_MIN_HEIGHT  480
#define _CCIR_WIN_MIN_WIDTH  768
#define _CCIR_WIN_MIN_HEIGHT 576
#define DEFAULT_HW_SUBSAMPLE  2

/* Default GUI window settings */
#define BORDER_WIDTH 5
#define WIN_X_ROOT 10
#define WIN_Y_ROOT 10
#define WIN_MIN_WIDTH  (_EIA_WIN_MIN_WIDTH/DEFAULT_HW_SUBSAMPLE) 
#define WIN_MIN_HEIGHT (_EIA_WIN_MIN_HEIGHT/DEFAULT_HW_SUBSAMPLE) 


/* Define default image array size */
#define NUM_VID_IMAGES 4

/* Command structure to contain command line switch information */
struct Cmd
{
   ui32       dwCmd;
   char       szParam[MAX_FIELD_LENGTH];
   struct Cmd *pscNextCmd;
};

/* Event structure to contain event information translated from OS specific messages to 
 * events our program can understand
 */
struct Event
{
   ui32          dwCmd;
   ui32          dwParam;
   struct Event* pseRoot;
   struct Event* pseNextEvent;
};

/* Wrap the TMG images in a structure that allows us 
 * to test the buffers for full and empty status.
 */
struct Image
{
   Thandle hVidImage;
   /* Store byte offset of image data area from start of large single shared memory block 
    * for use in SG table.
    */
   ui32 dwByteOffset;   
   /* store byte transfer size (ie image size) for use with SG table.*/
   ui32 dwByteTransferSize;

   ui32 bFull;
   ui32 bEmpty;
};



#if defined _X_GRAPHICS
/* X defined & used in some string definitions */
#define _MAX_MAPPED_STRING_LENGTH MAX_FIELD_LENGTH
#endif

struct GuiInfo
{
   /* Program names */
   char szProgName[MAX_FIELD_LENGTH];

   /* Window sizing information to pass to the GUI layer */
   ui32 dwWinWidth;
   ui32 dwWinHeight;

   Thandle hTempImage; /* Temporary image used for format conversions */

#if defined _CONSOLE_ONLY
#define GUI_FILENO STDIN_FILENO
   int nAsyncFileNo;

#elif defined _MACINTOSH
   WindowPtr   pWin;     /* Ptr to window list entry */
   Rect        windRect;   /* Window size parameters */

#elif defined _X_GRAPHICS
   Display *pdDisplay;
   Window wWin;      /* Window handle */
   i32 iScreenNum; 

   /* Graphics context & display status variables */
   GC gcView;
   XFontStruct *xfsFont;

#elif defined _WIN95 || defined _WINNT
   HWND      hWnd;     /* Window handle */
   HANDLE  hInstance;  /* Instance handle */
     
   /* Graphics context & display status variables */

#else
#if! defined __MWERKS__
#error 'No OS platform defined for GUI'
#endif
#endif


   /* TMG image handles
    * The hVidImage array stores an number of image handles, which can be
    * used for general image acquisition or as temporary buffers during
    * format conversion
    */
   struct Image sVidImage[NUM_VID_IMAGES];
   ui32 hPalettedImage;
   ui32 hDisplay;

   /* TMG display format - not strictly necessary to save because available by TMG call  */
   ui16 wDisplayDepth;
   ui16 wDisplayFormat;

   /* If TRUE then the acquired image format is not the same as the display image format */
   Tboolean fConvertForDisplay;
};



#endif
