/*************************************************************************
 *                          ACTIVE SILICON LIMITED                      
 *                                                                        
 *  File name   : xpl_gui.c
 *  Project     : ASL maintained example code OS-dependent Gui interface
 *  Author      : Mike Harrison, Martin Bone
 *  Systems     : ANSI C, X Windows, MS-Windows
 *  Written     : 25-Feb-98
 *  Updated     :
 *  Version     : 1:00:00
 *
 *  Copyright   : ASL
 **************************************************************************
 * Comments: Refer to Xlib Programming Manual chapter 3.0, pp66 & on for
 *           X11 R4 source code discussion.
 *           Refer to Petzold 'Programming Windows3.1' for Win32
 *           source code discussion.
 *           Refer to 'Ultimate Mac Programming' for Mac code discussion.
 * Assumptions:
 * Comments :
 * Revisions:
 */

/* Active Silicon Include files - pulls in X includes as required */
#include <asl_inc.h>
#include <xpl_inc.h>


/* Any ANSI-C console OS. eg Dos, Unix, Lynx. */
#if defined _CONSOLE_ONLY
/*
ConsoleOnly (ForBrief)
**************************************************************************
*/

/* Local function prototypes */
ui32 GUI_AsyncIoOpen  ( struct GuiInfo *psGui );
ui32 GUI_AsyncIoGet   ( struct GuiInfo *psGui, char *pcInputChar );
ui32 GUI_AsyncIoClose ( struct GuiInfo *psGui );


/* Asynchronous I/O Handling Routines
 * **********************************
 */
/* Open terminal for Async IO */
ui32 GUI_AsyncIoOpen(struct GuiInfo *psGui)
{
   static char *szFnName = "(Console)GUI_AsyncIoOpen";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

#if defined _SOLARIS2 || defined _LYNXOS || defined _LINUX
   psGui->nAsyncFileNo = open ( "/dev/tty", O_RDONLY | O_NONBLOCK );
   if ( psGui->nAsyncFileNo < 0 )
   {
      DBG_WriteString( 10, "\n%s: Couldn't open /dev/tty/", szFnName);
      return -1;
   }

   {
      struct termios attr;

      tcgetattr( psGui->nAsyncFileNo,  &attr);
      attr.c_lflag &= ~(ICANON);
      attr.c_cc[VMIN] = 1;
      tcsetattr( GUI_FILENO, TCSANOW, &attr);
   }
#endif

   return ASL_OK;
}


/* Close terminal after Async IO */
ui32 GUI_AsyncIoClose(struct GuiInfo *psGui)
{
   static char *szFnName = "(Console)GUI_AsyncIoClose";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

#if defined _SOLARIS2 || defined _LYNXOS || defined _LINUX
   close( psGui->nAsyncFileNo );
#endif

   return ASL_OK;
}


/* Get Async IO */
ui32 GUI_AsyncIoGet(struct GuiInfo *psGui, char *pcInputChar)
{
   static char *szFnName = "(Console)GUI_AsyncIoGet";
   ui32 dwStatus = FALSE;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

#if defined _DOS32 || defined _WIN95 || defined _WINNT
   if ( kbhit() )
   {
      *pcInputChar = (char) ASL_getch();
      dwStatus = TRUE;
   }
#elif defined _SOLARIS2 || defined _LYNXOS || defined _LINUX
   if ( read ( psGui->nAsyncFileNo, (void *) pcInputChar, 1) == 1)
   {
      dwStatus = TRUE;
      DBG_WriteString( 1, "\n%s: Found 0x%x, %c", szFnName, (int) *pcInputChar, *pcInputChar);
   }
#elif defined _MACINTOSH
   {
      EventRecord erNextEvent;
      WindowPtr theWindow;

      OS_process_yield();
      if (GetNextEvent(everyEvent, &erNextEvent))
      {
         switch (erNextEvent.what)
         {
            case autoKey:
            case keyDown:
               if (!(erNextEvent.modifiers & cmdKey))
               {
                  dwStatus = TRUE;
                  *pcInputChar = (char) (erNextEvent.message & charCodeMask);
               }
               break;

            case mouseDown:
               switch( FindWindow(erNextEvent.where, &theWindow) )
               {
                  /* Allow system events to be handled */
                  case inSysWindow:
                     SystemClick( &erNextEvent, theWindow );
                     break;

                  /* Allow Sioux window to be dragged around */
                  case inDrag:
                     DragWindow( theWindow, erNextEvent.where, &qd.screenBits.bounds );
                     break;

                  /* A mouse click in the Sioux window is equivalent to
                   * the user pressing Q
                   */
                  case inContent:
                     dwStatus = TRUE;
                     *pcInputChar = (char) 'Q';
                     break;

                  default:
                     break;
               }
               break;

            default:
               break;
         }
      }
   }
#endif

   return dwStatus;
}


/* Message Handling Routines
 * *************************
 */
ui32 GUI_CheckMessageQueue( struct GuiInfo *psGui, struct Event **pseCurrentEventList )
{
   static char *szFnName = "(Console)GUI_CheckMessageQueue";
   struct Event *pseNewEvent = NULL;
   struct Event *pseCurrentEventPtr = NULL;
   struct Event *pseCurrentEventRoot = NULL;
   struct Event  seCurrentEvent;
   char  cInputChar;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Open terminal for asynchronous input */
   GUI_AsyncIoOpen(psGui);

   while( GUI_AsyncIoGet(psGui, &cInputChar) )
   {
      /* set defaults */
      seCurrentEvent.dwCmd = GUI_INVALID_CMD;
      seCurrentEvent.dwParam = GUI_INVALID_PARAM;
      seCurrentEvent.pseNextEvent = NULL;

      switch ( (int) cInputChar )
      {
         case (int) 'Q':
         case (int) 'q':
            seCurrentEvent.dwCmd = 'Q';
            seCurrentEvent.dwParam = GUI_INVALID_PARAM;
            break;

         /* Add to handling chain here if required e.g re-sizing commands etc. */
         default:
            break;
      }

      if ( seCurrentEvent.dwCmd != GUI_INVALID_CMD )
      {
         /* Allocate an event structure */
         pseNewEvent = (struct Event *) calloc( sizeof ( struct Event ), (size_t) 1);
         if( pseNewEvent != NULL )
         {
            /* If previous event exists to add to ... */
            if( pseCurrentEventPtr != NULL )
            {
               /* Enter the event into the event list, adjust pointers and go round for next */
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;

               /* Update to current event */
               pseCurrentEventPtr->pseNextEvent = pseNewEvent;
               pseCurrentEventPtr = pseNewEvent;
            }
            else
            {
               /* Store root of the list */
               pseCurrentEventRoot = pseNewEvent;
               pseCurrentEventPtr = pseNewEvent;

               /* Initialise the root */
               pseCurrentEventPtr->pseNextEvent = NULL;
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;
            }
         }
         else
            teStatus = ASLERR_OUT_OF_MEMORY;
      } /* End of adding to list */
   } /* End of while */

   GUI_AsyncIoClose(psGui);
   *pseCurrentEventList = pseCurrentEventRoot;
   return teStatus;
}


/* Display Handling Routines
 * *************************
 */
ui32 GUI_DisplayClose( struct GuiInfo *psGui )
{
   static char *szFnName = "(Console)GUI_DisplayClose";
     
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   psGui->hDisplay = TMG_INVALID_HANDLE;

   return ASL_OK;
}


ui32 GUI_DisplayImage( struct GuiInfo *psGui, ui32 dwBuffer )
{
   static char *szFnName = "(Console)GUI_DisplayImage";
   static ui32 dwImageCount = 0;
     
   DBG_WriteString( 1,  "\n%s: Entering..", szFnName);
   DBG_WriteString( 10, "\n%s: Image display [ %4d ]", szFnName, dwImageCount++ );

   return ASL_OK;
}


ui32 GUI_DisplayInit( struct GuiInfo *psGui )
{
   static char *szFnName = "(Console)GUI_DisplayInit";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Nothing to do for console mode */
   psGui->hDisplay = TMG_INVALID_HANDLE;

   /* Required solely for data transfer by Snapper into a TMG image structure */
   psGui->wDisplayFormat = TMG_BGRX32; 
   psGui->wDisplayDepth = 0; 
   return ASL_OK;
}


/* Initialise display info prior to calling DisplayInit */
ui32 GUI_DisplayInitInfo( struct GuiInfo *psGui, char *pszProgName )
{
   static char *szFnName = "(Console)GUI_DisplayInitInfo";
   ui32 dwImageBuffer;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Set up default TMG variables */
   for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
      psGui->sVidImage[dwImageBuffer].hVidImage = TMG_INVALID_HANDLE;
   psGui->hDisplay     = TMG_INVALID_HANDLE;
   
   /* Initialise to a generally valid BASE_set_mapper format */
   psGui->wDisplayFormat = 0; 
   psGui->wDisplayDepth = 0; 

   _UNUSED_VAR( pszProgName );

   return ASL_OK;
}


/* Resize the Image window - As there is no image window in _CONSOLE_ONLY
 * mode, simply do nothing
 */
ui32 GUI_DisplayReSize( struct GuiInfo *psGui )
{
   static char *szFnName = "(Console)GUI_DisplayReSize";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   return ASL_OK;
}

/* End of console-only */


#elif (defined _WIN95 || defined _WINNT)
/*
Win32 (ForBrief)
**************************************************************************
*/
LRESULT CALLBACK GUI_WndProc(HWND, UINT, WPARAM, LPARAM);

/* This function must act as the message queue process
 * for any window. Instead of returning any messages to windows,
 * we translate as necessary ( noting that this reposts the message 
 * to the message queue and so we leave it) and act upon it.
 * Un-used mesages must be acted upon by the DefWindowProc
 * The GUI_WndProc is kept as a safety catch-all in case this
 * mechanism misses anything, the alternative is to set it 
 * to NULL or DefWindProc in DisplayInit.
 */
ui32 GUI_CheckMessageQueue( struct GuiInfo *psGui, struct Event **pseCurrentEventList )
{
   static char *szFnName = "(WIN32)GUI_CheckMessageQueue";
   Terr teStatus;
   ui32 dwMsgCount = 0;
   ui32 bStoreEvent = FALSE;
   ui32 dwImageCnt;
   Thandle hLocalImage;
   MSG mLocalMsg;
   int nMsgStatus;
   
   struct Event *pseNewEvent = NULL;
   struct Event *pseCurrentEventPtr = NULL;
   struct Event *pseCurrentEventRoot = NULL;
   struct Event  seCurrentEvent;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Check for the correct window size by examining the data associated 
    * with each image handle & resize window accordingly.
    */
   for ( dwImageCnt = 0; dwImageCnt < NUM_VID_IMAGES; dwImageCnt++)
   {
      hLocalImage = GUI_TmgGetImage( psGui, dwImageCnt );
      /* Get image size*/
      /* Find max window size required */
      /* resize current window to max window size */
   }
      
   /* defaults */
   teStatus = ASL_OK;
   
   /* Check for any user input - ie mouse or keystrokes.
    * Use peek because GetMessage blocks until one is available.
    */
   while ( PeekMessage( &mLocalMsg, psGui->hWnd, 0, 0, PM_NOREMOVE) == TRUE && !ASL_is_err(teStatus) )
   {      
      /* defaults */
      seCurrentEvent.dwCmd     = GUI_INVALID_CMD;
      seCurrentEvent.dwParam   = GUI_INVALID_PARAM;
      if ((nMsgStatus = GetMessage( &mLocalMsg, psGui->hWnd, 0, 0 )) == -1)
         teStatus =  ASL_ERROR;

      /* Try to translate message from virtual key to ASCII and place back
       * in the queue
       */
      else if ( TranslateMessage( &mLocalMsg ) )
         /* Message translated OK */
         teStatus = ASL_OK;

      /* Process the message ourselves */
      else
      {
         switch( mLocalMsg.message )
         {
            /* This is sent when a window is being destroyed.*/
            case WM_DESTROY:
            /* Should never be detected here due to GetMessage filtering */
            case WM_QUIT:
               break;
              
            /* RightClick in windows client area...*/
            case WM_RBUTTONDOWN: 
            case WM_LBUTTONDOWN: 
               MessageBeep(0);
               seCurrentEvent.dwCmd = 'Q';
               seCurrentEvent.dwParam = GUI_INVALID_PARAM;
               dwMsgCount++;
               break;

            case WM_CHAR:
               switch( mLocalMsg.wParam )
               {
                  case (int)'q':/* User keyboard quit msg */
                  case (int)'Q':
                     MessageBeep(0);
                     seCurrentEvent.dwCmd = 'Q';
                     seCurrentEvent.dwParam = GUI_INVALID_PARAM;
                     dwMsgCount++;
                     break;
                  case (int)'X':/* Not handled yet */
                  case (int)'x':
                  case (int)'Y':
                  case (int)'y':
                  case (int)'1':
                  case (int)'2':
                  case (int)'4':
                  case (int)'8':
                  case (int)'i':/* Set sync off video */
                  case (int)'I':
                  case (int)'O':/* Set sync of CSYNC_NEG */
                  case (int)'o':
                  case (int)'c':/* Set to colour display */
                  case (int)'C':
                  case (int)'g':/* Set to grayscale display */
                  case (int)'G':
                  default: 
                     break;/* ignore the rest */
               } /* end switch( mLocalMsg.wParam ) */

            default:
               DefWindowProc( psGui->hWnd, mLocalMsg.message, mLocalMsg.wParam, mLocalMsg.lParam);
               break;
         } /* end switch (mLocalMsg.message) */

         teStatus = ASL_OK; /* for successful completion of message handling */
              
         if ( seCurrentEvent.dwCmd != GUI_INVALID_CMD )
         {   
            bStoreEvent = FALSE;
                 
            /* Add message to message list*/
                
            /* Allocate an event structure */
            pseNewEvent = (struct Event *) calloc( sizeof ( struct Event ), (size_t) 1);
            if ( pseNewEvent != NULL )
            {
               /* If previous event exists to add to ... */
               if ( pseCurrentEventPtr != NULL )
               {
                  /* Enter the event into the event list, adjust pointers and go round for next */
                  pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
                  pseNewEvent->dwParam = seCurrentEvent.dwParam;

                  pseCurrentEventPtr->pseNextEvent = pseNewEvent;
                  /* Update to current event */
                  pseCurrentEventPtr = pseNewEvent;
               }
               else
               {
                  /* Store root of the list */
                  pseCurrentEventRoot = pseNewEvent;
                  pseCurrentEventPtr = pseNewEvent;
                  /* Initialise the root */
                  pseCurrentEventPtr->pseNextEvent = NULL;
                  pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
                  pseNewEvent->dwParam = seCurrentEvent.dwParam;
               }
            }
            else
               teStatus = ASLERR_OUT_OF_MEMORY;
         } /* end if (seCurrentEvent.dwCmd != GUI_INVALID_CMD) */
      } /* end of processing message ourselves */
   } /* end while (PeekMessage... */

   *pseCurrentEventList = pseCurrentEventRoot;
   return teStatus;
}


/* Display Handling Routines
 * *************************
 */
ui32 GUI_DisplayClose( struct GuiInfo *psGui )
{
   static char *szFnName = "(Win32)GUI_DisplayClose";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   DestroyWindow( (HWND) psGui->hWnd );
   TMG_image_destroy( psGui->hTempImage );
   return ASL_OK;
}


ui32 GUI_DisplayImage( struct GuiInfo *psGui, ui32 dwBuffer )
{
   static char *szFnName = "(WIN32)GUI_DisplayImage";
   PAINTSTRUCT ps;
   HDC hDC;
   Thandle hImage;
   Terr teStatus = ASL_OK;
      
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Guarantee clipping area is entire client area for repainting,
    * needs to be done before BeginPain(t) is used
    */
   InvalidateRect( psGui->hWnd, NULL, FALSE );

   /* Add any drawing code here... */
   hDC = BeginPaint( psGui->hWnd, &ps );

   /* Get handle to this image buffer */
   hImage = GUI_TmgGetImage( psGui, dwBuffer);

#if defined _WIN32_DISPLAY_DDI
   DBG_WriteString( 8, "\n%s: Displaying DCI/DDI image", psGui->szProgName );   

   if ( psGui->fConvertForDisplay )
   {
      if( ASL_is_ok( teStatus = TMG_image_convert( hImage, psGui->hTempImage, psGui->wDisplayFormat, TMG_IS_DIB , TMG_RUN) ) )
         teStatus = TMG_display_image( psGui->hDisplay, psGui->hTempImage, TMG_RUN);
   }
   else
      teStatus = TMG_display_image( psGui->hDisplay, hImage, TMG_RUN);

#elif defined _WIN32_DISPLAY_DDB
   DBG_WriteString( 8, "\n%s: Displaying DDB image", psGui->szProgName );

   if ( psGui->fConvertForDisplay )
   {
      if( ASL_is_ok( teStatus = TMG_display_set_paint_hDC( psGui->hDisplay, hDC ) ) )
         if( ASL_is_ok( TMG_image_convert( hImage, psGui->hTempImage, psGui->wDisplayFormat, 0, TMG_RUN) ) )
            teStatus = TMG_display_image( psGui->hDisplay, psGui->hTempImage, TMG_RUN);
   }
   else
      teStatus = TMG_display_image( psGui->hDisplay, hImage, TMG_RUN);

#elif defined _WIN32_DISPLAY_DIB
   DBG_WriteString( 8, "\n%s: Displaying DIB image", psGui->szProgName );      

   if( psGui->wDisplayFormat != TMG_BGR24 )
   {
      if( ASL_is_ok( teStatus = TMG_display_set_paint_hDC( psGui->hDisplay, hDC ) ) )
         if( ASL_is_ok( TMG_image_convert( hImage, psGui->hTempImage, TMG_BGR24, TMG_IS_DIB , TMG_RUN) ) )
            teStatus = TMG_display_image( psGui->hDisplay, psGui->hTempImage, TMG_RUN);
   }
   else
      teStatus = TMG_display_image( psGui->hDisplay, hImage, TMG_RUN);
#endif

   EndPaint (psGui->hWnd, &ps);
   return teStatus;
}


ui32 GUI_DisplayInit( struct GuiInfo *psGui )
{
   static char *szFnName = "(Win32)GUI_DisplayInit";
   Terr teStatus = ASL_OK;
#if defined _WIN32_DISPLAY_DDI   
   ui32 dwImageCount;
#endif

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Create window but do not make it visible.
    * It will be resized to fit the particular video source being making it visible
    */
   psGui->hWnd = CreateWindow( psGui->szProgName, psGui->szProgName, WS_OVERLAPPED,
      CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
      NULL, NULL, psGui->hInstance, NULL);

   if ( psGui->hWnd == NULL )
      return ( ASL_ERROR );

   psGui->hDisplay = TMG_display_create();
   teStatus = TMG_display_init( psGui->hDisplay, psGui->hWnd); 
   if( ASL_is_err ( teStatus) )
      MessageBox(0, "The current graphics mode is not supported.\n\
      Please select a 32K, 64K or 16.7 million (32 bit) colours mode.", "FAILED TO INITIALIZE GRAPHICS", MB_OK);

   psGui->wDisplayFormat = (ui16) TMG_display_get_parameter( psGui->hDisplay, TMG_PIXEL_FORMAT);
   psGui->wDisplayDepth = (ui16) TMG_display_get_parameter( psGui->hDisplay, TMG_DEPTH );
   DBG_WriteString( 8, "\n%s: Display initialised, display type = %d", psGui->szProgName, (ui16) TMG_display_get_parameter( psGui->hDisplay, TMG_PIXEL_FORMAT) );
   DBG_WriteString( 8, "\n%s: DirectDraw/DCI Caps = %08lx", psGui->szProgName, TMG_display_get_parameter( psGui->hDisplay, TMG_DISPLAY_DIRECT_CAPS) );
   
   if (ASL_is_err( psGui->hTempImage = TMG_image_create() ) )
   {
      DBG_WriteString( 10, "\n%s: Failed to create temporary image", psGui->szProgName);
      return ( ASL_ERROR );
   }

   if( !ASL_is_err ( teStatus ) )
   {
#if defined _WIN32_DISPLAY_DDI
      /* Use DirectDraw/DCI */
      DBG_WriteString( 10, "\n%s: Using DirectDraw display mode", szFnName );
      TMG_display_set_flags( psGui->hDisplay, TMG_DISPLAY_DIRECT, TRUE); 

      /* We set the half_aspect to false, so that single field images are displayed as actual half height
       * images, otherwise the DirectDraw display function interprets it as a single video field and displays it
       * it on alternate lines (like it should do in sequence mode - so that live video can be displayed).
       */
      for( dwImageCount = 0; dwImageCount < NUM_VID_IMAGES; dwImageCount++)
         TMG_image_set_flags( GUI_TmgGetImage( psGui, dwImageCount), TMG_HALF_ASPECT, FALSE);
         
      if (TMG_display_get_parameter( psGui->hDisplay, TMG_DISPLAY_DIRECT_CAPS) == 0) 
      {  
         /* No DirectDraw/DCI */
         MessageBox( psGui->hWnd, "A DirectDraw/DCI driver is not present - please select an alternative method.", "FAILURE", MB_OK);
         /* switch off to keep tidy. */
         TMG_display_set_flags( psGui->hDisplay, TMG_DISPLAY_DIRECT, FALSE);  
         teStatus = ASL_ERROR;
      }

#elif defined _WIN32_DISPLAY_DDB
      /* Use DDB */
      /* 999 problem: either the user needs to know about the display format
       * and pre-convert the image to that format, or we do it here.
       * This is equally problematic because then we need an image to store
       * it in, because we cannot guarantee availability of one of the users images
       * For now, the user must know and pre-convert. (!)
       */
      
#elif defined _WIN32_DISPLAY_DIB
      /* Use DIB */
      DBG_WriteString( 10, "\n%s: Using DIB (Device Independent Bitmap) display mode", szFnName );
      DBG_WriteString( 10, "\n%s: DirectDraw (if available) is faster", szFnName );
      TMG_display_set_flags( psGui->hDisplay, TMG_IS_DIB, TRUE); 

#else
#if ! defined __MWERKS__
#error 'Define a drawing method- eg _WIN32_DISPLAY_DCI/DDB/DIB'
#endif
#endif
   }

   return ( teStatus );
}


ui32 GUI_DisplayInitInfo( struct GuiInfo* psGui, char *pszProgName )
{
   static char *szFnName = "(Win32)GUI_DisplayInitInfo";
   WNDCLASS  wc;
   ui32 dwImageCnt;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Set up default TMG variables */
   for( dwImageCnt = 0; dwImageCnt < NUM_VID_IMAGES; dwImageCnt++)
      psGui->sVidImage[dwImageCnt].hVidImage = TMG_INVALID_HANDLE;
   
   /* Set up screen variables */
   psGui->hDisplay =  TMG_INVALID_HANDLE;
   psGui->dwWinWidth  = 0;
   psGui->dwWinHeight = 0;
   psGui->fConvertForDisplay = FALSE;
   strcpy(psGui->szProgName, pszProgName);


   /* Set up default Windows variables */
   psGui->hWnd = NULL;
   psGui->hInstance = (HINSTANCE) 0;

   /* Fill in window class structure with parameters that describe
    the main window. */
   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC) GUI_WndProc;
   wc.cbClsExtra    = 0;
   wc.cbWndExtra    = 0;
   wc.hInstance     = psGui->hInstance;
   wc.hIcon         = LoadIcon ( psGui->hInstance, psGui->szProgName);
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_GRAYTEXT);
   wc.lpszMenuName  = psGui->szProgName;
   wc.lpszClassName = psGui->szProgName;

   /* Register the window class and return success/failure code. */
   if ( RegisterClass(&wc) == 0 )
      return ASL_ERROR;
   
   return ASL_OK;
}


/* Resize the Image window */
ui32 GUI_DisplayReSize( struct GuiInfo *psGui )
{
   static char *szFnName = "(Win32)GUI_DisplayReSize";
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* If the width and height have not already been set, then use the full image size */
   if (psGui->dwWinWidth == 0)
      psGui->dwWinWidth  = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_WIDTH );
   if (psGui->dwWinHeight == 0)
      psGui->dwWinHeight = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_HEIGHT );

   if (! SetWindowPos( psGui->hWnd, 0, 0, 0, (int) psGui->dwWinWidth, (int) psGui->dwWinHeight, SWP_NOMOVE | SWP_NOZORDER | SWP_SHOWWINDOW ) )
      teStatus = ASL_ERROR;

   return teStatus;
}


LRESULT CALLBACK GUI_WndProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   static char *szFnName = "(Win32)GUI_WndProc";
   int wmId, wmEvent;
   SIZE szScreen;
   PAINTSTRUCT ps;
   HDC hdc;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   switch (message) 
   { 
      case WM_COMMAND:
         wmId    = LOWORD(wParam); // Remember, these are...
         wmEvent = HIWORD(wParam); // ...different for Win32!

      case WM_RBUTTONDOWN: // RightClick in windows client area...
         MessageBeep(0);
         PostQuitMessage(0);
         break;

      /* Only comes through on plug'n'play systems */
      case WM_DISPLAYCHANGE:
         szScreen.cx = LOWORD(lParam);
         szScreen.cy = HIWORD(lParam);

         MessageBeep(0);
         /* If the display 'has' changed, then szScreen reflects the new size,
          * otherwise szScreen reflects the original size
          */
         if ((BOOL) wParam) 
            MessageBeep(0);
         break;

      case WM_PAINT:
         hdc = BeginPaint (hWnd, &ps);
         /* Add any drawing code here... */
         EndPaint (hWnd, &ps);
         break;        

      case WM_CHAR:
         switch( wParam )
         {
            case (int)'q':
            case (int)'Q':
               MessageBeep(0);
               PostQuitMessage(0);
               break;
            default:
               break;/* ignore  */
         }
         break;
            
      case WM_DESTROY:
         PostQuitMessage(0);
         break;

      default:
         return (DefWindowProc(hWnd, message, wParam, lParam));
   }

   return (0);
}


#elif (defined _X_GRAPHICS)
/*
XGraphics (ForBrief)
**************************************************************************
*/

/* Local function prototypes */
ui32 GUI_DisplayEntryScreen( struct GuiInfo* psGui );
ui32 GUI_GetXGC( struct GuiInfo *psGui );
ui32 extract_options( ui16 wNumArgs, char **pszArgPtr, struct Cmd **pscListPtr );

/* Message Handling Routines
 * *************************
 */
ui32 GUI_CheckMessageQueue( struct GuiInfo *psGui, struct Event **pseCurrentEventList )
{
   static char *szFnName = "(X)GUI_CheckMessageQueue";
   /* Event information structure */
   XEvent xeReport;
   int nEventsQueued;
   struct Event *pseNewEvent = NULL;
   struct Event *pseCurrentEventPtr = NULL;
   struct Event *pseCurrentEventRoot = NULL;
   struct Event  seCurrentEvent;

   /* Keyboard event interpretation variables */
   XComposeStatus sxcsKeyCompStat;
   KeySym ksKey;
   char pcBuffer[_MAX_MAPPED_STRING_LENGTH];
   ui16 wBuffSize = _MAX_MAPPED_STRING_LENGTH;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Check for events waiting, if there are - process them */
   nEventsQueued = XEventsQueued( psGui->pdDisplay, QueuedAfterFlush );
   DBG_WriteString( 8, "\n%s: %d events queued", psGui->szProgName, nEventsQueued );

   while( nEventsQueued-- > 0 )
   {
      /* Initialise our temporary event structure */
      seCurrentEvent.dwCmd   = GUI_INVALID_CMD;
      seCurrentEvent.dwParam = GUI_INVALID_PARAM;

      /* Get the event */
      XNextEvent( psGui->pdDisplay, &xeReport );
      DBG_WriteString( 8, "\n%s: Event reporting %i", psGui->szProgName, nEventsQueued );
      DBG_WriteString( 8, "\n%s: Event type %i", psGui->szProgName, xeReport.type );

      /* Translate the event type */
      switch ( xeReport.type )
      {
         case ButtonPress:
            seCurrentEvent.dwCmd = 'Q';
            break;
         case KeyPress:
            /* keysyms are defined in X11/keysymdef.h */
            XLookupString( (XKeyEvent*)&xeReport, pcBuffer, wBuffSize, &ksKey, &sxcsKeyCompStat );
/*          XLookupString( (XKeyEvent*)&xeReport, pcBuffer, wBuffSize, &ksKey, &sxcsKeyCompStat );
		NOTE: original line above - typecasting forced to solve Linux compilation issue
		from more fussy gcc compiler type checking.
		Edited by Colin / Jon Rennie 23 Oct 2002  */

            switch ( (KeySym) ksKey )
            {
               case XK_q:
               case XK_Q:
                  seCurrentEvent.dwCmd = 'Q';
                  break;

               default: /* do nothing */
                  break;
            }
            break;

         default: /* do nothing */
            break;
      }

      if ( seCurrentEvent.dwCmd != GUI_INVALID_CMD )
      {
         /* Allocate an event structure */
         pseNewEvent = (struct Event *) calloc( sizeof ( struct Event ), (size_t) 1);
         DBG_WriteString( 8, "\n%s: Event root = 0x%08x", psGui->szProgName, pseNewEvent );
         if( pseNewEvent != NULL )
         {
            /* If previous event exists to add to ... */
            if( pseCurrentEventPtr != NULL )
            {
               /* Enter the event into the event list, adjust pointers and go round for next */
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;

               pseCurrentEventPtr->pseNextEvent = pseNewEvent;
               /* Update to current event */
               pseCurrentEventPtr = pseNewEvent;
            }
            else
            {
               /* Store root of the list */
               pseCurrentEventRoot = pseNewEvent;
               /* Initialise the root */
               pseCurrentEventPtr = pseNewEvent;
               pseCurrentEventPtr->pseNextEvent = NULL;
            
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;
            }
         }
         else
         {
            teStatus = ASLERR_OUT_OF_MEMORY;
            break;
         }
      }
   }

   /* Pass the list back */
   *pseCurrentEventList = pseCurrentEventRoot;
   return ( teStatus );
}


/* Display Handling Routines
 * *************************
 */
ui32 GUI_DisplayClose( struct GuiInfo *psGui )
{
   static char *szFnName = "(X)GUI_DisplayClose";
   ui32 teStatus = ASL_OK;
   ui32 dwImageCount;
     
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Free TMG resources */
   TMG_display_destroy( psGui->hDisplay );
   psGui->hDisplay = TMG_INVALID_HANDLE;
 
   for( dwImageCount = 0; dwImageCount < NUM_VID_IMAGES; dwImageCount++)
   {
      TMG_image_destroy( psGui->sVidImage[dwImageCount].hVidImage );
      psGui->sVidImage[dwImageCount].hVidImage = TMG_INVALID_HANDLE;
   }   
 
   /* Free Gui resources */
   if( psGui->xfsFont != NULL )
      XUnloadFont( psGui->pdDisplay, psGui->xfsFont->fid );
   psGui->xfsFont = NULL;
 
   if( psGui->gcView != GUI_INVALID_GC )
      XFreeGC( psGui->pdDisplay, psGui->gcView );
   psGui->gcView = GUI_INVALID_GC;

   if( psGui->pdDisplay != GUI_INVALID_DISPLAY )
      XCloseDisplay( psGui->pdDisplay );
   psGui->pdDisplay = GUI_INVALID_DISPLAY;
   psGui->wWin = GUI_INVALID_WINDOW;

   return ( ASL_OK );
}


ui32 GUI_DisplayImage( struct GuiInfo *psGui, ui32 dwImageNumber )
{
   static char *szFnName = "(X)GUI_DisplayImage";
   Thandle hCroppedImage;
   Thandle hImage;
   Terr teStatus;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   hImage = GUI_TmgGetImage( psGui, dwImageNumber );
   teStatus = TMG_display_image( psGui->hDisplay, hImage, TMG_RUN ); 

   return teStatus;
}


ui32 GUI_DisplayInit( struct GuiInfo* psGui )
{
   static char *szFnName = "(X)GUI_DisplayInit";
   char *pszPtr[2] = {NULL,NULL};   /* Server device name */ 
   Terr teStatus;
   ui32 dwImageCount;
   Pixmap pxmpIcon;                 /* X window x-manager hints variables */
   XSizeHints xshSizeHint;
   XWMHints xwmWHint;
   XClassHint xchCHint;
   XTextProperty xtpropWnName;
   XTextProperty xtpropIconName;
   XEvent xeReport;                 /* Event information structure */


   DBG_WriteString( 1, "\n%s: Entering...", szFnName );

   /* Connect to the Xserver - defaults to the value of the DISPLAY environment variable if NULL*/
   DBG_WriteString( 1, "\n%s: Connecting to X server %s", szFnName, XDisplayName( NULL ));
   if ( (psGui->pdDisplay = XOpenDisplay( XDisplayName(NULL) ) ) == NULL )
   {
      DBG_WriteString( 8, "\n%s: Cannot connect to X server %s", szFnName, XDisplayName( NULL )); 
      return ( ASL_ERROR );
   }

   /* get screen size etc from display structure macro */
   psGui->iScreenNum = DefaultScreen( psGui->pdDisplay );
  

   /* 999 add check for valid window */
   psGui->wWin = XCreateSimpleWindow( psGui->pdDisplay, RootWindow( psGui->pdDisplay, psGui->iScreenNum),
                                      WIN_X_ROOT, WIN_Y_ROOT, 
                                      (unsigned int) 1, (unsigned int) 1,
                                      BORDER_WIDTH, 
                                      BlackPixel( psGui->pdDisplay, psGui->iScreenNum ), 
                                      WhitePixel( psGui->pdDisplay, psGui->iScreenNum )  );
   DBG_WriteString( 8, "\n%s: Simple window created: %li", szFnName, (long int) psGui->wWin);
   
  /* Set up the X Windows IDs before TMG_init_display 
   * The frame, canvas and view terminology comes from Solaris View 
   * but set them all to the same value for now. 
   */

   teStatus = ASL_OK;
   for ( dwImageCount = 0; dwImageCount< NUM_VID_IMAGES; dwImageCount++)
      if( psGui->sVidImage[dwImageCount].hVidImage == TMG_INVALID_HANDLE )
         teStatus |= ASL_ERROR;
  
   if( ASL_is_err(teStatus))
   {
      DBG_WriteString( 10, "\n%s: Failed due to no valid TMG image handles present", szFnName );
      return (ASL_ERROR);
   }
  
   /* Create a  handle to the screen */
   if ( ( psGui->hDisplay = TMG_display_create() ) == TMG_INVALID_HANDLE)
      return ASL_ERROR;
 
   TMG_display_set_Xid( psGui->hDisplay, TMG_XID_FRAME,  psGui->wWin);
   TMG_display_set_Xid( psGui->hDisplay, TMG_XID_CANVAS, psGui->wWin);
   TMG_display_set_Xid( psGui->hDisplay, TMG_XID_WINDOW, psGui->wWin);
  
   /* Can only do TMG_init after server connection because we need a valid wWin XID to have anything to initialise */
   teStatus = TMG_display_init( psGui->hDisplay, TMG_X_WINDOWS );
   if ( ASL_is_err( teStatus ) )
   {
      DBG_WriteString( 10, "\n%s: TMG_display_init failed with error %i", szFnName, (int) ASL_get_err(teStatus) );
      return ASL_ERROR;
   }
   DBG_WriteString( 8, "\n%s: Display initialised OK", szFnName );

   /* Detect the display format and save for use later. */
   psGui->wDisplayDepth = (int) TMG_display_get_parameter( psGui->hDisplay, TMG_DEPTH );
   DBG_WriteString( 8, "\n%s: TMG_display depth reports %d", szFnName, psGui->wDisplayDepth);
   psGui->wDisplayFormat = (int) TMG_display_get_parameter( psGui->hDisplay, TMG_PIXEL_FORMAT );
   DBG_WriteString( 8, "\n%s: TMG_display format reports %d", szFnName, psGui->wDisplayFormat);
 
   /* Attempt to manage the display of our new window properly */
   /* Set size hints for the window manager*/  

   xshSizeHint.flags=PPosition | PSize | PMinSize;
   xshSizeHint.min_width = WIN_MIN_WIDTH ;
   xshSizeHint.min_height = WIN_MIN_HEIGHT;

   if( psGui->szProgName != NULL )
   {
      DBG_WriteString( 8, "\n%s: psGui->szProgName = %s", szFnName, psGui->szProgName );
   }

   /*  Use the actual not the default */
   if ( XStringListToTextProperty( pszPtr, 1, &xtpropWnName ) == 0 )
   {
      DBG_WriteString( 10, "\n%s: Structure allocation for xtpropWnName failed!", szFnName);
      return ASL_ERROR;
   }
     
   if ( XStringListToTextProperty( pszPtr, 1, &xtpropIconName) == 0 )
   {
      DBG_WriteString( 10, "\n%s: Structure allocation for xtpropIconName failed!", szFnName);
      return ASL_ERROR;
   }

   /* Not iconised to start, wants keyboard input and has an icon */
   xwmWHint.initial_state = NormalState;
   xwmWHint.input = TRUE;
   xwmWHint.icon_pixmap = pxmpIcon;
   xwmWHint.flags = StateHint | IconPixmapHint | InputHint;
     
   /* For use by the window manager in the program database */
   xchCHint.res_name =  psGui->szProgName;
   xchCHint.res_class = psGui->szProgName;
     
   XSetWMProperties( psGui->pdDisplay, psGui->wWin, &xtpropWnName, &xtpropIconName, NULL, 0, &xshSizeHint, &xwmWHint, &xchCHint);
  

   /* Select notification sources */
   XSelectInput( psGui->pdDisplay, psGui->wWin, ExposureMask | KeyPressMask | ButtonPressMask | StructureNotifyMask );
  
   /*Initialise resources - select & load font, create a view context to draw on in the window */
   if ( (psGui->xfsFont = XLoadQueryFont( psGui->pdDisplay, "9x15") ) == NULL )
   {
      DBG_WriteString( 10, "\n%s: Cannot open 9x15 font", szFnName);
      return ASL_ERROR;
   }

   /* Get a graphics context to display on */
   if ( ASL_is_err( GUI_GetXGC( psGui ) ) )
   {
      /* GC context information is saved to psGui within GetXGC */
      DBG_WriteString( 10, "\n%s: Failed to create and initialise screen context", szFnName );
      return ASL_ERROR;
   }
  
   /*Display our window on the screen */
   XMapWindow( psGui->pdDisplay, psGui->wWin);
   DBG_WriteString( 8, "\n%s: Window mapped - Is it visible ?", szFnName );
  
   /* Window must not be drawn on until an expose event is received - so loop on expose event
    * Check for events waiting, if there are - process them, downside is we junk any other event.
    */
   do
   {
      XNextEvent( psGui->pdDisplay, &xeReport);
   }
   while (xeReport.type != Expose);

   return ASL_OK;
} 


ui32 GUI_DisplayInitInfo( struct GuiInfo *psGui, char *pszProgName )
{
   static char *szFnName = "(X)GUI_DisplayInitInfo";
   ui32 dwImageBuffer;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);
   
   /* Set up default TMG variables */
   for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
      psGui->sVidImage[dwImageBuffer].hVidImage = TMG_INVALID_HANDLE;
   psGui->hDisplay     = TMG_INVALID_HANDLE;

   psGui->dwWinWidth  = 0;
   psGui->dwWinHeight = 0;
   psGui->fConvertForDisplay = FALSE;

   strcpy(psGui->szProgName, pszProgName);
   psGui->pdDisplay = GUI_INVALID_DISPLAY;
   psGui->xfsFont = NULL;
   psGui->gcView = GUI_INVALID_GC;
   psGui->wWin =GUI_INVALID_WINDOW;

   return ASL_OK;
}


ui32 GUI_DisplayReSize( struct GuiInfo *psGui )
{
   static char *szFnName = "(X)GUI_DisplayReSize";
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* If the width and height have not already been set, then use the full image size */
   if (psGui->dwWinWidth == 0)
      psGui->dwWinWidth  = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_WIDTH );
   if (psGui->dwWinHeight == 0)
      psGui->dwWinHeight = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_HEIGHT );

   XResizeWindow( psGui->pdDisplay, psGui->wWin, psGui->dwWinWidth, psGui->dwWinHeight);
   XFlush( psGui->pdDisplay );

   return teStatus;
}


ui32 GUI_GetXGC( struct GuiInfo* psGui )
{
   static char *szFnName = "(X)GUI_GetXGC";
   static char szDashList[] = {12,24};
   unsigned long ulValMask = 0L;
   unsigned int uiLineWidth = 6;
   int nDashListLength = 2;
   int nLineStyle = LineOnOffDash;
   int nCapStyle = CapRound;
   int nJoinStyle = JoinRound;
   int nDashOffset = 0;
   XGCValues xgcvValues;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Create and initialise graphics context */
   psGui->gcView = XCreateGC( psGui->pdDisplay, psGui->wWin, ulValMask, &xgcvValues);
   if ( psGui->gcView == NULL )
   {
      DBG_WriteString( 8, "\n%s: Failed to create GC", szFnName);
      return ASL_ERROR;
   }
   XSetFont( psGui->pdDisplay, psGui->gcView, psGui->xfsFont->fid);
   XSetForeground( psGui->pdDisplay, psGui->gcView, BlackPixel( psGui->pdDisplay, psGui->iScreenNum ));
   XSetLineAttributes( psGui->pdDisplay, psGui->gcView, uiLineWidth, nLineStyle, nCapStyle, nJoinStyle);
   XSetDashes( psGui->pdDisplay, psGui->gcView, nDashOffset, szDashList, nDashListLength);
   return ASL_OK;
}
/* End of _X_GRAPHICS */


#elif (defined _MACINTOSH)
/*
Macintosh (ForBrief)
**************************************************************************
*/

/* Local function prototypes */
ui32 GUI_AsyncIoGet ( char *pcInputChar );


/* Asynchronous I/O Handling Routines
 * **********************************
 */
/* Get Async IO */
ui32 GUI_AsyncIoGet( char *pcInputChar )
{
   static char *szFnName = "(MAC)GUI_AsyncIoGet";
   ui32 dwStatus = FALSE;
   EventRecord erNextEvent;
   WindowPtr theWindow;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);
   
   OS_process_yield();
   if (GetNextEvent(everyEvent, &erNextEvent))
   {
      switch (erNextEvent.what)
      {
         case autoKey:
         case keyDown:
            if (!(erNextEvent.modifiers & cmdKey))
            {
               dwStatus = TRUE;
               *pcInputChar = (char) (erNextEvent.message & charCodeMask);
            }
            break;
            
         case mouseDown:
            switch( FindWindow(erNextEvent.where, &theWindow) )
            {
               /* Allow system events to be handled */
               case inSysWindow:
                  SystemClick( &erNextEvent, theWindow );
                  break;
                      
               /* Allow Sioux window to be dragged around */
               case inDrag:
                  DragWindow( theWindow, erNextEvent.where, &qd.screenBits.bounds );
                  break;
                      
               /* A mouse click in the Sioux window is equivalent to
                * the user pressing Q
                */
               case inContent:
                  dwStatus = TRUE;
                  *pcInputChar = (char) 'Q';
                  break;

               default:
                  break;
            }
            break;

         default:
            SIOUXHandleOneEvent( &erNextEvent );
            break;
      }
   }

   return dwStatus;
}


/* Message Handling Routines
 * *************************
 */
ui32 GUI_CheckMessageQueue( struct GuiInfo *psGui, struct Event **pseCurrentEventList )
{
   static char *szFnName = "(MAC)GUI_CheckMessageQueue";
   struct Event *pseNewEvent = NULL;
   struct Event *pseCurrentEventPtr = NULL;
   struct Event *pseCurrentEventRoot = NULL;
   struct Event  seCurrentEvent;
   char  cInputChar;
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   while( GUI_AsyncIoGet( &cInputChar ) )
   {
      /* set defaults */
      seCurrentEvent.dwCmd = GUI_INVALID_CMD;
      seCurrentEvent.dwParam = GUI_INVALID_PARAM;
      seCurrentEvent.pseNextEvent = NULL;

      switch ( (int) cInputChar )
      {
         case (int) 'Q':
         case (int) 'q': 
            seCurrentEvent.dwCmd = 'Q';
            seCurrentEvent.dwParam = GUI_INVALID_PARAM;
            break;
      
         /* Add to handling chain here if required e.g re-sizing commands etc. */
         default:
            break;
      }

      if ( seCurrentEvent.dwCmd != GUI_INVALID_CMD )
      {
         /* Allocate an event structure */
         pseNewEvent = (struct Event *) calloc( sizeof ( struct Event ), (size_t) 1);
         if( pseNewEvent != NULL )
         {
            /* If previous event exists to add to ... */
            if( pseCurrentEventPtr != NULL )
            {
               /* Enter the event into the event list, adjust pointers and go round for next */
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;

               /* Update to current event */
               pseCurrentEventPtr->pseNextEvent = pseNewEvent;
               pseCurrentEventPtr = pseNewEvent;
            }
            else
            {
               /* Store root of the list */
               pseCurrentEventRoot = pseNewEvent;
               pseCurrentEventPtr = pseNewEvent;

               /* Initialise the root */
               pseCurrentEventPtr->pseNextEvent = NULL;
               pseNewEvent->dwCmd   = seCurrentEvent.dwCmd;
               pseNewEvent->dwParam = seCurrentEvent.dwParam;
            }
         }
         else
            teStatus = ASLERR_OUT_OF_MEMORY;
      } /* End of adding to list */
   } /* End of while */

   _UNUSED_VAR( psGui );

   *pseCurrentEventList = pseCurrentEventRoot;
   return teStatus;
}


/* Display Handling Routines
 * *************************
 */
ui32 GUI_DisplayClose     ( struct GuiInfo *psGui )
{
   static char *szFnName = "(MAC)GUI_DisplayClose";
   Terr teStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);
 
   /* Since we created these without a resource we must destroy them explicitly */
   DisposeWindow( psGui->pWin );
   TMG_image_destroy( psGui->hTempImage );

   return( teStatus );
}


ui32 GUI_DisplayImage     ( struct GuiInfo *psGui, ui32 dwBufferNumber )
{
   static char *szFnName = "(MAC)GUI_DisplayImage";
   Terr teStatus = ASL_OK;
 
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);
    
   /* set window to current graf port */
   SetPort( psGui->pWin );   
   if ( psGui->fConvertForDisplay )
   {
      if( ASL_is_ok( TMG_image_convert( GUI_TmgGetImage( psGui, dwBufferNumber ), psGui->hTempImage, psGui->wDisplayFormat, TMG_USE_LUT, TMG_RUN) ) )
         teStatus = TMG_display_image( psGui->hDisplay, psGui->hTempImage, TMG_RUN);
   }
   else
      teStatus = TMG_display_image( psGui->hDisplay, GUI_TmgGetImage( psGui, dwBufferNumber ), TMG_RUN);

   ValidRect( &psGui->windRect );

   return( teStatus );
}


ui32 GUI_DisplayInit      ( struct GuiInfo *psGui )
{
   static char *szFnName = "(MAC)GUI_DisplayInit";
   Terr teStatus = ASL_OK;
   PixMapHandle  hpmPlayThru;
   char TempString[MAX_FIELD_LENGTH];

 
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   InitGraf(&qd.thePort);
   InitFonts();
   InitWindows(); /* Loads the window resource */
   InitMenus();   /* Loads the menu resource */
   TEInit();      /* Text-editor init - needed for window title display apparently */
   InitDialogs(nil); /* Needed for system alert message box & other stuff- apparently */
   InitCursor();

   /* Make a new window for drawing in, and it must be a color window.  
    * The window is sized in init info.
    */
   strcpy( TempString, psGui->szProgName );
   psGui->pWin = NewCWindow( nil, &psGui->windRect, c2pstr(TempString), false, noGrowDocProc, 
                  (WindowPtr) -1, true, 0);
                  
   /* Set window to current graf port - how do we avoid losing it ?*/
   SetPort( psGui->pWin );   
 
   psGui->hDisplay = TMG_display_create( );
   if( psGui->hDisplay == TMG_INVALID_HANDLE )
      TMG_err_ret( ASL_ERROR, "Failed to acquire display ", 0 , szFnName );
     
   if (ASL_is_err( psGui->hTempImage = TMG_image_create() ) )
   {
      DBG_WriteString( 10, "\n%s: Failed to create temporary image", psGui->szProgName);
      return ( ASL_ERROR );
   }
   
   /* Obtain a valid PixMapWindow to display with */
   hpmPlayThru = GetWindowPort( psGui->pWin )->portPixMap;

   /* Initialise display with PixMapWindow */
   teStatus = TMG_display_init( psGui->hDisplay, hpmPlayThru ); 
   psGui->wDisplayFormat = TMG_display_get_parameter( psGui->hDisplay, TMG_PIXEL_FORMAT );
   psGui->wDisplayDepth = TMG_display_get_parameter( psGui->hDisplay, TMG_DEPTH );
 

   return( teStatus );
}


ui32 GUI_DisplayInitInfo  ( struct GuiInfo *psGui , char *pszProgName)
{
   static char *szFnName = "(MAC)GUI_DisplayInitInfo";
   Terr teStatus = ASL_OK;
   ui32 dwImageBuffer;
   OSErr  MacOSErr;
   SysEnvRec   theWorld;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Test the computer to be sure we can do color.  
    * If not we would crash, which would be bad due to the Macs process
    * model bringing the entire machine down.
    * If we cannot run, just beep and exit.
    */
   MacOSErr = SysEnvirons( 1, &theWorld);
   if( MacOSErr != 0 )
   {
      TMG_err_ret( ASL_ERROR, "\SysEnvirons call failed", 0, szFnName );
   }

   if ( theWorld.hasColorQD == false) 
   {
      SysBeep(50);
      /* If no color QD, we must leave. */
      TMG_err_ret( ASL_ERROR, "No colour QuickDraw manager available", 0, szFnName );
   }


   /* Set up default TMG variables */
   for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
      psGui->sVidImage[dwImageBuffer].hVidImage = TMG_INVALID_HANDLE; 
 
   strcpy( psGui->szProgName, "");
   psGui->hDisplay     = TMG_INVALID_HANDLE;
   psGui->pWin = nil;

   psGui->dwWinWidth  = 0;
   psGui->dwWinHeight = 0;
   psGui->fConvertForDisplay = FALSE;
   psGui->windRect.top = 50;
   psGui->windRect.left = 50;
   psGui->windRect.right = psGui->windRect.left + psGui->dwWinWidth;
   psGui->windRect.bottom = psGui->windRect.top + psGui->dwWinHeight;
 
   _UNUSED_VAR( pszProgName );
   
   return( teStatus );
}


ui32 GUI_DisplayReSize  ( struct GuiInfo *psGui )
{
   static char *szFnName = "(MAC)GUI_DisplayReSize";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* If the width and height have not already been set, then use the full image size */
   if (psGui->dwWinWidth == 0)
      psGui->dwWinWidth  = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_WIDTH );
   if (psGui->dwWinHeight == 0)
      psGui->dwWinHeight = TMG_image_get_parameter( psGui->sVidImage[0].hVidImage, TMG_HEIGHT );

   psGui->windRect.right = psGui->windRect.left + psGui->dwWinWidth;
   psGui->windRect.bottom = psGui->windRect.top + psGui->dwWinHeight;

   SizeWindow( psGui->pWin, (int) psGui->dwWinWidth, (int) psGui->dwWinHeight, TRUE );
   ShowWindow( psGui->pWin );

   return ASL_OK;
}


#else
/***************************************************************************/
#if ! defined  __MWERKS__
#error 'Must define an OS platform for Gui '
#endif
#endif


/* OS-dependent delay function */
#if defined _WIN95 || defined _WINNT
Terr ASL_Sleep( ui32 dwUSeconds )
{
   Sleep( dwUSeconds / 1000 );
   return ( ASL_OK );
}
#elif defined _MACINTOSH
Terr ASL_Sleep( ui32 dwUSeconds )
{
   sleep( dwUSeconds/1e6 );
   return ( ASL_OK );
}
#elif defined _LYNXOS
Terr ASL_Sleep( ui32 dwUSeconds )
{
   usleep( dwUSeconds );
   return ( ASL_OK );
}
#elif defined _LINUX
Terr ASL_Sleep( ui32 dwUSeconds )
{
   usleep( dwUSeconds );
   return ( ASL_OK );
}
#elif defined _VXWORKS
Terr ASL_Sleep( ui32 dwUSeconds )
{
   return ( ASL_OK );
}
#elif defined _SOLARIS2
Terr ASL_Sleep( ui32 dwUSeconds )
{
   return ( ASL_OK );
}

#elif defined _DOS32
Terr ASL_Sleep( ui32 dwUSeconds )
{
   return ( ASL_OK );
}
#else
#if ! defined __MWERKS__
#error ' undetermined OS for sleep function '
#endif
#endif


/* OS independent code */
/***************************************************************************/
/* Command line option handling */
Terr GUI_SetCmdParams( struct Info *psInfo, ui16 argc, char **argv )
{
   static char *szFnName = "GUI_SetCmdParams";
   char szFileName[MAX_FIELD_LENGTH]="";
   struct Cmd scRoot;  
   struct Cmd *pscPrevCmdPtr = NULL;  
   struct Cmd *pscCmdPtr = NULL;
#if ! defined _MACINTOSH
   ui32 dwXsize;
   ui32 dwYsize;
   ui32 dwNumCmds;
#endif
 
   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Initialise */
   scRoot.dwCmd = GUI_INVALID_CMD;
   scRoot.szParam[0] = '\0';
   scRoot.pscNextCmd = NULL;


   /* Macintosh programs do not have command line parameters */
#if defined _MACINTOSH   
   _UNUSED_VAR( argc );
   _UNUSED_VAR( argv );
   _UNUSED_VAR( psInfo );
#else
   
   dwNumCmds = ASL_ParseCmdLine( &scRoot, (i16) argc, &argv[0] );

   /* Search for command line args identifying the image size, hardware subsampling and board ID*/
   pscCmdPtr = &scRoot; 
  

   while ( pscCmdPtr != NULL )
   {
      switch ( (int) pscCmdPtr->dwCmd )
      {
         case 'f':/*Image filename to display or write to */
            sscanf( pscCmdPtr->szParam, "%s", szFileName );
            TMG_image_set_infilename(  GUI_TmgGetImage( &psInfo->sGuiInfo, 0), szFileName );
            TMG_image_set_outfilename( GUI_TmgGetImage( &psInfo->sGuiInfo, 0), szFileName );
            break;
         case 'b':/* Snapper ID*/
            sscanf( pscCmdPtr->szParam, "%x", &psInfo->sSnapInfo.dwBoardID ); 
            break;
         case 'x':
            sscanf( pscCmdPtr->szParam, "%d", &dwXsize ); 
            psInfo->sGuiInfo.dwWinWidth = dwXsize;
            break;
         case 'y':
            sscanf( pscCmdPtr->szParam, "%d", &dwYsize ); 
            psInfo->sGuiInfo.dwWinHeight = dwYsize;
            break;

        /* Add extra command letters here as required. 
         * Note that the only snapper specific command is the board address, 
         * which is passed by reference.
         * This can be replaced with a Snapper info structure; 
         */
         default:
            break;
      }

      pscCmdPtr = pscCmdPtr->pscNextCmd;  
   }
#endif

   return ASL_OK;
}


ui32 GUI_SnapInit( struct SnapInfo* psSnap )
{
   static char *szFnName = "GUI_SnapInit";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   psSnap->dwFrameCount = 0;
   psSnap->dwDisplayCount = 0;
   psSnap->dwFrameBufferNumber = 0;
   psSnap->dwDisplayBufferNumber = 0;
   psSnap->dwBoardID = BASE_AUTO;

   return (ASL_OK);
}


/* TMG Handling Routines
 * *********************
 */
Terr GUI_TmgClose( struct GuiInfo* psGui )
{
   static char *szFnName = "GUI_TmgClose";
   ui32 dwStatus;
   ui32 dwImageBuffer;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* Free TMG resources */
   /* This function will also delete the display resources if they have been created */
   dwStatus = ASL_OK;
   for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
      dwStatus |= TMG_image_destroy( GUI_TmgGetImage( psGui, dwImageBuffer ) );
   if ( psGui->hDisplay != TMG_INVALID_HANDLE )
      dwStatus |= TMG_display_destroy( psGui->hDisplay );

   if( ASL_is_err( dwStatus ) )
   {
      TMG_err_ret( dwStatus, "Unable to release TMG resources", 0, szFnName);
   }
   else
   {
      for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
         psGui->sVidImage[dwImageBuffer].hVidImage = TMG_INVALID_HANDLE;
      psGui->hDisplay     = TMG_INVALID_HANDLE;
   }
   return( ASL_OK );
}


/* Wrapper function to hide Gui contents */
ui32 GUI_TmgGetImage( struct GuiInfo* psGui, ui32 dwFrameBufferNumber )
{
   static char *szFnName = "GUI_TmgGetImage";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   if ( dwFrameBufferNumber < NUM_VID_IMAGES && dwFrameBufferNumber >= 0)
      return(psGui->sVidImage[dwFrameBufferNumber].hVidImage );
   else
      TMG_err_ret( ASL_ERROR, "Image index outside valid range- check NUM_VID_IMAGES", 0, szFnName);
}


/* Wrapper function to access image buffer ready status */
ui32 GUI_TmgGetImageFull( struct GuiInfo* psGui, ui32 dwFrameBufferNumber, ui32 *pdwStatus )
{
   static char *szFnName = "GUI_TmgGetFull";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   if ( dwFrameBufferNumber < NUM_VID_IMAGES && dwFrameBufferNumber >= 0)
   {
      *pdwStatus = psGui->sVidImage[dwFrameBufferNumber].bFull;
      return ASL_OK;
   }
   else
      TMG_err_ret( ASL_ERROR, "Image index outside valid range- check NUM_VID_IMAGES", 0, szFnName);
}


/* Wrapper function to access image buffer ready status */
ui32 GUI_TmgSetImageFull( struct GuiInfo* psGui, ui32 dwFrameBufferNumber, ui32 bFull )
{
   static char *szFnName = "GUI_TmgGetFull";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   if ( dwFrameBufferNumber < NUM_VID_IMAGES && dwFrameBufferNumber >= 0)
   {
      psGui->sVidImage[dwFrameBufferNumber].bFull = bFull;
      return ASL_OK;
   }
   else
      TMG_err_ret( ASL_ERROR, "Image index outside valid range- check NUM_VID_IMAGES", 0, szFnName);
}


ui32 GUI_TmgInit( struct GuiInfo* psGui )
{
   static char *szFnName = "GUI_TmgInit";
   ui32 dwImageBuffer;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   for (dwImageBuffer = 0; dwImageBuffer < NUM_VID_IMAGES; dwImageBuffer++)   
   {   
      if ( (psGui->sVidImage[dwImageBuffer].hVidImage = TMG_JPEG_image_create() ) == TMG_INVALID_HANDLE )
      {
         TMG_err_ret( ASLERR_BAD_IMAGE, "Unable to create image buffer", 0, szFnName);
      }
      else
      {
         psGui->sVidImage[dwImageBuffer].bFull = FALSE;
         psGui->sVidImage[dwImageBuffer].bEmpty = FALSE;
      }
   }

   psGui->hDisplay = TMG_INVALID_HANDLE;/* One more time */
   return (ASL_OK);
}


ui32 GUI_TmgWriteFile( struct GuiInfo *psGui, char *szFileBase, ui32 dwFileFormat, ui32 dwHandleIndex )
{
   static char *szFnName = "GUI_TmgWriteFile";
   static ui32 dwFileCount = 1;
   char szFileName[256];
   Thandle hWriteImage;
   ui32 dwStatus = ASL_OK;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   if ( dwHandleIndex >= NUM_VID_IMAGES )
      return ASL_ERROR;

   sprintf( szFileName, "%.5s%03d.", szFileBase, dwFileCount );

   switch (dwFileFormat)
   {
      case TMG_TIFF: strcat( szFileName, "tif" ); break;
      case TMG_JPEG: strcat( szFileName, "jpg" ); break;
      case TMG_EPS:  strcat( szFileName, "eps" ); break;
      case TMG_BMP:  strcat( szFileName, "bmp" ); break;
      case TMG_RAW:  strcat( szFileName, "raw" ); break;
      default:
         DBG_WriteString( 10, "\n%s: Unsupported File Format", szFnName );
         return ASL_ERROR;
         break;
   }
   DBG_WriteString( 10, "\n%s: Writing image to file %s", psGui->szProgName, szFileName);

   /* Convert the image to RGB24 data format, before writing to file */
   if ( ( hWriteImage = TMG_image_create() ) == TMG_NULL )
      return ASL_ERROR;
   dwStatus = TMG_image_convert( GUI_TmgGetImage( psGui, dwHandleIndex ), hWriteImage, TMG_RGB24, 0, TMG_RUN );
   if ( ASL_is_ok( dwStatus ) )
      dwStatus = TMG_image_set_outfilename( hWriteImage, szFileName );
   if ( ASL_is_ok( dwStatus ) )
      dwStatus = TMG_image_write( hWriteImage, TMG_NULL, (ui16) dwFileFormat, TMG_RUN );
   dwStatus |= TMG_image_destroy( hWriteImage );

   if ( dwStatus != ASL_OK )
      return ASL_ERROR;
   else
      return ASL_OK;
}


ui32 ASL_ParseCmdLine( struct Cmd *pscListPtr, i16 wArgs, char** pszArgPtr )
{
   static char *szFnName = "ASL_ParseCmdLine";
   ui32 dwCmdCount = (ui32) 0;
 
   struct Cmd scNewCmd;
   struct Cmd *pscNewCmd;
   struct Cmd *pscThisCmd = NULL;

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   /* The first argument is the program name - therefore skip it */
   while (--wArgs > 0)
   {
      scNewCmd.dwCmd   = GUI_INVALID_CMD;
      scNewCmd.szParam[0] = '\0';

     DBG_WriteString( 8, "\n%s: Arg=%s", szFnName, pszArgPtr[wArgs] );      
   
      /* Look for arg separator at start of command string */
      if ( ( pszArgPtr[wArgs][0] == '-' || pszArgPtr[wArgs][0] == '/' ) &&
             pszArgPtr[wArgs][1] != '\0' )
      {
         scNewCmd.dwCmd = pszArgPtr[wArgs][1];
         /* strcpy only with a non-zero length string */
         if ( pszArgPtr[wArgs][2] != '\0' ) 
            strcpy( scNewCmd.szParam, &(pszArgPtr[wArgs][2]) );
  
         /* Allocate a cmd list block */
         if( ( pscNewCmd = ( struct Cmd * ) calloc ( 1, sizeof( struct Cmd ) ) ) == NULL )
         {
            DBG_WriteString( 8, "\n%s: Failed to allocate key command structure %d", szFnName, wArgs );
            break;
         }   
         else
         {
            /* Copy temporary stack member data to list member */
            pscNewCmd->pscNextCmd = NULL;          
            pscNewCmd->dwCmd = scNewCmd.dwCmd;
            strcpy( pscNewCmd->szParam, scNewCmd.szParam );
      
            /* Maintain list pointers */
            if ( pscListPtr == NULL )
            {   
               DBG_WriteString( 8,"\n%s: Allocated key list head", szFnName );
               pscListPtr = pscNewCmd;               
            }
            else
            {
               /* Get last in list */ 
               pscThisCmd = pscListPtr;
               while ( pscThisCmd->pscNextCmd != NULL )
               {
                  pscThisCmd = pscThisCmd->pscNextCmd;
               } 

               /* & add to it */
               DBG_WriteString( 8, "\n%s: Adding switch %d to list", szFnName, wArgs );
               pscThisCmd->pscNextCmd = pscNewCmd;
            }               
            dwCmdCount++;
         }
      }
   }

   return dwCmdCount;   
}



#if defined _MACINTOSH
void MAC_err_display( char* szFnName, Terr dwProblem, char* szDescription )
{
   DBG_WriteString( 10, "Error 0x%08x in %s : %s \n", dwProblem, szFnName, szDescription );
}
#endif


void GUI_EnableErrorHandler( Tboolean Enable )
{
   static char *szFnName = "GUI_EnableErrorHandler";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   if ( Enable )
#if defined _MACINTOSH
      ASL_err_set_reporting( ASL_ERR_SET_HANDLER, MAC_err_display );
#else
      ASL_err_set_reporting( ASL_ERR_SET_HANDLER, ASL_err_display );
#endif
   else
      ASL_err_set_reporting( ASL_ERR_SET_HANDLER, NULL );
}


void GUI_DisplayRates( struct Info *psInfo )
{
   static char *szFnName = "GUI_DisplayRates";

   DBG_WriteString( 1, "\n%s: Entering..", szFnName);

   DBG_WriteString( 10, "\n%s: %d images acquired in %.2f seconds",  psInfo->sGuiInfo.szProgName, psInfo->sSnapInfo.dwFrameCount,   psInfo->sSnapInfo.dFinish - psInfo->sSnapInfo.dStart );
   DBG_WriteString( 10, "\n%s: %d images displayed in %.2f seconds", psInfo->sGuiInfo.szProgName, psInfo->sSnapInfo.dwDisplayCount, psInfo->sSnapInfo.dFinish - psInfo->sSnapInfo.dStart );
   if ( psInfo->sSnapInfo.dwFrameCount != 0 )
   {
      DBG_WriteString( 10, "\n%s: Acquisition rate = %.2f fps", psInfo->sGuiInfo.szProgName, (float) psInfo->sSnapInfo.dwFrameCount   / ( psInfo->sSnapInfo.dFinish - psInfo->sSnapInfo.dStart )  );
      DBG_WriteString( 10, "\n%s: Display rate     = %.2f fps", psInfo->sGuiInfo.szProgName, (float) psInfo->sSnapInfo.dwDisplayCount / ( psInfo->sSnapInfo.dFinish - psInfo->sSnapInfo.dStart )  );
   }
}
