/*************************************************************************
 *                          ACTIVE SILICON LIMITED                      
 *                                                                        
 *  File name   : xpl_thrd.c
 *  Project     : ASLEXAMP maintained Snapper example code
 *  Author      : Mike Harrison, Martin Bone
 *  Systems     : ANSI C, POSIX & WIN32 threading.
 *  Written     : 4-MAR-1998
 *  Updated     : 
 *  Version     : 1:00:00                                                     
 *                                                                        
 *  Copyright 
 **************************************************************************
 * Comments: 
 * Assumptions:
 * Revisions:
 *  Extension 23 Oct 2002 to add Linux pthread definitions
 */

/* Active Silicon Include files */
#include <asl_inc.h>
#include <xpl_inc.h>

#if defined _LYNXOS || defined _SOLARIS2 || defined _LINUX
ui32 ASL_Thrd_Create( struct ThrdInfo *psThrdInfo, void *pfFunction, void* pArg, ui32 dwPriority )
{
   static char *szFnName = "ASL_Thrd_Create";
   ui32 dwStatus = ASL_OK;

#if defined _LYNXOS
/* The extra brackets are required because of the way Lynx uses nasty macros
 * to implement thread creation via the ST_NEW function, we don't want to separate casts
 * from their arguments by bad parsing in macro expansions.
 */
   if( pthread_create( (&psThrdInfo->ptThreadID), psThrdInfo->ptaThreadAttr, ((void*(*)(void*))pfFunction),( (void*) pArg) ) != ASL_OK )
#elif defined _SOLARIS2
   if( pthread_create( (&psThrdInfo->ptThreadID), &psThrdInfo->ptaThreadAttr, (void*(*)(void*))pfFunction, (void*) pArg ) != ASL_OK )
#elif defined _LINUX
   /* Need to pass NULL as the pthread attribute pointer - known limitation of early Linux pthread implementations */
   if( pthread_create( (&psThrdInfo->ptThreadID), NULL, (void*(*)(void*))pfFunction, (void*) pArg ) != ASL_OK )
#endif
   {
      printf("%s: Unable to start thread \n", szFnName );
      dwStatus = ASL_ERROR;
   }
   return dwStatus;
}

ui32 ASL_Thrd_Delete( struct ThrdInfo *psThrdInfo )
{
   return ASLERR_NOT_IMPLEMENTED;
}


ui32 ASL_Thrd_Gettime( double* dTime )
{
#if defined _LINUX
 /* Linux does not include the POSIX4 clock_gettime function */
   struct timeval stTime;
   ui32 dwStatus = ASL_OK;
   if ( gettimeofday( &stTime, NULL ) != ASL_OK )
   {
      *dTime = (double) 0.0;
      return ASL_ERROR;
   }
   else
   {
      *dTime = stTime.tv_sec + 1e-6 * stTime.tv_usec;
      return ASL_OK;
   }
#elif defined _LYNXOS || defined _SOLARIS2
   struct timespec stTime;
   ui32 dwStatus = ASL_OK;
   if ( clock_gettime( CLOCK_REALTIME, &stTime ) != ASL_OK )
   {
      *dTime = (double) 0.0;
      return ASL_ERROR;
   }
   else
   {
      *dTime = stTime.tv_sec + 1e-9 * stTime.tv_nsec;
      return ASL_OK;
   }
#endif

}

#elif defined _WINNT || defined _WIN95
ui32 ASL_Thrd_Create( struct ThrdInfo *psThrdInfo, void *pfFunction, void* pArg, ui32 dwPriority )
{
   psThrdInfo->hThread = CreateThread(
       (LPSECURITY_ATTRIBUTES) NULL,	// pointer to thread security attributes
       (DWORD) 0,	// initial thread stack size, in bytes
       (LPTHREAD_START_ROUTINE)  pfFunction,	// pointer to thread function
       (LPVOID) pArg,	// argument for new thread
       (DWORD) CREATE_SUSPENDED,	// creation flags
       (LPDWORD)  &psThrdInfo->ThrdID 	// pointer to returned thread identifier
      );
   if ( ! SetThreadPriority( psThrdInfo->hThread, THREAD_PRIORITY_TIME_CRITICAL) )
   {
       return ( ASL_ERROR );
   }
   else
   {
      /* Un-suspend thread */
      ResumeThread( psThrdInfo->hThread );
   }
   return ASL_OK;
}

/* Last-resort function - can damage kernel if running
 * kernel code, similarly for dll's. Presumably WIN32 problem - no safe cancellation.
 */
ui32 ASL_Thrd_Delete( struct ThrdInfo *psThrdInfo )
{
   return ASL_ERROR;
}

ui32 ASL_Thrd_Gettime( double* dTime )
{
   /* Get Timing information */
   *dTime = ( double ) clock() / CLOCKS_PER_SEC;
   return ASL_OK;
}
/* end of WIN32 */

#elif defined _MACINTOSH

ui32 ASL_Thrd_Create( struct ThrdInfo *psThrdInfo, void *pfFunction, void* pArg, ui32 dwPriority )
{
   static char *szFnName = "(MAC)ASL_Thrd_Create";

   _UNUSED_VAR( psThrdInfo );
   _UNUSED_VAR( pfFunction );
   _UNUSED_VAR( pArg );
   _UNUSED_VAR( dwPriority );
      
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS is not pre-emptive multithread capable", (Terr) 0, (char*) szFnName );
}

ui32 ASL_Thrd_Delete( struct ThrdInfo *psThrdInfo )
{
   static char *szFnName = "(MAC)ASL_Thrd_Delete";

   _UNUSED_VAR( psThrdInfo );
   ASLGEN_err_ret( ASLERR_NOT_IMPLEMENTED, "MacOS is not pre-emptive multithread capable", 0, szFnName );
}

ui32 ASL_Thrd_Gettime( double* dTime )
{
   static char *szFnName = "(MAC)ASL_Thrd_Gettime";
   
   /* Get Timing information */
   *dTime = ( double ) clock() / CLOCKS_PER_SEC;
   return ASL_OK;
}


#else
#if ! defined __MWERKS__
#error Undefined OS in thread.c
#endif
#endif
