#!/bin/bash

#########################################################################
## install.sh                                                          ##
#########################################################################
# Copyright (c) 2002 Active Silicon Ltd.                                #
# Author: Jon Rennie <jon@jonrennie.com>                                #
#                                                                       #
# This BASH script creates a menu for users to install and test the     #
# Active Silicon Linux device driver for the Snapper framegrabber card. #
# Options are:                                                          #
#  1. install the Active Silicon proprietary snpmod.o kernel module     #
#     a running Linux system, append the required entry to the system   #
#     /etc/modules.conf file, attempt to load the kernel module into    #
#     memory space.                                                     #
#  2. launch a console based test program                               #
#  3. launch selected graphical live video window test apps (XWindows)  #
#                                                                       #
# Option execution is by chaining off further scripts.                  #
#########################################################################
# Version 1.1                                                           #
# Last Modification : 28 Oct 2002                                       #
# Last Author : Jon Rennie                                              #
#                                                                       #
# CHANGELOG                                                             #
# Version 1.0
#   Started from base of LFG-sdk-linux install.sh version 1.2
# 
#########################################################################
# This program is free software; you can redistribute it and/or# modify #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation; either version 2 of the License, or (at #
# your option) any later version.                                       #
#                                                                       #
# This program is distributed in the hope that it will be useful, but   #
# WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     #
# General Public License for more details.                              #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 #
# USA.                                                                  #
#########################################################################

SCRIPTPATH=`pwd`

# Function to prevent the shell from exiting after execution
do_wait() {
echo -n "[Return]"
read R
}

# Find out whether we are running on 2.2 or 2.4 kernel and set vars     #
FAMILY=`uname -r | sed -e 's/2.4.*/2.4/g' | sed -e 's/2.2.*/2.2/g'`

#########################################################################
# End of functions. Start the action :-)                                #
#########################################################################

# check whether tmp filesystem is rw - if not exit
if [ "touch /tmp/testrw" = "0" ]; then
  echo ""
  echo "**********************************************"
  echo " Your /tmp filesystem is currently mounted as"
  echo " read-only. I cannot perform the installation."
  echo " Please remount as read-write and try again."
  echo "**********************************************"
  echo ""
  exit;
fi

##########################################################################
## check we are running as root and that the root filesystem is not ro
##########################################################################
#if [ "$UID" != "0" ]; then
#  echo ""
#  echo "**********************************************"
#  echo " You must be the root user to install this"
#  echo " kernel driver module. Please gain the correct"
#  echo " privileges and try again."
#  echo "**********************************************"
#  echo ""
#  exit;
#fi

# check whether tmp filesystem is rw - if not exit
if [ "touch /root/testrw" = "0" ]; then
  echo ""
  echo "**********************************************"
  echo " Your / filesystem is currently mounted as"
  echo " read-only. I cannot perform the installation."
  echo " Please remount as read-write and try again."
  echo "**********************************************"
  echo ""
  exit;
fi


##########################################################################
## check to see whether we have libncurses
##########################################################################
if [ ! -e /usr/lib/libncurses.so.5 ]; then
  # version of libncurses req for dialog not avail in /usr/lib
  # echo "Failed to find /usr/lib/libncurses.so.5"
  if [ -e /usr/lib/libncurses.so ]; then 
    # we have an earlier version in /usr/lib. Symlink it to libncurses.so.5
    # echo "Found /usr/lib/libncurses.so - will symlink it as /usr/lib/libncurses.so.5"
    ln -s /usr/lib/libncurses.so /usr/lib/libncurses.so.5
  else
    if [ ! -e /lib/libncurses.so.5 ]; then 
      # version of libncurses req for dialog not avail in /lib either
      # echo "Failed to find /lib/libncurses.so.5 either"
      if [ -e /lib/libncurses.so ]; then 
        # we have an earlier version in /lib. Symlink it to libncurses.so.5
        # echo "Found /lib/libncurses.so - will symlink it as /lib/libncurses.so.5"
        ln -s /lib/libncurses.so /lib/libncurses.so.5
      else
        # we do not have libncurses in /lib or /usr/lib. Dump time :-)
        echo ""
        echo "###############################################################################"
        echo "Problem. Your system does not have the required libraries to run this graphical"
        echo "mode installer. You can either install the modules manually (see README.TXT) or"
        echo "install the libncurses libraries and try again. Exiting this installer now."
        echo "###############################################################################"
        echo ""
        exit;
      fi
    fi
  fi
fi


# Create a user unique lockfile
touch /tmp/`whoami`
MENU=/tmp/`whoami`


#########################################################################
# Create the main GUI                                                   #
#########################################################################

# --menu text height width menu-height [ tag item ] ...

$SCRIPTPATH/installer/dialog --title "Active Silicon / Datacell Snapper Cards - Linux 2.4" \
--menu "Device Driver and SDK installation, basic test programs." 24 65 17 \
"Install" "Install the Active Silicon Snapper driver" \
"-------" "----------------------------------------------" \
"Test" "Test the driver in text (console) mode" \
"-------" "----------------------------------------------" \
"Test1" "Area Scan test application - frame capture" \
"Test2" "Area Scan test application - field capture" \
"Test3" "Area Scan test application - threading mode" \
"-------" "----------------------------------------------" \
"readtst" "Documentation for the test applications" \
"-------" "----------------------------------------------" \
"Readme" "View the readme.txt file" \
"Licence" "View the licence for this software" \
"-------" "----------------------------------------------" \
"Modules" "List currently installed module drivers" \
"PCI" "List installed PCI hardware devices" \
"-------" "----------------------------------------------" \
"Exit" "Exit menu" 2> $MENU
SELECT="`cat $MENU`"
# The menu functions

# The menu functions
if [ "$SELECT" = "Install" ]; then
  installer/run-install.sh ;
 ./install.sh
fi

if [ "$SELECT" = "Test" ]; then 
  2.4-kernel-progs/run-test.sh 
 ./install.sh
fi

if [ "$SELECT" = "Test1" ]; then 
  2.4-kernel-progs/run-area.sh s24ansq; 
 ./install.sh
fi

if [ "$SELECT" = "Test2" ]; then 
  2.4-kernel-progs/run-area.sh s24asq; 
 ./install.sh
fi

if [ "$SELECT" = "Test3" ]; then 
  2.4-kernel-progs/run-area.sh s24async; 
 ./install.sh
fi

if [ "$SELECT" = "readtst" ]; then 
  $SCRIPTPATH/installer/dialog --textbox $SCRIPTPATH/$FAMILY-kernel-progs/apps/readme.txt 20 65
 ./install.sh
fi

if [ "$SELECT" = "Readme" ]; then 
  $SCRIPTPATH/installer/dialog --textbox $SCRIPTPATH/readme.txt 20 65
 ./install.sh
fi

if [ "$SELECT" = "Licence" ]; then 
  $SCRIPTPATH/installer/dialog --textbox $SCRIPTPATH/licence.txt 20 65
 ./install.sh
fi

if [ "$SELECT" = "Modules" ]; then
  /sbin/lsmod > /tmp/modlist.txt 
  $SCRIPTPATH/installer/dialog --textbox /tmp/modlist.txt 14 65
  rm /tmp/modlist.txt
 ./install.sh
fi

if [ "$SELECT" = "PCI" ]; then
  cp /proc/pci /tmp/pcilist.txt 
  $SCRIPTPATH/installer/dialog --textbox /tmp/pcilist.txt 20 65
  rm /tmp/pcilist.txt
 ./install.sh
fi

if [ "$SELECT" = "-------" ]; then
 ./install.sh
fi

if [ "$SELECT" = "Exit" ]; then rm /tmp/`whoami` ; exit ;fi
