#!/bin/bash

#########################################################################
# Copyright (c) 2002 Active Silicon Ltd.                                #
# Author: Jon Rennie <jon@jonrennie.com>                                #
#                                                                       #
# This BASH script will install the Active Silicon proprietary snpmod.o #
# kernel module onto a running Linux system, append the required entry  #
# to the system's /etc/modules.conf file, attempt to load the kernel    #
# module into memory space.                                             #
#                                                                       #
#########################################################################
#                                                                       #
# PREQUISITES                                                           #
# 1. A kernel version for which a snpmod.o module is available.         #
# 2. The 'mknod' utility program.                                       #
# 3. The 'modutils' suite.                                              #
# 4. An Active Silicon Snapper framegrabber card installed.             #
#                                                                       #
#########################################################################
#                                                                       #
# LICENSING                                                             #
#                                                                       #
# The Active Silicon snpmod.o module is supplied as a precompiled       #
# binary. It is NOT free software, and is subject to acceptance of the  #
# Active Silicon licensing agreement. Source code is NOT supplied.      #
#                                                                       #
# The test programs supplied are NOT free software, and use is subject  #
# the same license agreement. Source code IS supplied as a reference    #
# for developers, but remains copyright Active Silicon.                 #
#                                                                       #
# The test programs make use of an image compression library (the TMG   #
# JPEG compression library) which is proprietary to Active Silicon. The #
# TMG library is supplied as a linked runtime object file and the use   #
# of it is subject to the licesning agreement referred to above/.       #
#                                                                       #
# This installation script IS free software, covered by the GPL License #
# quoted below.                                                         #
#                                                                       #
# This script needs to be run as root.                                  #
#########################################################################
# This program is free software; you can redistribute it and/or# modify #
# it under the terms of the GNU General Public License as published by  #
# the Free Software Foundation; either version 2 of the License, or (at #
# your option) any later version.                                       #
#                                                                       #
# This program is distributed in the hope that it will be useful, but   #
# WITHOUT ANY WARRANTY; without even the implied warranty of            #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     #
# General Public License for more details.                              #
#                                                                       #
# You should have received a copy of the GNU General Public License     #
# along with this program; if not, write to the Free Software           #
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307 #
# USA.                                                                  #
#########################################################################
# clear
cd installer

SCRIPTPATH=`pwd`
LOGFILE="/tmp/asl-install-log.txt"

#########################################################################
# Find out whether we are running on 2.2 or 2.4 kernel and set vars     #
#########################################################################
VERSION=`uname -r`
FAMILY=`uname -r | sed -e 's/2.4.*/2.4/g' | sed -e 's/2.2.*/2.2/g'`
MODPATH="driver/$FAMILY"
# so FAMILY is either "2.2" or 2.4" and MODPATH is "driver/2.2" or "driver/2.4"

#########################################################################
# Some functions to start with                                          #
#########################################################################
# Function to draw a pretty banner on the screen
Banner () {
$SCRIPTPATH/dialog --infobox \
"  ACTIVE SILICON SNAPPER FRAMEGRABBER CARD

 Active Silicon kernel module driver
  Active Silicon SDK and libraries
 Active Silicon example test programs

Copyright Active Silicon Limited 2002" 10 45
sleep 3
}

# clear out any leftover /tmp/tmpmsg files from previous runs
if [ -f /tmp/tmpmsg ]; then
 rm /tmp/tmpmsg
fi

# Banner 

# Logfile setup
if [ -e $LOGFILE ]; then  # logfile exists - let's copy to backup
  mv $LOGFILE $LOGFILE.bak
  echo "[previous installation logfile preserved as $LOGFILE.bak" >> $LOGFILE
  echo >> $LOGFILE
fi
echo 'Active Silicon Snapper Card driver installation log' >> $LOGFILE
date >> $LOGFILE
echo >> $LOGFILE

##########################################################################
## check we are running as root and that the root filesystem is not ro
##########################################################################
if [ "$UID" != "0" ]; then
cat << EOF > /tmp/tmpmsg

You must be the root user to install this
kernel driver.                   
                                         
Please gain the correct privilege and try
again.                                   
EOF
 $SCRIPTPATH/dialog --title "YOU MUST RUN AS ROOT" --msgbox "`cat /tmp/tmpmsg`" 10 50
 exit;
fi

# check whether tmp filesystem is rw - if not exit
if [ "touch /root/testrw" = "0" ]; then
cat << EOF > /tmp/tmpmsg

**********************************************

 Your root filesystem is currently mounted as
 read-only. I cannot perform the installation.
 Please remount as read-write and try again.

**********************************************
EOF
 $SCRIPTPATH/dialog --title "READ ONLY ROOT FILESYSTEM!" --msgbox "`cat /tmp/tmpmsg`" 12 50
 exit;
fi



############################################################################
#  Check we are being called from the right directory
#  by looking for our $MODULES subdirectory
############################################################################

if [ ! -d $SCRIPTPATH/$MODPATH ]; then
cat << EOF > /tmp/tmpmsg

You must run this script from the directory
it is located in. For example:

 cd /mnt/cdrom
 install.sh

Please try again.
EOF
  $SCRIPTPATH/dialog --title "INCORRECT EXECUTION" --msgbox "`cat /tmp/tmpmsg`" 12 50
  exit;
fi

############################################################################
## control loop that ends when user accepts settings
############################################################################
COMPLETE=no
while [ "$COMPLETE" = "no" ]; do


############################################################################
#    Question and answer.
############################################################################



cat << EOF > /tmp/tmpmsg

This application will install the Active
Silicon Snapper card Linux device driver on to
your system.

A log of files added to your system will be
saved to /tmp/asl-install-log.txt, so you
can manually undo this installation at a
later time if you wish.

EOF
$SCRIPTPATH/dialog --title "DRIVER INSTALLATION" --msgbox "`cat /tmp/tmpmsg`" 13 50

# now, we need to let the user know what we support

cat << EOF > /tmp/tmpmsg

Your kernel is from the $FAMILY 'family' 

In that series, Active Silicon only supports
systems that are running one of the following
kernels:

EOF

# read the kernel versions we support from the directory
# list and append them to the next message we are going
# to show the user
# also need to measure size of the array as $KERNELSNUM
KERNELSNUM=0
for file in $( ls $SCRIPTPATH/$MODPATH | sort )
do
 echo "$file" >> /tmp/tmpmsg
 KERNELSNUM=$(($KERNELSNUM+1))
done
DIALOGHEIGHT=$(($KERNELSNUM+11))

$SCRIPTPATH/dialog --title "DRIVER COMPATIBILITY" --msgbox "`cat /tmp/tmpmsg`" $DIALOGHEIGHT 50

# check that what the user has is one of our supported kernels
# the list of supported ones is currently in /tmp/tmpmsg so if
# grep that for the current $VERSION we can check............

if grep $VERSION /tmp/tmpmsg 
  then rm -f /tmp/tmpmsg
else
  cat << EOF > /tmp/tmpmsg

Your system does not appear to meet the
requirements to use the precompiled ASL
kernel device driver.

Please contact us to check whether we
have a precompiled device driver that
will suit your system. Our email address
is snapper.support@activesilicon.co.uk

Your kernel version is $VERSION
EOF
  $SCRIPTPATH/dialog --title "UNSUPPORTED" --msgbox "`cat /tmp/tmpmsg`" 16 45
  COMPLETE=yes
  exit;
fi

############################################################################
#    Have we guessed right?
############################################################################
cat << EOF > /tmp/tmpmsg
I have detected that you are running
kernel version $VERSION

If this is correct press yes, and I shall
proceed to install your new device driver.

You will be asked to confirm each change
I want to make to your system before it
takes place.
EOF

$SCRIPTPATH/dialog --title "VERSION IDENTIFICATION CHECK" --yesno \
"`cat /tmp/tmpmsg`" 13 50
if [ $? = 0 ]; then
  COMPLETE="yes"
  rm -f /tmp/tmpmsg
else
 cat << EOF > /tmp/tmpmsg

You have chosen not to automatically install
the Active Silicon device driver.

You may be able to install the driver
manually. Please refer to the README.TXT
file.

EOF
$SCRIPTPATH/dialog --title "EXITING" --msgbox "`cat /tmp/tmpmsg`" 11 50
  rm -f /tmp/tmpmsg
  COMPLETE=yes
  exit;
fi

############################################################################
#    Show license agreement and get user agreement.                        #
############################################################################

$SCRIPTPATH/dialog --textbox ../licence.txt 20 62

$SCRIPTPATH/dialog --title "LICENSE AGREEMENT ACCEPTANCE?" --yesno \
"Do you accept the terms of the preceding License Agreement?" 7 50
if [ $? = 0 ]; then
  COMPLETE="yes"
else
 exit;
fi


############################################################################
#    Now go ahead and try and install the new kernel module.
############################################################################


############################################################################
# next, see whether this installation is using modules.conf or conf.modules
# The latter is deprecated, but we need to support both
############################################################################

if [ -e /etc/modules.conf ]; then
  echo "Found /etc/modules.conf"
  MODULES="modules.conf"
else
  MODULES="conf.modules"
fi

############################################################################
# now copy over the new modules
############################################################################

cat << EOF > /tmp/tmpmsg
Do you want me to place a copy of the driver
kernel module in your /lib/modules directory?

This is a required installation step, and 
will not affect or interfere with any other
part of your system.
EOF
$SCRIPTPATH/dialog --title "INSTALL DRIVER?" --yesno \
"`cat /tmp/tmpmsg`" 10 50
if [ $? = 0 ]; then
  rm -f /tmp/tmpmsg
else
  # dump time
  cat << EOF > /tmp/tmpmsg

You chose not to install the driver.

This program will now exit.
EOF
  $SCRIPTPATH/dialog --title "DRIVER INSTALLATION ABORTED" --msgbox "`cat /tmp/tmpmsg`" 8 50
  rm -f /tmp/tmpmsg
  exit;
fi


# Now copy over the modules - bearing in mind its different names to different places

if [ "$FAMILY" = "2.4" ]; then

  cp -f $SCRIPTPATH/$MODPATH/$VERSION/snpmod.o /lib/modules/$VERSION/kernel/drivers/media/video/snpmod.o
  # check the return code
  RETCODE=$?
  if [ "$RETCODE" != "0" ]; then
    cat << EOF > /tmp/tmpmsg

There was a problem copying over the new kernel module.

I tried to copy
 $SCRIPTPATH/$MODPATH/$VERSION/snpmod.o
to
 /lib/modules/$VERSION/kernel/drivers/media/video/snpmod.o

This program will now exit - please check file paths and refer to
the README.TXT file for manual installation instructions.
EOF
    $SCRIPTPATH/dialog --title "INSTALLATION FAILURE" --msgbox "`cat /tmp/tmpmsg`" 14 70
    rm -f /tmp/tmpmsg
    exit;
  else
    echo "Installed new binary module snpmod.o to /lib/modules/$VERSION/kernel/drivers/media/video/snpmod.o" >> $LOGFILE
    cat << EOF > /tmp/tmpmsg

New kernel module was successfully installed to
/lib/modules/$VERSION/kernel/drivers/media/video/snpmod.o

The new driver can be automatically loaded on machine
start up if I create the required entries at the tail of
your /etc/$MODULES file. Do you want me to do this?

EOF
  fi


elif [ "$FAMILY" = "2.2" ]; then
  cp -f $SCRIPTPATH/$MODPATH/$VERSION/snpmod.o /lib/modules/$VERSION/misc/snpmod.o
  # check the return code
  RETCODE=$?
  if [ "$RETCODE" != "0" ]; then
    cat << EOF > /tmp/tmpmsg

There was a problem copying over the new kernel module.

I tried to copy
 $SCRIPTPATH/$MODPATH/$VERSION/snpmod.o
to
 /lib/modules/$VERSION/misc/snpmod.o

This program will now exit - please check file paths and refer to
the README.TXT file for manual installation instructions.
EOF
    $SCRIPTPATH/dialog --title "INSTALLATION FAILURE" --msgbox "`cat /tmp/tmpmsg`" 14 70
    rm -f /tmp/tmpmsg
    exit;
  else
    echo "Installed new binary module snpmod.o to /lib/modules/$VERSION/misc/snpmod.o" >> $LOGFILE
    cat << EOF > /tmp/tmpmsg

New kernel module was successfully installed to
/lib/modules/$VERSION/misc/snpmod.o

The new driver can be automatically loaded on machine
start up if I create the required entries at the tail of
your /etc/$MODULES file. Do you want me to do this?

EOF
  fi

fi

$SCRIPTPATH/dialog --title "KERNEL MODULE INSTALLED" --yesno "`cat /tmp/tmpmsg`" 11 65
if [ $? = 0 ]; then    # user wants me to add the entries to modules.conf

  ############################################################################
  # now append the required lines to /etc/modules.conf
  ############################################################################
  # and populate it with new variables
  if [ "$FAMILY" = "2.4" ]; then
    cat << EOF >> /etc/$MODULES

# Entry for Active Silicon Snapper Card device driver snpmod.o
alias char-major-241      snpmod
# End of entries for Active Silicon Snapper card snpmod.o device driver
EOF
      cat << EOF >> $LOGFILE
Added the following lines to /etc/$MODULES
 # Entry for Active Silicon Snapper Card device driver snpmod.o
 alias char-major-241      snpmod
 # End of entries for Active Silicon Snapper card snpmod.o device driver

EOF
  elif [ "$FAMILY" = "2.2" ]; then
    cat << EOF >> /etc/$MODULES

# Entry for Active Silicon Snapper Card device driver snpmod.o
alias char-major-241      snpmod
# End of entries for Active Silicon Snapper card snpmod.o device driver
EOF
      cat << EOF >> $LOGFILE
Added the following lines to /etc/$MODULES
 # Entry for Active Silicon Snapper Card device driver snpmod.o
 alias char-major-241      snpmod
 # End of entries for Active Silicon Snapper card snpmod.o device driver

EOF
  fi
fi

############################################################################
# now see whether we need to make the snpmod device nodes
############################################################################
cat << EOF > /tmp/tmpmsg

The device driver needs some special /dev 
nodes to function properly. I can now check
whether those nodes already exist, and if 
they do not, create them. Your Snapper card
will not function properly without them.

If I do create them, they will not interfere
with any other part of your system. You are
recommended to let me carry out this check.
May I?
EOF
$SCRIPTPATH/dialog --title "CHECK AND CREATE /dev NODES?" --yesno "`cat /tmp/tmpmsg`" 15 50
if [ $? = 0 ]; then    # user wants me to check / create the /dev nodes
   # is /dev/snapper0 -- /dev/snapper15 there and already a special character device?
   for device in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 ; do
      if [ ! -c /dev/snapper$device ]   # it's not a character device
      then
         if [ -f /dev/snapper$device ] # but it's there. Let's dump it 
         then
            rm -f /dev/snapper$device
            echo "Removed existing device file /dev/snapper$device" >> $LOGFILE
         fi
         # either it wasn't there in the first place or we dumped it as the wrong type
         # so let's make a new snapperX node
         mknod /dev/snapper$device c 241 $device
         echo "Created Snapper device node /dev/snapper$device" >> $LOGFILE
      fi
  done
  chmod 666 /dev/snapper*

  $SCRIPTPATH/dialog --title "DEVICE NODES OKAY" --msgbox \
"Required /dev nodes okay. I shall now try and
load the device driver into memory" 6 50
else
  $SCRIPTPATH/dialog --title "DEVICE NODES NOT CHECKED" --msgbox \
"/dev nodes NOT checked. I shall now try and
load the device driver into memory" 6 50
fi

############################################################################
# now run a depmod and modprobe the snpmod (2.4 series) or snpmod (2.2) module
############################################################################
/sbin/depmod -ae
if [ $FAMILY = "2.2" ]; then
  DRIVER="snpmod"
else
  DRIVER="snpmod"
fi

echo "/sbin/modprobe $DRIVER"
/sbin/modprobe $DRIVER

# check the return code
RETCODE=$?
if [ "$RETCODE" != "0" ]; then
SUCCESS="yes"
cat << EOF > /tmp/tmpmsg

The device driver was successfully installed,
but could not be loaded into memory.

Perhaps the Snapper card is not yet installed?

Check your hardware, perhaps by examining
the /proc/pci file, and try to load the
card manually.

Refer to the README.TXT file for instructions.
EOF
 $SCRIPTPATH/dialog --title "MODULE LOADING FAILURE" --msgbox "`cat /tmp/tmpmsg`" 14 50
 rm -f /tmp/tmpmsg
else
 cat << EOF > /tmp/tmpmsg

New kernel module was successfully loaded into
memory. It is now available as for use.

Installation completed.

EOF
$SCRIPTPATH/dialog --title "INSTALLATION COMPLETE" --msgbox "`cat /tmp/tmpmsg`" 9 50

fi

rm -f /tmp/tmpmsg


cat << EOF > /tmp/tmpmsg

Do you want to view the installation log?

EOF
$SCRIPTPATH/dialog --title "View installation logile?" --yesno \
"`cat /tmp/tmpmsg`" 6 50
if [ $? = 0 ]; then    # user wants to view the log
  $SCRIPTPATH/dialog --textbox $LOGFILE 20 65
fi

rm -f /tmp/tmpmsg
done
